# $Header: /cvsroot/NorthStar/Makefile.PL,v 1.3 2002/05/21 23:48:30 brownkid Exp $

# Check for required modules (borrowed from libwww).
use ExtUtils::MakeMaker;

$| = 1;
my $missing_modules = 0;

print "Checking for DBI...";
eval {
      require DBI;
      DBI->require_version('1.20');
      };
if ($@) {
   print "failed\n";
   $missing_modules++;
   print <<"EOT";

ERROR: $@
DBI is required but wasn't found.  You must install the DBI-1.20 module
   (or a newer version) to use NorthStar.

EOT
}
else {
   print "ok\n";
}

print "Checking for CGI...";
eval {
      require CGI;
      };
if ($@) {
   print "failed\n";
   $missing_modules++;
   print <<"EOT";

ERROR: $@
CGI is required but wasn't found.  You must install the CGI-2.79 module
(or a newer version) to use NorthStar.

EOT
}
else {
   print "ok\n";
}


if ($missing_modules) {
   print <<"EOT";
Missing modules can be obtained from the Comprehensive Perl Archive
Network (CPAN) at <URL:http://www.perl.org/CPAN/>.

*** You can't use NorthStar until any missing modules are installed ***

EOT
exit(1);
}

print << "EOT";


Welcome to the NorthStar installation script.  This script
will help guide you through most of the steps required to
get NorthStar up and running.  

EOT

while(($k,$v) = each %Config) {
   print "$k=$v\n";
}

my($r);
$r = AskBool('Would you like to configure NorthStar now',1);

if($r == 1) {
   %dirs = Configure();
} else {
   $notconfigured = 1;
   print << "EOT";

You have chosen not to configure NorthStar.  Please be 
sure that your config file (/etc/NorthStar.conf) is
present before continuing.  Make sure you copy the files
in the 'html' directory to your public NorthStar directory

EOT
exit(1);
}

open(F, ">.dirs") || die "could not open file .dirs: $!\n";
print F "$dirs{'tmpldir'}\n";
print F "$dirs{'cgidir'}\n";
close(F);

WriteMakefile(
   'NAME'		=> 'NorthStar',
   'VERSION_FROM'	=> 'lib/NorthStar.pm',
   'PMLIBDIRS'		=> ['lib'],
);

sub AskBool {
   my($question) = shift;
   my($default) = shift;
   my($a);
   if($default == 1) {
      $a = "[Y/n]";
   } else {
      $a = "[y/N]";
   }
   my($done);
   while($done != 1) {
      print "$question $a? ";
      $c = <STDIN>;
      chop($c);
      if($c =~ /^n$/i) {
         $ret = 0;
         $done = 1;
      }
      if($c =~ /^y$/i) {
         $ret = 1;
         $done = 1;
      }
      if($c =~ /^$/) {
         $ret = $default;
         $done = 1;
      }
   }
   return($ret);
}


sub AskString {
   my($question) = shift;
   my($default) = shift || undef;
   my($a);
   if($default) {
      $a = "[$default]";
   } else {
      $a = "[]";
   }
   my($done);
   while($done != 1) {
      print "$question $a? ";
      $c = <STDIN>;
      chop($c);
      if($c =~ /^.*$/) {
         $ret = $c;
         $done = 1;
      }
      if($c =~ /^$/ && $default) {
         $ret = $default;
         $done = 1;
      }
   }
   return($ret);
}

sub AskList {
   my($question) = shift;
   my($options) = shift;
   my($returns) = shift;
   my($done);
   while($done != 1) {
      print "$question\n";
      $i = 1;
      foreach(@{$options}) {
         print "  $i) $_\n";
         $i++;
      }

      print "Enter your choice: ";
      $c = <STDIN>;
      chop($c);
      if($c =~ /\d+/) {
         if($c > 0 && $c < $i) {
            $ret = ${$returns}[$c-1];
            $done = 1;
         }
      }
   }
   return($ret)
}
sub Configure {
   print "\n";
   $email = AskString('The NorthStar administrator email address','user@email.com');
   $htdoc = AskString('Please enter the path to your website document root','/usr/local/htdocs');
   if(!-d $htdoc) {
      print "\nThe document root does not exist.  Please create it and try again\n\n";
      exit(1);
   }
   $htmldir = AskString("  The directory to install the HTML files in","$htdoc/NorthStar");

   $cgidir = AskString("Please enter your cgi-bin directory path",'/usr/local/httpd/cgi-bin');
   if(!-d $cgidir) {
      print "\nThe CGI directory does not exist.  Please create it and try again\n\n";
      exit(1);
   }

   $hostname = AskString("Please enter your webserver hostname",'www.yourserver.com');
   $cgiurl = AskString("  The URL to your cgi-bin directory","http://$hostname/cgi-bin");

   $temp = $htmldir;
   $temp =~ s/$htdoc//g;
   $tmplurl = AskString("  The URL to your template directory","http://$hostname$temp");

   print "\n";
   $dbtype = AskList('Select your database type',[qw(PostgreSQL MySQL)],[qw(Pg mysql)]);

   $dbhost = AskString('The hostname of your database server','db.yourserver.com');
   if($dbtype eq 'mysql') {
      $dbport = AskString('The port number to connect to','3306');
   }
   if($dbtype eq 'Pg') {
      $dbport = AskString('The port number to connect to','5432');
   }

   $dbname = AskString('The name of the database to use','northstar');
   $dbuser = AskString('The username to connect to the database with','northstar');
   $dbpass = AskString('The password to use when connecting');

   if(-f "/etc/NorthStar.conf") {
      $r = AskBool('Would you like to overwrite your existing configuration file',1);
      if($r == 1) {
         $filename = "/etc/NorthStar.conf";
      } else {
         $filename = AskString('Enter a name to save the configuration as');
      }
   }
   open(CONFIG, ">$filename") || die("open of $filename failed: $!\n");
   print CONFIG << "EOT";
<Global>
    AdminEmail      $email
    ScriptURL       $cgiurl/NorthStar.cgi
    TemplateURL     $tmplurl
    HTMLDir         $htmldir
#   UseModules		ModuleName
</Global>

<Database>
    DatabaseDriver  $dbtype
    DatabaseHost    $dbhost
    DatabasePort    $dbport
    DatabaseName    $dbname
    DatabaseUser    $dbuser
    DatabasePass    $dbpass
</Database>

EOT
   close(CONFIG) || die("close of $filename failed: $!\n");

   my(%ret) = (
         'tmpldir' => $htmldir,
         'cgidir' => $cgidir
      );
   return(%ret);
}
