#!/usr/bin/perl
package NorthStarUpdater;

# 
# Copyright (C) 2001, Hitesh Patel <hitesh@presys.com>
# 
# This file is part of NorthStar.
#
# NorthStar is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# NorthStar is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with NorthStar; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#

use DBI;
use Net::IP;

$VERSION = "1.3";
my(@searchdir) = qw(/opt/NorthStar /usr/NorthStar /usr/local/NorthStar);
my($t);

foreach(@searchdir) {
    if(-f "$_/NorthStar.conf") {
        ConfigInit("$_/NorthStar.conf");
        $t = 1;
        last;
    }
}

if(!$t) {
    die("Unable to find the configuration file.\n");
}

my(%config) = ConfigGetSection('Global');
my(%dbconfig) = ConfigGetSection('Database');
my($lib_dir) = $config{'InstallDir'} . "/lib/";

require $lib_dir . 'dbutil.lib';

my(%dbinfo) = (
  'fh'      => 'db',		              # Database FH, DONT change this
  'driver'	=> $dbconfig{'DatabaseDriver'},# DBI database driver
  'host'    => $dbconfig{'DatabaseHost'}, # The host address of your db server
  'port'    => $dbconfig{'DatabasePort'}, # Database port
  'options' => '',		                  # Unused
  'tty'     => '',		                  # Unused
  'dbname'  => $dbconfig{'DatabaseName'}, # The database to connect to
  'dbuser'  => $dbconfig{'DatabaseUser'}, # The user to connect as
  'dbpass'  => $dbconfig{'DatabasePass'}  # The password to use
);

if(db_connect(\%dbinfo) != 0) {
	ExitError("db_connect: " . $DBI::errstr);
}

main();

sub main {
    my($c, $sth, $sql);

    print 
"





Welcome to the NorthStar Database Updater.  This updater
should only be used with version $VERSION of NorthStar.  

Please make sure you have backed up your existing database
before running this program... we wouldn't want you to 
loose all your data would we? ;-)

";
    print "Would you like to continue with the update [y/n]? ";
    $c = <STDIN>;
    chop($c);
    lc($c);
    if($c ne 'y') {
        ExitError("Aborting update");
    }


    $sql = "SELECT label FROM networks WHERE id = '0'";
    $sth = $db->prepare($sql);
    $sth->execute();
    if(db_rescheck(\$sth,'check_dbver','0') < 0) {
        UpdateNetworks();
        UpdateAddresses();
    }
    UpdateUsers();
    UpdateSchema();
    print << 'EOF';


Ok, were done.  Give it a run and see how it
works!!

EOF

}

sub UpdateNetworks {
    my($sql, $sth, %nets, @x, @row);

    $sql = "ALTER TABLE networks RENAME TO networks_old";
    $sth = $db->prepare($sql);
	$sth->execute();
	db_rescheck(\$sth,'rename_old_net_table','1');

    $sql = "DROP INDEX networks_pkey";
    $sth = $db->prepare($sql);
	$sth->execute();
	db_rescheck(\$sth,'delete_old_net_pkeyindex','1');

    $sql =
"CREATE TABLE networks (".
"	id		INT4 NOT NULL DEFAULT nextval('networks_id_seq') PRIMARY KEY,".
"	parent		INT4 NOT NULL REFERENCES networks(id) MATCH FULL ON DELETE CASCADE ON UPDATE CASCADE DEFAULT '0',".
"	name		VARCHAR(256) NOT NULL,".
"	descr		TEXT,".
"	label		VARCHAR(256) NOT NULL,".
"	network		NUMERIC(40,0) NOT NULL,".
"	masklen		INT4 NOT NULL,".
"	owner		INT4 NOT NULL REFERENCES owners(id) MATCH FULL ON DELETE CASCADE ON UPDATE CASCADE,".
"	device		INT4 NOT NULL REFERENCES devices(id) MATCH FULL ON DELETE CASCADE ON UPDATE CASCADE,".
"	assignuser	INT4 NOT NULL REFERENCES users(id) MATCH FULL ON UPDATE CASCADE,".
"	assigndate	TIMESTAMP DEFAULT now(),".
"	updateuser	INT4 REFERENCES users(id) MATCH FULL ON UPDATE CASCADE,".
"	upddate		TIMESTAMP DEFAULT now()".
");";

    $sth = $db->prepare($sql);
	$sth->execute();
	db_rescheck(\$sth,'create_new_net_table','1');

	$sth = $db->prepare("SELECT * FROM networks_old ORDER BY id");
	$sth->execute;
	db_rescheck(\$sth,'get_old_net_data','1');

	for($i = 0; $i < $sth->rows; $i++) {
		@row = $sth->fetchrow_array;
		$nets{$row[0]}{id} = $row[0];
		$nets{$row[0]}{parent} = $row[1];
		$nets{$row[0]}{name} = $row[2];
		$nets{$row[0]}{descr} = $row[3];
		$nets{$row[0]}{network} = $row[4];
		$nets{$row[0]}{owner} = $row[5];
		$nets{$row[0]}{device} = $row[6];
		$nets{$row[0]}{assignuser} = $row[7];
		$nets{$row[0]}{assigndate} = $row[8];
		$nets{$row[0]}{updateuser} = $row[9];
		$nets{$row[0]}{upddate} = $row[10];
		push(@x, $row[0]);
	}

	foreach(@x) {
		$ip = new Net::IP($nets{$_}{network},4) or ExitError(Net::IP::Error());	
		$nets{$_}{masklen} = $ip->prefixlen || 0;
		$nets{$_}{int} = $ip->intip || 0;
		$r = $nets{$_};
		print "$r->{id} $r->{network} = $r->{int}/$r->{masklen}\n";
		$sth = $db->prepare("INSERT INTO networks VALUES ('$r->{id}','$r->{parent}','$r->{name}','$r->{descr}','$r->{network}','$r->{int}','$r->{masklen}','$r->{owner}','$r->{device}','$r->{assignuser}','$r->{assigndate}','$r->{updateuser}','$r->{upddate}');\n");
		$sth->execute;
		db_rescheck(\$sth,'add_new_net_entry','1');

	}

    $sql = "DROP TABLE networks_old";
    $sth = $db->prepare($sql);
	$sth->execute();
	db_rescheck(\$sth,'drop_old_net_table','1');

}

sub UpdateAddresses {
    my($sql, $sth, %addr, @x, @row);

    $sql = "ALTER TABLE addresses RENAME TO addresses_old";
    $sth = $db->prepare($sql);
	$sth->execute();
	db_rescheck(\$sth,'rename_old_addr_table','1');

    $sql = "DROP INDEX addresses_pkey";
    $sth = $db->prepare($sql);
	$sth->execute();
	db_rescheck(\$sth,'delete_old_addr_pkeyindex','1');

    $sql = 
"CREATE TABLE addresses (".
"	id              INT4 NOT NULL DEFAULT nextval('addresses_id_seq') PRIMARY KEY,".
"	parent		INT4 NOT NULL REFERENCES networks(id) MATCH FULL ON DELETE CASCADE ON UPDATE CASCADE,".
"	address		VARCHAR(256) NOT NULL,".
"	owner		INT4 NOT NULL REFERENCES owners(id) MATCH FULL ON DELETE CASCADE ON UPDATE CASCADE,".
"	device		INT4 NOT NULL REFERENCES devices(id) MATCH FULL ON DELETE CASCADE ON UPDATE CASCADE,".    
"	dnsname		VARCHAR(128),".
"	macaddr		VARCHAR(20),".
"	misc		TEXT,".
"	assignuser	INT4 NOT NULL REFERENCES users(id) MATCH FULL ON UPDATE CASCADE,".
"	assigndate	TIMESTAMP DEFAULT now(),".
"	updateuser	INT4 REFERENCES users(id) MATCH FULL ON UPDATE CASCADE,".
"	upddate		TIMESTAMP DEFAULT now()".
");";

    $sth = $db->prepare($sql);
	$sth->execute();
	db_rescheck(\$sth,'create_new_net_table','1');
       
    $sth = $db->prepare("SELECT * FROM addresses_old ORDER BY id");
	$sth->execute;
	db_rescheck(\$sth,'get1','1');

	for($i = 0; $i < $sth->rows; $i++) {
		@row = $sth->fetchrow_array;
		$addr{$row[0]}{id} = $row[0];
		$addr{$row[0]}{parent} = $row[1];
		$addr{$row[0]}{address} = $row[2];
		$addr{$row[0]}{owner} = $row[3];
		$addr{$row[0]}{device} = $row[4];
		$addr{$row[0]}{dnsname} = $row[5];
		$addr{$row[0]}{macaddr} = $row[6];
		$addr{$row[0]}{misc} = $row[7];
		$addr{$row[0]}{assignuser} = $row[8];
		$addr{$row[0]}{assigndate} = $row[9];
		$addr{$row[0]}{updateuser} = $row[10];
		$addr{$row[0]}{upddate} = $row[11];
		push(@x, $row[0]);
	}

	foreach(@x) {
		$r = $addr{$_};
		print "$r->{id} $r->{address}\n";
		$sth = $db->prepare("INSERT INTO addresses VALUES ('$r->{id}','$r->{parent}','$r->{address}','$r->{owner}','$r->{device}','$r->{dnsname}','$r->{macaddr}','$r->{misc}','$r->{assignuser}','$r->{assigndate}','$r->{updateuser}','$r->{upddate}');\n");
		$sth->execute;
		db_rescheck(\$sth,'add_new_addr_entry','1');
	}

    $sql = "DROP TABLE addresses_old";
    $sth = $db->prepare($sql);
	$sth->execute();
	db_rescheck(\$sth,'drop_old_addr_table','1');

}


sub UpdateUsers {
    my($sth, $i, %users, @s, $permmask);
    $sth = $db->prepare("SELECT * FROM users");
    $sth->execute;
    db_rescheck(\$sth, 'get_users','1');

    my(@perms) = qw(
perm_addloc perm_updloc perm_delloc perm_addowner perm_updowner perm_delowner perm_adddev perm_upddev perm_deldev perm_addnet perm_updnet perm_delnet perm_addaddr perm_addaddrrange perm_updaddr perm_deladdr perm_adduser perm_upduser perm_deluser
);


    for($i = 0; $i < $sth->rows; $i++) {
        my(@row) = $sth->fetchrow_array;
        $users{$row[0]}{'id'} = $row[0];
        $users{$row[0]}{'username'} = $row[1];
		$users{$row[0]}{'passwd'} = $row[2];
		$users{$row[0]}{'realname'} = $row[3];
		$users{$row[0]}{'enabled'} = $row[4];
		$users{$row[0]}{'perm_addnet'} = $row[5];
		$users{$row[0]}{'perm_updnet'} = $row[6];
		$users{$row[0]}{'perm_delnet'} = $row[7];
		$users{$row[0]}{'perm_addaddr'} = $row[8];
		$users{$row[0]}{'perm_addaddrrange'} = $row[8];
		$users{$row[0]}{'perm_updaddr'} = $row[9];
		$users{$row[0]}{'perm_deladdr'} = $row[10];
		$users{$row[0]}{'perm_adddev'} = $row[11];
		$users{$row[0]}{'perm_upddev'} = $row[12];
		$users{$row[0]}{'perm_deldev'} = $row[13];
		$users{$row[0]}{'perm_adduser'} = $row[14];
		$users{$row[0]}{'perm_upduser'} = $row[15];
		$users{$row[0]}{'perm_deluser'} = $row[16];
		$users{$row[0]}{'perm_addowner'} = $row[17];
		$users{$row[0]}{'perm_updowner'} = $row[18];
		$users{$row[0]}{'perm_delowner'} = $row[19];
		$users{$row[0]}{'perm_addloc'} = $row[20];
		$users{$row[0]}{'perm_updloc'} = $row[21];
		$users{$row[0]}{'perm_delloc'} = $row[22];
        push(@s, $row[0]);
    }

    $sth = $db->prepare("DROP TABLE users");
    $sth->execute;
    db_rescheck(\$sth, 'drop_users_table','1');

    $sth = $db->prepare(
"CREATE TABLE users (".
"	id		INT4 NOT NULL DEFAULT nextval('users_id_seq') PRIMARY KEY,".
"	username	VARCHAR(32) NOT NULL UNIQUE,".
"	passwd		VARCHAR(32) NOT NULL,".
"	realname	VARCHAR(128) NOT NULL,".
"	enabled		BOOL DEFAULT 'T',".
"	permmask	VARCHAR(128)".
");");
    $sth->execute;
    db_rescheck(\$sth,'create_users_table','1');

    foreach(@s) {
        $permmask = '';
        if($_ == 0) {
            $permmask = "1111111111111111111111111111XXXXXXXXXXXXXXXX11111111111111111111XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX";
        } else {
            foreach $p (@perms) {
                if($users{$_}{$p} == 1) {
                    $permmask .= "1";
                } else {
                    $permmask .= "0";
                }
            }
            $permmask .= "000000000XXXXXXXXXXXXXXXX00000111111111111111XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX";
        }
        my(@x) = ($users{$_}{'id'},$users{$_}{'username'},$users{$_}{'password'},$users{$_}{'realname'},
                  $users{$_}{'enabled'}, $permmask);
        $sth = $db->prepare("INSERT INTO users VALUES('$x[0]','$x[1]','$x[2]','$x[3]','$x[4]','$x[5]');");
        $sth->execute;
        db_rescheck(\$sth,'insert_new_user','1');
        print "user: $_ $permmask\n";
    }
}

sub UpdateSchema {
    my($sth);

    my(@sql) = (
"UPDATE users SET passwd = 'ekGaihSJjmIL.' WHERE id = '0' AND passwd = 'UNUSED';",
"CREATE SEQUENCE templates_id_seq START 6;",
"CREATE TABLE templates (
	id		INT4 NOT NULL DEFAULT nextval('templates_id_seq') PRIMARY KEY,
	objtype		INT2,
	name		VARCHAR(64) NOT NULL,
	descr		TEXT
);",
"GRANT ALL ON templates TO northstar;",
"CREATE SEQUENCE fielddefs_id_seq START 16;",
"CREATE TABLE fielddefs (
	id 		INT4 NOT NULL DEFAULT nextval('fielddefs_id_seq') PRIMARY KEY,
	templateid	INT4,
	name		VARCHAR(64),
	descr		TEXT,
	required	BOOL DEFAULT 'F',
	regex		INT4,
	minlength	INT4,
	maxlength	INT4,
	sortkey		INT4,
	fieldtype	VARCHAR(32),
	fieldopt	VARCHAR(255),
	updatemode	INT2
);",
"GRANT ALL ON fielddefs TO northstar;",
"CREATE SEQUENCE regex_id_seq START 14;",
"CREATE TABLE regex (
	id		INT4 NOT NULL DEFAULT nextval('regex_id_seq') PRIMARY KEY,
	name		VARCHAR(64),
	example		VARCHAR(64),
	regex		VARCHAR(255)
);",
"GRANT ALL ON regex TO northstar;",
"CREATE TABLE fielddata (
	id		    INT4 NOT NULL,
    objtype     INT4 NOT NULL,
	fieldid		INT4 NOT NULL,
	data		TEXT
);",
"GRANT ALL ON fielddata TO northstar;",
"INSERT INTO templates VALUES('0','0','Default Template','Default Template');",
"INSERT INTO templates VALUES('1','1','Default Owner Template','Default Owner Template');",
"INSERT INTO templates VALUES('2','2','Default Location Template','Default Location Template');",
"INSERT INTO templates VALUES('3','3','Default Device Template','Default Device Template');",
"INSERT INTO templates VALUES('4','4','Default Network Template','Default Network Template');",
"INSERT INTO templates VALUES('5','5','Default Address Template','Default Address Template');",
"INSERT INTO regex VALUES('1','Email Address','xxx\@yyy.zzz','([\\\\w\\\\-\\\\.\\\\+]+)\\\\\@([\\\\w\\\\-\\\\.]+)\\\\.(\\\\w{3}|\\\\w{2})');",
"INSERT INTO regex VALUES('2','Phone Number','111-222-3333','^[\\\\d\\\\-\\\\(\\\\)\\\\ ]+\$');",
"INSERT INTO regex VALUES('3','State','XX','([.\\\\w{2}])');",
"INSERT INTO regex VALUES('4','Zip Code','55555','([.\\\\d{5}])');",
"INSERT INTO regex VALUES('5','Zip Code Long','55555-4444','([.\\\\d{5}]\\\\-[.\\\\d{4}])');",
"INSERT INTO regex VALUES('6','Zip Code Both','55555, 55555-4444','^(\\\\d{5}\\\\-\\\\d{4}|\\\\d{5})\$');",
"INSERT INTO regex VALUES('7','Alphanumeric','A-Z, a-z, 0-9, .#&-\@\,','^[\\\\d\\\\w\\\\ \\\\.\\\\#\\\\&\\\\,\\\\\\'\\\\-\\\\\@\\\\W]+\$');",
"INSERT INTO regex VALUES('8','Number','0-9','^\\\\d+\$');",
"INSERT INTO regex VALUES('9','Date','mm-dd-yy','^(\\\\d{1,2}\\\\/\\\\d{1,2}\\\\/\\\\d{2})\$');",
"INSERT INTO regex VALUES('10','IPv4 Address','255.255.255.255','^[0-9]+\\\\.[0-9]+\\\\.[0-9]+\\\\.[0-9]+\$');",
"INSERT INTO regex VALUES('11','CIDR Address','10.0.0.0/8','^[0-9]+\\\\.[0-9]+\\\\.[0-9]+\\\\.[0-9]+\\\\/\\\\d+\$');",
"INSERT INTO regex VALUES('12','MAC Address','00:22:55:aa:bb:ff','^[0-9a-fA-F]{2}\\\\:[0-9a-fA-F]{2}\\\\:[0-9a-fA-F]{2}\\\\:[0-9a-fA-F]{2}\\\\:[0-9a-fA-F]{2}\\\\:[0-9a-fA-F]{2}\$');",
"INSERT INTO regex VALUES('13','Anything','anything','.*');",
"INSERT INTO fielddefs VALUES('1','5','MAC Address','MAC Address of the interface using this address','F',
                             '12','0','20','1','TEXT','','2');",
"INSERT INTO fielddefs VALUES('2','1','Street Address','Street Address of the Owner','T','7','0','100','1','TEXT','','2');",
"INSERT INTO fielddefs VALUES('3','1','City','City of the Owner','T','7','0','100','2','TEXT','','2');",
"INSERT INTO fielddefs VALUES('4','1','State','State of the Owner','F','3','0','50','3','TEXT','','2');",
"INSERT INTO fielddefs VALUES('5','1','Zip/Postal Code','Zip Code of the Owner','T','6','0','50','4','TEXT','','2');",
"INSERT INTO fielddefs VALUES('6','1','POC Name','Point of Contact Name','T','7','0','50','5','TEXT','','2');",
"INSERT INTO fielddefs VALUES('7','1','POC Email','Point of Contact Email','T','1','0','50','6','TEXT','','2');",
"INSERT INTO fielddefs VALUES('8','1','POC Phone','Point of Contact Phone','T','2','0','50','7','TEXT','','2');",
"INSERT INTO fielddefs VALUES('9','1','POC Pager','Point of Contact Pager','F','2','0','50','8','TEXT','','2');",
"INSERT INTO fielddefs VALUES('10','1','POC Cell','Point of Contact Cell','F','2','0','50','9','TEXT','','2');",
"INSERT INTO fielddefs VALUES('11','1','Tech Name','Tech Point of Contact Name','F','7','0','50','10','TEXT','','2');",
"INSERT INTO fielddefs VALUES('12','1','Tech Email','Tech Point of Contact Email','F','1','0','50','11','TEXT','','2');",
"INSERT INTO fielddefs VALUES('13','1','Tech Phone','Tech Point of Contact Phone','F','2','0','50','12','TEXT','','2');",
"INSERT INTO fielddefs VALUES('14','1','Tech Pager','Tech Point of Contact Pager','F','2','0','50','13','TEXT','','2');",
"INSERT INTO fielddefs VALUES('15','1','Tech Cell','Tech Point of Contact Cell','F','2','0','50','14','TEXT','','2');",
"INSERT INTO fielddata SELECT id, 5 as objtype, 1 as fieldid, macaddr FROM addresses;",
"INSERT INTO fielddata SELECT id, 1 as objtype, 2 as fieldid, address FROM owners;",
"INSERT INTO fielddata SELECT id, 1 as objtype, 3 as fieldid, city FROM owners;",
"INSERT INTO fielddata SELECT id, 1 as objtype, 4 as fieldid, state FROM owners;",
"INSERT INTO fielddata SELECT id, 1 as objtype, 5 as fieldid, zip FROM owners;",
"INSERT INTO fielddata SELECT id, 1 as objtype, 6 as fieldid, pocname FROM owners;",
"INSERT INTO fielddata SELECT id, 1 as objtype, 7 as fieldid, pocemail FROM owners;",
"INSERT INTO fielddata SELECT id, 1 as objtype, 8 as fieldid, pocphone FROM owners;",
"INSERT INTO fielddata SELECT id, 1 as objtype, 9 as fieldid, pocpager FROM owners;",
"INSERT INTO fielddata SELECT id, 1 as objtype, 10 as fieldid, poccell FROM owners;",
"INSERT INTO fielddata SELECT id, 1 as objtype, 11 as fieldid, techname FROM owners;",
"INSERT INTO fielddata SELECT id, 1 as objtype, 12 as fieldid, techemail FROM owners;",
"INSERT INTO fielddata SELECT id, 1 as objtype, 13 as fieldid, techphone FROM owners;",
"INSERT INTO fielddata SELECT id, 1 as objtype, 14 as fieldid, techpager FROM owners;",
"INSERT INTO fielddata SELECT id, 1 as objtype, 15 as fieldid, techcell FROM owners;",
"CREATE TABLE owners_old AS SELECT
	id, 1+0 as templateid, name, descr 
FROM owners;",
"DROP TABLE owners;",
"CREATE TABLE owners (
	id              INT4 NOT NULL DEFAULT nextval('owners_id_seq') PRIMARY KEY,  	
	templateid	INT4 NOT NULL DEFAULT '1',
	name		VARCHAR(256) NOT NULL,
	descr		TEXT
);",
"INSERT INTO owners SELECT * FROM owners_old;",
"DROP TABLE owners_old;",
"GRANT ALL ON owners TO northstar;",
"CREATE TABLE locations_old AS SELECT 
	id, parent, 2+0 as templateid, name, descr, assignuser, assigndate,
	updateuser, upddate 
FROM locations;",
"DROP TABLE locations;",
"CREATE TABLE locations (
	id              INT4 NOT NULL DEFAULT nextval('locations_id_seq') PRIMARY KEY,    
	parent		INT4 NOT NULL REFERENCES locations(id) MATCH FULL ON DELETE CASCADE ON UPDATE CASCADE DEFAULT '0',	
	templateid	INT4 NOT NULL DEFAULT '2',
	name		VARCHAR(256) NOT NULL,
	descr		TEXT,
	assignuser	INT4 REFERENCES users(id) MATCH FULL ON UPDATE CASCADE,
	assigndate	TIMESTAMP DEFAULT now(),
	updateuser	INT4 REFERENCES users(id) MATCH FULL ON UPDATE CASCADE,
	upddate		TIMESTAMP DEFAULT now()
);",
"INSERT INTO locations SELECT * FROM locations_old;",
"DROP TABLE locations_old;",
"GRANT ALL ON locations TO northstar;",
"CREATE TABLE devices_old AS SELECT 
	id, parent, 3+0 as templateid, name, descr, owner, location, assignuser,
	assigndate, updateuser, upddate 
FROM devices;",
"DROP TABLE devices;",
"CREATE TABLE devices (
	id		INT4 NOT NULL DEFAULT nextval('devices_id_seq') PRIMARY KEY, 
	parent		INT4 NOT NULL REFERENCES devices(id) MATCH FULL ON DELETE CASCADE ON UPDATE CASCADE DEFAULT '0',	
	templateid	INT4 NOT NULL DEFAULT '3',
	name		VARCHAR(256) NOT NULL,
	descr		TEXT,
	owner		INT4 NOT NULL REFERENCES owners(id) MATCH FULL ON DELETE CASCADE ON UPDATE CASCADE, 
	location	INT4 NOT NULL REFERENCES locations(id) MATCH FULL ON DELETE CASCADE ON UPDATE CASCADE, 
	assignuser	INT4 NOT NULL REFERENCES users(id) MATCH FULL ON UPDATE CASCADE,
	assigndate	TIMESTAMP DEFAULT now(),
	updateuser	INT4 REFERENCES users(id) MATCH FULL ON UPDATE CASCADE,
	upddate		TIMESTAMP DEFAULT now()
);",
"INSERT INTO devices SELECT * FROM devices_old;",
"DROP TABLE devices_old;",
"GRANT ALL ON devices TO northstar;",
"CREATE TABLE networks_old AS SELECT
	id, parent, 4+0 as templateid, name, descr, label, network, masklen, 
	owner, device, assignuser, assigndate, updateuser, upddate 
FROM networks;",
"DROP TABLE networks;",
"CREATE TABLE networks (
	id		INT4 NOT NULL DEFAULT nextval('networks_id_seq') PRIMARY KEY,
	parent		INT4 NOT NULL REFERENCES networks(id) MATCH FULL ON DELETE CASCADE ON UPDATE CASCADE DEFAULT '0',
	templateid	INT4 NOT NULL DEFAULT '4',
	name		VARCHAR(256) NOT NULL,
	descr		TEXT,
	label		VARCHAR(256) NOT NULL,
	network		NUMERIC(40,0) NOT NULL,
	masklen		INT4 NOT NULL,
	owner		INT4 NOT NULL REFERENCES owners(id) MATCH FULL ON DELETE CASCADE ON UPDATE CASCADE,
	device		INT4 NOT NULL REFERENCES devices(id) MATCH FULL ON DELETE CASCADE ON UPDATE CASCADE,
	assignuser	INT4 NOT NULL REFERENCES users(id) MATCH FULL ON UPDATE CASCADE,
	assigndate	TIMESTAMP DEFAULT now(),
	updateuser	INT4 REFERENCES users(id) MATCH FULL ON UPDATE CASCADE,
	upddate		TIMESTAMP DEFAULT now()
);",
"INSERT INTO networks SELECT * FROM networks_old;",
"DROP TABLE networks_old;",
"GRANT ALL ON networks TO northstar;",
"CREATE TABLE addresses_old AS SELECT
	id, parent, 5+0 as templateid, address, owner, device, dnsname,
	misc, assignuser, assigndate, updateuser, upddate 
FROM addresses;",
"DROP TABLE addresses;",
"CREATE TABLE addresses (
	id              INT4 NOT NULL DEFAULT nextval('addresses_id_seq') PRIMARY KEY,        
	parent		INT4 NOT NULL REFERENCES networks(id) MATCH FULL ON DELETE CASCADE ON UPDATE CASCADE,
	templateid	INT4 NOT NULL DEFAULT '5',
	address		VARCHAR(256) NOT NULL,
	owner		INT4 NOT NULL REFERENCES owners(id) MATCH FULL ON DELETE CASCADE ON UPDATE CASCADE,  
	device		INT4 NOT NULL REFERENCES devices(id) MATCH FULL ON DELETE CASCADE ON UPDATE CASCADE,     
	dnsname		VARCHAR(128),
	misc		TEXT,
	assignuser	INT4 NOT NULL REFERENCES users(id) MATCH FULL ON UPDATE CASCADE,
	assigndate	TIMESTAMP DEFAULT now(),
	updateuser	INT4 REFERENCES users(id) MATCH FULL ON UPDATE CASCADE,
	upddate		TIMESTAMP DEFAULT now()
);",
"INSERT INTO addresses SELECT * FROM addresses_old;",
"DROP TABLE addresses_old;",
"GRANT ALL ON addresses TO northstar;"
);
    foreach(@sql) {
        #print $_ . "\n";

        $sth = $db->prepare($_);
        $sth->execute;
        db_rescheck(\$sth,"update_db_schema: $_",'1');
    }
}


sub ExitError {
    my($msg) = $_[0];
    my($package, $filename, $line) = caller;
    print "\nERROR: $msg at $package:$filename:$line\n";
    exit(1);
}
    
sub ConfigGetSection {
        my $sec = shift;
        my(%t);
        
        my($h) = $NorthStar::CONFIG{$sec};
        while(my($k,$v) = each %$h) {
                $t{$k} = $v;
        }
        return(%t);
}

sub ConfigGetValue {
        if(scalar(@_) == 3) {
                return($NorthStar::CONFIG{$_[0]}{$_[1]}{$_[2]});
        } elsif(scalar(@_) == 2) {
                return($NorthStar::CONFIG{$_[0]}{$_[1]});
        } else {
                return(undef);
        }
}

sub ConfigInit {
        my $cf = shift;
        my($cs, $csv, $cn, $cv, @j, $x);
        open(_CONFIG, "<$cf") || die("config file open failed: $cf: $!\n");
        while(<_CONFIG>) {
                chomp;
                s/^[\s\t]+//g; #strip leading whitespace
                next if /^#/;
                next if /^$/;
                s/\s+$//g; #strip trailing whitespace

                if(/\#/) {
                    /(.*)\#.*/;
                    $x = $1;
                } else {
                    $x = $_;
                }

                # given:
                # <Section SectionVariable>
                #                Name = Value
                # </Section>
                # $cs  = Section                Config Section        
                # $csv = SectionVariable        Config Section Variable 
                # $cn  = Name                   Config Name
                # $cv  = Value                  Config Variable
                # 
                # $csv may not be defined if a SectionVariable was not present
                if($x =~ /^<(.*)>$/ && $x !~ /^<\/.*>/) {           # handler for section starts <Section>
                        if(!defined($cs)) {              # check to see if the previous section was closed with </Section> 
                                $cs = $1;
                        } else {
                                die("syntax error in configuration file \"$cf\" at line $.: " .
                                    "previous configuration section not closed (maybe you forgot </$cs>)\n");
                        }

                        # sub-handler for sections with a variable <Section SectionVariable>
                        if($x =~ /^<(.*) (.*)>$/ && (@j = split(/\s+/, $x)) == 2) {
                                $cs = $1;
                                $csv = $2;
                        } elsif($x =~ /^<(.*)>$/) { # dont clobber sections with no SectionVariable (<Section> ..)
                                $cs = $1;
                        } else {
                                die("syntax error in configuration file \"$cf\" at line $.: " .
                                    "configuration sections may only have one parameter (<Section Parameter>)\n");
                        }
                        next;
                } elsif($x =~ /^<\/(.*)>$/) {            #handler for section closes </Section>
                        if(defined($cs)) {         #check to see if this section was ever opened
                                undef($cs);
                                undef($csv);
                        } else {
                                $cs = $1;          #set section name for error message
                                $cs =~ s/\///g;    #strip leading '/'
                                die("syntax error in configuration file \"$cf\" at line $.: " .
                                    "configuration section not opened (maybe you forgot <$cs>)\n");
                        }
                        next;
                } else {
                        if($x =~ /(.*) \"(.*)\"/) {               #process line "Name = "Value In Here"
                                $cn = $1;                                
                                $cv = $2;
                                $cv =~ s/\\(.)/$1/g;              #normalize escaped charecters like: \"
                        } elsif((@j = split(/\s+/, $x)) > 2) {    #process line "Name = Value1 Value2..."
                                $cn = shift(@j);
                                $cv = join(',', @j);              #make a comma delimited list
                        } elsif((@j = split(/\s+/, $x)) == 2) {   #process line "Name = Value"
                                $cn = $j[0];
                                $cv = $j[1];
                        } else {
                                die("syntax error in configuration file \"$cf\" at line $.\n");
                        }

                        if($csv) { #if SectionVariable was set this as CONFIG->Section->SectionVariable->Name=Value
                                #$self->Debug("add value: CONFIG->$cs->$csv->$cn=$cv");
                                $NorthStar::CONFIG{$cs}{$csv}{$cn} = $cv;
                        } else {   #else set this value under CONFIG->Section->Name=Value
                                #$self->Debug("add value: CONFIG->$cs->$cn=$cv");
                                $NorthStar::CONFIG{$cs}{$cn} = $cv;
                        }
                        
                        next;
                }
                
                
        }
        #$self->Debug("closing config file");
        close(_CONFIG) || die("close failed: $cf: $!\n");

        return 1;
}

sub ConfigPrint {
        if(!defined(%NorthStar::CONFIG)) {
                return;
        }
        
        while(my($k,$v) = each %NorthStar::CONFIG) {
                print STDERR "$k\n";
                while(my($y,$z) = each %$v) {
                        if(ref($z) eq "HASH") {
                                print STDERR " $y\n";
                                while(my($a,$b) = each %$z) {
                                        print STDERR "  $a=$b\n";
                                }
                        } else {
                                print STDERR " $y=$z\n";
                        }
                }
                print STDERR "\n";
        }
}
