#!/usr/bin/perl
package NorthStarUpdater;

use DBI;

$VERSION = "1.6";
my(@searchdir) = qw(/opt/NorthStar /usr/NorthStar /usr/local/NorthStar);
my($t);

foreach(@searchdir) {
    if(-f "$_/NorthStar.conf") {
        ConfigInit("$_/NorthStar.conf");
        $t = 1;
        last;
    }
}

if(!$t) {
    die("Unable to find the configuration file.\n");
}

my(%config) = ConfigGetSection('Global');
my(%dbconfig) = ConfigGetSection('Database');
my($lib_dir) = $config{'InstallDir'} . "/lib/";

require $lib_dir . 'dbutil.lib';

my(%dbinfo) = (
  'fh'      => 'db',		              # Database FH, DONT change this
  'driver'	=> $dbconfig{'DatabaseDriver'},# DBI database driver
  'host'    => $dbconfig{'DatabaseHost'}, # The host address of your db server
  'port'    => $dbconfig{'DatabasePort'}, # Database port
  'options' => '',		                  # Unused
  'tty'     => '',		                  # Unused
  'dbname'  => $dbconfig{'DatabaseName'}, # The database to connect to
  'dbuser'  => $dbconfig{'DatabaseUser'}, # The user to connect as
  'dbpass'  => $dbconfig{'DatabasePass'}  # The password to use
);

if(db_connect(\%dbinfo) != 0) {
	ExitError("db_connect: " . $DBI::errstr);
}

main();

sub main {
    my($c, $sth, $sql);

    print 
"





Welcome to the NorthStar Database Updater.  This updater
should only be used with version $VERSION of NorthStar.  

Please make sure you have backed up your existing database
before running this program... we wouldn't want you to 
loose all your data would we? ;-)

";
    print "Would you like to continue with the update [y/n]? ";
    $c = <STDIN>;
    chop($c);
    lc($c);
    if($c ne 'y') {
        ExitError("Aborting update");
    }
    if($dbconfig{'DatabaseDriver'} =~ /pg/i) {
        PgsqlUpdateNetworks();
    } elsif($dbconfig{'DatabaseDriver'} =~ /mysql/i) {
        MysqlUpdateNetworks();
    } else {
        ExitError("Database type not supported.");
    }
    UpdateAddresses();
    UpdateUsers();

    print << 'EOF';


Ok, were done.  Give it a run and see how it
works!!

EOF

}

sub PgsqlUpdateNetworks {
    my($sql, $sth, %nets, @x, @row);


    $sql = "CREATE TABLE networks_old AS SELECT ".
             "id, parent, templateid, viewid, status, name, descr, ".
	         "label, network, masklen, owner, device, location, ".
	         "5+0, assignuser, assigndate, updateuser, upddate ".
           "FROM networks;";
    $sth = $db->prepare($sql);
	$sth->execute();
	db_rescheck(\$sth,'create_old_net_table','1');

    $sql = "DROP TABLE networks";
    $sth = $db->prepare($sql);
	$sth->execute();
	db_rescheck(\$sth,'delete_net_table','1');

    $sql =
"CREATE TABLE networks (".
"	id		INT4 NOT NULL DEFAULT nextval('networks_id_seq') PRIMARY KEY,".
"	parent		INT4 NOT NULL REFERENCES networks(id) MATCH FULL ON DELETE CASCADE ON UPDATE CASCADE DEFAULT '0',".
"	templateid	INT4 NOT NULL DEFAULT '4',".
"	viewid		INT4 NOT NULL,".
"	status		INT4 NOT NULL,".
"	name		VARCHAR(256) NOT NULL,".
"	descr		TEXT,".
"	label		VARCHAR(256) NOT NULL,".
"	network		NUMERIC(40,0) NOT NULL,".
"	masklen		INT4 NOT NULL,".
"	owner		INT4 NOT NULL REFERENCES owners(id) MATCH FULL ON DELETE CASCADE ON UPDATE CASCADE,".
"	device		INT4 NOT NULL REFERENCES devices(id) MATCH FULL ON DELETE CASCADE ON UPDATE CASCADE,".
"	location	INT4 NOT NULL REFERENCES locations(id) MATCH FULL ON DELETE CASCADE ON UPDATE CASCADE,".
"	priority	INT4 NOT NULL DEFAULT '5',".
"	assignuser	INT4 NOT NULL REFERENCES users(id) MATCH FULL ON UPDATE CASCADE,".
"	assigndate	TIMESTAMP DEFAULT now(),".
"	updateuser	INT4 REFERENCES users(id) MATCH FULL ON UPDATE CASCADE,".
"	upddate		TIMESTAMP DEFAULT now()".
");";

    $sth = $db->prepare($sql);
	$sth->execute();
	db_rescheck(\$sth,'create_new_net_table','1');

	$sth = $db->prepare("INSERT INTO networks SELECT * FROM networks_old;");
	$sth->execute;
	db_rescheck(\$sth,'copy_net_data','1');

    $sql = "DROP TABLE networks_old";
    $sth = $db->prepare($sql);
	$sth->execute();
	db_rescheck(\$sth,'drop_old_net_table','1');

    $sql = "GRANT ALL ON networks TO northstar;";
    $sth = $db->prepare($sql);
	$sth->execute();
	db_rescheck(\$sth,'set_permissions_net_table','1');

}

sub MysqlUpdateNetworks {
    my($sql, $sth, %nets, @x, @row);


    $sql = "CREATE TABLE networks_old AS SELECT ".
             "id, parent, templateid, viewid, status, name, descr, ".
	         "label, network, masklen, owner, device, location, ".
	         "5+0, assignuser, assigndate, updateuser, upddate ".
           "FROM networks;";
    $sth = $db->prepare($sql);
	$sth->execute();
	db_rescheck(\$sth,'mysql_create_old_net_table','1');

    $sql = "DROP TABLE networks";
    $sth = $db->prepare($sql);
	$sth->execute();
	db_rescheck(\$sth,'mysql_delete_net_table','1');

    $sql =

"CREATE TABLE networks (".
"	id		INT4 NOT NULL AUTO_INCREMENT PRIMARY KEY,".
"	parent		INT4 NOT NULL DEFAULT '0' REFERENCES networks(id) MATCH FULL ON DELETE CASCADE ON UPDATE CASCADE,".
"	templateid	INT4 NOT NULL DEFAULT '4',".
"	viewid		INT4 NOT NULL,".
"	status		INT4 NOT NULL,".
"	name		VARCHAR(255) NOT NULL,".
"	descr		TEXT,".
"	label		VARCHAR(255) NOT NULL,".
"	network		NUMERIC(40,0) NOT NULL,".
"	masklen		INT4 NOT NULL,".
"	owner		INT4 NOT NULL REFERENCES owners(id) MATCH FULL ON DELETE CASCADE ON UPDATE CASCADE,".
"	device		INT4 NOT NULL REFERENCES devices(id) MATCH FULL ON DELETE CASCADE ON UPDATE CASCADE,".
"	location	INT4 NOT NULL REFERENCES locations(id) MATCH FULL ON DELETE CASCADE ON UPDATE CASCADE,".
"	priority	INT4 NOT NULL DEFAULT '5',".
"	assignuser	INT4 NOT NULL REFERENCES users(id) MATCH FULL ON UPDATE CASCADE,".
"	assigndate	TIMESTAMP,".
"	updateuser	INT4 REFERENCES users(id) MATCH FULL ON UPDATE CASCADE,".
"	upddate		TIMESTAMP".
");";

    $sth = $db->prepare($sql);
	$sth->execute();
	db_rescheck(\$sth,'mysql_create_new_net_table','1');

	$sth = $db->prepare("INSERT INTO networks SELECT * FROM networks_old WHERE id = 0;");
	$sth->execute;
	db_rescheck(\$sth,'mysql_copy_net_data_default','1');

	$sth = $db->prepare("UPDATE networks SET id = '0' WHERE label = '0.0.0.0/0';");
	$sth->execute;
	db_rescheck(\$sth,'mysql_copy_net_data_setdefault','1');

	$sth = $db->prepare("INSERT INTO networks SELECT * FROM networks_old WHERE id != 0;");
	$sth->execute;
	db_rescheck(\$sth,'mysql_copy_net_data_all','1');
    
    $sql = "DROP TABLE networks_old";
    $sth = $db->prepare($sql);
	$sth->execute();
	db_rescheck(\$sth,'mysql_drop_old_net_table','1');

}

sub UpdateAddresses {
    my($sql, $sth, %addr, @x, @row);

    $sth = $db->prepare("SELECT * FROM addresses ORDER BY id");
	$sth->execute;
	db_rescheck(\$sth,'get1','1');

	for($i = 0; $i < $sth->rows; $i++) {
		@row = $sth->fetchrow_array;
		$addr{$row[0]}{id} = $row[0];
		$addr{$row[0]}{parent} = $row[1];
		$addr{$row[0]}{templateid} = $row[2];
		$addr{$row[0]}{viewid} = $row[3];
		$addr{$row[0]}{address} = $row[4];
		$addr{$row[0]}{owner} = $row[5];
		$addr{$row[0]}{device} = $row[6];
		$addr{$row[0]}{dnsname} = $row[7];
		$addr{$row[0]}{misc} = $row[8];
		$addr{$row[0]}{assignuser} = $row[9];
		$addr{$row[0]}{assigndate} = $row[10];
		$addr{$row[0]}{updateuser} = $row[11];
		$addr{$row[0]}{upddate} = $row[12];
		push(@x, $row[0]);
	}

	foreach(@x) {
		$r = $addr{$_};
        $r->{label} = "$r->{address}/32";
        $r->{netmask} = 32;
        $r->{network} = IP_DQToInt($r->{address});
        $sth = $db->prepare("SELECT location FROM networks WHERE id = '$r->{parent}';");
		$sth->execute;
		db_rescheck(\$sth,'get_parent_location','1');
        $r->{location} = ($sth->fetchrow_array)[0];
		print "$r->{id} $r->{address} ";

		$sql = "INSERT INTO networks ".
            "(parent,templateid,viewid,status,name,descr,label,network,masklen,".
            "owner,device,location,priority,assignuser,assigndate,updateuser,".
            "upddate) VALUES ('$r->{'parent'}','$r->{'templateid'}','$r->{'viewid'}',".
            "'2','$r->{'dnsname'}','$r->{'misc'}','$r->{'label'}','$r->{'network'}',".
            "'$r->{'netmask'}','$r->{'owner'}','$r->{'device'}','$r->{'location'}',".
            "'5','$r->{'assignuser'}','$r->{'assigndate'}','$r->{'updateuser'}',".
            "'$r->{'upddate'}');";
    
        $sth = $db->prepare($sql);
        $sth->execute;
		db_rescheck(\$sth,'add_new_addr_entry','1');

        $sql = "SELECT id FROM networks WHERE label = '$r->{'label'}';";
        $sth = $db->prepare($sql);
        $sth->execute;
		db_rescheck(\$sth,'get_new_addr_id','1');
        $r->{'newid'} = ($sth->fetchrow_array)[0];
        print "fielddata($r->{'id'} -> $r->{'newid'}) ";
        $sql = "UPDATE fielddata SET id = '$r->{'newid'}' WHERE id = '$r->{'id'}' AND objtype = '5';";
        $sth = $db->prepare($sql);
        $sth->execute;
		db_rescheck(\$sth,'update_template_id','1');
        print $sth->rows . "\n";
	}

    $sql = "DROP TABLE addresses";
    $sth = $db->prepare($sql);
	$sth->execute();
	db_rescheck(\$sth,'drop_addr_table','1');

}



sub UpdateUsers {
    my($sth, $i, %users, @s, $permmask);
    $sth = $db->prepare("UPDATE users SET permmask = '11111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111' WHERE id = '0';");
    $sth->execute;
    db_rescheck(\$sth, 'update_admin_permmask','1');
}

sub IP_DQToBin {
    my @dq;
	my $q;
	my $i;
	my $bin;

	foreach $q (split /\./,$_[0]) {
		push @dq,$q;
	}
	for ($i = 0; $i < 4 ; $i++) {
		if (! defined $dq[$i]) {
			push @dq,0;
		}
	}
	$bin    = pack("CCCC",@dq);      # 4 unsigned chars
	return $bin;
}

sub IP_BinToInt {
	return unpack("N",$_[0]);
}

sub IP_DQToInt {
    return(IP_BinToInt(IP_DQToBin($_[0])));
}

sub ExitError {
    my($msg) = $_[0];
    my($package, $filename, $line) = caller;
    print "\nERROR: $msg at $package:$filename:$line\n";
    exit(1);
}
    
sub ConfigGetSection {
        my $sec = shift;
        my(%t);
        
        my($h) = $NorthStar::CONFIG{$sec};
        while(my($k,$v) = each %$h) {
                $t{$k} = $v;
        }
        return(%t);
}

sub ConfigGetValue {
        if(scalar(@_) == 3) {
                return($NorthStar::CONFIG{$_[0]}{$_[1]}{$_[2]});
        } elsif(scalar(@_) == 2) {
                return($NorthStar::CONFIG{$_[0]}{$_[1]});
        } else {
                return(undef);
        }
}

sub ConfigInit {
        my $cf = shift;
        my($cs, $csv, $cn, $cv, @j, $x);
        open(_CONFIG, "<$cf") || die("config file open failed: $cf: $!\n");
        while(<_CONFIG>) {
                chomp;
                s/^[\s\t]+//g; #strip leading whitespace
                next if /^#/;
                next if /^$/;
                s/\s+$//g; #strip trailing whitespace

                if(/\#/) {
                    /(.*)\#.*/;
                    $x = $1;
                } else {
                    $x = $_;
                }

                # given:
                # <Section SectionVariable>
                #                Name = Value
                # </Section>
                # $cs  = Section                Config Section        
                # $csv = SectionVariable        Config Section Variable 
                # $cn  = Name                   Config Name
                # $cv  = Value                  Config Variable
                # 
                # $csv may not be defined if a SectionVariable was not present
                if($x =~ /^<(.*)>$/ && $x !~ /^<\/.*>/) {           # handler for section starts <Section>
                        if(!defined($cs)) {              # check to see if the previous section was closed with </Section> 
                                $cs = $1;
                        } else {
                                die("syntax error in configuration file \"$cf\" at line $.: " .
                                    "previous configuration section not closed (maybe you forgot </$cs>)\n");
                        }

                        # sub-handler for sections with a variable <Section SectionVariable>
                        if($x =~ /^<(.*) (.*)>$/ && (@j = split(/\s+/, $x)) == 2) {
                                $cs = $1;
                                $csv = $2;
                        } elsif($x =~ /^<(.*)>$/) { # dont clobber sections with no SectionVariable (<Section> ..)
                                $cs = $1;
                        } else {
                                die("syntax error in configuration file \"$cf\" at line $.: " .
                                    "configuration sections may only have one parameter (<Section Parameter>)\n");
                        }
                        next;
                } elsif($x =~ /^<\/(.*)>$/) {            #handler for section closes </Section>
                        if(defined($cs)) {         #check to see if this section was ever opened
                                undef($cs);
                                undef($csv);
                        } else {
                                $cs = $1;          #set section name for error message
                                $cs =~ s/\///g;    #strip leading '/'
                                die("syntax error in configuration file \"$cf\" at line $.: " .
                                    "configuration section not opened (maybe you forgot <$cs>)\n");
                        }
                        next;
                } else {
                        if($x =~ /(.*) \"(.*)\"/) {               #process line "Name = "Value In Here"
                                $cn = $1;                                
                                $cv = $2;
                                $cv =~ s/\\(.)/$1/g;              #normalize escaped charecters like: \"
                        } elsif((@j = split(/\s+/, $x)) > 2) {    #process line "Name = Value1 Value2..."
                                $cn = shift(@j);
                                $cv = join(',', @j);              #make a comma delimited list
                        } elsif((@j = split(/\s+/, $x)) == 2) {   #process line "Name = Value"
                                $cn = $j[0];
                                $cv = $j[1];
                        } else {
                                die("syntax error in configuration file \"$cf\" at line $.\n");
                        }

                        if($csv) { #if SectionVariable was set this as CONFIG->Section->SectionVariable->Name=Value
                                #$self->Debug("add value: CONFIG->$cs->$csv->$cn=$cv");
                                $NorthStar::CONFIG{$cs}{$csv}{$cn} = $cv;
                        } else {   #else set this value under CONFIG->Section->Name=Value
                                #$self->Debug("add value: CONFIG->$cs->$cn=$cv");
                                $NorthStar::CONFIG{$cs}{$cn} = $cv;
                        }
                        
                        next;
                }
                
                
        }
        #$self->Debug("closing config file");
        close(_CONFIG) || die("close failed: $cf: $!\n");

        return 1;
}

sub ConfigPrint {
        if(!defined(%NorthStar::CONFIG)) {
                return;
        }
        
        while(my($k,$v) = each %NorthStar::CONFIG) {
                print STDERR "$k\n";
                while(my($y,$z) = each %$v) {
                        if(ref($z) eq "HASH") {
                                print STDERR " $y\n";
                                while(my($a,$b) = each %$z) {
                                        print STDERR "  $a=$b\n";
                                }
                        } else {
                                print STDERR " $y=$z\n";
                        }
                }
                print STDERR "\n";
        }
}
