/*
# 
# Copyright (C) 2001, Hitesh Patel <hitesh@presys.com>.
# All rights reserved.
# 
# This file is part of NorthStar and is covered under
# the terms of the Apache Software License version 1.1
#
# A copy of this license is included in the LICENSE file
#
# $Header: /cvsroot/NorthStar/java/BKTreeApplet.java,v 1.16 2002/04/29 09:41:05 brownkid Exp $
#
*/

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import java.util.*;
import java.net.*;
import java.io.*;
import java.net.MalformedURLException;
import java.applet.AppletContext.*; 
import java.applet.*;

public class BKTreeApplet extends JApplet {
	private URL url = null;
	private Hashtable objList;
	private ObjInfo selectedObject;
	private String treePath;
	private String treePathParent;
	private TreePath myPath;
	private PopupMenu popup;
	JButton topButtons[];
	JButton botButtons[];
	MenuItem popupItems[];
	Hashtable buttonHash;
	Hashtable popupHash;

	class InitTree extends JScrollPane implements TreeWillExpandListener {
		final JTree tree;
		ObjInfo objLoading;
		DefaultMutableTreeNode nodeLoading;

		public InitTree(final String clickURL, final String shiftClickURL) {
	        DefaultMutableTreeNode root = new DefaultMutableTreeNode(getParameter("viewName"));
	        createObjNodes(root);
		
	        tree = new JTree(root);
	        tree.setEditable(false);
	        tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
			tree.setScrollsOnExpand(true);
	
			tree.addTreeSelectionListener(new TreeSelectionListener() {
				public void valueChanged(TreeSelectionEvent e) {
					DefaultMutableTreeNode node = (DefaultMutableTreeNode)tree.getLastSelectedPathComponent();
	        
					if (node == null) return;
					if (node.isRoot() == true) return;

					Object nodeInfo = node.getUserObject();
					ObjInfo obj = (ObjInfo)nodeInfo;
					selectedObject = obj;

					StringTokenizer blockParts = new StringTokenizer(obj.getName(), getParameter("botButtonDisableDelim"));
					String left = new String(blockParts.nextToken());
					String right = new String(blockParts.nextToken());
					if(right.equals(new String(getParameter("botButtonDisableStr"))) == true) {
						String one = new String("1");

						// Disable bottom row buttons
						int tempint = Integer.parseInt(getParameter("botNumButtons"));
						for(int i = 0; i < tempint; i++) {
							if(getParameter("botButtonDisable" + i) != null) {
								if(one.equals(new String(getParameter("botButtonDisable" + i))) == true) {
									botButtons[i].setEnabled(false);
								}
							} else {
								botButtons[i].setEnabled(true);
							}
						}

						// Disable popup menuitems
						tempint = Integer.parseInt(getParameter("popupItems"));
						for(int i = 0; i < tempint; i++) {		
							if(getParameter("popupDisable" + i) != null) {
								if(one.equals(new String(getParameter("popupDisable" + i))) == true) {
									popupItems[i].setEnabled(false);
								}
							} else {
								popupItems[i].setEnabled(true);
							}
						}
					} else {
						int tempint = Integer.parseInt(getParameter("botNumButtons"));
						for(int i = 0; i < tempint; i++) {		
							botButtons[i].setEnabled(true);
						}

						tempint = Integer.parseInt(getParameter("popupItems"));
						for(int i = 0; i < tempint; i++) {		
							popupItems[i].setEnabled(true);
						}
					}	

					TreePath fullPath = tree.getSelectionPath();
					int pathCount = fullPath.getPathCount();
					TreePath tempPath;
					treePath = "";
					treePathParent = "";
					for(int i = 1; i < pathCount; i++) {
						DefaultMutableTreeNode tempNode = (DefaultMutableTreeNode)fullPath.getPathComponent(i);
						Object tempObj = tempNode.getUserObject();
						ObjInfo tempObjInfo = (ObjInfo)tempObj;
						Integer tempInt = tempObjInfo.getID();
						if(treePath.length() > 0) {
							treePath = treePath + ":" + tempInt.toString();
						} else {
							treePath = tempInt.toString();
						}

						if(i != (pathCount - 1)) {
							if(treePathParent.length() > 0) {
								treePathParent = treePathParent + ":" + tempInt.toString();
							} else {
								treePathParent = tempInt.toString();
							}
						}
					}
				}
			});

			MouseListener ml = new MouseAdapter() {
			public void mousePressed(MouseEvent e) {
				if(e.getClickCount() == 2 && e.isShiftDown()) {
			        int selRow = tree.getRowForLocation(e.getX(), e.getY());
			        if(selRow != -1) {
						String tempURL;
						tempURL = strReplace(shiftClickURL, new String("$PATH$"), treePath);
						tempURL = strReplace(tempURL, new String("$PATHPARENT$"), treePathParent);
						tempURL = strReplace(tempURL, new String("$NAME$"), selectedObject.getName());
						tempURL = strReplace(tempURL, new String("$ID$"), selectedObject.getID().toString());
						showURL(tempURL, new String("bottom"));
					}
				} 
				
				if(e.getClickCount() == 2 && !e.isShiftDown()) {
			        int selRow = tree.getRowForLocation(e.getX(), e.getY());
			        if(selRow != -1) {
						String tempURL;
						tempURL = strReplace(clickURL, new String("$PATH$"), treePath);
						tempURL = strReplace(tempURL, new String("$PATHPARENT$"), treePathParent);
						tempURL = strReplace(tempURL, new String("$NAME$"), selectedObject.getName());
						tempURL = strReplace(tempURL, new String("$ID$"), selectedObject.getID().toString());
						showURL(tempURL, new String("bottom"));
					}
		        }
				if (e.isPopupTrigger()) {
					popup.show(e.getComponent(), e.getX(), e.getY());
				}
			}
			public void mouseReleased(MouseEvent e) {
				if (e.isPopupTrigger()) {
					popup.show(e.getComponent(), e.getX(), e.getY());
				}
			}
			};
			tree.addMouseListener(ml);
			tree.setShowsRootHandles(true);
	        tree.putClientProperty("JTree.lineStyle", "Angled");
            setViewportView(tree);

			tree.makeVisible(myPath);
			if(myPath.getLastPathComponent() != null) {
				tree.expandRow(tree.getRowForPath(myPath));
			}
	
			// Add the expand listener
			// ** Do this last, it breaks the path code for some reason
			tree.addTreeWillExpandListener(this);
		}

	    private void createObjNodes(DefaultMutableTreeNode root) {
			int i;

			addChildNodes(root, new Integer(0));
			StringTokenizer paramTreePath = new StringTokenizer(getParameter("treePath"), ":");
			int depth = paramTreePath.countTokens();
			Object[] pathObj = new Object[depth+1];

			pathObj[0] = root;
			for(i = 0; i < depth; i++) {
				Integer id = new Integer(paramTreePath.nextToken());
				Object nodeObj = objList.get(id.toString());
				DefaultMutableTreeNode node = (DefaultMutableTreeNode)nodeObj;
				if(node != null) {
					node.removeAllChildren();
					pathObj[i+1] = node;
					addChildNodes(node, id);
				}
			}

			myPath = new TreePath(pathObj);
		}

        public void treeWillExpand(TreeExpansionEvent e) throws ExpandVetoException {
			Integer nodeID = null;
			
			// Get the node for the item expanded
			TreePath expandedPath = e.getPath();
			DefaultMutableTreeNode tempNode = (DefaultMutableTreeNode)expandedPath.getLastPathComponent();

			// Set the id to 0 if the root node was selected
			// othwerside set to the ID of the node selected
			if(tempNode.isRoot() == false) {
				Object tempObj = tempNode.getUserObject();
				ObjInfo tempObjInfo = (ObjInfo)tempObj;
				nodeID = tempObjInfo.getID();
			} else {
				nodeID = new Integer(0);
			}

			// Clear the "Loading..." placeholder node
			tempNode.removeAllChildren();
			addChildNodes(tempNode, nodeID);
        }

		private void addChildNodes(DefaultMutableTreeNode node, Integer parent) {
			URL url = null;
			String inputLine;
			BufferedReader in = null;
			// Create our URL to the CGI
			try {
				url = new URL(getParameter("scriptURL") + getParameter("scriptGetChildCmd") + parent);
			}
			catch (MalformedURLException v) {
				System.out.println(v);
			}

			// Try to get the data.
			try	{
				in = new BufferedReader(new InputStreamReader(url.openStream()));
			}
			catch (IOException i) {
				System.out.println(i);
			}

			try {
				while ((inputLine = in.readLine()) != null) {
					addNodeByLine(node, inputLine);
				}
			}
			catch (IOException i) {					
				System.out.println(i); 
			}
		
			try	{
				in.close();
			}
			catch (IOException i) {	
				System.out.println(i);
			}
		}


		private void addNodeByLine(DefaultMutableTreeNode node, String line) {
			StringTokenizer st = new StringTokenizer(line, ";");
			Integer id = new Integer(st.nextToken());
			String idstr = new String(id.toString());
			Integer parentid = new Integer(st.nextToken());
			String descr = new String(st.nextToken());
			String name = new String(st.nextToken());
			Integer children = new Integer(st.nextToken());
			int intChildren = children.intValue();
			ObjInfo newObj = new ObjInfo(id, parentid, intChildren, descr, name);
			DefaultMutableTreeNode newNode = new DefaultMutableTreeNode(newObj);
			objList.put(idstr, newNode);
			node.add(newNode);

			if(intChildren > 0) {
				ObjInfo objLoading = new ObjInfo(new Integer(-1), new Integer(-1), -1, new String("Loading..."), new String(""));
				DefaultMutableTreeNode nodeLoading = new DefaultMutableTreeNode(objLoading);
				newNode.add(nodeLoading);
			}
		}

        public void treeWillCollapse(TreeExpansionEvent e) {
			;
        }
        
		public void makeVisible(TreePath path) {
			tree.makeVisible(path);
		}

		public void expandRow(int row) {
			tree.expandRow(row);
		}

		public int getRowForPath(TreePath path) {
			return(tree.getRowForPath(path));
		}
	}

	public void showURL(String urlString, String frame) {
		try {
			System.out.println("showURL: url=" + urlString + " frame=" + frame);
            url = new URL(urlString);
		}
		catch (MalformedURLException v) {
		}
	   	getAppletContext().showDocument(url, frame);
	}

	public String strReplace(String str, String tag, String replace) {
		String ret;
		ret = new String();
		try {
			ret = str.substring(0, str.indexOf(tag)) + replace + str.substring(str.indexOf(tag) + tag.length());
		}
		catch (IndexOutOfBoundsException i) {
			ret = str;
		}
		return(ret);
	}

	public void init() {
		int i;
		int tempint;
		buttonHash = new Hashtable();
		treePath = "";
		treePathParent = "";
  
		// Top row of buttons
		tempint = Integer.parseInt(getParameter("topNumButtons"));
		topButtons = new JButton[tempint];
		for(i = 0; i < tempint; i++) {
			buttonHash.put(new String(getParameter("topButtonName" + i)), new String(getParameter("topButtonURL" + i)));

			topButtons[i] = new JButton(getParameter("topButtonName" + i));
			topButtons[i].addActionListener(new ActionListener() {
	            public void actionPerformed(ActionEvent e) {
					String buttonURL = (String)buttonHash.get(e.getActionCommand());
					String tempURL;
					tempURL = strReplace(buttonURL, new String("$PATH$"), treePath);
					tempURL = strReplace(tempURL, new String("$PATHPARENT$"), treePathParent);
					showURL(tempURL, new String("bottom"));
				}
	        });
		}

		// Bottom row of buttons
		tempint = Integer.parseInt(getParameter("botNumButtons"));
		botButtons = new JButton[tempint];

		for(i = 0; i < tempint; i++) {
			buttonHash.put(new String(getParameter("botButtonName" + i)), new String(getParameter("botButtonURL" + i)));
			botButtons[i] = new JButton(getParameter("botButtonName" + i));
			botButtons[i].addActionListener(new ActionListener() {
	            public void actionPerformed(ActionEvent e) {
					String buttonURL = (String)buttonHash.get(e.getActionCommand());
					String tempURL;
					tempURL = strReplace(buttonURL, new String("$PATH$"), treePath);
					tempURL = strReplace(tempURL, new String("$PATHPARENT$"), treePathParent);
					tempURL = strReplace(tempURL, new String("$NAME$"), selectedObject.getName());
					tempURL = strReplace(tempURL, new String("$ID$"), selectedObject.getID().toString());
					showURL(tempURL, new String("bottom"));
				}
	        });
		}
		
	
		// Popup menu items
	    popup = new PopupMenu();

		tempint = Integer.parseInt(getParameter("popupItems"));
		popupItems = new MenuItem[tempint];
		popupHash = new Hashtable();
		for(i = 0; i < tempint; i++) {
			popupHash.put(new String(getParameter("popupName" + i)), new String(getParameter("popupURL" + i)));
			popupItems[i] = new MenuItem(getParameter("popupName" + i));
	        popupItems[i].addActionListener(new ActionListener() {
	            public void actionPerformed(ActionEvent e) {
					String popupURL = (String)popupHash.get(e.getActionCommand());
					String tempURL;
					tempURL = strReplace(popupURL, new String("$PATH$"), treePath);
					tempURL = strReplace(tempURL, new String("$PATHPARENT$"), treePathParent);
					tempURL = strReplace(tempURL, new String("$NAME$"), selectedObject.getName());
					tempURL = strReplace(tempURL, new String("$ID$"), selectedObject.getID().toString());
					showURL(tempURL, new String("bottom"));
				}
			});
			popup.add(popupItems[i]);
		}


		objList = new Hashtable();

		InitTree treePanel;
		treePanel = new InitTree(getParameter("dblClickURL"), getParameter("shiftDblClickURL"));
        getContentPane().setLayout(new GridLayout(1,0));
        getContentPane().add(treePanel);
        getContentPane().setLayout(new BorderLayout());
        treePanel.setPreferredSize(new Dimension(600, 300));

		JPanel topbuttonpanel = new JPanel();
		topbuttonpanel.setLayout(new GridLayout(1,2));

		tempint = Integer.parseInt(getParameter("topNumButtons"));
		for(i = 0; i < tempint; i++) {		
			topbuttonpanel.add(topButtons[i]);
		}

		JPanel toppanel = new JPanel();
		toppanel.setLayout(new GridLayout(1,0));
		toppanel.add(topbuttonpanel);

		getContentPane().add(toppanel, BorderLayout.NORTH);
		getContentPane().add(treePanel, BorderLayout.CENTER);

        JPanel botpanel = new JPanel();
        botpanel.setLayout(new GridLayout(1,6));

		tempint = Integer.parseInt(getParameter("botNumButtons"));
		for(i = 0; i < tempint; i++) {		
			botpanel.add(botButtons[i]);
			botButtons[i].setEnabled(false);
		}

		getContentPane().add(botpanel, BorderLayout.SOUTH);
		getContentPane().add(popup); // add popup menu to applet
        enableEvents(AWTEvent.MOUSE_EVENT_MASK); 

		tempint = Integer.parseInt(getParameter("popupItems"));
		for(i = 0; i < tempint; i++) {		
			popupItems[i].setEnabled(false);
		}
	}

	private class ObjInfo {
		public Integer objID;
		public Integer objParent;
		public int objChildren;
		public String objName;
		public String objDescr;

        public ObjInfo(Integer id, Integer parent, int children, String name, String descr) {
			objName = name;
			objDescr = descr;
			objID = id;
			objParent = parent;
			objChildren = children;
		}

		public String getDescr() {
			return objDescr;
		}
		
		public String getName() {
			return objName;
		}

		public Integer getID() {
			return objID;
		}

		public Integer getParent() {
			return objParent;
		}

		public int getChildren() {
			return objChildren;
		}

		public String toString() {
			if(objDescr.length() == 0) {
				return objName;
			} else {
				return(objName + " (" + objDescr + ")");
			}
		}
	}
}
