# 
# Copyright (C) 2001, Hitesh Patel <hitesh@presys.com>.
# All rights reserved.
# 
# This file is part of NorthStar and is covered under
# the terms of the Apache Software License version 1.1
#
# A copy of this license is included in the LICENSE file
#
# $Header: /cvsroot/NorthStar/lib/NorthStar.pm,v 1.17 2002/05/21 00:56:30 brownkid Exp $
#

package NorthStar;

BEGIN {
   use Exporter;
   @ISA = qw(Exporter);
   @EXPORT = qw(ShowAudit Audit ShowMain ShowTmpl ShowMenu);
}
use DBI;
use CGI qw(-no_debug :standard);

use NorthStar::API qw(:ALL);
use NorthStar::Search;
use NorthStar::Location;
use NorthStar::Owner;
use NorthStar::Device;
use NorthStar::Network;
use NorthStar::User;
use NorthStar::Session;
use NorthStar::Template;
use NorthStar::Regex;
use NorthStar::View;


$VERSION = "1.65";

sub new {
   my($self) = {};
   bless($self);
   return($self);
}

sub init {
   Debug("CORE: initializing module");

   my(%modinfo) = (
         export_hooks => [
            'CORE/ShowMain/OutputMainPre',
            'CORE/ShowMain/OutputMainPost',
            'CORE/ShowTmpl/OutputTmplPre',
            'CORE/ShowTmpl/OutputTmplPost',
            'CORE/AuditAdd/InsertPre',
            'CORE/AuditAdd/InsertPost',
            'CORE/ShowAudit/OutputSearchPagePre',
            'CORE/ShowAudit/OutputSearchPagePost',
            'CORE/ShowAudit/OutputResultsHeadPre',
            'CORE/ShowAudit/OutputResultsHeadPost',
            'CORE/ShowAudit/OutputResultsItemPre',
            'CORE/ShowAudit/OutputResultsItemPost',
            'CORE/ShowAudit/OutputResultsFootPre',
            'CORE/ShowAudit/OutputResultsFootPost',
         ],
         user_hooks => {
            'shmain'		=>	\&NorthStar::ShowMain,
            'shtmpl'    => \&NorthStar::ShowTmpl,
            'shaudit'	=>	\&NorthStar::ShowAudit,
            'shmenu'    => \&NorthStar::ShowMenu,
            },
         import_hooks => {},
      );

   API_PermissionsRegisterPerm(42, 0, 'shaudit', 'Show Audit Log', 'CORE');    

   API_MenuAddSection('Browse', 1);
   API_MenuAddSection('Search', 201);
   API_MenuAddSection('Preferences', 401);
   API_MenuAddSection('Wizards', 601);
   API_MenuAddSection('Administration', 801);

   my($script) = ConfigGetValue('Global','ScriptURL');
   API_MenuAddItem('Administration','Audit Log',"$script?r=shaudit", undef, 101); 

   return(%modinfo);
}

sub run {
   if(API_Init(1,1,1,0) < 1) {
      API_ExitError();
   }
   $| = 1;
   main();
}


sub main {
   my($sth);
   my($sid) = $cgi->cookie('NorthStarSID');
   if(!$sid && $cgi->param('sid')) {
      $sid = $cgi->param('sid');
   }
   $NorthStar::sid = $sid;

   if($cgi->param('r') eq 'logout') {
      my($c) = $cgi->cookie(-name  => 'NorthStarSID',
            -value => ''
         );

      API_OutputHeader(
         -type  => 'text/html',
         -cookie=>$c,
      ); 

      if(API_Init(0,0,0,1) < 1) {
         API_ExitError();
      }

      $NorthStar::username = SessionInit($sid, $ENV{'REMOTE_ADDR'});
      SessionDelete($sid);
      Audit('session','logout',"session $sid logged out");
      my(%pitems) = ( 
            'msg' => 'You are logged out.  Have a nice day!' 
         );
      API_OutputPage('login_page.html', \%pitems);
      exit(0);
   } elsif($sid) {
      if(API_Init(0,0,0,1) < 1) {
         API_ExitError();
      }
      $NorthStar::username = SessionInit($sid, $ENV{'REMOTE_ADDR'});
   } elsif($cgi->param('username') && $cgi->param('passwd')) {
      $cgi->param('username', lc($cgi->param('username')));
      if(API_Init(0,0,0,1) < 1) {
         API_ExitError();
      }
      SessionCreate($cgi->param('username'), $cgi->param('passwd'),
         $ENV{'REMOTE_ADDR'});
      $NorthStar::username = $cgi->param('username');
      Audit('session','login',
         "user $NorthStar::username ($ENV{'REMOTE_ADDR'}) logged in");
   } else {
      API_OutputHeader();
      API_OutputPage('login_page.html');
      exit(0);
   }

   API_OutputHeader();

   $sth = $db->prepare("SELECT id,realname FROM users WHERE username = '" .
         $NorthStar::username . "'");

   $sth->execute();
   API_DBResCheck(\$sth,'get_uid','1');
   ($NorthStar::user_id, $NorthStar::user_realname) = $sth->fetchrow_array;

   VoodooMagic();
   undef($sth);
   $db->disconnect;
}

sub VoodooMagic {
   my($sth) = $db->prepare("SELECT permmask FROM users WHERE id = '$user_id'");
   $sth->execute;
   API_DBResCheck(\$sth,'permmask_get','1');
   my($pm) = ($sth->fetchrow_array)[0];

   API_PermissionsSetPerms($pm);

   if($cgi->param('r') && $NorthStar::_USERHOOKDB{$cgi->param('r')}) {
      my($r) = $NorthStar::_USERHOOKDB{$cgi->param('r')};
      if(API_PermissionsCheckPerm($cgi->param('r')) != 1) {
         API_ExitError("You do not have permission to execute this function");
      }
      &$r();
   } else {
      ShowMain();
   }
}

sub ShowMain {
   my(%pitems) = ();
   ModuleDispatch('CORE','CORE/ShowMain/OutputMainPre', \%pitems);
   API_OutputPage('main_page.html', \%pitems);
   ModuleDispatch('CORE','CORE/ShowMain/OutputMainPost', \%pitems);
}

sub ShowTmpl {
   my(%pitems) = ();
   ModuleDispatch('CORE','CORE/ShowTmpl/OutputTmplPre', \%pitems);
   API_OutputPage($cgi->param('p'), \%pitems);
   ModuleDispatch('CORE','CORE/ShowTmpl/OutputTmplPost', \%pitems);
}

sub ShowMenu {
   my(%pitems) = ();
   my($section, $item, $k, $v);
   API_OutputPage('menu_head.html');
   my(@sections) = sort { $NorthStar::_usermenusections{$a} <=> 
      $NorthStar::_usermenusections{$b} } keys %NorthStar::_usermenusections;

   foreach $section (@sections) {
      Debug($section, 8);
      %pitems = (
            'section' => $section
         );

      API_OutputPage('menu_section.html', \%pitems);
      my(@items) = sort { $NorthStar::_usermenuitems{$section}{$a}{'sort'} <=> 
         $NorthStar::_usermenuitems{$section}{$b}{'sort'} } 
      keys %{$NorthStar::_usermenuitems{$section}};

      foreach $item (@items) {
         Debug("$section->$item link=" . 
            $NorthStar::_usermenuitems{$section}{$item}{'link'} . " ap=" .
            $NorthStar::_usermenuitems{$section}{$item}{'append'}, 8);

         %pitems = (
               'section' => $section,
               'name'    => $item,
               'link'    => $NorthStar::_usermenuitems{$section}{$item}{'link'},
               'append'  => $NorthStar::_usermenuitems{$section}{$item}{'append'}
            );
         API_OutputPage('menu_item.html', \%pitems);
      }
   }


   API_OutputPage('menu_foot.html');
}

sub Audit {
   my($section) = shift || return;
   my($action) = shift || return;
   my($msg) = shift || return;
   AuditAdd($section, $action, $msg);
}

sub AuditAdd {
   my($section) = shift || return;
   my($action) = shift || return;
   my($msg) = shift || return;

   my(%audititems) = (
         'section'   => $section,
         'action'    => $action,
         'msg'       => $msg,
      );
   ModuleDispatch('CORE','CORE/AuditAdd/InsertPre', \%audititems);
   if(API_AddAudit($section, $action, $msg) < 1) {
      API_ExitError();
   } 

   ModuleDispatch('CORE','CORE/AuditAdd/InsertPost', \%audititems);

   return(0);
}

sub ShowAudit {
   my(%pitems, $t, $sth);
   if(!$cgi->param('doproc')) {
      API_UserDropdown(\$pitems{'user_select'},undef,'All',undef);

      ModuleDispatch('CORE','CORE/ShowAudit/OutputSearchPagePre', \%pitems);
      API_OutputPage('auditlog_main.html', \%pitems);
      ModuleDispatch('CORE','CORE/ShowAudit/OutputSearchPagePost', \%pitems);
      return();
   }

   my($sql) = "SELECT * FROM auditlog";
   if($cgi->param('objtype')) {
      if($t) {
         $sql .= " section = '" . $cgi->param('objtype') . "'";
      } else {
         $sql .= " WHERE section = '" . $cgi->param('objtype') . "'";
         $t = 1;
      }
   }

   if($cgi->param('action')) {
      if($t) {
         $sql .= " AND action = '" . $cgi->param('action') . "'";
      } else {
         $sql .= " WHERE action = '" . $cgi->param('action') . "'";
         $t = 1;
      }
   }

   if($cgi->param('user') && $cgi->param('user') ne 'All') {
      if($t) {
         $sql .= " AND username = '" . $cgi->param('user') . "'";
      } else {
         $sql .= " WHERE username = '" . $cgi->param('user') . "'";
         $t = 1;
      }
   }

   $sql .= " ORDER BY date DESC";
   $sth = $db->prepare($sql);
   $sth->execute();
   API_DBResCheck(\$sth, 'ShowAudit: GetLogs', '1');

   %pitems = ();
   ModuleDispatch('CORE','CORE/ShowAudit/OutputResultsHeadPre', \%pitems);
   API_OutputPage('auditlog_head.html', \%pitems);
   ModuleDispatch('CORE','CORE/ShowAudit/OutputResultsHeadPost', \%pitems);

   my($i);
   for($i = 0; $i < $sth->rows; $i++) {
      my(@row) = $sth->fetchrow_array;
      my(%pitems) = (
            'id'        =>  $row[0],
            'date'      =>  $row[1],
            'user'      =>  $row[2],
            'section'   =>  $row[3],
            'action'    =>  $row[4],
            'msg'       =>  $row[5]
         );
      ModuleDispatch('CORE','CORE/ShowAudit/OutputResultsItemPre', \%pitems);
      API_OutputPage('auditlog_item.html',\%pitems);
      ModuleDispatch('CORE','CORE/ShowAudit/OutputResultsItemPost', \%pitems);
   }
   %pitems = ();
   ModuleDispatch('CORE','CORE/ShowAudit/OutputResultsFootPre', \%pitems);
   API_OutputPage('auditlog_foot.html', \%pitems);
   ModuleDispatch('CORE','CORE/ShowAudit/OutputResultsFootPost', \%pitems);
}
