# 
# Copyright (C) 2001, Hitesh Patel <hitesh@presys.com>.
# All rights reserved.
# 
# This file is part of NorthStar and is covered under
# the terms of the Apache Software License version 1.1
#
# A copy of this license is included in the LICENSE file
#
# $Header: /cvsroot/NorthStar/lib/NorthStar/Device.pm,v 1.16 2002/05/21 00:56:17 brownkid Exp $
#

package NorthStar::Device;
use Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(AddDevice UpdDevice DelDevice ShowDevice ShowDeviceTree);

use NorthStar::API qw(:ALL);

sub init {
   Debug("DEVICE: initializing module");
   my(%modinfo) = (
         export_hooks => [
            'CORE/Device/AddDevice/OutputAddPagePre',
            'CORE/Device/AddDevice/OutputAddPagePost',
            'CORE/Device/AddDevice/OutputAddPageTemplatePre',
            'CORE/Device/AddDevice/OutputAddPageTemplatePost',
            'CORE/Device/AddDevice/OutputAddPageFootPre',
            'CORE/Device/AddDevice/OutputAddPageFootPost',
            'CORE/Device/AddDevice/AddPre',
            'CORE/Device/AddDevice/AddPost',
            'CORE/Device/UpdDevice/OutputUpdPagePre',
            'CORE/Device/UpdDevice/OutputUpdPagePost',
            'CORE/Device/UpdDevice/OutputUpdPageTemplatePre',
            'CORE/Device/UpdDevice/OutputUpdPageTemplatePost',
            'CORE/Device/UpdDevice/OutputUpdPageFootPre',
            'CORE/Device/UpdDevice/OutputUpdPageFootPost',
            'CORE/Device/UpdDevice/UpdPre',
            'CORE/Device/UpdDevice/UpdPost',
            'CORE/Device/DelDevice/OutputConfirmPre',
            'CORE/Device/DelDevice/OutputConfirmPost',
            'CORE/Device/DelDevice/DelPre',
            'CORE/Device/DelDevice/DelPost',
            'CORE/Device/ShowDevice/OutputHeadPre',
            'CORE/Device/ShowDevice/OutputHeadPost',
            'CORE/Device/ShowDevice/OutputBodyPre',
            'CORE/Device/ShowDevice/OutputBodyPost',
            'CORE/Device/ShowDevice/OutputTemplatePre',
            'CORE/Device/ShowDevice/OutputTemplatePost',
            'CORE/Device/ShowDevice/OutputChildrenHeadPre',
            'CORE/Device/ShowDevice/OutputChildrenHeadPost',
            'CORE/Device/ShowDevice/OutputChildrenItemPre',
            'CORE/Device/ShowDevice/OutputChildrenItemPost',
            'CORE/Device/ShowDevice/OutputChildrenFootPre',
            'CORE/Device/ShowDevice/OutputChildrenFootPost',
            'CORE/Device/ShowDevice/OutputNetblocksHeadPre',
            'CORE/Device/ShowDevice/OutputNetblocksHeadPost',
            'CORE/Device/ShowDevice/OutputNetblocksItemPre',
            'CORE/Device/ShowDevice/OutputNetblocksItemPost',
            'CORE/Device/ShowDevice/OutputNetblocksFootPre',
            'CORE/Device/ShowDevice/OutputNetblocksFootPost',
            'CORE/Device/ShowDevice/OutputAddrHeadPre',
            'CORE/Device/ShowDevice/OutputAddrHeadPost',
            'CORE/Device/ShowDevice/OutputAddrItemPre',
            'CORE/Device/ShowDevice/OutputAddrItemPost',
            'CORE/Device/ShowDevice/OutputAddrFootPre',
            'CORE/Device/ShowDevice/OutputAddrFootPost',
            'CORE/Device/ShowDevice/OutputFootPre',
            'CORE/Device/ShowDevice/OutputFootPost'
         ],
         user_hooks => {
            'adddev'    => \&NorthStar::Device::AddDevice,
            'upddev'    => \&NorthStar::Device::UpdDevice,
            'deldev'    => \&NorthStar::Device::DelDevice,
            'shdev'	    => \&NorthStar::Device::ShowDevice,
            'shdevtree' => \&NorthStar::Device::ShowDeviceTree,
            },
         import_hooks => {},
      );

   API_PermissionsRegisterPerm(6, 0, 'adddev', 'Add Device', 'CORE');
   API_PermissionsRegisterPerm(7, 0, 'upddev', 'Update Device', 'CORE');
   API_PermissionsRegisterPerm(8, 0, 'deldev', 'Delete Device', 'CORE');   
   API_PermissionsRegisterPerm(52, 1, 'shdev', 'Show Device', 'CORE');
   API_PermissionsRegisterPerm(59, 1, 'shdevtree', 'Show Device Tree', 'CORE');

   my($script) = ConfigGetValue('Global','ScriptURL');
   API_MenuAddItem('Browse','Devices', "$script?r=shdevtree",undef, 301); 

   return(%modinfo);
}

sub AddDevice {
   my(%pitems, %fields);
   if(!$cgi->param('doproc')) {
      %pitems = ();

      if(!$cgi->param('template_select')) {
         %pitems = (
               'r' => 'adddev',
               'parent' => $cgi->param('parent'),
               'netaddr'=> $cgi->param('netaddr'),
               'owner'  => $cgi->param('owner')
            );

         my($tmplid) = API_TemplateCheckSingle('3');
         if($tmplid < 1) {
            API_TemplateSelectPage('3',\%pitems);
            return();
         } else {
            $cgi->param('template_select', $tmplid);
         }
      }

      if(!$cgi->param('parent')) {
         $pitems{'parent_device'} = "Top-level Device" . "<input type=\"hidden\" name=\"parent\" value=\"0\">";
         API_OwnerDropdown(\$pitems{'owner_table'},undef,undef,'Global Owner');
         if(API_LocationDropdown(\$pitems{'location_table'},undef,undef,1) < 1) {
            API_ExitError();
         }
      } else {
         my(%devinfo) = API_GetDevice($cgi->param('parent'));
         $pitems{'parent_device'} = "$devinfo{'name'}" . 
            "<input type=\"hidden\" name=\"parent\" value=\"" . $cgi->param('parent') . "\">";
         API_OwnerDropdown(\$pitems{'owner_table'}, $devinfo{'owner'},undef,'Global Owner');
         if(API_LocationDropdown(\$pitems{'location_table'}, $devinfo{'location'},undef,1) < 1) {
            API_ExitError();
         }
      }

      $pitems{'template_select'} = $cgi->param('template_select');
      ModuleDispatch('Device','CORE/Device/AddDevice/OutputAddPagePre', \%pitems);
      API_OutputPage('device_add.html', \%pitems);
      ModuleDispatch('Device','CORE/Device/AddDevice/OutputAddPagePost', \%pitems);

      my(%fields, @order);
      %fields = API_TemplateFieldGetAssignedFields($cgi->param('template_select'));
      @order = API_TemplateFieldGetAssignedFieldsOrder($cgi->param('template_select'));

      ModuleDispatch('Device','CORE/Device/AddDevice/OutputAddPageTemplatePre', \%pitems);
      API_OutputPage('template_fieldedit_head.html');
      foreach(@order) {
         %pitems = (
               'id'	=> $_,
               'value'	=> '',
               'name'	=> $fields{$_},
               'example' => API_TemplateFieldGetExample($_),
               'required'=> API_TemplateFieldIsRequired($_) == 1 ? "*" : ""
            );
         $pitems{'formhtml'} = API_TemplateFieldCreateHTML($pitems{'id'}, $pitems{'value'});
         API_OutputPage('template_fieldedit_item.html', \%pitems);
      }
      API_OutputPage('template_fieldedit_foot.html', \%pitems);			
      ModuleDispatch('Device','CORE/Device/AddDevice/OutputAddPageTemplatePost', \%pitems);

      ModuleDispatch('Device','CORE/Device/AddDevice/OutputAddPageFootPre', \%pitems);
      API_OutputPage('device_add_foot.html', \%pitems);
      ModuleDispatch('Device','CORE/Device/AddDevice/OutputAddPageFootPost', \%pitems);
   } else {	
      %fields = (
            'required' => {
               'name'      => 'alpha',
               'location'	=> 'num',
               'owner'		=> 'num',
               'parent'    => 'num',
               'template_select' => 'num',
               },
            'optional' => {
               'descr'	=> 'alpha'
               },
            'names' => {
               'name'	=> 'Device Name',
               'descr'	=> 'Device Description',
               'location'=>'Device Location',
               'owner'	=> 'Device Owner',
               'parent'=> 'Device Parent',
               'template_select' => 'Device Template',
               }
         );
      API_CheckFormFields(\%fields);

      my(%fields, @order);
      %fields = API_TemplateFieldGetAssignedFields($cgi->param('template_select'));	
      @order = API_TemplateFieldGetAssignedFieldsOrder($cgi->param('template_select'));

      foreach(@order) {
         my($x) = API_TemplateFieldCheckInput($_, \$cgi->param($_));

         if($x == 2) {			
            API_ExitError("The field $fields{$_} was not filled in");
         }
         if($x == 3) {			
            API_ExitError("The field $fields{$_} was too short");
         }
         if($x == 4) {			
            API_ExitError("The field $fields{$_} was too long");
         }				
         if($x == 5) {			
            API_ExitError("The field $fields{$_} was not correctly formatted");
         }				
      }

      my(%devinfo) = (
            'parent'    => $cgi->param('parent'),
            'templateid'=> $cgi->param('template_select'),
            'name'      => $cgi->param('name'),
            'descr'     => $cgi->param('descr'),
            'owner'     => $cgi->param('owner'),
            'location'  => $cgi->param('location')
         );


      ModuleDispatch('Device','CORE/Device/AddDevice/AddPre');
      my($id) = API_AddDevice(\%devinfo);
      if($id < 1) {
         API_ExitError();
      }
      ModuleDispatch('Device','CORE/Device/AddDevice/AddPost');

      foreach(@order) {
         API_TemplateFieldSetValue($id, '3', $_, $cgi->param($_));
      }

      $cgi->delete('owner');
      $cgi->delete('location');
      if($cgi->param('parent')) {
         $cgi->param('id', $cgi->param('parent'));
      }
      ShowDeviceTree();
   }
}

sub UpdDevice {
   my(@row, %pitems, %fields);
   if(!$cgi->param('id')) {
      API_ExitError("An device id to update was not specified");
   }

   if(!$cgi->param('doproc')) {
      %pitems = API_GetDevice($cgi->param('id'));

      API_OwnerDropdown(\$pitems{'owner_table'}, $pitems{'owner'},undef,'Global Owner');
      if(API_LocationDropdown(\$pitems{'location_table'}, $pitems{'location'},undef,1) < 1) {
         API_ExitError();
      }

      if($pitems{'parent'} != 0) {
         my(%devinfo) = API_GetDevice($pitems{'parent'}, 1);
         $pitems{'parent_device'} = $devinfo{'name'} . "<input type=\"hidden\" name=\"parent\" value=\"$pitems{'parent'}\">";
      } else {
         $pitems{'parent_device'} = "Top-Level Device<input type=\"hidden\" name=\"parent\" value=\"$pitems{'parent'}\">";      
      }

      ModuleDispatch('Device','CORE/Device/UpdDevice/OutputUpdPagePre', \%pitems);
      API_OutputPage('device_upd.html', \%pitems);
      ModuleDispatch('Device','CORE/Device/UpdDevice/OutputUpdPagePost', \%pitems);

      my(%fields, @order);
      %fields = API_TemplateFieldGetAssignedFields($pitems{'templateid'});
      @order = API_TemplateFieldGetAssignedFieldsOrder($pitems{'templateid'});

      ModuleDispatch('Device','CORE/Device/UpdDevice/OutputUpdPageTemplatePre', \%pitems);
      API_OutputPage('template_fieldedit_head.html');
      foreach(@order) {
         %pitems = (
               'id'	    => $_,
               'value'	 => API_TemplateFieldGetValue($cgi->param('id'), '3', $_) || "",
               'name'	 => $fields{$_},
               'example' => API_TemplateFieldGetExample($_),
               'required'=> API_TemplateFieldIsRequired($_) == 1 ? "*" : ""
            );
         $pitems{'formhtml'} = API_TemplateFieldCreateHTML($pitems{'id'}, $pitems{'value'});
         API_OutputPage('template_fieldedit_item.html', \%pitems);
      }
      API_OutputPage('template_fieldedit_foot.html', \%pitems);			
      ModuleDispatch('Device','CORE/Device/UpdDevice/OutputUpdPageTemplatePost', \%pitems);

      ModuleDispatch('Device','CORE/Device/UpdDevice/OutputUpdPageFootPre', \%pitems);
      API_OutputPage('device_upd_foot.html', \%pitems);
      ModuleDispatch('Device','CORE/Device/UpdDevice/OutputUpdPageFootPost', \%pitems);
   } else {	
      %fields = (
            'required' => {
               'name'	=>	'alpha',
               'owner'	=>	'num',
               'location'=>'num',
               },
            'optional' => {
               'descr'	=>	'alpha'
               },
            'names' => {
               'name'	=>	'Device Name',
               'descr'	=>	'Device Description',
               'owner'	=>	'Device Owner',
               'location'=>'Device Location',
               }
         );

      API_CheckFormFields(\%fields);

      my(%fields, @order);
      my(%devinfo) = API_GetDevice($cgi->param('id'));

      my($id) = $devinfo{'templateid'};
      %fields = API_TemplateFieldGetAssignedFields($id);
      @order = API_TemplateFieldGetAssignedFieldsOrder($id);

      foreach(@order) {
         my($x) = API_TemplateFieldCheckInput($_, \$cgi->param($_));

         if($x == 2) {			
            API_ExitError("The field $fields{$_} was not filled in");
         }
         if($x == 3) {			
            API_ExitError("The field $fields{$_} was too short");
         }
         if($x == 4) {			
            API_ExitError("The field $fields{$_} was too long");
         }				
         if($x == 5) {			
            API_ExitError("The field $fields{$_} was not correctly formatted");
         }				
      }

      %devinfo = (
            'id'     => $cgi->param('id'),
            'name'   => $cgi->param('name'),
            'descr'  => $cgi->param('descr'),
            'owner'  => $cgi->param('owner'),
            'location'=>$cgi->param('location')
         );

      ModuleDispatch('Device','CORE/Device/UpdDevice/UpdPre');
      if(API_UpdDevice(\%devinfo) < 1) {
         API_ExitError();
      }
      ModuleDispatch('Device','CORE/Device/UpdDevice/UpdPost');

      foreach(@order) {
         API_TemplateFieldSetValue($cgi->param('id'), '3', $_, $cgi->param($_));
      }

      if($cgi->param('parent')) {
         $cgi->delete('parent');
      }
      ShowDeviceTree();
   }
}

sub DelDevice {
   my(%pitems);
   if(!$cgi->param('id')) {
      API_ExitError("A device id to delete was not specified");
   }

   if(!API_GetDevice($cgi->param('id'))) {
      API_ExitError();
   }

   if(!$cgi->param('confirm')) {
      %pitems = (
            'object'=> 'Device',
            'r'	=> $cgi->param('r'),
            'id'	=> $cgi->param('id'),
            'move_delete' => '<input type="hidden" value="1" name="confirm">'
         );
      ModuleDispatch('Device','CORE/Device/DelDevice/OutputConfirmPre', \%pitems);
      API_OutputPage('delete_confirm.html',\%pitems);
      ModuleDispatch('Device','CORE/Device/DelDevice/OutputConfirmPost', \%pitems);
   } else {
      my(@delids);
      ModuleDispatch('Device','CORE/Device/DelDevice/DelPre');
      if(API_DelDevice($cgi->param('id'), \@delids) < 1) {
         API_ExitError();
      }
      ModuleDispatch('Device','CORE/Device/DelDevice/DelPost');

      foreach(@delids) {
         if(API_TemplateFieldDeleteAllValues($_, '3') < 1) {
            API_ExitError();
         }
      }

      $cgi->delete('id');
      $cgi->delete('owner');
      $cgi->delete('location');
      ShowDeviceTree();
   }

}

sub ShowDevice {
   my(@row, %pitems, $i, @data);
   if(!$cgi->param('id')) {
      API_ExitError("A device id to view was not specified");
   }

   %pitems = API_GetDevice($cgi->param('id'), 1);
   if(!%pitems) {
      API_ExitError();
   }

   my(%devinfo) = %pitems;

   if($pitems{'parentid'} == 0) {
      $pitems{'parentlink'} = $pitems{'parent'};
   } else {
      $pitems{'parentlink'} = '<A HREF="'. ConfigGetValue('Global','ScriptURL').
         "?r=shdev&id=$pitems{'parentid'}\">".
         "$pitems{'parent'}</A>";
   }

   ModuleDispatch('Device','CORE/Device/ShowDevice/OutputHeadPre', \%pitems);
   API_OutputPage('device_detailshow_head.html', \%pitems);
   ModuleDispatch('Device','CORE/Device/ShowDevice/OutputHeadPost', \%pitems);

   ModuleDispatch('Device','CORE/Device/ShowDevice/OutputBodyPre', \%pitems);
   API_OutputPage('device_detailshow_body.html', \%pitems);
   ModuleDispatch('Device','CORE/Device/ShowDevice/OutputBodyPost', \%pitems);

   my(%fields, @order);
   %fields = API_TemplateFieldGetAssignedFields($pitems{'templateid'});
   @order = API_TemplateFieldGetAssignedFieldsOrder($pitems{'templateid'});

   ModuleDispatch('Device','CORE/Device/ShowDevice/OutputTemplatePre', \%pitems);
   API_OutputPage('template_fieldview_head.html', \%pitems);
   foreach(@order) {
      %pitems = (
            'id'	=> $_,
            'value'	=> API_TemplateFieldGetValue($cgi->param('id'), '3', $_) || "",
            'name'	=> $fields{$_},
            'required'=> ''
         );
      $pitems{'formhtml'} = API_TemplateFieldCreateHTML($pitems{'id'}, $pitems{'value'});
      API_OutputPage('template_fieldview_item.html', \%pitems);
   }
   API_OutputPage('template_fieldview_foot.html', \%pitems);			
   ModuleDispatch('Device','CORE/Device/ShowDevice/OutputTemplatePost');

   ModuleDispatch('Device','CORE/Device/ShowDevice/OutputChildrenHeadPre', \%pitems);
   API_OutputPage('device_detailshow_devlisthead.html', \%pitems);
   ModuleDispatch('Device','CORE/Device/ShowDevice/OutputChildrenHeadPost', \%pitems);

   my($aryref) = $devinfo{'children'};
   foreach(@{$aryref}) {
      my(%pitems) = API_GetDevice($_);
      ModuleDispatch('Device','CORE/Device/ShowDevice/OutputChildrenItemPre', \%pitems);
      API_OutputPage('device_detailshow_devlistitem.html',\%pitems);
      ModuleDispatch('Device','CORE/Device/ShowDevice/OutputChildrenItemPost', \%pitems);
   }
   ModuleDispatch('Device','CORE/Device/ShowDevice/OutputChildrenFootPre', \%pitems);
   API_OutputPage('device_detailshow_devlistfoot.html', \%pitems);
   ModuleDispatch('Device','CORE/Device/ShowDevice/OutputChildrenFootPost', \%pitems);


   ModuleDispatch('Device','CORE/Device/ShowDevice/OutputNetblocksHeadPre', \%pitems);
   API_OutputPage('device_detailshow_netlisthead.html', \%pitems);
   ModuleDispatch('Device','CORE/Device/ShowDevice/OutputNetblocksHeadPost', \%pitems);

   @data = ();
   API_GetNetworkList([qw(id name label)], \@data, [qw(name)], [{ 'field' => 'device',
            'type'  => '=',
            'data'  => $cgi->param('id')
            },
         { 'field' => 'masklen',
            'type'  => '<',
            'data'  => '32'
            }]);

   foreach(@data) {
      %pitems = (
            'id'     => ${$_}{'id'},
            'name'   => ${$_}{'name'},
            'network'=> ${$_}{'label'}
         );
      ModuleDispatch('Device','CORE/Device/ShowDevice/OutputNetblocksItemPre', \%pitems);
      API_OutputPage('device_detailshow_netlistitem.html',\%pitems);
      ModuleDispatch('Device','CORE/Device/ShowDevice/OutputNetblocksItemPost', \%pitems);
   }

   ModuleDispatch('Device','CORE/Device/ShowDevice/OutputNetblocksFootPre', \%pitems);
   API_OutputPage('device_detailshow_netlistfoot.html', \%pitems);
   ModuleDispatch('Device','CORE/Device/ShowDevice/OutputNetblocksFootPost', \%pitems);


   ModuleDispatch('Device','CORE/Device/ShowDevice/OutputAddrHeadPre', \%pitems);
   API_OutputPage('device_detailshow_addrlisthead.html', \%pitems);
   ModuleDispatch('Device','CORE/Device/ShowDevice/OutputAddrHeadPost', \%pitems);
   @data = ();
   API_GetNetworkList([qw(id network name)], \@data, 
      [qw(network)], 
      [{ 'field' => 'device',
            'type'  => '=',
            'data'  => $cgi->param('id')
            },
         { 'field' => 'masklen',
            'type'  => '=',
            'data'  => '32'
            }]
   );

   foreach(@data) {
      %pitems = (
            'id'     => ${$_}{'id'},
            'address'=> IP_IntToDQ(${$_}{'network'}),
            'dnsname'=> ${$_}{'name'}
         );
      ModuleDispatch('Device','CORE/Device/ShowDevice/OutputAddrItemPre', \%pitems);
      API_OutputPage('device_detailshow_addrlistitem.html',\%pitems);
      ModuleDispatch('Device','CORE/Device/ShowDevice/OutputAddrItemPost', \%pitems);
   }

   ModuleDispatch('Device','CORE/Device/ShowDevice/OutputAddrFootPre', \%pitems);
   API_OutputPage('device_detailshow_addrlistfoot.html', \%pitems);		
   ModuleDispatch('Device','CORE/Device/ShowDevice/OutputAddrFootPost', \%pitems);

   ModuleDispatch('Device','CORE/Device/ShowDevice/OutputFootPre', \%pitems);
   API_OutputPage('device_detailshow_foot.html', \%pitems);
   ModuleDispatch('Device','CORE/Device/ShowDevice/OutputFootPost', \%pitems);
}

sub ShowDeviceTree {
   my($i, $p, $id, $a, $b, $t, $d, @sort, %labels);
   my(@data);
   API_GetDeviceList([qw(id parent name)], \@data, [qw(parent name)]);
   foreach(@data) {
      push(@sort, ${$_}{'id'});
      $parents{${$_}{'id'}} = ${$_}{'parent'};
      $labels{${$_}{'id'}}{'name'} = ${$_}{'name'};
   }


   my($n0) = {
      'name'          => 'Devices',
      'id'            => '0',
      'parent'        => '0',
      'children'      => [],
   };

   my(%lasthash);

   $d = 1;
   foreach(@sort) {
      $id = $_;
      $p = $parents{$_};
      $a = "n$id";
      $b = "n$p";
      $lasthash{$p} = $id;
      ${$a} = {                
         'name'          => $labels{$_}{'name'},
         'id'            => $_,
         'parent'        => $p,
         'last'          => '0',
         'children'      => [],
      };
      if($p == 0) {
         API_TreeGrow($n0, ${$a});
      } else {
         API_TreeGrow(${$b}, ${$a});
      }
   }

   while(($a,$b) = each %lasthash) {
      $t = "n$b";
      if(${$t}->{'parent'} == 0) {
         ${$t}->{'last'} = '2';
      } else {
         ${$t}->{'last'} = '1';
      }
   }

   API_OutputPage('device_showtree_head.html');
   API_TreeDumpTable('device_showtree_item.html', $n0, 
      ($cgi->param('path') || undef), 
      ($cgi->param('depth') || undef), 
      '', 'shdev', 'shdevtree');
   API_OutputPage('device_showtree_foot.html');
}

return 1;
