# 
# Copyright (C) 2001, Hitesh Patel <hitesh@presys.com>.
# All rights reserved.
# 
# This file is part of NorthStar and is covered under
# the terms of the Apache Software License version 1.1
#
# A copy of this license is included in the LICENSE file
#
# $Header: /cvsroot/NorthStar/lib/NorthStar/Location.pm,v 1.14 2002/05/21 00:56:17 brownkid Exp $
#
package NorthStar::Location;
use Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(AddLocation UpdLocation DelLocation ShowLocation ShowLocationTree);

use NorthStar::API qw(:ALL);

sub init {
   Debug("LOCATION: initializing module");
   my(%modinfo) = (
         export_hooks => [
            'CORE/Location/AddLocation/OutputAddPagePre',
            'CORE/Location/AddLocation/OutputAddPagePost',
            'CORE/Location/AddLocation/OutputAddPageTemplatePre',
            'CORE/Location/AddLocation/OutputAddPageTemplatePost',
            'CORE/Location/AddLocation/OutputAddPageFootPre',
            'CORE/Location/AddLocation/OutputAddPageFootPost',
            'CORE/Location/AddLocation/AddPre',
            'CORE/Location/AddLocation/AddPost',
            'CORE/Location/UpdLocation/OutputUpdPagePre',
            'CORE/Location/UpdLocation/OutputUpdPagePost',
            'CORE/Location/UpdLocation/OutputUpdPageTemplatePre',
            'CORE/Location/UpdLocation/OutputUpdPageTemplatePost',
            'CORE/Location/UpdLocation/OutputUpdPageFootPre',
            'CORE/Location/UpdLocation/OutputUpdPageFootPost',
            'CORE/Location/UpdLocation/UpdPre',
            'CORE/Location/UpdLocation/UpdPost',
            'CORE/Location/DelLocation/OutputConfirmPre',
            'CORE/Location/DelLocation/OutputConfirmPost',
            'CORE/Location/DelLocation/DelPre',
            'CORE/Location/DelLocation/DelPost',
            'CORE/Location/ShowLocation/OutputHeadPre',
            'CORE/Location/ShowLocation/OutputHeadPost',
            'CORE/Location/ShowLocation/OutputBodyPre',
            'CORE/Location/ShowLocation/OutputBodyPost',
            'CORE/Location/ShowLocation/OutputTemplatePre',
            'CORE/Location/ShowLocation/OutputTemplatePost',
            'CORE/Location/ShowLocation/OutputChildrenHeadPre',
            'CORE/Location/ShowLocation/OutputChildrenHeadPost',
            'CORE/Location/ShowLocation/OutputChildrenItemPre',
            'CORE/Location/ShowLocation/OutputChildrenItemPost',
            'CORE/Location/ShowLocation/OutputChildrenFootPre',
            'CORE/Location/ShowLocation/OutputChildrenFootPost',
            'CORE/Location/ShowLocation/OutputNetblocksHeadPre',
            'CORE/Location/ShowLocation/OutputNetblocksHeadPost',
            'CORE/Location/ShowLocation/OutputNetblocksItemPre',
            'CORE/Location/ShowLocation/OutputNetblocksItemPost',
            'CORE/Location/ShowLocation/OutputNetblocksFootPre',
            'CORE/Location/ShowLocation/OutputNetblocksFootPost',
            'CORE/Location/ShowLocation/OutputDevHeadPre',
            'CORE/Location/ShowLocation/OutputDevHeadPost',
            'CORE/Location/ShowLocation/OutputDevItemPre',
            'CORE/Location/ShowLocation/OutputDevItemPost',
            'CORE/Location/ShowLocation/OutputDevFootPre',
            'CORE/Location/ShowLocation/OutputDevFootPost',
            'CORE/Location/ShowLocation/OutputFootPre',
            'CORE/Location/ShowLocation/OutputFootPost'
         ],
         user_hooks => {
            'addloc'    => \&NorthStar::Location::AddLocation,
            'updloc'    => \&NorthStar::Location::UpdLocation,
            'delloc'    => \&NorthStar::Location::DelLocation,
            'shloc'	    => \&NorthStar::Location::ShowLocation,
            'shloctree' => \&NorthStar::Location::ShowLocationTree,
            },
         import_hooks => {},
      );

   API_PermissionsRegisterPerm('0', 0, 'addloc', 'Add Location', 'CORE');
   API_PermissionsRegisterPerm(1, 0, 'updloc', 'Update Location', 'CORE');
   API_PermissionsRegisterPerm(2, 0, 'delloc', 'Delete Location', 'CORE');
   API_PermissionsRegisterPerm(50, 1, 'shloc', 'Show Location', 'CORE');
   API_PermissionsRegisterPerm(60, 1,'shloctree', 'Show Location Tree', 'CORE');

   my($script) = ConfigGetValue('Global','ScriptURL');
   API_MenuAddItem('Browse','Locations', "$script?r=shloctree",undef, 401); 

   return(%modinfo);
}

sub AddLocation {
   my(%pitems);
   if(!$cgi->param('doproc')) {
      %pitems = ();

      if(!$cgi->param('template_select')) {
         %pitems = (
               'r' => 'addloc',
               'parent' => $cgi->param('parent'),
               'netaddr'=> $cgi->param('netaddr')
            );

         my($tmplid) = API_TemplateCheckSingle('2');
         if($tmplid < 1) {
            API_TemplateSelectPage('2',\%pitems);
            return();
         } else {
            $cgi->param('template_select', $tmplid);
         }
      }

      if(!$cgi->param('parent')) {
         $pitems{'parent_location'} = "Top-level Location" . "<input type=\"hidden\" name=\"parent\" value=\"0\">";
      } else {
         my(%locinfo) = API_GetLocation($cgi->param('parent'));
         $pitems{'parent_location'} = "$locinfo{'name'}" . 
            '<input type="hidden" name="parent" value="' . 
            $cgi->param('parent') .'">';
      }

      $pitems{'template_select'} = $cgi->param('template_select');
      ModuleDispatch('Location','CORE/Location/AddLocation/OutputAddPagePre', \%pitems);
      API_OutputPage('location_add.html', \%pitems);
      ModuleDispatch('Location','CORE/Location/AddLocation/OutputAddPagePost', \%pitems);

      my(%fields, @order);
      %fields = API_TemplateFieldGetAssignedFields($cgi->param('template_select'));
      @order = API_TemplateFieldGetAssignedFieldsOrder($cgi->param('template_select'));

      ModuleDispatch('Location','CORE/Location/AddLocation/OutputAddPageTemplatePre', \%pitems);
      API_OutputPage('template_fieldedit_head.html');
      foreach(@order) {
         %pitems = (
               'id'	=> $_,
               'value'	=> '',
               'name'	=> $fields{$_},
               'example' => API_TemplateFieldGetExample($_),
               'required'=> API_TemplateFieldIsRequired($_) == 1 ? "*" : ""
            );
         $pitems{'formhtml'} = API_TemplateFieldCreateHTML($pitems{'id'}, $pitems{'value'});
         API_OutputPage('template_fieldedit_item.html', \%pitems);
      }
      API_OutputPage('template_fieldedit_foot.html', \%pitems);			
      ModuleDispatch('Location','CORE/Location/AddLocation/OutputAddPageTemplatePost', \%pitems);

      ModuleDispatch('Location','CORE/Location/AddLocation/OutputAddPageFootPre', \%pitems);
      API_OutputPage('location_add_foot.html', \%pitems);
      ModuleDispatch('Location','CORE/Location/AddLocation/OutputAddPageFootPost', \%pitems);
   } else {	
      my(%fields) = (
            'required' => {
               'name'	=> 'alpha',
               'parent'=> 'num',
               },
            'optional' => {
               'descr'	=> 'alpha'
               },
            'names' => {
               'name'	=> 'Location Name',
               'parent'=> 'Location Parent',
               'descr'	=> 'Location Description'
               }
         );

      API_CheckFormFields(\%fields);

      %fields = API_TemplateFieldGetAssignedFields($cgi->param('template_select'));	
      my(@order) = API_TemplateFieldGetAssignedFieldsOrder($cgi->param('template_select'));

      foreach(@order) {
         my($x) = API_TemplateFieldCheckInput($_, \$cgi->param($_));

         if($x == 2) {			
            API_ExitError("The field $fields{$_} was not filled in");
         }
         if($x == 3) {			
            API_ExitError("The field $fields{$_} was too short");
         }
         if($x == 4) {			
            API_ExitError("The field $fields{$_} was too long");
         }				
         if($x == 5) {			
            API_ExitError("The field $fields{$_} was not correctly formatted");
         }				
      }

      my(%locinfo) = (
            'parent'    => $cgi->param('parent'),
            'templateid'=> $cgi->param('template_select'),
            'name'      => $cgi->param('name'),
            'descr'     => $cgi->param('descr'),
         );  

      ModuleDispatch('Location','CORE/Location/AddLocation/AddPre');
      my($id) = API_AddLocation(\%locinfo);
      if($id < 1) {
         API_ExitError();
      }
      ModuleDispatch('Location','CORE/Location/AddLocation/AddPost');

      foreach(@order) {
         API_TemplateFieldSetValue($id, '2', $_, $cgi->param($_));
      }
      if($cgi->param('parent')) {
         $cgi->param('id', $cgi->param('parent'));
      }
      ShowLocationTree();
   }
}

sub UpdLocation {
   if(!$cgi->param('id')) {
      API_ExitError("An location id to update was not specified");
   }

   if(!$cgi->param('doproc')) {
      %pitems = API_GetLocation($cgi->param('id'));

      if($pitems{'parent'} != 0) {
         my(%locinfo) = API_GetLocation($pitems{'parent'}, 1);
         $pitems{'parent_location'} = $locinfo{'name'} . "<input type=\"hidden\" name=\"parent\" value=\"$pitems{'parent'}\">";
      } else {
         $pitems{'parent_location'} = "Top-Level Location<input type=\"hidden\" name=\"parent\" value=\"$pitems{'parent'}\">";            
      }

      ModuleDispatch('Location','CORE/Location/UpdLocation/OutputUpdPagePre', \%pitems);
      API_OutputPage('location_upd.html', \%pitems);
      ModuleDispatch('Location','CORE/Location/UpdLocation/OutputUpdPagePost', \%pitems);

      my(%fields, @order);
      %fields = API_TemplateFieldGetAssignedFields($pitems{'templateid'});
      @order = API_TemplateFieldGetAssignedFieldsOrder($pitems{'templateid'});

      ModuleDispatch('Location','CORE/Location/UpdLocation/OutputUpdPageTemplatePre', \%pitems);
      API_OutputPage('template_fieldedit_head.html');
      foreach(@order) {
         %pitems = (
               'id'	=> $_,
               'value'	=> API_TemplateFieldGetValue($cgi->param('id'), '2', $_) || "",
               'name'	=> $fields{$_},
               'example' => API_TemplateFieldGetExample($_),
               'required'=> API_TemplateFieldIsRequired($_) == 1 ? "*" : ""
            );
         $pitems{'formhtml'} = API_TemplateFieldCreateHTML($pitems{'id'}, $pitems{'value'});
         API_OutputPage('template_fieldedit_item.html', \%pitems);
      }
      API_OutputPage('template_fieldedit_foot.html', \%pitems);			
      ModuleDispatch('Location','CORE/Location/UpdLocation/OutputUpdPageTemplatePost', \%pitems);

      ModuleDispatch('Location','CORE/Location/UpdLocation/OutputUpdPageFootPre', \%pitems);
      API_OutputPage('location_upd_foot.html', \%pitems);
      ModuleDispatch('Location','CORE/Location/UpdLocation/OutputUpdPageFootPost', \%pitems);
   } else {	
      my(%fields) = (
            'required' => {
               'name'	=>	'alpha',
               },
            'optional' => {
               'descr'	=>	'alpha'
               },
            'names' => {
               'name'	=>	'Location Name',
               'descr'	=>	'Location Description'
               }
         );

      API_CheckFormFields(\%fields);

      my(%locinfo) = API_GetLocation($cgi->param('id'));
      my($id) = $locinfo{'templateid'};
      %fields = API_TemplateFieldGetAssignedFields($id);
      my(@order) = API_TemplateFieldGetAssignedFieldsOrder($id);

      foreach(@order) {
         my($x) = API_TemplateFieldCheckInput($_, \$cgi->param($_));

         if($x == 2) {			
            API_ExitError("The field $fields{$_} was not filled in");
         }
         if($x == 3) {			
            API_ExitError("The field $fields{$_} was too short");
         }
         if($x == 4) {			
            API_ExitError("The field $fields{$_} was too long");
         }				
         if($x == 5) {			
            API_ExitError("The field $fields{$_} was not correctly formatted");
         }				
      }
      %locinfo = (
            'id'     => $cgi->param('id'),
            'name'   => $cgi->param('name'),
            'descr'  => $cgi->param('descr'),
         );  

      ModuleDispatch('Location','CORE/Location/UpdLocation/UpdPre');
      if(API_UpdLocation(\%locinfo) < 1) {
         API_ExitError();
      }
      ModuleDispatch('Location','CORE/Location/UpdLocation/UpdPost');

      foreach(@order) {
         API_TemplateFieldSetValue($cgi->param('id'), '2', $_, $cgi->param($_));
      }
      ShowLocationTree();
   }
}

sub DelLocation {
   if(!$cgi->param('id')) {
      API_ExitError("A location id to delete was not specified");
   }

   if(!API_GetLocation($cgi->param('id'))) {
      API_ExitError();
   }

   if(!$cgi->param('confirm')) {
      my(%pitems) = (
            'object'      => 'Location',
            'r'	        => $cgi->param('r'),
            'id'	        => $cgi->param('id'),
            'move_delete' => '<input type="hidden" value="1" name="confirm">'
         );
      ModuleDispatch('Location','CORE/Location/DelLocation/OutputConfirmPre', \%pitems);
      API_OutputPage('delete_confirm.html',\%pitems);
      ModuleDispatch('Location','CORE/Location/DelLocation/OutputConfirmPost', \%pitems);
   } else {
      my(@delids);
      ModuleDispatch('Location','CORE/Location/DelLocation/DelPre');
      if(API_DelLocation($cgi->param('id'), \@delids) < 1) {
         API_ExitError();
      }
      ModuleDispatch('Location','CORE/Location/DelLocation/DelPost');

      foreach(@delids) {
         if(API_TemplateFieldDeleteAllValues($_, '2') < 1) {
            API_ExitError();
         }
      }

      $cgi->delete('id');
      ShowLocationTree();
   }
}

sub ShowLocation {
   my(@data);


   if(!$cgi->param('id')) {
      API_ExitError("A location id to view was not specified");
   }

   my(%pitems) = API_GetLocation($cgi->param('id'), 1);
   if(!%pitems) {
      API_ExitError();
   }
   my(%locinfo) = %pitems;

   if($pitems{'parentid'} == 0) {
      $pitems{'parentlink'} = $pitems{'parent'};
   } else {
      $pitems{'parentlink'} = '<A HREF="'. ConfigGetValue('Global','ScriptURL').
         "?r=shloc&id=$pitems{'parentid'}\">".
         "$pitems{'parent'}</A>";
   }

   ModuleDispatch('Location','CORE/Location/ShowLocation/OutputHeadPre', \%pitems);
   API_OutputPage('location_detailshow_head.html', \%pitems);
   ModuleDispatch('Location','CORE/Location/ShowLocation/OutputHeadPost', \%pitems);

   ModuleDispatch('Location','CORE/Location/ShowLocation/OutputBodyPre', \%pitems);
   API_OutputPage('location_detailshow_body.html', \%pitems);
   ModuleDispatch('Location','CORE/Location/ShowLocation/OutputBodyPost', \%pitems);

   my(%fields, @order);
   %fields = API_TemplateFieldGetAssignedFields($pitems{'templateid'});
   @order = API_TemplateFieldGetAssignedFieldsOrder($pitems{'templateid'});

   %pitems = ();
   ModuleDispatch('Location','CORE/Location/ShowLocation/OutputTemplatePre', \%pitems);
   API_OutputPage('template_fieldview_head.html', \%pitems);
   foreach(@order) {
      %pitems = (
            'id'	=> $_,
            'value'	=> API_TemplateFieldGetValue($cgi->param('id'), '2', $_) || "",
            'name'	=> $fields{$_},
            'required'=> ''
         );
      $pitems{'formhtml'} = API_TemplateFieldCreateHTML($pitems{'id'}, $pitems{'value'});
      API_OutputPage('template_fieldview_item.html', \%pitems);
   }
   API_OutputPage('template_fieldview_foot.html', \%pitems);			
   ModuleDispatch('Location','CORE/Location/ShowLocation/OutputTemplatePost', \%pitems);

   ModuleDispatch('Location','CORE/Location/ShowLocation/OutputChildrenHeadPre', \%pitems);
   API_OutputPage('location_detailshow_loclisthead.html', \%pitems);
   ModuleDispatch('Location','CORE/Location/ShowLocation/OutputChildrenHeadPost', \%pitems);

   my($aryref) = $locinfo{'children'};
   foreach(@{$aryref}) {
      my(%pitems) = API_GetLocation($_);
      ModuleDispatch('Location','CORE/Location/ShowLocation/OutputChildrenItemPre', \%pitems);
      API_OutputPage('location_detailshow_loclistitem.html',\%pitems);
      ModuleDispatch('Location','CORE/Location/ShowLocation/OutputChildrenItemPost', \%pitems);
   }
   ModuleDispatch('Location','CORE/Location/ShowLocation/OutputChildrenFootPre', \%pitems);
   API_OutputPage('location_detailshow_loclistfoot.html');
   ModuleDispatch('Location','CORE/Location/ShowLocation/OutputChildrenFootPost', \%pitems);


   ModuleDispatch('Location','CORE/Location/ShowLocation/OutputNetblocksHeadPre', \%pitems);
   API_OutputPage('location_detailshow_netlisthead.html', %pitems);
   ModuleDispatch('Location','CORE/Location/ShowLocation/OutputNetblocksHeadPost', \%pitems);
   @data = ();
   API_GetNetworkList([qw(id name label)], \@data, [qw(name)], [{ 'field' => 'location',
            'type'  => '=',
            'data'  => $cgi->param('id')
            },
         { 'field' => 'masklen',
            'type'  => '<',
            'data'  => '32'
            }]);

   foreach(@data) {
      %pitems = (
            'id'     => ${$_}{'id'},
            'name'   => ${$_}{'name'},
            'network'=> ${$_}{'label'}
         );
      ModuleDispatch('Location','CORE/Location/ShowLocation/OutputNetblocksItemPre', \%pitems);
      API_OutputPage('location_detailshow_netlistitem.html',\%pitems);
      ModuleDispatch('Location','CORE/Location/ShowLocation/OutputNetblocksItemPost', \%pitems);
   }

   %pitems = ();
   ModuleDispatch('Location','CORE/Location/ShowLocation/OutputNetblocksFootPre', \%pitems);
   API_OutputPage('location_detailshow_netlistfoot.html', \%pitems);
   ModuleDispatch('Location','CORE/Location/ShowLocation/OutputNetblocksFootPost', \%pitems);




   ModuleDispatch('Location','CORE/Location/ShowLocation/OutputDevHeadPre', \%pitems);
   API_OutputPage('location_detailshow_devlisthead.html', \%pitems);
   ModuleDispatch('Location','CORE/Location/ShowLocation/OutputDevHeadPost', \%pitems);
   @data = ();
   API_GetDeviceList([qw(id name)], \@data, [qw(name)], [{ 'field' => 'location',
            'type'  => '=',
            'data'  => $cgi->param('id')
            }]);
   foreach(@data) {
      %pitems = (
            'id'  => ${$_}{'id'},
            'name'=> ${$_}{'name'}
         );
      ModuleDispatch('Location','CORE/Location/ShowLocation/OutputDevItemPre', \%pitems);
      API_OutputPage('location_detailshow_devlistitem.html',\%pitems);
      ModuleDispatch('Location','CORE/Location/ShowLocation/OutputDevItemPost', \%pitems);
   }

   %pitems = ();
   ModuleDispatch('Location','CORE/Location/ShowLocation/OutputDevFootPre', \%pitems);
   API_OutputPage('location_detailshow_devlistfoot.html', \%pitems);		
   ModuleDispatch('Location','CORE/Location/ShowLocation/OutputDevFootPost', \%pitems);

   ModuleDispatch('Location','CORE/Location/ShowLocation/OutputFootPre', \%pitems);
   API_OutputPage('location_detailshow_foot.html');
   ModuleDispatch('Location','CORE/Location/ShowLocation/OutputFootPost', \%pitems);
}

sub ShowLocationTree {
   my($i, $p, $id, $a, $b, $t, $d, @sort, %labels);
   my(@data);
   API_GetLocationList([qw(id parent name)], \@data, [qw(parent name)]);
   foreach(@data) {
      push(@sort, ${$_}{'id'});
      $parents{${$_}{'id'}} = ${$_}{'parent'};
      $labels{${$_}{'id'}}{'name'} = ${$_}{'name'};
   }

   my($n0) = {
      'name'          => 'Locations',
      'id'            => '0',
      'parent'        => '0',
      'children'      => [],
   };

   my(%lasthash);

   $d = 1;
   foreach(@sort) {
      $id = $_;
      $p = $parents{$_};
      $a = "n$id";
      $b = "n$p";
      $lasthash{$p} = $id;
      ${$a} = {
         'name'          => $labels{$_}{'name'},
         'id'            => $_,
         'parent'        => $p,
         'last'          => '0',
         'children'      => [],
      };
      if($p == 0) {
         API_TreeGrow($n0, ${$a});
      } else {
         API_TreeGrow(${$b}, ${$a});
      }
   }

   while(($a,$b) = each %lasthash) {
      $t = "n$b";
      if(${$t}->{'parent'} == 0) {
         ${$t}->{'last'} = '2';
      } else {
         ${$t}->{'last'} = '1';
      }
   }

   API_OutputPage('location_showtree_head.html');
   API_TreeDumpTable('location_showtree_item.html', $n0, 
      ($cgi->param('path') || undef), 
      ($cgi->param('depth') || undef), '', 'shloc', 'shloctree');
   API_OutputPage('location_showtree_foot.html');
}

return 1;
