# 
# Copyright (C) 2001, Hitesh Patel <hitesh@presys.com>.
# All rights reserved.
# 
# This file is part of NorthStar and is covered under
# the terms of the Apache Software License version 1.1
#
# A copy of this license is included in the LICENSE file
#
# $Header: /cvsroot/NorthStar/lib/NorthStar/Network.pm,v 1.19 2002/05/21 00:56:17 brownkid Exp $
#

package NorthStar::Network;
use Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(AddNet AddNetRange UpdNet DelNet ShowNet 
             ShowNetTree ShowNetTreeJava NetworkAlloc);

use NorthStar::API qw(:ALL);

sub init {
   Debug("NETWORK: initializing module");
   my(%modinfo) = (
         export_hooks => [
            'CORE/Network/AddNetwork/OutputAddAddrPagePre',
            'CORE/Network/AddNetwork/OutputAddAddrPagePost',
            'CORE/Network/AddNetwork/OutputAddNetPagePre',
            'CORE/Network/AddNetwork/OutputAddNetPagePost',
            'CORE/Network/AddNetwork/OutputAddPageTemplatePre',
            'CORE/Network/AddNetwork/OutputAddPageTemplatePost',
            'CORE/Network/AddNetwork/OutputAddPageAddrFootPre',
            'CORE/Network/AddNetwork/OutputAddPageAddrFootPost',
            'CORE/Network/AddNetwork/OutputAddPageNetFootPre',
            'CORE/Network/AddNetwork/OutputAddPageNetFootPost',
            'CORE/Network/AddNetwork/AddPre',
            'CORE/Network/AddNetwork/AddPost',
            'CORE/Network/AddNetworkRange/OutputAddAddrPagePre',
            'CORE/Network/AddNetworkRange/OutputAddAddrPagePost',
            'CORE/Network/AddNetworkRange/OutputAddNetPagePre',
            'CORE/Network/AddNetworkRange/OutputAddNetPagePost',
            'CORE/Network/AddNetworkRange/OutputAddPageTemplatePre',
            'CORE/Network/AddNetworkRange/OutputAddPageTemplatePost',
            'CORE/Network/AddNetworkRange/OutputAddPageAddrFootPre',
            'CORE/Network/AddNetworkRange/OutputAddPageAddrFootPost',
            'CORE/Network/AddNetworkRange/OutputAddPageNetFootPre',
            'CORE/Network/AddNetworkRange/OutputAddPageNetFootPost',
            'CORE/Network/AddNetworkRange/AddPre',
            'CORE/Network/AddNetworkRange/AddPost',
            'CORE/Network/UpdNetwork/OutputUpdPagePre',
            'CORE/Network/UpdNetwork/OutputUpdPagePost',
            'CORE/Network/UpdNetwork/OutputUpdPageTemplatePre',
            'CORE/Network/UpdNetwork/OutputUpdPageTemplatePost',
            'CORE/Network/UpdNetwork/OutputUpdPageFootPre',
            'CORE/Network/UpdNetwork/OutputUpdPageFootPost',
            'CORE/Network/UpdNetwork/UpdPre',
            'CORE/Network/UpdNetwork/UpdPost',
            'CORE/Network/DelNetwork/OutputConfirmPre',
            'CORE/Network/DelNetwork/OutputConfirmPost',
            'CORE/Network/DelNetwork/DelPre',
            'CORE/Network/DelNetwork/DelPost',
            'CORE/Network/ShowNetwork/OutputHeadPre',
            'CORE/Network/ShowNetwork/OutputHeadPost',
            'CORE/Network/ShowNetwork/OutputBodyPre',
            'CORE/Network/ShowNetwork/OutputBodyPost',
            'CORE/Network/ShowNetwork/OutputTemplatePre',
            'CORE/Network/ShowNetwork/OutputTemplatePost',
            'CORE/Network/ShowNetwork/OutputAddrHeadPre',
            'CORE/Network/ShowNetwork/OutputAddrHeadPost',
            'CORE/Network/ShowNetwork/OutputAddrItemPre',
            'CORE/Network/ShowNetwork/OutputAddrItemPost',
            'CORE/Network/ShowNetwork/OutputAddrFootPre',
            'CORE/Network/ShowNetwork/OutputAddrFootPost',
            'CORE/Network/ShowNetwork/OutputFootPre',
            'CORE/Network/ShowNetwork/OutputFootPost'
         ],
         user_hooks => {
            'addnet'        => \&NorthStar::Network::AddNet,
            'addnetrange'   => \&NorthStar::Network::AddNetRange,
            'updnet'        => \&NorthStar::Network::UpdNet,
            'delnet'        => \&NorthStar::Network::DelNet,
            'shnet'         => \&NorthStar::Network::ShowNet,
            'shnettree'     => \&NorthStar::Network::ShowNetTree,
            'addaddr'       => \&NorthStar::Network::AddNet,
            'addaddrrange'  => \&NorthStar::Network::AddNetRange,
            'allocwiz'      => \&NorthStar::Network::NetworkAlloc,
            },
         import_hooks => {},
      );

   API_PermissionsRegisterPerm(9, 0, 'addnet', 'Add Network', 'CORE');
   API_PermissionsRegisterPerm(10, 0, 'updnet', 'Update Network/Address', 'CORE');
   API_PermissionsRegisterPerm(11, 0, 'delnet', 'Delete Network/Address', 'CORE');
   API_PermissionsRegisterPerm(12, 0, 'addaddr', 'Add Address', 'CORE');
   API_PermissionsRegisterPerm(13, 0, 'addaddrrange', 'Add Address Range', 'CORE');
   API_PermissionsRegisterPerm(43, 0, 'addnetrange', 'Add Network Range', 'CORE');
   API_PermissionsRegisterPerm(53, 1, 'shnet', 'Show Network', 'CORE');
   API_PermissionsRegisterPerm(57, 1, 'shnettree', 'Show Network Tree', 'CORE');
   API_PermissionsRegisterPerm(31, 0, 'allocwiz', 'Allocation Wizard', 'CORE');    

   my($script) = ConfigGetValue('Global','ScriptURL');
   API_MenuAddItem('Browse','Networks', "$script?r=shnettree&mode=", 
      "(<A HREF=\"$script?r=shnettree&mode=java\" target='main'>Java</A>)", 101);
   API_MenuAddItem('Wizards','Allocation Wizard', "$script?r=allocwiz",undef); 

   return(%modinfo);
}

sub AddNet {
   my(%netinfo, $maxlen, $ipaddr, %pitems, %fields, $t, $sql, $ip, $i,$addaddr);

   if(!$cgi->param('view')) {
      API_ExitError("A view id was not specified");
   }

   if($cgi->param('r') eq "addaddr" || $cgi->param('netmask') == '32') {
      if ($cgi->param('r') ne 'addaddr') {
         $cgi->param('template_select', "");
         $cgi->param('netaddr', $cgi->param('netblock'));
         $cgi->param('netblock', '');
      }
      $cgi->param('r', 'addaddr');
      $addaddr = -1;
      %netinfo = API_GetNetwork($cgi->param('parent'));
      $cgi->param('netmask', '32');
   } else {
      $addaddr = 0;
   }

   if($cgi->param('location') && $cgi->param('owner') && 
         !$cgi->param('device') && $cgi->param('done')) {
      $cgi->param('doproc', 1);
   }

   if($cgi->param('doproc') == 0 || !$cgi->param('doproc') || 
         $cgi->param('showaddnet') == 1) {
      if(!$cgi->param('template_select')) {
         if ($addaddr) {
            $objtype = '5';
         } else {
            $objtype = '4';
         }
         %pitems = (
               'r' 			=> $cgi->param('r'),
               'view' 		=> $cgi->param('view'),
               'mode' 		=> $cgi->param('mode') || '',
               'netaddr' 	=> $cgi->param('netaddr'),
               'netblock'  => $cgi->param('netblock') || "",
               'parent' 	=> $cgi->param('parent') || "",
               'netmask'   => '32',
               'status'    => '2'
            );

         my($tmplid) = API_TemplateCheckSingle($objtype);
         if($tmplid < 1) {
            API_TemplateSelectPage($objtype,\%pitems);
            return();
         } else {
            $cgi->param('template_select', $tmplid);
         }
      }
      %pitems = (
            'parent'    => $cgi->param('parent') || "",
            'name'      => $cgi->param('name') || "",
            'descr'     => $cgi->param('descr') || "",
            'netaddr'   => $cgi->param('netaddr') || "",
            'netblock'  => $cgi->param('netblock') || "",
         );
      if($cgi->param('shnet')) { 
         $pitems{'shnet'} = 1;
      } else {
         $pitems{'shnet'} = 0;
      }

      unless ($pitems{'netblock'}) {
         $pitems{'netblock'} = $pitems{'netaddr'};
      }
      ($ipaddr,$maxlen) = split(/\//,$pitems{'netblock'});


      API_OwnerDropdown(\$pitems{'owner_table'}, 
         ($cgi->param('owner') || undef), 
         'document.form.submit();');

      if($cgi->param('owner') || $cgi->param('location')) {
         if(API_DeviceDropdown(\$pitems{'device_table'}, undef, 
                  ($cgi->param('owner') || undef), 
                  ($cgi->param('location') || undef), 
                  'document.form.doproc.value=1;') < 1) {;
            API_ExitError();
         }
      } else {
         $pitems{'device_table'} = "<SELECT NAME=\"device\"><OPTION>Select an Owner or Location Object First</OPTION></SELECT>";
      }


      if ($cgi->param('netaddr') =~ /\// || $cgi->param('netblock') =~ /\// ) {
         $pitems{'netblock_out'} = "<input type=hidden name=\"netblock\" value=\"$pitems{'netblock'}\">";
      }

      my($submit) = 'document.form.submit();';
      if ($cgi->param('netaddr') !~ /\// && !$cgi->param('netblock')) {
         undef $submit;
         $pitems{'netaddr2_table'} = '<input type="text" name="netaddr" '.
            'value=' . $cgi->param('netaddr') . '>';
      } elsif ($cgi->param('netmask')) {
         API_NetworkAddrDropdown(\$pitems{'netaddr2_table'},
            $ipaddr,$maxlen,$cgi->param('netmask'),
            $cgi->param('view'),'');
      } else {
         $pitems{'netaddr2_table'} = "<select name=\"netaddr\"><option value=\"\">Select a Netmask First</option></select>";
      }
      API_CIDRDropdown(\$pitems{'netmask_table'},
         ($cgi->param('netmask')||undef),undef, $maxlen,$submit);
      if(API_LocationDropdown(\$pitems{'location_table'}, 
               ($cgi->param('location') || undef), 
               'document.form.submit();') < 1) {
         API_ExitError();
      }

      API_NetStatusDropdown(\$pitems{'status_table'}, 
         ($cgi->param('status') || undef));		
      API_NetPriorityDropdown(\$pitems{'priority_table'}, 
         ($cgi->param('priority') || '5'));

      $pitems{'parent_net'} = "$netinfo{'name'} ($netinfo{'label'})";
      $pitems{'template_select'} = $cgi->param('template_select');
      $pitems{'view'} = $cgi->param('view');
      $pitems{'mode'} = $cgi->param('mode') || '';
      if ($addaddr) {
         ModuleDispatch('Network','CORE/Network/AddNetwork/OutputAddAddrPagePre', \%pitems);
         API_OutputPage('address_addaddr.html', \%pitems);
         ModuleDispatch('Network','CORE/Network/AddNetwork/OutputAddAddrPagePre', \%pitems);
      } else {
         ModuleDispatch('Network','CORE/Network/AddNetwork/OutputAddNetPagePre', \%pitems);
         API_OutputPage('network_addnet.html', \%pitems);
         ModuleDispatch('Network','CORE/Network/AddNetwork/OutputAddNetPagePost', \%pitems);
      }
      my(%fields, @order);
      %fields = API_TemplateFieldGetAssignedFields($cgi->param('template_select'));
      @order = API_TemplateFieldGetAssignedFieldsOrder($cgi->param('template_select'));

      %pitems = ();
      ModuleDispatch('Network','CORE/Network/AddNetwork/OutputAddPageTemplatePre', \%pitems);
      API_OutputPage('template_fieldedit_head.html', \%pitems);

      foreach(@order) {
         %pitems = (
               'id'	=> $_,
               'value'	=> '',
               'name'	=> $fields{$_},
               'example' => API_TemplateFieldGetExample($_),
               'required'=> API_TemplateFieldIsRequired($_) == 1 ? "*" : ""
            );
         $pitems{'formhtml'} = API_TemplateFieldCreateHTML($pitems{'id'}, $pitems{'value'});
         API_OutputPage('template_fieldedit_item.html', \%pitems);
      }
      API_OutputPage('template_fieldedit_foot.html', \%pitems);			
      ModuleDispatch('Network','CORE/Network/AddNetwork/OutputAddPageTemplatePost', \%pitems);

      if ($addaddr) {
         ModuleDispatch('Network','CORE/Network/AddNetwork/OutputAddPageAddrFootPre', \%pitems);
         API_OutputPage('address_addaddr_foot.html', \%pitems);
         ModuleDispatch('Network','CORE/Network/AddNetwork/OutputAddPageAddrFootPost', \%pitems);
      } else {
         ModuleDispatch('Network','CORE/Network/AddNetwork/OutputAddPageNetFootPre', \%pitems);
         API_OutputPage('network_addnet_foot.html', \%pitems);
         ModuleDispatch('Network','CORE/Network/AddNetwork/OutputAddPageNetFootPost', \%pitems);
      }
   } else {
      %fields = (
            'required' => {
               'name'      => 'alpha',
               'owner'     => 'num',	
               'location'  => 'num',
               'status'    => 'num',
               'netaddr'   => 'ipv4addr',
               'netmask'   => 'num',
               'priority'  => 'num',
               },
            'optional' => {
               'device'    => 'num',
               'descr'     => 'alpha'
               },
            'names' => {
               'name'      => 'Network Name',
               'descr'     => 'Network Description',
               'owner'     => 'Network Owner',
               'location'  => 'Network Location',
               'status'    => 'Network Block Status',
               'device'    => 'Network Device',
               'netaddr'   => 'Network Address',
               'netmask'   => 'Network Address Mask',
               'priority'  => 'Network Block Priority',
               }
         );

      API_CheckFormFields(\%fields);
      if(API_NetCheckExist($cgi->param('netaddr'), $cgi->param('netmask'), $cgi->param('view')) > 0) {
         API_ExitError("The network you are trying to add already exists");
      }

      if(IP_CheckPrefix($cgi->param('netaddr'), $cgi->param('netmask')) != 1) {
         API_ExitError("The block you are trying to add does not have a valid network address and netmask");
      }

      my(%fields, @order);
      %fields = API_TemplateFieldGetAssignedFields($cgi->param('template_select'));	
      @order = API_TemplateFieldGetAssignedFieldsOrder($cgi->param('template_select'));

      foreach(@order) {
         my($x) = API_TemplateFieldCheckInput($_, \$cgi->param($_));

         if($x == 2) {			
            API_ExitError("The field $fields{$_} was not filled in");
         }
         if($x == 3) {			
            API_ExitError("The field $fields{$_} was too short");
         }
         if($x == 4) {			
            API_ExitError("The field $fields{$_} was too long");
         }				
         if($x == 5) {			
            API_ExitError("The field $fields{$_} was not correctly formatted");
         }				
      }

      if(API_CheckNetworkStatus($cgi->param('netaddr'), $cgi->param('netmask'), $cgi->param('view')) < 1) {
         API_ExitError();
      }

      if(!$cgi->param('device')) {
         $cgi->param('device', 0);
      }


      my(%netinfo) = (
            'name'      => $cgi->param('name'),
            'templateid'=> $cgi->param('template_select'),
            'viewid'    => $cgi->param('view'),
            'status'    => $cgi->param('status'),
            'descr'     => $cgi->param('descr'),
            'block'     => $cgi->param('netaddr') . '/' . $cgi->param('netmask'),
            'owner'     => $cgi->param('owner'),
            'device'    => $cgi->param('device'),
            'location'  => $cgi->param('location'),
            'priority'  => $cgi->param('priority')
         );

      ModuleDispatch('Network','CORE/Network/AddNetwork/AddPre');
      my($id) = API_AddNetwork(\%netinfo);
      if($id < 1) {
         API_ExitError();
      }
      ModuleDispatch('Network','CORE/Network/AddNetwork/AddPost');

      foreach(@order) {
         API_TemplateFieldSetValue($id, '4', $_, $cgi->param($_));
      }

      if($cgi->param('shnet')) {
         $cgi->param('id', $cgi->param('parent'));
         ShowNet();
      } else {
         if($cgi->param('mode') eq 'java') {
            %pitems = ( 
                  'url' => ConfigGetValue('Global','ScriptURL') . 
                  "?r=shnettree&mode=java&view=" . $cgi->param('view') . 
                  '&path=' . $cgi->param('path')
               );
            API_OutputPage('reload_frame.html', \%pitems);
         } else {
            ShowNetTree();
         }
      }
   }
}

sub AddNetRange {
   my(%netinfo, %pitems, %fields, $t, $i, $addrmode, $objtype);

   if($cgi->param('r') eq 'addaddrrange') {
      $addrmode = 1;
   }

   if ($addrmode) {
      $objtype = '5';
   } else {
      $objtype = '4';
   }

   if(!$cgi->param('view')) {
      API_ExitError("A view id was not specified");
   }

   if($cgi->param('location') && $cgi->param('owner') && 
         !$cgi->param('device') && $cgi->param('done')) {
      $cgi->param('doproc', 1);
   }

   if($cgi->param('doproc') == 0 || !$cgi->param('doproc') || 
         !$cgi->param('start') || !$cgi->param('stop')) {

      if(!$cgi->param('template_select')) {
         %pitems = (
               'r' => $cgi->param('r'),
               'view' => $cgi->param('view'),
               'mode' => $cgi->param('mode') || '',
               'start' => $cgi->param('start'),
               'stop'  => $cgi->param('stop') || "",
               'status'=> $cgi->param('status') || "1",
            );

         my($tmplid) = API_TemplateCheckSingle($objtype);
         if($tmplid < 1) {
            API_TemplateSelectPage($objtype,\%pitems);
            return();
         } else {
            $cgi->param('template_select', $tmplid);
         }            
      }

      %pitems = (
            'start' => $cgi->param('start') || "",
            'stop'  => $cgi->param('stop') || "",
            'descr'	=> $cgi->param('descr') || "",
            'name'  => $cgi->param('name') || ""
         );

      API_OwnerDropdown(\$pitems{'owner_table'}, 
         ($cgi->param('owner') || undef), 
         'document.form.submit();');

      if($cgi->param('owner') || $cgi->param('location')) {
         API_DeviceDropdown(\$pitems{'device_table'}, undef, 
            ($cgi->param('owner') || undef), 
            ($cgi->param('location') || undef), 
            'document.form.doproc.value=1;');
      } else {
         $pitems{'device_table'} = "<SELECT NAME=\"device\"><OPTION>Select an Owner or Location Object First</OPTION></SELECT>";
      }

      if(API_LocationDropdown(\$pitems{'location_table'}, 
               ($cgi->param('location') || undef), 
               'document.form.submit();') < 1) {
         API_ExitError();
      }

      API_NetStatusDropdown(\$pitems{'status_table'}, 
         ($cgi->param('status') || undef));
      API_CIDRDropdown(\$pitems{'netmask_table'},
         ($cgi->param('netmask') || undef),
         undef, $maxlen,$submit);
      API_NetPriorityDropdown(\$pitems{'priority_table'}, 
         ($cgi->param('priority') || '5'));

      $pitems{'template_select'} = $cgi->param('template_select');
      $pitems{'view'} = $cgi->param('view');
      $pitems{'mode'} = $cgi->param('mode') || '';
      if($addrmode) {
         ModuleDispatch('Network','CORE/Network/AddNetworkRange/OutputAddAddrPagePre', \%pitems);
         API_OutputPage('address_addaddrrange.html', \%pitems);
         ModuleDispatch('Network','CORE/Network/AddNetworkRange/OutputAddAddrPagePost', \%pitems);
      } else {
         ModuleDispatch('Network','CORE/Network/AddNetworkRange/OutputAddNetPagePre', \%pitems);
         API_OutputPage('network_addnetrange.html', \%pitems);
         ModuleDispatch('Network','CORE/Network/AddNetworkRange/OutputAddNetPagePost', \%pitems);
      }
      my(%fields, @order);
      %fields = API_TemplateFieldGetAssignedFields($cgi->param('template_select'));
      @order = API_TemplateFieldGetAssignedFieldsOrder($cgi->param('template_select'));

      ModuleDispatch('Network','CORE/Network/AddNetworkRange/OutputAddPageTemplatePre', \%pitems);
      API_OutputPage('template_fieldedit_head.html', \%pitems);
      foreach(@order) {
         %pitems = (
               'id'	=> $_,
               'value'	=> '',
               'name'	=> $fields{$_},
               'example' => API_TemplateFieldGetExample($_),
               'required'=> API_TemplateFieldIsRequired($_) == 1 ? "*" : ""
            );
         $pitems{'formhtml'} = API_TemplateFieldCreateHTML($pitems{'id'}, $pitems{'value'});
         API_OutputPage('template_fieldedit_item.html', \%pitems);
      }
      API_OutputPage('template_fieldedit_foot.html', \%pitems);			
      ModuleDispatch('Network','CORE/Network/AddNetworkRange/OutputAddPageTemplatePost', \%pitems);

      if($addrmode) {
         ModuleDispatch('Network','CORE/Network/AddNetworkRange/OutputAddPageAddrFootPre', \%pitems);
         API_OutputPage('address_addaddrrange_foot.html', \%pitems);
         ModuleDispatch('Network','CORE/Network/AddNetworkRange/OutputAddPageAddrFootPost', \%pitems);
      } else {
         ModuleDispatch('Network','CORE/Network/AddNetworkRange/OutputAddPageNetFootPre', \%pitems);
         API_OutputPage('network_addnetrange_foot.html', \%pitems);
         ModuleDispatch('Network','CORE/Network/AddNetworkRange/OutputAddPageNetFootPost', \%pitems);
      }
   } else {
      %fields = (
            'required' => {
               'owner'     => 'num',	
               'start'     => 'ipv4addr',
               'stop'      => 'ipv4addr',
               'netmask'   => 'num',
               'location'	=> 'num',
               'status'    => 'num',
               'priority'  => 'num'
               },
            'optional' => {
               'device'    => 'num',
               'descr'     => 'alpha',
               'name'      => 'alpha',
               },
            'names' => {
               'descr'     => 'Network Description',
               'owner'     => 'Network Owner',
               'device'    => 'Network Device',
               'start'     => 'Starting Network Address',
               'stop'      => 'Stopping Network Address',
               'netmask'   => 'Network Mask Length',
               'name'      => 'Network Name',
               'location'  => 'Network Location',
               'status'    => 'Network Block Status',
               'priority'  => 'Network Block Priority'
               }
         );
      API_CheckFormFields(\%fields);

      my(%fields, @order);
      %fields = API_TemplateFieldGetAssignedFields($cgi->param('template_select'));	
      @order = API_TemplateFieldGetAssignedFieldsOrder($cgi->param('template_select'));

      foreach(@order) {
         my($x) = API_TemplateFieldCheckInput($_, \$cgi->param($_));

         if($x == 2) {			
            API_ExitError("The field $fields{$_} was not filled in");
         }
         if($x == 3) {			
            API_ExitError("The field $fields{$_} was too short");
         }
         if($x == 4) {			
            API_ExitError("The field $fields{$_} was too long");
         }				
         if($x == 5) {			
            API_ExitError("The field $fields{$_} was not correctly formatted");
         }				
      }
      my(@nets) = IP_GetRange($cgi->param('start'), $cgi->param('stop'), $cgi->param('netmask'));
      if(!@nets) {
         API_ExitError("The Starting Network is greater than the Stopping Network.");
      }
      foreach(@nets) {
         my(@start) = split(/\./, $_);
         my($name) = $cgi->param('name');
         $name =~ s/\%ADDR/$_/g;
         $name =~ s/\%OC1/$start[0]/g;
         $name =~ s/\%OC2/$start[1]/g;
         $name =~ s/\%OC3/$start[2]/g;
         $name =~ s/\%OC4/$start[3]/g;
         if(API_NetCheckExist($_, $cgi->param('netmask'), $cgi->param('view')) > 0) {
            if($addrmode) {
               API_ExitError("The address $_ already exists");
            } else {
               API_ExitError("The block $_/" . $cgi->param('netmask') . 
                  " already exists");
            }
         }

         my(%netinfo) = (
               'name'      => $name,
               'templateid'=> $cgi->param('template_select'),
               'viewid'    => $cgi->param('view'),
               'status'    => $cgi->param('status'),
               'descr'     => $cgi->param('descr'),
               'block'     => $_ . '/' . $cgi->param('netmask'),
               'owner'     => $cgi->param('owner'),
               'device'    => $cgi->param('device'),
               'location'  => $cgi->param('location'),
               'priority'  => $cgi->param('priority')
            );

         ModuleDispatch('Network','CORE/Network/AddNetworkRange/AddPre');
         my($id) = API_AddNetwork(\%netinfo);
         if($id < 1) {
            API_ExitError();
         }
         ModuleDispatch('Network','CORE/Network/AddNetworkRange/AddPost');

         foreach(@order) {
            API_TemplateFieldSetValue($id, $objtype, $_, $cgi->param($_));
         }
      }

      if($cgi->param('mode') eq 'java') {
         %pitems = ( 
               'url' => ConfigGetValue('Global','ScriptURL') . 
               "?r=shnettree&mode=java&view=" . $cgi->param('view') . 
               "&path=" . $cgi->param('path')
            );
         API_OutputPage('reload_frame.html', \%pitems);
      } else {
         ShowNetTree();
      }
   }	
}

sub UpdNet {
   my(@row, %pitems, %netinfo, $sql);
   if(!$cgi->param('id')) {
      API_ExitError("An network id to update was not specified");
   }

   %netinfo = API_GetNetwork($cgi->param('id'));
   if(!%netinfo) {
      API_ExitError("The block specified does not exist");
   }

   if(!$cgi->param('doproc')) {
      %pitems = %netinfo;

      if(!$cgi->param('owner')) {
         API_OwnerDropdown(\$pitems{'owner_table'}, $pitems{'owner'}, 
            'document.form.submit();');
      } else {
         API_OwnerDropdown(\$pitems{'owner_table'}, 
            ($cgi->param('owner') || undef), 
            'document.form.submit();');
      }

      if($cgi->param('owner') || $cgi->param('location')) {
         API_DeviceDropdown(\$pitems{'device_table'}, undef, 
            ($cgi->param('owner') || undef), 
            ($cgi->param('location') || undef), 
            'document.form.doproc.value=1;');
      } else {
         API_DeviceDropdown(\$pitems{'device_table'}, $pitems{device}, 
            $pitems{owner}, ($cgi->param('location') || undef),
            'document.form.doproc.value=1;');
      }

      if(!$cgi->param('location')) {
         if(API_LocationDropdown(\$pitems{'location_select'}, $pitems{'location'}, 'document.form.submit();') < 1) {
            API_ExitError();
         }
      } else {
         if(API_LocationDropdown(\$pitems{'location_select'}, 
                  ($cgi->param('location') || undef),
                  'document.form.submit();') < 1) {
            API_ExitError();
         } 
      }

      if(!$cgi->param('status')) {
         API_NetStatusDropdown(\$pitems{'status_select'}, $pitems{'status'});
      } else {
         API_NetStatusDropdown(\$pitems{'status_select'}, 
            ($cgi->param('status') || undef));
      }

      if(!$cgi->param('priority')) {
         API_NetPriorityDropdown(\$pitems{'priority_select'}, $pitems{'priority'});
      } else {
         API_NetPriorityDropdown(\$pitems{'priority_select'}, 
            ($cgi->param('priority') || undef));
      }

      if($pitems{'parent'} == 0) {
         $pitems{'parent_net'} = "Top-level Network" . "<input type=\"hidden\" name=\"parent\" value=\"0\">";
      } else {
         %parentinfo = API_GetNetwork($pitems{'parent'});
         $pitems{'parent_net'} = "$parentinfo{'name'} ($parentinfo{'label'})" . "<input type=\"hidden\" name=\"parent\" value=\"$pitems{'parent'}\">";
      }

      $pitems{'view'} = $cgi->param('view');
      $pitems{'mode'} = $cgi->param('mode') || '';
      $pitems{'depth'} = $cgi->param('depth');
      $pitems{'path'} = $cgi->param('path');

      ModuleDispatch('Network','CORE/Network/UpdNetwork/OutputUpdPagePre', \%pitems);
      API_OutputPage('network_updnet.html', \%pitems);
      ModuleDispatch('Network','CORE/Network/UpdNetwork/OutputUpdPagePost', \%pitems);
      my(%fields, @order);

      %fields = API_TemplateFieldGetAssignedFields($netinfo{'templateid'});
      @order = API_TemplateFieldGetAssignedFieldsOrder($netinfo{'templateid'});

      ModuleDispatch('Network','CORE/Network/UpdNetwork/OutputUpdPageTemplatePre', \%pitems);
      API_OutputPage('template_fieldedit_head.html');
      foreach(@order) {
         %pitems = (
               'id'	   => $_,
               'name'	=> $fields{$_},
               'example' => API_TemplateFieldGetExample($_),
               'required'=> API_TemplateFieldIsRequired($_) == 1 ? "*" : ""
            );
         if($netinfo{'masklen'} == 32) {
            $pitems{'value'} = API_TemplateFieldGetValue($cgi->param('id'), '5', $_) || "";
         } else {
            $pitems{'value'} = API_TemplateFieldGetValue($cgi->param('id'), '4', $_) || "";
         }

         $pitems{'formhtml'} = API_TemplateFieldCreateHTML($pitems{'id'}, $pitems{'value'});
         API_OutputPage('template_fieldedit_item.html', \%pitems);
      }
      %pitems = ();
      API_OutputPage('template_fieldedit_foot.html', \%pitems);			
      ModuleDispatch('Network','CORE/Network/UpdNetwork/OutputUpdPageTemplatePost', \%pitems);

      ModuleDispatch('Network','CORE/Network/UpdNetwork/OutputUpdPageFootPre', \%pitems);
      API_OutputPage('network_updnet_foot.html', \%pitems);
      ModuleDispatch('Network','CORE/Network/UpdNetwork/OutputUpdPageFootPost', \%pitems);

   } else {	
      %fields = (
            'required' => {
               'name'      => 'alpha',
               'owner'     => 'num',	
               'location'  => 'num',
               'status'    => 'num',
               'priority'  => 'num',
               },
            'optional' => {
               'device'    => 'num',
               'descr'     => 'alpha',
               'status_recurse'=>'num',
               },
            'names' => {
               'name'      => 'Network Name',
               'descr'     => 'Network Description',
               'owner'     => 'Network Owner',
               'device'    => 'Network Device',
               'location'  => 'Network Location',
               'status'    => 'Network Block Status',
               'status_recurse' => 'Network Block Status Recursion',
               'priority'  => 'Network Block Priority',
               }
         );
      API_CheckFormFields(\%fields);

      my(%fields, @order);
      %netinfo = API_GetNetwork($cgi->param('id'));
      %fields = API_TemplateFieldGetAssignedFields($netinfo{'templateid'});
      @order = API_TemplateFieldGetAssignedFieldsOrder($netinfo{'templateid'});

      foreach(@order) {
         my($x) = API_TemplateFieldCheckInput($_, \$cgi->param($_));

         if($x == 2) {			
            API_ExitError("The field $fields{$_} was not filled in");
         }
         if($x == 3) {			
            API_ExitError("The field $fields{$_} was too short");
         }
         if($x == 4) {			
            API_ExitError("The field $fields{$_} was too long");
         }				
         if($x == 5) {			
            API_ExitError("The field $fields{$_} was not correctly formatted");
         }				
      }

      my($ip,$l) = split(/\//, $netinfo{'label'});

      if(API_CheckNetworkStatus($ip, $l, $netinfo{'viewid'}) < 1) {
         API_ExitError();
      }

      if(!$cgi->param('device')) {
         $cgi->param('device', "0");
      }

      my(%netinfo) = (
            'id'     => $cgi->param('id'),
            'name'   => $cgi->param('name'),
            'status' => $cgi->param('status'),
            'descr'  => $cgi->param('descr'),
            'owner'  => $cgi->param('owner'),
            'device' => $cgi->param('device'),
            'location'=>$cgi->param('location'),
            'priority'=>$cgi->param('priority')
         );

      ModuleDispatch('Network','CORE/Network/UpdNetwork/UpdPre');
      if(API_UpdNetwork(\%netinfo) < 1) {
         API_ExitError();
      }
      ModuleDispatch('Network','CORE/Network/UpdNetwork/UpdPost');

      %netinfo = API_GetNetwork($cgi->param('id'));

      if($cgi->param('status') == 3 || $cgi->param('status_recurse') == 1) {
         if(API_NetStatusUpdate($netinfo{'id'}, $cgi->param('status')) < 1) {
            API_ExitError();
         }
      }

      if($cgi->param('priority_recurse') == 1) {
         if(API_NetPriorityUpdate($cgi->param('id'), $cgi->param('priority')) < 1) {
            API_ExitError();
         }
      }

      foreach(@order) {
         if($netinfo{'masklen'} == 32) {
            API_TemplateFieldSetValue($cgi->param('id'), '5', $_, $cgi->param($_));
         } else {
            API_TemplateFieldSetValue($cgi->param('id'), '4', $_, $cgi->param($_));
         }
      }

      if($cgi->param('mode') eq 'java') {
         %pitems = ( 
               'url' => ConfigGetValue('Global','ScriptURL') . 
               "?r=shnettree&mode=java&view=" . $cgi->param('view') .
               "&path=" . $cgi->param('path')
            );
         API_OutputPage('reload_frame.html', \%pitems);
      } else {
         ShowNetTree();
      }
   }
}

sub DelNet {
   my(%pitems);
   if(!$cgi->param('id')) {
      API_ExitError("A network id to delete was not specified");
   }

   my(%netinfo) = API_GetNetwork($cgi->param('id'));
   if(!%netinfo) {
      API_ExitError();
   }

   my($mask) = $netinfo{'masklen'};
   if($cgi->param('confirm') == '2' ) {
      if(API_NetReassignParent($cgi->param('id'), $netinfo{'parent'}) < 1) {
         API_ExitError();
      }
   }

   if($cgi->param('confirm')) {
      my(@delids);
      ModuleDispatch('Network','CORE/Network/DelNetwork/DelPre');
      if(API_DelNetwork($cgi->param('id'), \@delids) < 1) {
         API_ExitError();
      }
      ModuleDispatch('Network','CORE/Network/DelNetwork/DelPost');

      foreach(@delids) {
         if(API_TemplateFieldDeleteAllValues($_, '4') < 1) {
            API_ExitError();
         }
         if(API_TemplateFieldDeleteAllValues($_, '5') < 1) {
            API_ExitError();
         }
      }

      $cgi->param('id', $netinfo{'parent'});

      if($cgi->param('mode') eq 'java') {
         %pitems = ( 
               'url' => ConfigGetValue('Global','ScriptURL') . 
               "?r=shnettree&mode=java&view=" . $cgi->param('view') . 
               "&path=" . $cgi->param('path')
            );
         API_OutputPage('reload_frame.html', \%pitems);
      } else {
         ShowNetTree();
      }
   } else {
      %pitems = (
            'object'=>'Network',
            'r'     => $cgi->param('r'),
            'id'    => $cgi->param('id'),
            'view'  => $cgi->param('view'),
            'mode'  => $cgi->param('mode') || ''
         );
      %netinfo = API_GetNetwork($cgi->param('id'));
      my($aryref) = $netinfo{'children'};
      if($mask == 32 || !@{$aryref}) {
         $pitems{'move_delete'} = '<input type="hidden" value="1" name="confirm">';
      } else {
         $pitems{'move_delete'} = 'Select a delete action:<br><table border=0><tr>'.
            '<td><input type="radio" value="1" name="confirm" '.
            'checked>&nbsp;Delete ALL Children</td></tr><td>'.
            '<input type="radio" value="2" name="confirm">'.
            '&nbsp;Move Children up the Tree</td></tr></table>';
      }

      ModuleDispatch('Network','CORE/Network/DelNetwork/OutputConfirmPre', \%pitems);
      API_OutputPage('delete_confirm.html',\%pitems);
      ModuleDispatch('Network','CORE/Network/DelNetwork/OutputConfirmPost', \%pitems);
   }
}

sub ShowNet {
   my(%pitems, @row, $i, $ip);
   if(!$cgi->param('id')) {
      API_ExitError("A network ID to view was not specified");
   }

   %pitems = API_GetNetwork($cgi->param('id'), 1);			
   if(!%pitems) {
      API_ExitError();
   }

   my(%netinfo) = %pitems;

   if($pitems{'parentid'} == 0) {
      $pitems{'parentlink'} = $pitems{'parent'};
   } else {
      $pitems{'parentlink'} = '<A HREF="'. ConfigGetValue('Global','ScriptURL').
         "?r=shnet&id=$pitems{'parentid'}\">".
         "$pitems{'parent'}</A>";
   }

   ModuleDispatch('Network','CORE/Network/ShowNetwork/OutputHeadPre', \%pitems);
   API_OutputPage('net_detailshow_head.html', \%pitems);
   ModuleDispatch('Network','CORE/Network/ShowNetwork/OutputHeadPost', \%pitems);

   ModuleDispatch('Network','CORE/Network/ShowNetwork/OutputBodyPre', \%pitems);
   API_OutputPage('net_detailshow_body.html', \%pitems);
   ModuleDispatch('Network','CORE/Network/ShowNetwork/OutputBodyPost', \%pitems);

   my(%fields, @order);
   %fields = API_TemplateFieldGetAssignedFields($netinfo{'templateid'});
   @order = API_TemplateFieldGetAssignedFieldsOrder($netinfo{'templateid'});

   %pitems = ();
   ModuleDispatch('Network','CORE/Network/ShowNetwork/OutputTemplatePre', \%pitems);
   API_OutputPage('template_fieldview_head.html', \%pitems);

   foreach(@order) {
      %pitems = (
            'id'	=> $_,
            'name'	=> $fields{$_},
            'required'=> '',
            'view' => $cgi->param('view'),
         );
      if($netinfo{'masklen'} == 32) {
         $pitems{'value'} = API_TemplateFieldGetValue($cgi->param('id'), '5', $_) || "";
      } else {
         $pitems{'value'} = API_TemplateFieldGetValue($cgi->param('id'), '4', $_) || "";
      }

      $pitems{'formhtml'} = API_TemplateFieldCreateHTML($pitems{'id'}, $pitems{'value'});
      API_OutputPage('template_fieldview_item.html', \%pitems);
   }
   API_OutputPage('template_fieldview_foot.html', \%pitems);			
   ModuleDispatch('Network','CORE/Network/ShowNetwork/OutputTemplatePost', \%pitems);

   my($aryref) = $netinfo{'children'};
   if($netinfo{'masklen'} != 32 && @{$aryref}) {
      my(%addrs, @order);
      foreach(@{$aryref}) {
         Debug("child=$_");
         my(%childinfo) = API_GetNetwork($_);
         if(!%childinfo) {
            API_ExitError();
         }
         if($childinfo{'masklen'} == 32) {
            my($addr) = IP_IntToDQ($childinfo{'network'});
            $addrs{$addr}{'id'} 	= $_;
            $addrs{$addr}{'address'}	= $addr;
            $addrs{$addr}{'dnsname'} 	= $childinfo{'name'};
            push(@order, $addr);
         }
      }
      my($network, $masklen) = split(/\//, $netinfo{'label'});
      my($broadcast) = IP_LastIP($network, $masklen);

      %pitems = ();
      ModuleDispatch('Network','CORE/Network/ShowNetwork/OutputAddrHeadPre', \%pitems);
      API_OutputPage('net_detailshow_addrlisthead.html', \%pitems);
      ModuleDispatch('Network','CORE/Network/ShowNetwork/OutputAddrHeadPost', \%pitems);

      if(@{$aryref} && $masklen >= '24' && $masklen < '32') {
         @order = ();

         my(@netparts) = split(/\./, $network);	
         my(@broadparts) = split(/\./, $broadcast);	
         for($i = $netparts[3]; $i <= $broadparts[3]; $i++) {
            my($ip) = "$netparts[0].$netparts[1].$netparts[2].$i";
            if(!$addrs{$ip}) {
               $addrs{$ip}{id} = "NOASSIGN";
               $addrs{$ip}{address} = "$ip";
            }
            push(@order, $ip);
         }
      } 

      foreach(@order) { 
         %pitems = (
               'id'		   =>	$addrs{$_}{id},
               'address'	=>	$addrs{$_}{address},
               'dnsname'	=>	$addrs{$_}{dnsname},
               'parent'	   =>	$cgi->param('id'),
               'view'    	=> $cgi->param('view'),
            );
         ModuleDispatch('Network','CORE/Network/ShowNetwork/OutputAddrItemPre', \%pitems);
         if($pitems{'address'} eq $network || $pitems{'address'} eq $broadcast) {
            API_OutputPage('net_detailshow_addrlistitemnetwork.html',\%pitems);
         } elsif($pitems{'id'} =~ /NOASSIGN/) {
            API_OutputPage('net_detailshow_addrlistitemavailable.html',\%pitems);
         } else {
            API_OutputPage('net_detailshow_addrlistitemused.html',\%pitems);
         }
         ModuleDispatch('Network','CORE/Network/ShowNetwork/OutputAddrItemPost', \%pitems);
      }	

      %pitems = ();
      ModuleDispatch('Network','CORE/Network/ShowNetwork/OutputAddrFootPre', \%pitems);
      API_OutputPage('net_detailshow_addrlistfoot.html', \%pitems);		
      ModuleDispatch('Network','CORE/Network/ShowNetwork/OutputAddrFootPost', \%pitems);
   }

   %pitems = ();
   ModuleDispatch('Network','CORE/Network/ShowNetwork/OutputFootPre', \%pitems);
   API_OutputPage('net_detailshow_foot.html', \%pitems);		 			
   ModuleDispatch('Network','CORE/Network/ShowNetwork/OutputFootPost', \%pitems);
}

sub ShowNetTree {
   my($i, $p, $id, $a, $b, $t, $d, @sort, %labels);

   if(!$cgi->param('view')) {    
      my($view_single) = API_GetViewSingle();
      if($view_single) { 
         $cgi->param('view', $view_single);
      } else {
         my(%form) = API_FormHash();
         if(API_ViewDropdown(\$form{'viewselect'}) < 1) {
            API_ExitError();
         }
         API_OutputPage('view_select.html',\%form);
         return();
      }
   }

   if($cgi->param('mode') eq 'java') {
      ShowNetTreeJava($cgi->param('view'));
      return();
   }

   my(@data);
   API_GetNetworkList([qw(id parent name label)], \@data, [qw(network masklen)],
      [{ 'field' => 'viewid',
            'type'  => '=',
            'data'  => $cgi->param('view')
            }]);
   foreach(@data) {
      push(@sort, ${$_}{'id'});
      $parents{${$_}{'id'}} = ${$_}{'parent'};
      $labels{${$_}{'id'}}{'name'} = ${$_}{'name'};
      $labels{${$_}{'id'}}{'network'} = ${$_}{'label'};
   }

   my($n0) = {
      'name'          => 'IPv4',
      'network'       => '0.0.0.0/0',
      'id'            => '0',
      'parent'        => '0',
      'children'      => [],
   };

   my(%lasthash);

   $d = 1;
   foreach(@sort) {
      $id = $_;
      $p = $parents{$_};
      $a = "n$id";
      $b = "n$p";
      $lasthash{$p} = $id;
      my($tnet,$tmask) = split(/\//, $labels{$_}{'network'});
      if($tmask == 32) {
         ${$a} = {                
            'name'          => $labels{$_}{'name'},
            'network'       => $labels{$_}{'network'},
            'id'            => $_,
            'html'          => 'address_showtree_item.html',
            'parent'        => $p,
            'last'          => '0',
            'children'      => [],
         };
      } else {
         ${$a} = {                
            'name'          => $labels{$_}{'name'},
            'network'       => $labels{$_}{'network'},
            'id'            => $_,
            'parent'        => $p,
            'last'          => '0',
            'children'      => [],
         };
      }

      if($p == 0) {
         API_TreeGrow($n0, ${$a});
      } else {
         API_TreeGrow(${$b}, ${$a});
      }
   }

   while(($a,$b) = each %lasthash) {
      $t = "n$b";
      if(${$t}->{'parent'} == 0) {
         ${$t}->{'last'} = '2';
      } else {
         ${$t}->{'last'} = '1';
      }
   }

   my(%viewinfo) = API_GetView($cgi->param('view'));
   if(!%viewinfo) {
      API_ExitError();
   }
   $pitems{'view'} = $viewinfo{'name'};

   $pitems{'viewid'} = $cgi->param('view');
   API_OutputPage('net_showtree_head.html',\%pitems);
   API_TreeDumpTable('net_showtree_item.html', $n0, 
      ($cgi->param('path') || undef), 
      ($cgi->param('depth') || undef), '', 'shnet', 
      ("shnettree&view=" . $cgi->param('view')),'','',\%pitems);
   API_OutputPage('net_showtree_foot.html');
}

sub ShowNetTreeJava {
   my($view) = shift;
   my(%pitems, $i, @sort, %parents, %labels);
   if(($cgi->param('parent') || $cgi->param('parent') == 0) && $cgi->param('x')) {
      my(@data);
      API_GetNetworkList([qw(id parent name label)], \@data, [qw(network masklen)],
         [{ 'field' => 'viewid',
               'type'  => '=',
               'data'  => $view
               },
            { 'field' => 'parent',
               'type'  => '=',
               'data'  => $cgi->param('parent')
               }]);
      foreach(@data) {
         my(%netinfo) = API_GetNetwork(${$_}{'id'});
         my($aryref) = $netinfo{'children'};
         push(@sort, ${$_}{'id'});
         $parents{${$_}{'id'}} = ${$_}{'parent'};
         $labels{${$_}{'id'}}{'name'} = ${$_}{'name'};
         $labels{${$_}{'id'}}{'network'} = ${$_}{'label'};
         $labels{${$_}{'id'}}{'children'} = @{$aryref};
      }

      $i = 0;
      foreach(@sort) {
         print "$_;$parents{$_};$labels{$_}{'network'};$labels{$_}{'name'};$labels{$_}{'children'}\n";
         $i++;
      }
      return;
   }

   my(%viewinfo) = API_GetView($cgi->param('view'));
   if(!%viewinfo) {
      API_ExitError();
   }
   $pitems{'viewName'} = $viewinfo{'name'};
   $pitems{'viewID'} = $view;
   $pitems{'treePath'} = $cgi->param('path') || "";
   API_OutputPage('net_showtree_java.html', \%pitems);
}


sub NetworkAlloc {
    # right.. groovy baby.. turn left for me.. oh yeah baby..
   my($sql, %pitems, %nets, %priority, $i);


    # no! no! no!
   if(!$cgi->param('view')) {    
      my($view_single) = API_GetViewSingle();
      if($view_single) { 
         $cgi->param('view', $view_single);
      } else {
         my(%form) = API_FormHash();
         if(API_ViewDropdown(\$form{'viewselect'}) < 1) {
            API_ExitError();
         }
         API_OutputPage('view_select.html',\%form);
         return();
      }
   }

   if($cgi->param('doproc') == 0 || !$cgi->param('doproc')) {
      if(API_LocationDropdown(\$pitems{'location_table'}, 
               ($cgi->param('location') || undef)) < 1) {
         API_ExitError();
      }      
      API_CIDRDropdown(\$pitems{'netmask_table'}, 
         ($cgi->param('netmask') || undef));
      API_OwnerDropdown(\$pitems{'owner_table'}, 
         ($cgi->param('owner') || undef));

      $pitems{'view'} = $cgi->param('view');
      API_OutputPage('network_alloc.html', \%pitems);
      return();
   }

   my(%fields) = (
         'required' => {
            'location'  => 'num',
            'netmask'   => 'num',
            },
         'optional' => {
            'owner'     => 'num',	
            },
         'names' => {
            'owner'     => 'Network Owner',
            'netmask'   => 'Network Address Mask',
            'location'  => 'Network Location'
            }
      );
   API_CheckFormFields(\%fields);
   my(@data);
   if($cgi->param('owner') > 0) {
      API_GetNetworkList([qw(id network masklen priority)], \@data, [qw(network masklen)],
         [{ 'field' => 'viewid',
               'type'  => '=',
               'data'  => $cgi->param('view')
               },
            { 'field' => 'masklen',
               'type'  => '<',
               'data'  => $cgi->param('netmask')
               },
            { 'field' => 'owner',
               'type'  => '=',
               'data'  => $cgi->param('owner')
               },
            { 'field' => 'location',
               'type'  => '=',
               'data'  => $cgi->param('location')
               },
            { 'field' => 'status',
               'type'  => '=',
               'data'  => '1'
               }]);
   } else {
      API_GetNetworkList([qw(id network masklen priority)], \@data, [qw(network masklen)],
         [{ 'field' => 'viewid',
               'type'  => '=',
               'data'  => $cgi->param('view')
               },
            { 'field' => 'masklen',
               'type'  => '<',
               'data'  => $cgi->param('netmask')
               },
            { 'field' => 'location',
               'type'  => '=',
               'data'  => $cgi->param('location')
               },
            { 'field' => 'status',
               'type'  => '=',
               'data'  => '1'
               }]);
   }

   if(!@data) {
      %pitems = (
            'r' => 'addnet',
            'view' => $cgi->param('view'),
            'netaddr' => '',
            'netmask' => $cgi->param('netmask'),
            'parent'  => '',
            'owner'   => $cgi->param('owner') || '',
            'location'=> $cgi->param('location')
         );

      my($tmplid) = API_TemplateCheckSingle('4');
      if($tmplid < 1) {
         API_TemplateSelectPage('4',\%pitems);
         return();
      } else {
         $cgi->param('template_select', $tmplid);
         $cgi->param('doproc', 0);
         AddNet();
      }  
      return();
   }

   foreach(@data) {
      $nets{${$_}{'id'}} = ${$_}{'network'} . '/' . ${$_}{'masklen'};
      $priority{${$_}{'id'}} = ${$_}{'priority'};
   }

   # oh yeah.. here comes the magic.. watch out
   my($block) = API_NetworkGetBlock(\%nets, \%priority, $cgi->param('netmask'), $cgi->param('view'));
   my($n, $m) = split(/\//, $block);

   my($tmplid) = API_TemplateCheckSingle('4');
   if($tmplid < 1) {
      %pitems = (
            'r' => 'addnet',
            'view' => $cgi->param('view'),
            'netaddr' => $n,
            'netmask' => $m,
            'parent'  => '',
            'owner'   => $cgi->param('owner') || '',
            'location'=> $cgi->param('location')
         );
      API_TemplateSelectPage('4',\%pitems);
   } else {
      $cgi->param('template_select', $tmplid);
      $cgi->param('doproc', 0);
      $cgi->param('netaddr', $n); 
      $cgi->param('netmask', $m);
      $cgi->param('parent', '');
      AddNet();
   }    
   # and... i'm spent
}

return 1;
