# 
# Copyright (C) 2001, Hitesh Patel <hitesh@presys.com>.
# All rights reserved.
# 
# This file is part of NorthStar and is covered under
# the terms of the Apache Software License version 1.1
#
# A copy of this license is included in the LICENSE file
#
# $Header: /cvsroot/NorthStar/lib/NorthStar/Owner.pm,v 1.14 2002/05/21 00:56:18 brownkid Exp $
#

package NorthStar::Owner;
use Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(AddOwner UpdOwner DelOwner ShowOwner ShowOwnerTree);

use NorthStar::API qw(:ALL);

sub init {
   Debug("OWNER: initializing module");
   my(%modinfo) = (
         export_hooks => [
            'CORE/Owner/AddOwner/OutputAddPagePre',
            'CORE/Owner/AddOwner/OutputAddPagePost',
            'CORE/Owner/AddOwner/OutputAddPageTemplatePre',
            'CORE/Owner/AddOwner/OutputAddPageTemplatePost',
            'CORE/Owner/AddOwner/OutputAddPageFootPre',
            'CORE/Owner/AddOwner/OutputAddPageFootPost',
            'CORE/Owner/AddOwner/AddPre',
            'CORE/Owner/AddOwner/AddPost',
            'CORE/Owner/UpdOwner/OutputUpdPagePre',
            'CORE/Owner/UpdOwner/OutputUpdPagePost',
            'CORE/Owner/UpdOwner/OutputUpdPageTemplatePre',
            'CORE/Owner/UpdOwner/OutputUpdPageTemplatePost',
            'CORE/Owner/UpdOwner/OutputUpdPageFootPre',
            'CORE/Owner/UpdOwner/OutputUpdPageFootPost',
            'CORE/Owner/UpdOwner/UpdPre',
            'CORE/Owner/UpdOwner/UpdPost',
            'CORE/Owner/DelOwner/OutputConfirmPre',
            'CORE/Owner/DelOwner/OutputConfirmPost',
            'CORE/Owner/DelOwner/DelPre',
            'CORE/Owner/DelOwner/DelPost',
            'CORE/Owner/ShowOwner/OutputHeadPre',
            'CORE/Owner/ShowOwner/OutputHeadPost',
            'CORE/Owner/ShowOwner/OutputBodyPre',
            'CORE/Owner/ShowOwner/OutputBodyPost',
            'CORE/Owner/ShowOwner/OutputTemplatePre',
            'CORE/Owner/ShowOwner/OutputTemplatePost',
            'CORE/Owner/ShowOwner/OutputNetblocksHeadPre',
            'CORE/Owner/ShowOwner/OutputNetblocksHeadPost',
            'CORE/Owner/ShowOwner/OutputNetblocksItemPre',
            'CORE/Owner/ShowOwner/OutputNetblocksItemPost',
            'CORE/Owner/ShowOwner/OutputNetblocksFootPre',
            'CORE/Owner/ShowOwner/OutputNetblocksFootPost',
            'CORE/Owner/ShowOwner/OutputDevHeadPre',
            'CORE/Owner/ShowOwner/OutputDevHeadPost',
            'CORE/Owner/ShowOwner/OutputDevItemPre',
            'CORE/Owner/ShowOwner/OutputDevItemPost',
            'CORE/Owner/ShowOwner/OutputDevFootPre',
            'CORE/Owner/ShowOwner/OutputDevFootPost',
            'CORE/Owner/ShowOwner/OutputFootPre',
            'CORE/Owner/ShowOwner/OutputFootPost'
         ],
         user_hooks => {
            'addowner'      => \&NorthStar::Owner::AddOwner,
            'updowner'      => \&NorthStar::Owner::UpdOwner,
            'delowner'      => \&NorthStar::Owner::DelOwner,
            'showner'       => \&NorthStar::Owner::ShowOwner,
            'shownertree'   => \&NorthStar::Owner::ShowOwnerTree,
            },
         import_hooks => {},
      );

   API_PermissionsRegisterPerm(3, 0, 'addowner', 'Add Owner', 'CORE');
   API_PermissionsRegisterPerm(4, 0, 'updowner', 'Update Owner', 'CORE');
   API_PermissionsRegisterPerm(5, 0, 'delowner', 'Delete Owner', 'CORE');
   API_PermissionsRegisterPerm(51, 1, 'showner', 'Show Owner', 'CORE');    
   API_PermissionsRegisterPerm(58, 1, 'shownertree', 'Show Owner Tree', 'CORE');

   my($script) = ConfigGetValue('Global','ScriptURL');
   API_MenuAddItem('Browse','Owners', "$script?r=shownertree",undef, 201); 

   return(%modinfo);
}

sub AddOwner {
   if(!$cgi->param('doproc')) {
      if(!$cgi->param('template_select')) {
         %pitems = (
               'r' => 'addowner',
               'parent' => $cgi->param('parent'),
               'netaddr'=> $cgi->param('netaddr')
            );

         my($tmplid) = API_TemplateCheckSingle('1');
         if($tmplid < 1) {
            API_TemplateSelectPage('1',\%pitems);
            return();
         } else {
            $cgi->param('template_select', $tmplid);
         }            
      }

      $pitems{'template_select'} = $cgi->param('template_select');

      ModuleDispatch('Owner','CORE/Owner/AddOwner/OutputAddPagePre', \%pitems);
      API_OutputPage('owner_add.html', \%pitems);
      ModuleDispatch('Owner','CORE/Owner/AddOwner/OutputAddPagePost', \%pitems);

      my(%fields, @order);
      %fields = API_TemplateFieldGetAssignedFields($cgi->param('template_select'));
      @order = API_TemplateFieldGetAssignedFieldsOrder($cgi->param('template_select'));

      %pitems = ();
      ModuleDispatch('Owner','CORE/Owner/AddOwner/OutputAddPageTemplatePre', \%pitems);
      API_OutputPage('template_fieldedit_head.html',\%pitems);

      foreach(@order) {
         %pitems = (
               'id'	=> $_,
               'value'	=> '',
               'name'	=> $fields{$_},
               'example' => API_TemplateFieldGetExample($_),
               'required'=> API_TemplateFieldIsRequired($_) == 1 ? "*" : ""
            );
         $pitems{'formhtml'} = API_TemplateFieldCreateHTML($pitems{'id'}, $pitems{'value'});
         API_OutputPage('template_fieldedit_item.html', \%pitems);
      }
      API_OutputPage('template_fieldedit_foot.html', \%pitems);			
      ModuleDispatch('Owner','CORE/Owner/AddOwner/OutputAddPageTemplatePost', \%pitems);

      ModuleDispatch('Owner','CORE/Owner/AddOwner/OutputAddPageFootPre', \%pitems);
      API_OutputPage('owner_add_foot.html', \%pitems);
      ModuleDispatch('Owner','CORE/Owner/AddOwner/OutputAddPageFootPost', \%pitems);
   } else {	
      my(%fields) = (
            'required' => {
               'name'		=>	'alpha',
               },
            'optional' => {
               'descr'		=>	'alpha',
               },
            'names' => {
               'name'		=>	'Owner Name',
               'descr'		=>	'Owner Description',
               }
         );

      API_CheckFormFields(\%fields);

      %fields = API_TemplateFieldGetAssignedFields($cgi->param('template_select'));	
      my(@order) = API_TemplateFieldGetAssignedFieldsOrder($cgi->param('template_select'));

      foreach(@order) {
         my($x) = API_TemplateFieldCheckInput($_, \$cgi->param($_));

         if($x == 2) {			
            API_ExitError("The field $fields{$_} was not filled in");
         }
         if($x == 3) {			
            API_ExitError("The field $fields{$_} was too short");
         }
         if($x == 4) {			
            API_ExitError("The field $fields{$_} was too long");
         }				
         if($x == 5) {			
            API_ExitError("The field $fields{$_} was not correctly formatted");
         }				
      }

      my(%ownerinfo) = (
            'templateid'   => $cgi->param('template_select'),
            'name'         => $cgi->param('name'),
            'descr'        => $cgi->param('descr'),
         );


      ModuleDispatch('Owner','CORE/Owner/AddOwner/AddPre');
      my($id) = API_AddOwner(\%ownerinfo);
      if($id < 1) {
         API_ExitError();
      }
      ModuleDispatch('Owner','CORE/Owner/AddOwner/AddPost');

      foreach(@order) {
         API_TemplateFieldSetValue($id, '1', $_, $cgi->param($_));
      }
      ShowOwnerTree();
   }
}

sub UpdOwner {
   if(!$cgi->param('id')) {
      API_ExitError("An owner id to update was not specified");
   }

   if(!$cgi->param('doproc')) {
      %pitems = API_GetOwner($cgi->param('id'));
      if(!%pitems) {
         API_ExitError();
      }
      ModuleDispatch('Owner','CORE/Owner/UpdOwner/OutputUpdPagePre', \%pitems);
      API_OutputPage('owner_upd.html', \%pitems);
      ModuleDispatch('Owner','CORE/Owner/UpdOwner/OutputUpdPagePost', \%pitems);

      my(%fields, @order);
      %fields = API_TemplateFieldGetAssignedFields($pitems{'templateid'});
      @order = API_TemplateFieldGetAssignedFieldsOrder($pitems{'templateid'});

      %pitems = ();
      ModuleDispatch('Owner','CORE/Owner/UpdOwner/OutputUpdPageTemplatePre', \%pitems);
      API_OutputPage('template_fieldedit_head.html', \%pitems);
      foreach(@order) {
         %pitems = (
               'id'	=> $_,
               'value'	=> API_TemplateFieldGetValue($cgi->param('id'), '1', $_) || "",
               'name'	=> $fields{$_},
               'example' => API_TemplateFieldGetExample($_),
               'required'=> API_TemplateFieldIsRequired($_) == 1 ? "*" : ""
            );
         $pitems{'formhtml'} = API_TemplateFieldCreateHTML($pitems{'id'}, $pitems{'value'});
         API_OutputPage('template_fieldedit_item.html', \%pitems);
      }
      API_OutputPage('template_fieldedit_foot.html', \%pitems);			
      ModuleDispatch('Owner','CORE/Owner/UpdOwner/OutputUpdPageTemplatePost', \%pitems);

      ModuleDispatch('Owner','CORE/Owner/UpdOwner/OutputUpdPageFootPre', \%pitems);
      API_OutputPage('owner_upd_foot.html', \%pitems); 
      ModuleDispatch('Owner','CORE/Owner/UpdOwner/OutputUpdPageFootPost', \%pitems);
   } else {	
      my(%fields) = (
            'required' => {
               'name'		=>	'alpha',
               },
            'optional' => {
               'descr'		=>	'alpha',
               },
            'names' => {
               'name'		=>	'Owner Name',
               'descr'		=>	'Owner Description',
               }
         );

      API_CheckFormFields(\%fields);

      my(%ownerinfo) = API_GetOwner($cgi->param('id'));

      my($id) = $ownerinfo{'templateid'};
      %fields = API_TemplateFieldGetAssignedFields($id);
      my(@order) = API_TemplateFieldGetAssignedFieldsOrder($id);

      foreach(@order) {
         my($x) = API_TemplateFieldCheckInput($_, \$cgi->param($_));

         if($x == 2) {			
            API_ExitError("The field $fields{$_} was not filled in");
         }
         if($x == 3) {			
            API_ExitError("The field $fields{$_} was too short");
         }
         if($x == 4) {			
            API_ExitError("The field $fields{$_} was too long");
         }				
         if($x == 5) {			
            API_ExitError("The field $fields{$_} was not correctly formatted");
         }				
      }

      %ownerinfo = (
            'id'     => $cgi->param('id'),
            'name'   => $cgi->param('name'),
            'descr'  => $cgi->param('descr'),
         );


      ModuleDispatch('Owner','CORE/Owner/UpdOwner/UpdPre');
      if(API_UpdOwner(\%ownerinfo) < 1) {
         API_ExitError();
      }
      ModuleDispatch('Owner','CORE/Owner/UpdOwner/UpdPost');

      foreach(@order) {
         API_TemplateFieldSetValue($cgi->param('id'), '1', $_, $cgi->param($_));
      }
      $cgi->delete('id');
      ShowOwnerTree();
   }
}

sub DelOwner {
   if(!$cgi->param('id')) {
      API_ExitError("A owner id to delete was not specified");
   }


   if(!API_GetOwner($cgi->param('id'))) {
      API_ExitError();
   }


   if(!$cgi->param('confirm')) {
      my(%pitems) = (
            'object'=> 'Owner',
            'r'	    => $cgi->param('r'),
            'id'	=> $cgi->param('id'),
            'move_delete' => '<input type="hidden" value="1" name="confirm">'
         );
      ModuleDispatch('Owner','CORE/Owner/DelOwner/OutputConfirmPre', \%pitems);
      API_OutputPage('delete_confirm.html',\%pitems);
      ModuleDispatch('Owner','CORE/Owner/DelOwner/OutputConfirmPost', \%pitems);
   } else {
      my(@delids);
      if(API_DelOwner($cgi->param('id'), \@delids) < 1) {
         API_ExitError();
      }

      foreach(@delids) {
         if(API_TemplateFieldDeleteAllValues($_, '1') < 1) {
            API_ExitError();
         }
      }

      $cgi->delete('id');
      ShowOwnerTree();
   }
}

sub ShowOwner {
   my($i, %pitems, @row, @data);
   if(!$cgi->param('id')) {
      API_ExitError("A owner id to view was not specified");
   }

   %pitems = API_GetOwner($cgi->param('id'));

   ModuleDispatch('Owner','CORE/Owner/ShowOwner/OutputHeadPre', \%pitems);
   API_OutputPage('owner_detailshow_head.html', \%pitems);
   ModuleDispatch('Owner','CORE/Owner/ShowOwner/OutputHeadPost', \%pitems);

   ModuleDispatch('Owner','CORE/Owner/ShowOwner/OutputBodyPre', \%pitems);
   API_OutputPage('owner_detailshow_body.html', \%pitems);
   ModuleDispatch('Owner','CORE/Owner/ShowOwner/OutputBodyPost', \%pitems);


   my(%fields, @order);
   %fields = API_TemplateFieldGetAssignedFields($pitems{'templateid'});
   @order = API_TemplateFieldGetAssignedFieldsOrder($pitems{'templateid'});

   %pitems = ();
   ModuleDispatch('Owner','CORE/Owner/ShowOwner/OutputTemplatePre', \%pitems);
   API_OutputPage('template_fieldview_head.html', \%pitems);
   foreach(@order) {
      %pitems = (
            'id'	=> $_,
            'value'	=> API_TemplateFieldGetValue($cgi->param('id'), '1', $_) || "",
            'name'	=> $fields{$_},
            'required'=> ''
         );
      $pitems{'formhtml'} = API_TemplateFieldCreateHTML($pitems{'id'}, $pitems{'value'});
      API_OutputPage('template_fieldview_item.html', \%pitems);
   }
   API_OutputPage('template_fieldview_foot.html', \%pitems);			
   ModuleDispatch('Owner','CORE/Owner/ShowOwner/OutputTemplatePost', \%pitems);

   %pitems = ();
   ModuleDispatch('Owner','CORE/Owner/ShowOwner/OutputNetblocksHeadPre', \%pitems);
   API_OutputPage('owner_detailshow_netlisthead.html', \%pitems);
   ModuleDispatch('Owner','CORE/Owner/ShowOwner/OutputNetblocksHeadPre', \%pitems);

   API_GetNetworkList([qw(id name label)], \@data, [qw(name)], [{ 'field' => 'owner',
            'type'  => '=',
            'data'  => $cgi->param('id')
            },
         { 'field' => 'masklen',
            'type'  => '<',
            'data'  => '32'
            }]);

   foreach(@data) {
      %pitems = (
            'id'     => ${$_}{'id'},
            'name'   => ${$_}{'name'},
            'network'=> ${$_}{'label'}
         );
      ModuleDispatch('Owner','CORE/Owner/ShowOwner/OutputNetblocksItemPre', \%pitems);
      API_OutputPage('owner_detailshow_netlistitem.html',\%pitems);
      ModuleDispatch('Owner','CORE/Owner/ShowOwner/OutputNetblocksItemPost', \%pitems);
   }

   %pitems = ();
   ModuleDispatch('Owner','CORE/Owner/ShowOwner/OutputNetblocksFootPre', \%pitems);
   API_OutputPage('owner_detailshow_netlistfoot.html', \%pitems);
   ModuleDispatch('Owner','CORE/Owner/ShowOwner/OutputNetblocksFootPost', \%pitems);

   %pitems = ();
   ModuleDispatch('Owner','CORE/Owner/ShowOwner/OutputDevHeadPre', \%pitems);
   API_OutputPage('owner_detailshow_devlisthead.html', \%pitems);
   ModuleDispatch('Owner','CORE/Owner/ShowOwner/OutputDevHeadPost', \%pitems);

   @data = ();
   API_GetDeviceList([qw(id name)], \@data, [qw(name)], [{ 'field' => 'owner',
            'type'  => '=',
            'data'  => $cgi->param('id')
            }]);
   foreach(@data) {
      %pitems = (
            'id'  => ${$_}{'id'},
            'name'=> ${$_}{'name'}
         );
      ModuleDispatch('Owner','CORE/Owner/ShowOwner/OutputDevItemPre', \%pitems);
      API_OutputPage('owner_detailshow_devlistitem.html',\%pitems);
      ModuleDispatch('Owner','CORE/Owner/ShowOwner/OutputDevItemPost', \%pitems);
   }

   ModuleDispatch('Owner','CORE/Owner/ShowOwner/OutputDevFootPre', \%pitems);
   API_OutputPage('owner_detailshow_devlistfoot.html', \%pitems);		
   ModuleDispatch('Owner','CORE/Owner/ShowOwner/OutputDevFootPre', \%pitems);

   ModuleDispatch('Owner','CORE/Owner/ShowOwner/OutputFootPre', \%pitems);
   API_OutputPage('owner_detailshow_foot.html', \%pitems);
   ModuleDispatch('Owner','CORE/Owner/ShowOwner/OutputFootPost', \%pitems);
}

sub ShowOwnerTree {
   my($i, $p, $id, $a, $b, $t, $d, @sort, %labels);
   my(@data);
   API_GetOwnerList([qw(id name)], \@data, [qw(name)]);
   foreach(@data) {
      push(@sort, ${$_}{'id'});
      $parents{${$_}{'id'}} = ${$_}{'parent'};
      $labels{${$_}{'id'}}{'name'} = ${$_}{'name'};
   }

   my($n0) = {
      'name'          => 'Owners',
      'id'            => '0',
      'parent'        => '0',
      'children'      => [],
   };

   my(%lasthash);

   $d = 1;
   foreach(@sort) {
      $id = $_;
      $p = $parents{$_};
      $a = "n$id";
      $b = "n$p";
      $lasthash{$p} = $id;
      ${$a} = {                
         'name'          => $labels{$_}{'name'},
         'id'            => $_,
         'parent'        => $p,
         'last'          => '0',
         'children'      => [],
      };
      if($p == 0) {
         API_TreeGrow($n0, ${$a});
      } else {
         API_TreeGrow(${$b}, ${$a});
      }
   }

   while(($a,$b) = each %lasthash) {
      $t = "n$b";
      if(${$t}->{'parent'} == 0) {
         ${$t}->{'last'} = '2';
      } else {
         ${$t}->{'last'} = '1';
      }
   }

   API_OutputPage('owner_showtree_head.html');
   API_TreeDumpTable('owner_showtree_item.html', $n0, 
      ($cgi->param('path') || undef), 
      ($cgi->param('depth') || undef), 
      '', 'showner', 'shownertree');
   API_OutputPage('owner_showtree_foot.html');
}

return 1;
