# 
# Copyright (C) 2001, Hitesh Patel <hitesh@presys.com>.
# All rights reserved.
# 
# This file is part of NorthStar and is covered under
# the terms of the Apache Software License version 1.1
#
# A copy of this license is included in the LICENSE file
#
# $Header: /cvsroot/NorthStar/lib/NorthStar/Regex.pm,v 1.11 2002/05/21 00:56:18 brownkid Exp $
#

package NorthStar::Regex;
use Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(ShowRegexTree ShowRegex AddRegex UpdRegex DelRegex);

use NorthStar::API qw(:ALL);

sub init {
   Debug("REGEX: initializing module");
   my(%modinfo) = (
         export_hooks => [],
         user_hooks => {
            'shregex'       => \&NorthStar::Regex::ShowRegex,
            'shregextree'   => \&NorthStar::Regex::ShowRegexTree,
            'addregex'      => \&NorthStar::Regex::AddRegex,
            'updregex'      => \&NorthStar::Regex::UpdRegex,
            'delregex'      => \&NorthStar::Regex::DelRegex,
            },
         import_hooks => {},
      );

   API_PermissionsRegisterPerm(25, 0, 'addregex', 'Add Regex', 'CORE');
   API_PermissionsRegisterPerm(26, 0, 'updregex', 'Update Regex', 'CORE');
   API_PermissionsRegisterPerm(27, 0, 'delregex', 'Delete Regex', 'CORE');
   API_PermissionsRegisterPerm(44, 0, 'shregextree', 'Show Regex Tree', 'CORE');
   API_PermissionsRegisterPerm(45, 0, 'shregex', 'Show Regex', 'CORE');   

   my($script) = ConfigGetValue('Global','ScriptURL');
   API_MenuAddItem('Administration','Regex\'s',"$script?r=shregextree", undef, 401); 

   return(%modinfo);
}

sub ShowRegexTree {
   my($i, $p, $id, $a, $b, $t, $d, @sort, %labels);

   API_GetRegexList([qw(id name)], \@data, [qw(name)]);
   foreach(@data) {
      push(@sort, ${$_}{'id'});
      $parents{${$_}{'id'}} = 0;
      $labels{${$_}{'id'}}{'name'} = ${$_}{'name'};
   }

   my($n0) = {
      'name'          => 'Regular Expressions',
      'id'            => '0',
      'parent'        => '0',
      'children'      => [],
   };

   my(%lasthash);

   $d = 1;
   foreach(@sort) {
      $id = $_;
      $p = $parents{$_};
      $a = "n$id";
      $b = "n$p";
      $lasthash{$p} = $id;
      ${$a} = {                
         'name'          => $labels{$_}{'name'},
         'id'            => $_,
         'parent'        => $p,
         'last'          => '0',
         'children'      => [],
      };
      if($p == 0) {
         API_TreeGrow($n0, ${$a});
      } else {
         API_TreeGrow(${$b}, ${$a});
      }
   }

   while(($a,$b) = each %lasthash) {
      $t = "n$b";
      if(${$t}->{'parent'} == 0) {
         ${$t}->{'last'} = '2';
      } else {
         ${$t}->{'last'} = '1';
      }
   }

   API_OutputPage('regex_showtree_head.html');
   API_TreeDumpTable('regex_showtree_item.html', $n0, 
      $cgi->param('path'), $cgi->param('depth'), 
      '', 'shregex', 'shregextree');
   API_OutputPage('regex_showtree_foot.html');
}

sub ShowRegex {
   my($i);
   if(!$cgi->param('id')) {
      API_ExitError("The ID specified was invalid");
   }

   my(%pitems) = API_GetRegex($cgi->param('id'));
   if(!%pitems) { 
      API_ExitError();
   }

   my(@data);
   if(API_GetTemplateFieldList([qw(id name)], \@data, [qw(id)],
            [{ 'field' => 'regex',
                  'type'  => '=',
                  'data'  => $cgi->param('id')
                  }]) < 1) {
      API_ExitError();
   }

   my(%r);
   foreach(@data) {
      $r{${$_}{'id'}} = ${$_}{'name'};
   }
   my(@s) = sort keys(%r);


   foreach(@s) {
      if(!$pitems{'fields'}) {
         $pitems{'fields'} = "<A HREF=\"" . ConfigGetValue('Global','ScriptURL') . "?r=shfield&id=$_\">" . $r{$_} . '</A>';
      } else {
         $pitems{'fields'} .= ", <A HREF=\"" . ConfigGetValue('Global','ScriptURL') . "?r=shfield&id=$_\">" . $r{$_} . '</A>';
      }
   }

   API_OutputPage('regex_view.html',\%pitems);
}


sub AddRegex {
   if(!$cgi->param('doproc')) {
      API_OutputPage('regex_add.html');
   } else {
      %fields = (
            'required' => {
               'name'		=> 'alpha',
               'regex'		=> 'anything'
               },
            'optional' => {
               'example'   => 'anything',
               },
            'names' => {
               'name'		=> 'Regex Name',
               'example'   => 'Regex Example',
               'regex'		=> 'Regular Expression'
               }
         );

      API_CheckFormFields(\%fields);

      my(%regexinfo) = (
            'name'   => $cgi->param('name'),
            'example'=> $cgi->param('example'),
            'regex'  => $cgi->param('regex')
         );

      my($id) = API_AddRegex(\%regexinfo);
      if($id < 1) {
         API_ExitError();
      }
      ShowRegexTree();
   }
}

sub UpdRegex {
   if(!$cgi->param('id')) {
      API_ExitError("A Regex ID to update was not specified");
   }

   if(!$cgi->param('doproc')) {
      my(%pitems) = API_GetRegex($cgi->param('id'));
      if(!%pitems) {
         API_ExitError();
      }
      API_OutputPage('regex_upd.html', \%pitems);
   } else {
      %fields = (
            'required' => {
               'name'		=> 'alpha',
               'regex'		=> 'anything'
               },
            'optional' => {
               'example'   => 'anything'
               },
            'names' => {
               'name'		=> 'Regex Name',
               'regex'		=> 'Regular Expression',
               'example'   => 'Regex Example'
               }
         );

      API_CheckFormFields(\%fields);

      my(%regexinfo) = (
            'id'     => $cgi->param('id'),
            'name'   => $cgi->param('name'),
            'example'=> $cgi->param('example'),
            'regex'  => $cgi->param('regex')
         );
      if(API_UpdRegex(\%regexinfo) < 1) {
         API_ExitError();
      }

      ShowRegexTree();
   }
}

sub DelRegex {
   if(!$cgi->param('id')) {
      API_ExitError("A Regex ID to delete was not specified");
   }

   if(API_DelRegex($cgi->param('id'))  < 1) {
      API_ExitError();
   }
   ShowRegexTree();
}

return 1;
