# 
# Copyright (C) 2001, Hitesh Patel <hitesh@presys.com>.
# All rights reserved.
# 
# This file is part of NorthStar and is covered under
# the terms of the Apache Software License version 1.1
#
# A copy of this license is included in the LICENSE file
#
# $Header: /cvsroot/NorthStar/lib/NorthStar/Search.pm,v 1.10 2002/05/21 00:56:18 brownkid Exp $
#

package NorthStar::Search;
use Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(ShowSearch);

use NorthStar::API qw(:PERMISSIONS :UTIL);

sub init {
   Debug("SEARCH: initializing module");
   my(%modinfo) = (
         export_hooks => [],
         user_hooks => {
            'search'    => \&NorthStar::Search::ShowSearch,
            },
         import_hooks => {},
      );
   API_PermissionsRegisterPerm(61, 1, 'search', 'Search', 'CORE');

   my($script) = ConfigGetValue('Global','ScriptURL');
   API_MenuAddItem('Search','Perform a Search', "$script?r=search",undef, 101); 

   return(%modinfo);
}

sub ShowSearch {
   my($sth, %pitems,$sql,@searchcol,@specialcol,@resultcol,$table,$x,$y);
   if(!$cgi->param('doproc')) {
      %pitems = ();
      API_OutputPage('search_main.html');
   } else {
      my(%fields) = (
            'required' => {
               'object'=> 'alpha',
               'query' => 'anything',
               },
            'optional' => {
               },
            'names' => {
               'object'=> 'Search Object',
               'query' => 'Search Query',
               }
         );

      API_CheckFormFields(\%fields);

      if($cgi->param('object') eq 'owner') {
         @searchcol = qw(name descr);
         @resultcol = qw(id name descr);
         $table = 'owners';
         $x = 'showner';
      } elsif($cgi->param('object') eq 'device') {
         @searchcol = qw(name descr);
         @resultcol = qw(id name descr);
         $table = 'devices';
         $x = 'shdev';
      } elsif($cgi->param('object') eq 'location') {
         @searchcol = qw(name descr);
         @resultcol = qw(id name descr);
         $table = 'locations';
         $x = 'shloc';
      } elsif($cgi->param('object') eq 'network') {
         @searchcol = qw(name descr label);
         @resultcol = qw(id name descr);
         $table = 'networks';           
         $x = 'shnet';
      } elsif($cgi->param('object') eq 'address') {
         @searchcol = qw(name descr label);
         @resultcol = qw(id name descr);
         $table = 'networks';
         $x = 'shnet';
      } else {
         API_ExitError("An invalid object type was specified");
      }


      $sql = "SELECT " . join(',',@resultcol) . " FROM $table WHERE ";
      foreach(@searchcol) {
         if(!$y) {
            $sql .= "$_ LIKE '%" . $cgi->param('query') . "%'";
            $y = 1;
         } else {
            $sql .= " OR $_ LIKE '%" . $cgi->param('query') . "%'";
         }
      }

      foreach(@specialcol) {
         if(!$y) {
            $sql .= "$_ = '" . $cgi->param('query') . "'";
            $y = 1;
         } else {
            $sql .= " OR $_ = '" . $cgi->param('query') . "'";
         }
      }   

      $sql .= " ORDER BY id;";
      $sth = $db->prepare($sql);
      $sth->execute;
      API_DBResCheck(\$sth,'search_dosearch','1');

      if($sth->rows == 0) {
         API_OutputPage('search_noresult.html');
         return();
      }

      %pitems =  (
            'results'   => $sth->rows
         );

      API_OutputPage('search_head.html', \%pitems);
      API_OutputPage('search_listhead.html', \%pitems);
      for($i = 0; $i < $sth->rows; $i++) {
         my(@row) = $sth->fetchrow_array;
         %pitems = (
               'id'    => $row[0],
               'field1'=> $row[1],
               'field2'=> $row[2],
               'cmd'   => $x,
            );
         API_OutputPage('search_listitem.html', \%pitems);
      }
      API_OutputPage('search_listfoot.html');
      API_OutputPage('search_foot.html');

   }
}

return 1;
