# 
# Copyright (C) 2001, Hitesh Patel <hitesh@presys.com>.
# All rights reserved.
# 
# This file is part of NorthStar and is covered under
# the terms of the Apache Software License version 1.1
#
# A copy of this license is included in the LICENSE file
#
# $Header: /cvsroot/NorthStar/lib/NorthStar/Session.pm,v 1.5 2002/05/16 01:46:49 brownkid Exp $
#

package NorthStar::Session;
use Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(SessionInit SessionCreate SessionUpdate SessionDelete);

use NorthStar::API qw(:UTIL);

sub init {
   Debug("SESSION: initializing module");
   return();
}

sub SessionInit {
   my($sid) = shift;
   my($ip) = shift;
   my($r, $sth, @row);
   SessionExpire();
   $sth = $db->prepare("SELECT ip,username FROM sessions WHERE id = '$sid';");
   $sth->execute;
   API_DBResCheck(\$sth,'sessioninit_getsessinfo','1');
   if($sth->rows != 1) {
      my($c) = $cgi->cookie(-name  => 'NorthStarSID',
            -value => ''
         );

      API_OutputHeader(-type     =>'text/html',
         -cookie   =>$c,
      ); 

      my(%pitems) = ( 'msg' => 'Your session has expired.  Please login again' );
      API_OutputPage('login_page.html', \%pitems);
      exit(0);
   }

   @row = $sth->fetchrow_array;
   my($sip) = $row[0];
   my($username) = $row[1];
   if($ip ne $sip) {
      my($c) = $cgi->cookie(-name  => 'NorthStarSID',
            -value => ''
         );

      API_OutputHeader(-type     =>'text/html',
         -cookie   =>$c,
      ); 
      my(%pitems) = ( 'msg' => 'Your IP Address has changed.  Please login again.' );
      API_OutputPage('login_page.html', \%pitems);
      exit(0);
   }

   SessionUpdate($sid);
   return($username);
}

sub SessionCreate {
   my($username) = shift;
   my($passwd) = shift;
   my($ip) = shift;
   my($r, $sth, @row);

   $sth = $db->prepare("SELECT passwd FROM users WHERE username = '$username' AND enabled = '1';");
   $sth->execute;
   API_DBResCheck(\$sth,'sessioncreate_getuinfo','1');

   if($sth->rows != 1) {
      API_OutputHeader();
      my(%pitems) = ( 
            'msg' => 'An authentication error has occured.  Your account is disabled or does not exist'
         );
      API_OutputPage('login_page.html', \%pitems);
      exit(0);
   }

   @row = $sth->fetchrow_array;
   my($cp) = $row[0];

   if(crypt($passwd, $cp) ne $cp) {
      API_OutputHeader();
      my(%pitems) = ( 'msg' => 'An authentication error has occured.  Please check your username and password and try again' );
      API_OutputPage('login_page.html', \%pitems);
      exit(0);
   }
   $r = SessionGenerateID();
   my($cd) = time();
   $sth = $db->prepare("INSERT INTO sessions VALUES('$r','$cd','$username','$ip');");
   $sth->execute;
   API_DBResCheck(\$sth,'sessioncreate_addsess','1');

   my($c) = $cgi->cookie(-name  => 'NorthStarSID',
         -value => $r
      );

   API_OutputHeader(-type     =>'text/html',
      -cookie   =>$c,
   ); 
}

sub SessionUpdate {
   my($sid) = shift || return;
   my($sth);
   my($cd) = time();
   $sth = $db->prepare("UPDATE sessions SET date='$cd' WHERE id = '$sid';");
   $sth->execute;
   API_DBResCheck(\$sth,'sessionupdate_updsess','1');
   return(1);
}

sub SessionDelete {
   my($sid) = shift || return;
   my($sth); 
   $sth = $db->prepare("DELETE FROM sessions WHERE id = '$sid';");
   $sth->execute;
   API_DBResCheck(\$sth,'sessiondelete_delsess','1');
   return(1);
}

sub SessionExpire {
   my($sth); 
   my($cd) = time();
   $cd -= 90 * 60;

   $sth = $db->prepare("DELETE FROM sessions WHERE date < '$cd';");
   $sth->execute;
   API_DBResCheck(\$sth,'sessionexpire_delsess','1');
   return(1);
}

sub SessionGenerateID {
   my($str);
   for(0..15) {
      $str .= chr int rand(50) + 64;
   }
   $str = unpack("H*", $str);
   return($str);
}

return 1;
