# 
# Copyright (C) 2001, Hitesh Patel <hitesh@presys.com>.
# All rights reserved.
# 
# This file is part of NorthStar and is covered under
# the terms of the Apache Software License version 1.1
#
# A copy of this license is included in the LICENSE file
#
# $Header: /cvsroot/NorthStar/lib/NorthStar/Template.pm,v 1.16 2002/05/21 00:56:18 brownkid Exp $
#

package NorthStar::Template;
use Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(AddTemplate UpdTemplate DelTemplate ShowTemplateTree 
             ShowTemplate ShowTemplateField TemplateFieldAdd
             TemplateFieldUpdate TemplateFieldDelete);

use NorthStar::API qw(:TEMPLATE :PERMISSIONS :TREE :UTIL :REGEX);

sub init {
   Debug("TEMPLATE: initializing module");
   my(%modinfo) = (
         export_hooks => [],
         user_hooks => {
            'addtemplate'   => \&NorthStar::Template::AddTemplate,
            'updtemplate'   => \&NorthStar::Template::UpdTemplate,
            'deltemplate'   => \&NorthStar::Template::DelTemplate,
            'shtemplate'    => \&NorthStar::Template::ShowTemplate,
            'shtemplatetree'=> \&NorthStar::Template::ShowTemplateTree,
            'addfield'      => \&NorthStar::Template::TemplateFieldAdd,
            'updfield'      => \&NorthStar::Template::TemplateFieldUpdate,
            'delfield'      => \&NorthStar::Template::TemplateFieldDelete,
            'shfield'       => \&NorthStar::Template::ShowTemplateField,
            },
         import_hooks => {},
      );

   API_PermissionsRegisterPerm(19, 0, 'addtemplate', 'Add Template', 'CORE');
   API_PermissionsRegisterPerm(20, 0, 'updtemplate', 'Update Template', 'CORE');
   API_PermissionsRegisterPerm(21, 0, 'deltemplate', 'Delete Template', 'CORE');
   API_PermissionsRegisterPerm(22, 0, 'addfield', 'Add Field', 'CORE');
   API_PermissionsRegisterPerm(23, 0, 'updfield', 'Update Field', 'CORE');
   API_PermissionsRegisterPerm(24, 0, 'delfield', 'Delete Field', 'CORE');    
   API_PermissionsRegisterPerm(46, 0, 'shfield', 'Show Field', 'CORE');
   API_PermissionsRegisterPerm(47, 0, 'shtemplate', 'Show Template', 'CORE');
   API_PermissionsRegisterPerm(48, 0, 'shtemplatetree', 'Show Template Tree', 
      'CORE');    

   my($script) = ConfigGetValue('Global','ScriptURL');
   API_MenuAddItem('Administration','Templates',"$script?r=shtemplatetree", undef, 301); 

   return(%modinfo);
}

sub AddTemplate {
   my($sth,%pitems);
   if(!$cgi->param('doproc')) {
      %pitems = ();
      if(!$cgi->param('parent')) {
         $pitems{'parent_template'} = "Top-level Template" . "<input type=\"hidden\" name=\"parent\" value=\"0\">";
      } else {
         my(%tmplinfo) = API_GetTemplate($cgi->param('parent'));
         $pitems{'parent_template'} = "$tmplinfo{'name'}" . 
            "<input type=\"hidden\" name=\"parent\" " .
            "value=\"" . $cgi->param('parent') . "\">";
      }
      $pitems{'objtype'} = $cgi->param('objtype');
      API_OutputPage('template_add.html', \%pitems);
   } else {	
      my(%fields) = (
            'required' => {
               'name'	=> 'alpha',
               'objtype'=> 'num',
               },
            'optional' => {
               'descr'	=> 'alpha'
               },
            'names' => {
               'name'	=> 'Template Name',
               'descr'	=> 'Template Description',
               'objtype'=>'Template Object Type'
               }
         );

      API_CheckFormFields(\%fields);
      my(%tmplinfo) = (
            'name'   => $cgi->param('name'),
            'descr'  => $cgi->param('descr'),
            'objtype'=> $cgi->param('objtype')
         );

      my($id) = API_AddTemplate(\%tmplinfo);
      if($id < 1) {
         API_ExitError();
      }
      ShowTemplateTree();
   }
}

sub UpdTemplate {
   my($sth);
   if(!$cgi->param('id')) {
      API_ExitError("A template id to update was not specified");
   }

   if(!$cgi->param('doproc')) {
      %pitems = API_GetTemplate($cgi->param('id'));
      if(!%pitems) {
         API_ExitError();
      }

      if(API_TemplateFieldDropdown(\$pitems{'fieldselect'}, undef, 
               $cgi->param('id'), '10') < 1) {
         API_ExitError();
      }

      API_OutputPage('template_upd.html', \%pitems);
   } else {	
      my(%fields) = (
            'required' => {
               'name'	=> 'alpha',
               },
            'optional' => {
               'descr'	=> 'alpha',
               'fieldorder' => 'alpha'
               },
            'names' => {
               'name'	=> 'Template Name',
               'descr'	=> 'Template Description',
               'fieldorder'=>'Field Ordering'
               }
         );

      API_CheckFormFields(\%fields);
      my(@fieldorder) = split(/\:/, $cgi->param('fieldorder'));
      my(%tmplinfo) = (
            'id'        => $cgi->param('id'),
            'name'      => $cgi->param('name'),
            'descr'     => $cgi->param('descr'),
            'fieldorder'=> \@fieldorder
         );
      if(API_UpdTemplate(\%tmplinfo) < 1) {
         API_ExitError();
      }

      ShowTemplateTree();
   }
}

sub DelTemplate {
   my($sth);
   if(!$cgi->param('id')) {
      API_ExitError("A template id to delete was not specified");
   }

   if(!$cgi->param('confirm')) {
      my(%pitems) = (
            'object'=> 'Template',
            'r'	  => $cgi->param('r'),
            'id'	  => $cgi->param('id'),
            'move_delete' => '<input type="hidden" value="1" name="confirm">'
         );
      API_OutputPage('delete_confirm.html',\%pitems);
   } else {
      if(API_DelTemplate($cgi->param('id')) < 1) {
         API_ExitError();
      }

      $cgi->delete('id');
      $cgi->delete('objtype');
      ShowTemplateTree();
   }
}




sub ShowTemplateTree {
   my($i, $p, $id, $a, $b, $t, $d, @sort, %labels, $sth);

   if(!$cgi->param('objtype')) {
      API_OutputPage('template_showtree_select.html');
      return();
   }
   my(@data);
   API_GetTemplateList([qw(id name)], \@data, [qw(id)],
      [{ 'field' => 'objtype',
            'type'  => '=',
            'data'  => $cgi->param('objtype')
            }]);   
   foreach(@data) {
      my(@fields, $f);
      push(@sort, ${$_}{'id'});
      API_GetTemplateFieldList([qw(id name)], \@fields, [qw(sortkey)],
         [{ 'field' => 'templateid',
               'type'  => '=',
               'data'  => ${$_}{'id'}
               }]);

      foreach $f (@fields) {    
         my($id) = 'f' . ${$f}{'id'};
         push(@sort, $id);
         $parents{$id} = ${$_}{'id'};
         $labels{$id}{'name'} = ${$f}{'name'};
      }

      $parents{${$_}{'id'}} = 0;
      $labels{${$_}{'id'}}{'name'} = ${$_}{'name'}
   }

   my($n0) = {
      'name'          => API_TemplateGetObjectType($cgi->param('objtype')) . ' Templates',
      'id'            => '0',
      'parent'        => '0',
      'children'      => [],
   };

   my(%lasthash);

   $d = 1;
   foreach(@sort) {
      if($_ !~ /^f/) {
         $id = $_;
         $p = $parents{$_};
         $a = "n$id";
         $b = "n$p";
         $lasthash{$p} = $id;
         ${$a} = {
            'name'          => $labels{$_}{'name'},
            'id'            => $_,
            'parent'        => $p,
            'last'          => '0',
            'children'      => [],
         };
      } else {
         my($s) = $_;
         $s =~ s/f//g;
         $id = $s;
         $p = $parents{$_};
         $a = "f$id";
         $b = "n$p";
         $lasthash{$p} = $id;
         ${$a} = {
            'name'          => $labels{$_}{'name'},
            'id'            => $s,
            'parent'        => $p,
            'last'          => '0',
            'html'          => 'field_showtree_item.html',
            'children'      => [],
         };
      }
      if($p == 0) {
         API_TreeGrow($n0, ${$a});
      } else {
         API_TreeGrow(${$b}, ${$a});
      }
   }

   while(($a,$b) = each %lasthash) {
      $t = "n$b";
      if(${$t}->{'parent'} == 0) {
         ${$t}->{'last'} = '2';
      } else {
         ${$t}->{'last'} = '1';
      }
   }

   %pitems = (
         'objtype'   => $cgi->param('objtype'),
         'objname'   => API_TemplateGetObjectType($cgi->param('objtype'))
      );

   API_OutputPage('template_showtree_head.html', \%pitems);
   API_TreeDumpTable('template_showtree_item.html', $n0, 
      ($cgi->param('path') || undef), 
      ($cgi->param('depth') || undef), 
      '', 'shtemplate', 
      "shtemplatetree&objtype=" . $cgi->param('objtype'),
      '','',\%pitems);
   API_OutputPage('template_showtree_foot.html');
}

sub ShowTemplate {
   if(!$cgi->param('id')) {
      API_ExitError("A Template ID to view was not specified");
   }

   my(%pitems) = API_GetTemplate($cgi->param('id'));
   if(!%pitems) {
      API_ExitError();
   }

   $pitems{'objname'} = API_TemplateGetObjectType($pitems{'objtype'});

   my(%tf) = API_TemplateFieldGetAssignedFields($cgi->param('id'));

   my(@s) = sort keys(%tf);
   foreach(@s) {
      my($fl) = "<A HREF=\"". ConfigGetValue('Global','ScriptURL') . "?r=viewfield&id=$_\">$tf{$_}</A>";
      if(!$pitems{'fieldnames'}) {
         $pitems{'fieldnames'} = $fl;
      } else {
         $pitems{'fieldnames'} .= ", $fl";
      }
   }

   API_OutputPage('template_view.html', \%pitems);
}


sub ShowTemplateField {
   my($i, $sth);
   if(!$cgi->param('id')) {
      API_ExitError("The ID specified was invalid");
   }

   my(%pitems) = API_GetTemplateField($cgi->param('id'));
   if(!%pitems) {
      API_ExitError();
   }

   if($pitems{'required'} == 0) {
      $pitems{'required'} = "No";
   } else {
      $pitems{'required'} = "Yes";
   }

   my(%tmplinfo) = API_GetTemplate($pitems{'templateid'});
   if(!%tmplinfo) {
      API_ExitError();
   }

   $pitems{'templatename'} = $tmplinfo{'name'};
   my(%regexinfo) = API_GetRegex($pitems{'regex'});
   if(!%regexinfo) {
      API_ExitError();
   }
   $pitems{'regexname'} = $regexinfo{'name'};
   $pitems{'updatemode'} = API_TemplateFieldUpdateName($pitems{'updatemode'});

   API_OutputPage('field_view.html', \%pitems);
}

sub TemplateFieldAdd {
   if(!$cgi->param('doproc')) {
      API_RegexDropdown(\$pitems{'regexselect'});
      API_TemplateFieldTypeDropdown(\$pitems{'fieldtypeselect'});
      API_TemplateFieldUpdateDropdown(\$pitems{'updatemodeselect'});
      $pitems{'path'} = $cgi->param('path');
      $pitems{'depth'} = $cgi->param('depth');
      $pitems{'parent'} = $cgi->param('parent');
      $pitems{'objtype'} = $cgi->param('objtype');
      API_OutputPage('field_add.html', \%pitems);
   } else {	
      my(%fields) = (
            'required' => {
               'name'		=>	'alpha',
               'regex_select'		=>	'num',
               'fieldtype_select'	=>	'alpha',
               'updatemode_select'	=>	'alpha',
               'parent'    => 'num'
               },
            'optional' => {
               'required'	=>	'alpha',
               'description'=>	'alpha',
               'minlength'	=>	'num',
               'maxlength'	=>	'num'
               },
            'names' => {
               'name'		=>	'Field Name',
               'description'=>	'Field Description',
               'required'	=>	'Field Required',
               'regex_select'		=>	'Field Regular Expression',
               'minlength'	=>	'Field Minimum Length',
               'maxlength'	=>	'Field Maximum Length',
               'fieldtype_select'	=>	'Field Type',
               'updatemode_select'	=>	'Field Update Mode',
               'parent'    => 'Field Template Assignment'
               }
         );

      API_CheckFormFields(\%fields);

      if(!$cgi->param('required')) { $cgi->param('required', 'f'); }

      my(%fieldinfo) = (
            'templateid'   => $cgi->param('parent'),
            'name'         => $cgi->param('name'),
            'descr'        => $cgi->param('description'),
            'required'     => $cgi->param('required'),
            'regex'        => $cgi->param('regex_select'),
            'minlength'    => $cgi->param('minlength'),
            'maxlength'    => $cgi->param('maxlength'),
            'fieldtype'    => $cgi->param('fieldtype_select'),
            'updatemode'   => $cgi->param('updatemode_select')
         );

      my($id) = API_AddTemplateField(\%fieldinfo);
      if($id < 1) {
         API_ExitError();
      }
      ShowTemplateTree();
   }
}

sub TemplateFieldUpdate {
   if(!$cgi->param('id')) {
      API_ExitError("A Field ID to update was not specified");
   }

   if(!$cgi->param('doproc')) {
      my(%pitems) = API_GetTemplateField($cgi->param('id'));
      if(!%pitems) {
         API_ExitError();
      }

      if($pitems{'required'} == 1) {
         $pitems{'required'} = "CHECKED";
      } else {
         $pitems{'required'} = "";
      }

      API_RegexDropdown(\$pitems{'regexselect'}, $pitems{'regex'});
      API_TemplateFieldTypeDropdown(\$pitems{'fieldtypeselect'}, $pitems{'fieldtype'});
      API_TemplateFieldUpdateDropdown(\$pitems{'updatemodeselect'}, $pitems{'updatemode'});
      $pitems{'objtype'} = $cgi->param('objtype');
      API_OutputPage('field_upd.html', \%pitems);
   } else {
      my(%fields) = (
            'required' => {
               'name'		=>	'alpha',
               'regex_select'		=>	'num',
               'fieldtype_select'	=>	'alpha',
               'updatemode_select'	=>	'alpha'
               },
            'optional' => {
               'required'	=>	'alpha',
               'description'=>	'alpha',
               'minlength'	=>	'num',
               'maxlength'	=>	'num'
               },
            'names' => {
               'name'		=>	'Field Name',
               'description'=>	'Field Description',
               'required'	=>	'Field Required',
               'regex_select'		=>	'Field Regular Expression',
               'minlength'	=>	'Field Minimum Length',
               'maxlength'	=>	'Field Maximum Length',
               'fieldtype_select'	=>	'Field Type',
               'updatemode_select'	=>	'Field Update Mode'
               }
         );

      API_CheckFormFields(\%fields);

      if(!$cgi->param('required')) { $cgi->param('required', 'f'); }
      else { $cgi->param('required', 't'); }

      my(%fieldinfo) = (
            'id'           => $cgi->param('id'),
            'name'         => $cgi->param('name'),
            'descr'        => $cgi->param('description'),
            'required'     => $cgi->param('required'),
            'regex'        => $cgi->param('regex_select'),
            'minlength'    => $cgi->param('minlength'),
            'maxlength'    => $cgi->param('maxlength'),
            'fieldtype'    => $cgi->param('fieldtype_select'),
            'updatemode'   => $cgi->param('updatemode_select')
         );
      if(API_UpdTemplateField(\%fieldinfo) < 1) {
         API_ExitError();
      }
      ShowTemplateTree();
   }
}

sub TemplateFieldDelete {
   if(!$cgi->param('id')) {
      API_ExitError("A Field ID to delete was not specified");
   }

   if(!$cgi->param('confirm')) {
      my(%pitems) = (
            'object'=> 'Field',
            'r'	  => $cgi->param('r'),
            'id'	  => $cgi->param('id'),
            'objtype'=>$cgi->param('objtype'),
            'move_delete' => '<input type="hidden" value="1" name="confirm">'
         );
      API_OutputPage('delete_confirm.html',\%pitems);
   } else {
      if(API_DelTemplateField($cgi->param('id')) < 1) {
         API_ExitError();
      }
      $cgi->delete('id');
      ShowTemplateTree();
   }
}



return 1;
