# 
# Copyright (C) 2001, Hitesh Patel <hitesh@presys.com>.
# All rights reserved.
# 
# This file is part of NorthStar and is covered under
# the terms of the Apache Software License version 1.1
#
# A copy of this license is included in the LICENSE file
#
# $Header: /cvsroot/NorthStar/lib/NorthStar/User.pm,v 1.14 2002/05/21 00:56:18 brownkid Exp $
#

package NorthStar::User;
use Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(AddUser UpdUser DelUser ShowUser ShowUserTree UserChangePass);

use NorthStar::API qw(:UTIL :USER :PERMISSIONS :TREE);

sub init {
   Debug("USER: initializing module");
   my(%modinfo) = (
         export_hooks => [],
         user_hooks => {
            'adduser'   => \&NorthStar::User::AddUser,
            'upduser'   => \&NorthStar::User::UpdUser,
            'deluser'   => \&NorthStar::User::DelUser,
            'shuser'    => \&NorthStar::User::ShowUser,
            'shusertree'=> \&NorthStar::User::ShowUserTree,
            'changepass'=> \&NorthStar::User::UserChangePass,
            },
         import_hooks => {},
      );

   API_PermissionsRegisterPerm(16, 0, 'adduser', 'Add User', 'CORE');
   API_PermissionsRegisterPerm(17, 0, 'upduser', 'Update User', 'CORE');
   API_PermissionsRegisterPerm(18, 0, 'deluser', 'Delete User', 'CORE');
   API_PermissionsRegisterPerm(49, 1, 'changepass', 'Change Password', 'CORE');   
   API_PermissionsRegisterPerm(55, 1, 'shuser', 'Show User', 'CORE');
   API_PermissionsRegisterPerm(56, 1, 'shusertree', 'Show User Tree', 'CORE'); 

   my($script) = ConfigGetValue('Global','ScriptURL');
   API_MenuAddItem('Preferences','Change Password', 
      "$script?r=changepass&p=main.html",undef); 
   API_MenuAddItem('Administration','Users',"$script?r=shusertree",undef, 201); 

   return(%modinfo);
}


sub AddUser {
   my($sth);

   if(!$cgi->param('doproc')) {
      my(%pitems, $i);
      API_PermissionsCreateCheckboxes(\$pitems{'permtable'});
      API_OutputPage('user_add.html', \%pitems);
   } else {	
      my(%fields) = (
            'required' => {
               'username'	=>	'alpha',
               'passwd' => 'alpha',
               'passwd2'=> 'alpha',
               },
            'optional' => {
               'realname'	=>	'alpha',
               },
            'names' => {
               'username'	=>	'User Login Name',
               'realname'  =>  'User Real Name',					
               'passwd' => 'Password',
               'passwd2'=> 'Verify Password'
               }
         );

      API_CheckFormFields(\%fields);

      if($cgi->param('passwd') ne $cgi->param('passwd2')) {
         API_ExitError("The passwords entered did not match.");
      }

      my(@perms) = $cgi->param('permissions');
      my(%userinfo) = (
            'username'  => $cgi->param('username'),
            'realname'  => $cgi->param('realname'),
            'passwd'    => $cgi->param('passwd'),
            'perms'     => \@perms
         );
      my($id) = API_AddUser(\%userinfo);
      if($id < 1) {
         API_ExitError();
      }
      ShowUserTree();
   }
}

sub UpdUser {
   my($sth, $i);

   if(!$cgi->param('id')) {
      API_ExitError("An user id to update was not specified");
   }

   if(!$cgi->param('doproc')) {
      my(%pitems) = API_GetUser($cgi->param('id'));
      if(!%pitems) {
         API_ExitError();
      }
      my(@d) = API_PermissionsCreateBitArray($pitems{permmask});
      API_PermissionsCreateCheckboxes(\$pitems{'permtable'}, \@d);

      if($pitems{'enabled'} == 1) {
         $pitems{'enabled'} = "CHECKED";
      } else {
         $pitems{'enabled'} = "";
      }

      API_OutputPage('user_upd.html', \%pitems);
   } else {	
      my(%fields) = (
            'required' => {
               },
            'optional' => {
               'realname'	=>	'alpha',
               },
            'names' => {
               'username'	=>	'User Login Name',
               'realname'  =>  'User Real Name',					
               }
         );

      API_CheckFormFields(\%fields);

      if($cgi->param('enabled') eq 't') {
         $cgi->param('enabled', '1');
      } else {
         $cgi->param('enabled', '0');
      }


      my(@perms) = $cgi->param('permissions');
      my(%userinfo) = (
            'id'        => $cgi->param('id'),
            'realname'  => $cgi->param('realname'),
            'enabled'    => $cgi->param('enabled'),
            'perms'     => \@perms
         );
      if(API_UpdUser(\%userinfo) < 1) {
         API_ExitError();
      }

      ShowUserTree();
   }
}

sub DelUser {
   my($sth);
   if(!$cgi->param('id') || $cgi->param('id') == 0) {
      API_ExitError("A user id to delete was not specified");
   }

   my(%userinfo) = API_GetUser($cgi->param('id'));
   if(!%userinfo) {
      API_ExitError();
   }

   if(!$cgi->param('confirm')) {
      my(%pitems) = (
            'object'=> 'User',
            'r'	  => $cgi->param('r'),
            'id'	  => $cgi->param('id'),
            'move_delete' => '<input type="hidden" value="1" name="confirm">'
         );
      API_OutputPage('delete_confirm.html',\%pitems);
   } else {

      if(API_DelUser($cgi->param('id')) < 1) {
         API_ExitError();
      }
      $cgi->delete('id');
      ShowUserTree();
   }
}

sub ShowUser {
   my($sth, $i, %pitems, @row);

   if(!$cgi->param('id')) {
      API_ExitError("A user id to view was not specified");   
   }

   API_OutputPage('user_detailshow_head.html', \%pitems);

   %pitems = API_GetUser($cgi->param('id'));
   if(!%pitems) {
      API_ExitError();
   }

   if($pitems{'enabled'} == 1) {
      $pitems{'enabled'} = "Yes";
   } else {
      $pitems{'enabled'} = "No";
   }

   API_PermissionsCreateTable($pitems{'permmask'}, \$pitems{'permtable'});
   API_OutputPage('user_detailshow_body.html', \%pitems);
   API_OutputPage('user_detailshow_foot.html');
}

sub ShowUserTree {
   my($i, $p, $id, $a, $b, $t, $d, @sort, %labels, $sth);
   my(@data);
   API_GetUserList([qw(id username realname)], \@data, [qw(username)]);

   foreach(@data) {  
      push(@sort, ${$_}{'id'});
      $parents{${$_}{'id'}} = '0';
      $labels{${$_}{'id'}}{'username'} = ${$_}{'username'};
      $labels{${$_}{'id'}}{'realname'} = ${$_}{'realname'};
   }

   my($u) = {
      'name'          => 'Users',
      'id'            => '0',
      'parent'        => '0',
      'children'      => [],
   };

   my(%lasthash);

   $d = 1;
   foreach(@sort) {
      $id = $_;
      $p = $parents{$_};
      $a = "n$id";
      $b = "n$p";
      $lasthash{$p} = $id;
      ${$a} = {                
         'username'      => $labels{$_}{'username'},
         'realname'      => $labels{$_}{'realname'},
         'id'            => $_,
         'parent'        => $p,
         'last'          => '0',
         'children'      => [],
      };
      if($p == 0) {
         API_TreeGrow($u, ${$a});
      } else {
         API_TreeGrow(${$b}, ${$a});
      }
   }

   while(($a,$b) = each %lasthash) {
      $t = "n$b";
      if(${$t}->{'parent'} == 0) {
         ${$t}->{'last'} = '2';
      } else {
         ${$t}->{'last'} = '1';
      }
   }

   API_OutputPage('user_showtree_head.html');
   API_TreeDumpTable('user_showtree_item.html', $u, 
      ($cgi->param('path') || undef), 
      ($cgi->param('depth') || undef), 
      '', 'shuser', 'shusertree');
   API_OutputPage('user_showtree_foot.html');
}

sub UserChangePass {
   if(!$cgi->param('doproc')) {
      my(%pitems) = (
            'p' => $cgi->param('p')
         );
      API_OutputPage('user_changepass.html', \%pitems);
   } else {
      my(%fields) = (
            'required' => {
               'passwd' => 'alpha',
               'passwd2'=> 'alpha',
               },
            'optional' => {
               },
            'names' => {
               'passwd' => 'Password',
               'passwd2'=> 'Verify Password'
               }
         );

      API_CheckFormFields(\%fields);

      if($cgi->param('passwd') ne $cgi->param('passwd2')) {
         API_ExitError("The passwords entered did not match.");
      }

      if(API_UserChangePass($NorthStar::user_id, $cgi->param('passwd')) < 1) {
         API_ExitError();
      }

      if($cgi->param('p')) {
         API_OutputPage($cgi->param('p'));
      } else {
         print "Password Changed";
      }
   }
}

return 1;
