# 
# Copyright (C) 2001, Hitesh Patel <hitesh@presys.com>.
# All rights reserved.
# 
# This file is part of NorthStar and is covered under
# the terms of the Apache Software License version 1.1
#
# A copy of this license is included in the LICENSE file
#
# $Header: /cvsroot/NorthStar/lib/NorthStar/View.pm,v 1.11 2002/05/21 00:56:18 brownkid Exp $
#

package NorthStar::View;
use Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(ShowViewTree ShowView AddView UpdView DelView);

use NorthStar::API qw(:VIEW :UTIL :PERMISSIONS :TREE);

sub init {
   Debug("VIEW: initializing module");
   my(%modinfo) = (
         export_hooks => [],
         user_hooks => {
            'shviewtree'=> \&NorthStar::View::ShowViewTree,
            'shview'    => \&NorthStar::View::ShowView,
            'addview'   => \&NorthStar::View::AddView,
            'updview'   => \&NorthStar::View::UpdView,
            'delview'   => \&NorthStar::View::DelView,
            },
         import_hooks => {},
      );

   API_PermissionsRegisterPerm(28, 0, 'addview', 'Add View', 'CORE');
   API_PermissionsRegisterPerm(29, 0, 'updview', 'Update View', 'CORE');
   API_PermissionsRegisterPerm(30, 0, 'delview', 'Delete View', 'CORE');
   API_PermissionsRegisterPerm(40, 0, 'shviewtree', 'Show View Tree', 'CORE');
   API_PermissionsRegisterPerm(41, 0, 'shview', 'Show View', 'CORE');  

   my($script) = ConfigGetValue('Global','ScriptURL');
   API_MenuAddItem('Administration','Views',"$script?r=shviewtree", undef, 501); 

   return(%modinfo);
}

sub ShowViewTree {
   my($i, $p, $id, $a, $b, $t, $d, @sort, %labels, $sth);

   my(@data);
   API_GetViewList([qw(id name)], \@data, [qw(id)]);
   foreach(@data) {
      push(@sort, ${$_}{'id'});
      $parents{${$_}{'id'}} = '0';
      $labels{${$_}{'id'}}{'name'} = ${$_}{'name'};
   }

   my($n0) = {
      'name'          => 'Views',
      'id'            => '0',
      'parent'        => '0',
      'children'      => [],
   };

   my(%lasthash);

   $d = 1;
   foreach(@sort) {
      $id = $_;
      $p = $parents{$_};
      $a = "n$id";
      $b = "n$p";
      $lasthash{$p} = $id;
      ${$a} = {                
         'name'          => $labels{$_}{'name'},
         'id'            => $_,
         'parent'        => $p,
         'last'          => '0',
         'children'      => [],
      };
      if($p == 0) {
         API_TreeGrow($n0, ${$a});
      } else {
         API_TreeGrow(${$b}, ${$a});
      }
   }

   while(($a,$b) = each %lasthash) {
      $t = "n$b";
      if(${$t}->{'parent'} == 0) {
         ${$t}->{'last'} = '2';
      } else {
         ${$t}->{'last'} = '1';
      }
   }

   API_OutputPage('view_showtree_head.html');
   API_TreeDumpTable('view_showtree_item.html', $n0, 
      ($cgi->param('path') || undef), 
      ($cgi->param('depth') || undef), 
      '', 'shview', 'shviewtree');
   API_OutputPage('view_showtree_foot.html');
}

sub ShowView {
   my($i);
   if(!$cgi->param('id')) {
      API_ExitError("The ID specified was invalid");
   }

   my(%pitems) = API_GetView($cgi->param('id'));
   if(!%pitems) {
      API_ExitError();
   }
   API_OutputPage('view_view.html',\%pitems);
}


sub AddView {
   if(!$cgi->param('doproc')) {
      API_OutputPage('view_add.html');
   } else {
      %fields = (
            'required' => {
               'name'		=> 'alpha',
               },
            'optional' => {
               'descr'   => 'alpha',
               },
            'names' => {
               'name'		=> 'View Name',
               'descr'   => 'View Description'
               }
         );

      API_CheckFormFields(\%fields);

      my(%viewinfo) = (
            'name'   => $cgi->param('name'),
            'descr'  => $cgi->param('descr')
         );

      my($id) = API_AddView(\%viewinfo);
      if($id < 1) {
         API_ExitError();
      }
      ShowViewTree();
   }
}

sub UpdView {
   if(!$cgi->param('id')) {
      API_ExitError("A View ID to update was not specified");
   }

   if(!$cgi->param('doproc')) {

      my(%pitems) = API_GetView($cgi->param('id'));
      if(!%pitems) {
         API_ExitError();
      }
      API_OutputPage('view_upd.html', \%pitems);
   } else {
      %fields = (
            'required' => {
               'name'		=> 'alpha',
               },
            'optional' => {
               'descr'   => 'alpha'
               },
            'names' => {
               'name'		=> 'View Name',
               'descr'		=> 'View Description',
               }
         );

      API_CheckFormFields(\%fields);

      my(%viewinfo) = (
            'id'     => $cgi->param('id'),
            'name'   => $cgi->param('name'),
            'descr'  => $cgi->param('descr')
         );

      if(API_UpdView(\%viewinfo) < 1) {
         API_ExitError();
      }

      ShowViewTree();
   }
}

sub DelView {
   if(!$cgi->param('id')) {
      API_ExitError("A View ID to delete was not specified");
   }

   if(API_DelView($cgi->param('id')) < 1) {
      API_ExitError();
   }
   ShowViewTree();
}

return(1);
