# 
# Copyright (C) 2001, Hitesh Patel <hitesh@presys.com>.
# All rights reserved.
# 
# This file is part of NorthStar and is covered under
# the terms of the Apache Software License version 1.1
#
# A copy of this license is included in the LICENSE file
#
# $Header: /cvsroot/NorthStar/lib/NorthStar/API.pm,v 1.21 2002/05/21 23:41:11 brownkid Exp $
#

package NorthStar::API;
use Carp;

sub BEGIN {

   use Exporter;
   #use AutoLoader 'AUTOLOAD';
   @ISA = qw(Exporter);
   @EXPORT = ();
   @EXPORT_OK = ();
   %EXPORT_TAGS = (
         'UTIL'   => [qw(IP_DQToBin IP_DQToBinStr IP_BinStrToDQ IP_LastIP IP_BinToDQ 
                                      IP_IntToBin IP_BinToInt IP_IntToDQ IP_DQToInt IP_CidrToBin 
                                      IP_GetSupernet IP_GetRange IP_CalculateNets IP_HostsInMasklen
                                      IP_CheckPrefix API_CIDRDropdown API_CheckFormFields
                                      API_CheckField API_OutputPage API_OutputHeader API_ExitError
                                      API_DBConnect API_DBResCheck API_GenericRecursiveUpdate
                                      API_GenericGetInfo API_BuildHash API_GenericDelete 
                                      API_GenericRecursiveDelete API_RecursiveGetChildren 
                                      API_SetError API_SetGenError API_GetError API_GetErrorStr
                                      API_MenuAddSection API_MenuAddItem API_FormHash ConfigGetSection 
                   						 ConfigGetValue ConfigInit Debug Die API_Init $cgi $db)],
         'MODULE' => [qw(ModuleAddHook ModuleAddUserHook ModuleDispatch 
                      							 ModuleGetRegisteredHooks ModuleRegisterHook ModulesInit)],
         'AUDIT'  => [qw(API_AddAudit)],
         'DEVICE' => [qw(API_AddDevice API_UpdDevice API_DelDevice API_DeviceDropdown
                                      API_GetDevice API_GetDeviceList )],
         'LOCATION'=>[qw(API_AddLocation API_UpdLocation API_DelLocation 
                                      API_LocationDropdown API_GetLocation API_GetLocationList)],
         'OWNER'  => [qw(API_AddOwner API_UpdOwner API_DelOwner API_OwnerDropdown 
                                      API_GetOwner API_GetOwnerList)],
         'NETWORK'=> [qw(API_AddNetwork API_UpdNetwork API_DelNetwork API_GetNetwork 
                                      API_CheckNetworkStatus API_GetNetworkList API_GetNetStatus
                                      API_NetPriorityDropdown API_GetNetStatusName 
                                      API_NetStatusDropdown API_NetStatusUpdate 
                                      API_NetPriorityUpdate API_NetReassignParent 
                                      API_NetCheckSmallerExist API_NetCheckExist
                                      API_NetworkAddrDropdown API_NetworkGetSubnets
                                      API_NetworkGetBlock API_NetRank 
                                      API_NetworkTreeUpdate)],
         'VIEW'   => [qw(API_AddView API_UpdView API_DelView API_ViewDropdown 
                                      API_GetView API_GetViewList API_GetViewSingle)],
         'USER'   => [qw(API_AddUser API_UserChangePass API_UpdUser API_DelUser 
                                      API_UserDropdown API_GetUser API_GetUserList)],
         'PERMISSIONS'=>[qw(API_PermissionsRegisterPerm 
                                      API_PermissionsGetInfo API_PermissionsSetInfo
                                      API_PermissionsSetPerms 
                                      API_PermissionsCheckPerm 
                                      API_PermissionsCreateCheckboxes
                                      API_PermissionsCreateTable
                                      API_PermissionsMaskCreate
                                      API_PermissionsCreateBitArray)],
         'TREE'   => [qw(API_TreeGrow API_TreeDump API_TreeDumpTable API_TreeArray 
                                      API_TreeHash API_TreeCreateDropdown)],
         'TEMPLATE'=>[qw(API_AddTemplate API_UpdTemplate
                                      API_DelTemplate API_GetTemplate
                                      API_GetTemplateList API_AddTemplateField
                                      API_UpdTemplateField API_DelTemplateField
                                      API_GetTemplateField API_GetTemplateFieldList
                                      API_TemplateSelectPage API_TemplateDropdown
                                      API_TemplateGetObjectType API_TemplateFieldSetValue 
                                      API_TemplateFieldGetValue API_TemplateFieldDeleteAllValues
                                      API_TemplateFieldIsRequired API_TemplateFieldGetExample 
                                      API_TemplateFieldCheckInput API_TemplateFieldDropdown 
                                      API_TemplateFieldSetSortKeys API_TemplateFieldGetAssignedFields
                                      API_TemplateFieldGetAssignedFieldsOrder
                                      API_TemplateFieldTypeDropdown API_TemplateFieldUpdateDropdown
                                      API_TemplateFieldUpdateName API_TemplateFieldCreateHTML
                                      API_TemplateCheckSingle)],
         'REGEX'  => [qw(API_AddRegex API_UpdRegex API_DelRegex API_RegexDropdown
                                      API_GetRegex API_GetRegexList API_RegexDropdown
                                      API_RegexCheck)],
      );

   $EXPORT_TAGS{'ALL'} = [
         @{$EXPORT_TAGS{'UTIL'}},
         @{$EXPORT_TAGS{'MODULE'}},
         @{$EXPORT_TAGS{'AUDIT'}},
         @{$EXPORT_TAGS{'DEVICE'}},
         @{$EXPORT_TAGS{'LOCATION'}},
         @{$EXPORT_TAGS{'OWNER'}},
         @{$EXPORT_TAGS{'NETWORK'}},
         @{$EXPORT_TAGS{'VIEW'}},
         @{$EXPORT_TAGS{'USER'}},
         @{$EXPORT_TAGS{'PERMISSIONS'}},
         @{$EXPORT_TAGS{'TREE'}},
         @{$EXPORT_TAGS{'TEMPLATE'}},
         @{$EXPORT_TAGS{'REGEX'}}];

   Exporter::export_tags('UTIL');
   Exporter::export_ok_tags('ALL');

}

$NorthStar::API::VERSION = "1.0";
$NorthStar::API::_headeroutputed = 0;

# API_Init:	Initialize NorthStar.  This function does the following
#					1. Initialize the CGI object reference
#				 	2. Read the config file
#					3. Initialize the core modules (if they haven't already)
#					4. Connect to the database
# Input:
# ** NOTE: All values are boolean.  1 initializes the 
#          particular item, 0 doesn't.  Default is 1.
#  $cgiinit  	CGI initialization
#  $configinit Config initialization
#  $coreinit	CORE Module initialization
#  $dbinit	   Database initialization
# Returns:
#	0 	General Error
#	1	Successful
sub API_Init {
   my($cgiinit) = shift;
   my($configinit) = shift;
   my($coreinit) = shift;
   my($dbinit) = shift;

   if($cgiinit != 1 && $cgiinit != 0) {
      $cgiinit = 1;
   }

   if($configinit != 1 && $configinit != 0) {
      $configinit = 1;
   }

   if($coreinit != 1 && $coreinit != 0) {
      $coreinit = 1;
   }

   if($dbinit != 1 && $dbinit != 0) {
      $dbinit = 1;
   }

   if($cgiinit == 1) {
      $cgi = new CGI;
   }

   if($configinit == 1) {
      my(@searchdir) = qw(/etc /opt/NorthStar /usr/NorthStar 
                       /usr/local/NorthStar .);
      my($t);
      foreach(@searchdir) {
         if(-f "$_/NorthStar.conf") {
            Debug("Found config in $_", 6);
            ConfigInit("$_/NorthStar.conf");
            $t = 1;
            last;
         }
      }

      if(!$t) {		
         API_SetGenError("Unable to find the configuration file");
         return(0);
      }
   }

   if($coreinit == 1 && !$NorthStar::API::_coreinit) {
      Debug("initializing core modules", 6);		
      ModulesInit();
      $NorthStar::API::_coreinit = 1;
   }

   if($dbinit == 1) {
      Debug("initializing database", 6);
      my(%dbconfig) = ConfigGetSection('Database');
      my(%dbinfo) = (
            'fh'      => \$db,		              
            'driver'	 => $dbconfig{'DatabaseDriver'},
            'host'    => $dbconfig{'DatabaseHost'}, 
            'port'    => $dbconfig{'DatabasePort'}, 
            'dbname'  => $dbconfig{'DatabaseName'}, 
            'dbuser'  => $dbconfig{'DatabaseUser'}, 
            'dbpass'  => $dbconfig{'DatabasePass'}  
         );

      if(API_DBConnect(\%dbinfo) < 1) {
         return(0);
      }
   }
   $NorthStar::API::_initdone = 1;
   return(1);
}

# IP_DQToBin: Convert a dotted-quad address to binary format
# Input:
#	$addr 	The address in dotted-quad format
# Returns:
#	$bin		The address in binary format
sub IP_DQToBin {
   my @dq;
   my $q;
   my $i;
   my $bin;
   foreach $q (split /\./,$_[0]) {
      push @dq,$q;
   }
   for ($i = 0; $i < 4 ; $i++) {
      if (! defined $dq[$i]) {
         push @dq,0;
      }
   }
   $bin    = pack("CCCC",@dq);      # 4 unsigned chars
   return $bin;
}

# IP_DQToBinStr: Convert a dotted-quad address to binary string format
# Input:
#	$addr 	The address in dotted-quad format
# Returns:
#	$binstr	The address in binary string format
sub IP_DQToBinStr {
   return(unpack('B32',IP_DQToBin($_[0])));
}

# IP_BinStrToDQ: Convert a binary string format address to dotted-quad format
# Input:
#	$addr 	The address in binary string format
# Returns:
#	$dq		The address in dotted-quad format
sub IP_BinStrToDQ {
   return(join '.', unpack('CCCC', pack('B32',$_[0])));
}

# IP_LastIP: Return the last ip in dotted-quad format
# Input:
#	$net 		The network address in dotted-quad format
#	$len		The mask length in CIDR format
# Returns:
#	$dq		The last address in dotted-quad format
sub IP_LastIP {
   my($net) = shift;
   my($len) = shift;
   my($bin) = IP_DQToBinStr($net);
   my($t) = substr($bin,0,$len);
   return(IP_BinStrToDQ($t . ('1' x (32 - length($t)))));
}

# IP_BinToDQ: Convert a binary address to dotted-quad format
# Input:
#	$addr 	The address in binary string format
# Returns:
#	$dq		The address in dotted-quad format
sub IP_BinToDQ {
   my($dq) = join ".",unpack("CCCC",$_[0]);
   return($dq);
}

# IP_IntToBin: Convert a integer address to binary format
# Input:
#	$int	 	The address in integer format
# Returns:
#	$bin		The address in binary format
sub IP_IntToBin {
   return pack("N",$_[0]);
}

# IP_BinToInt: Convert a binary address to integer format
# Input:
#	$bin		The address in binary format
# Returns:
#	$int	 	The address in integer format
sub IP_BinToInt {
   return unpack("N",$_[0]);
}

# IP_IntToDQ: Convert a integer address to dotted-quad format
# Input:
#	$int	 	The address in integer string format
# Returns:
#	$dq		The address in dotted-quad format
sub IP_IntToDQ {
   return(IP_BinToDQ(IP_IntToBin($_[0])));
}

# IP_DQToInt: Convert a dotted-quad address to integer format
# Input:
#	$dq		The address in dotted-quad format
# Returns:
#	$int	 	The address in integer format
sub IP_DQToInt {
   return(IP_BinToInt(IP_DQToBin($_[0])));
}


# IP_CidrToBin: Convert a cidr length to binary format
# Input:
#	$len		The cidr length
# Returns:
#	$bin	 	The address in binary format
sub IP_CidrToBin {
   my $cidr = $_[0];
   pack( "B*",(1 x $cidr) . (0 x (32 - $cidr)) );
}

# IP_GetSupernet: Get the supernet of a block.  Calls API_ExitError if
#                 something goes wrong
# Input:
#	$ip		The network block address in dotted-quad format
#	$mask 	The mask length of the block
#	$view 	The view id to look under
# Returns:
#  ($net,$mask)	The supernet address and mask length
#	undef 			Some 
sub IP_GetSupernet {
   my($ip) = IP_DQToBin(shift) || API_ExitError("IP_GetSupernet: A block was not given");
   my($mask) = shift || API_ExitError("IP_GetSupernet: A mask length was not given");
   my($view) = shift || API_ExitError("IP_GetSupernet: A view id was not given");
   my(%nethash, $val, $netmask, @row, $sth);
   $mask--;
   for($x = $mask; $x > 0; $x--) {
      $nethash{IP_BinToDQ($ip & IP_CidrToBin($x)). "/$x"} = 1;
   }
   $sth = $db->prepare("SELECT label FROM networks WHERE ".
         "masklen <= '$mask' AND viewid = '$view' ".
         "ORDER BY network DESC, masklen DESC");
   $sth->execute;
   API_DBResCheck(\$sth,"IP_GetSupernet: getnetid: ",'1');
   while (@row = $sth->fetchrow_array) {
      if ($nethash{$row[0]}) { 
         ($val,$netmask) = split('/',$row[0]); 
         last; 
      }
   }
   if(!$val && !$netmask) {
      $val = '0.0.0.0';
      $netmask = '0';
   }
   return($val,$netmask);
}

# IP_GetRange: Return an array of blocks in a given range
# Input:
#  $start		The starting block address in dotted-quad format
#	$stop 		The stopping block address in dotted-quad format
#	$mask 		The mask length of blocks to generate
# Returns:
#	@blocks		An array of block in the given range with the given mask
#	null   		Some error occured (stop was greater then start or
#					a required parameter not given)			
sub IP_GetRange {
   my($start) = shift || return();
   my($stop)  = shift || return();
   my($mask)  = shift || return();
   my($i, @blocks);
   my($startint) = IP_DQToInt($start);
   my($stopint)  = IP_DQToInt($stop);

   if($stop < $start) {
      API_SetGenError("The stopping address is less then the starting address");
      return();
   }
   my($hosts) = IP_HostsInMasklen($mask);
   for($i = $startint; $i <= $stopint; $i += $hosts) {
      push(@blocks, IP_IntToDQ($i));
   }
   return(@blocks);
}

# IP_CalculateNets: Return a list of blocks that are a specified mask length
# Input:
#  $net     The aggregate's network address
#  $mask    The aggregate's mask length
#  $mask2   The mask length of blocks we want
#  $max     The maximum number of blocks to generate
# Returns:
#  null     General Error
#  @blocks  A list of blocks
sub IP_CalculateNets {
   my($net) = shift || (API_SetError('CalculateNetsNoNet') && return());
   my($mask) = shift || (API_SetError('CalculateNetsNoMask') && return());
   my($mask2) = shift || (API_SetError('CalculateNetsNoMask2') && return());
   my($max) = shift || 300;
   my(@blocks);

   my($m)  = IP_CidrToBin($mask);
   my($h) = IP_DQToBin($net);
   my($n) = $h & $m;

   $hmin  = $h;
   $hmax  = $h | ~IP_CidrToBin($mask);

   my($subnet) = 0;
   $m = IP_CidrToBin($mask2);

   for ($subnet=0; $subnet < 2**($mask2 - $mask); $subnet++) {
      my($net) = IP_IntToBin((IP_BinToInt($n) | ($subnet << (32-$mask2))));
      if ($mask2 eq '32') {
         if(($net ne $h) && ($net ne $hmax)) {    
            push(@blocks,IP_BinToDQ($net));
         }
      } else {
         push(@blocks,IP_BinToDQ($net));
      }

      if ($subnet >= $max) {
         return(@blocks);
      }
   }
   return(@blocks);
}

# IP_HostsInMasklen: Get the number of hosts in a given mask length
# Input:
#	$len		The mask length
# Returns:
#	$hosts	The number of hosts in the given mask length
sub IP_HostsInMasklen {
   my($len) = shift;
   return(2**(32 - $len));
}

# IP_CheckPrefix: Check the given block address and mask length to make
#						sure their valid
# Input:
#  $net			The block address in dotted-quad format
#	$len			The mask length
# Returns:
#	-1 	Error (not valid)
#	0		Required parameter not given
#	1		Successful
sub IP_CheckPrefix {
   my($net) = shift || return(0);
   my($len) = shift || return(0);


   my($binip) = IP_DQToBinStr($net);
   my($last) = substr($binip, $len);

        # is len longer than the IP
   if($len > length($binip)) {
      return(-1);
   }

        # is the last part of the IP only 0's
   if($last !~ /^0*$/) {
      return(-1);
   }

        # is this a valid prefix length
   if((length($last) + $len) != 32) {
      return(-1);
   }

   return(1);
}

# API_CIDRDropdown: Create a dropdown of netmasks called 'netmask'
# Input:
#	$str			A reference to a string to put the dropdown in
#	$default 	The default value to select (the CIDR length)
#	$smallest	The smallest masklen to show (eg. giving '24' will
#					show everything from $biggest to '24'
#	$biggest 	Same as smallest but the biggest masklen to show
#	$append		The string to append to the <SELECT> tag
# Returns:
#  NONE
sub API_CIDRDropdown {
   my($str) = shift || return();
   my($default) = shift || undef;
   my($smallest) = shift || 32;
   my($biggest) = shift || 0;
   my($append) = shift || undef;
   my($i, @sort, %labels);
   my(%cidrtable) = (
         '32'	=> '255.255.255.255',
         '31'	=> '255.255.255.254',
         '30'	=> '255.255.255.252',
         '29'	=> '255.255.255.248',
         '28'	=> '255.255.255.240',
         '27'	=> '255.255.255.224',
         '26'	=> '255.255.255.192',
         '25'	=> '255.255.255.128',
         '24'	=> '255.255.255.0',
         '23'	=> '255.255.254.0',
         '22'	=> '255.255.252.0',
         '21'	=> '255.255.248.0',
         '20'	=> '255.255.240.0',
         '19'	=> '255.255.224.0',
         '18'	=> '255.255.192.0',
         '17'	=> '255.255.128.0',
         '16'	=> '255.255.0.0',
         '15'	=> '255.254.0.0',
         '14'	=> '255.252.0.0',
         '13'	=> '255.248.0.0',
         '12'	=> '255.240.0.0',
         '11'	=> '255.224.0.0',
         '10'	=> '255.192.0.0',
         '9'	=> '255.128.0.0',
         '8'	=> '255.0.0.0',
         '7'	=> '254.0.0.0',
         '6'	=> '252.0.0.0',
         '5'	=> '248.0.0.0',
         '4'	=> '240.0.0.0',
         '3'	=> '224.0.0.0',
         '2'	=> '192.0.0.0',
         '1'	=> '128.0.0.0',
      );

   push(@sort, "Choose a Netmask");		
   my($all) = ConfigGetValue('Global','ShowAllMaskLens');

   for($i = $smallest; $i > $biggest; $i--) {
      next if ($i == '31' && $all != 1);
      push(@sort, $i);
      $labels{$i} = '/' . $i . ' (' . $cidrtable{$i} . ')';
   }

   ${$str} = $cgi->popup_menu(
         -name	=>	'netmask',
         -values	=>	\@sort,
         -labels =>	\%labels,
         -default => $default,
         -onChange => $append,
      );
}

# API_CheckFormFields: Check form data for validity.  This function
#						     automatically calls API_ExitError on any error
# Input:
#	%fields			A reference to a hash-of-hashes containing field data
#		%required	A hash for each required field with the field
#						name as the key and the regex to use as the value
#		%optional	A hash for each optional field.  Same key/value
#					   format as above but these are only checked if a 
#					   value is specified
#		%names		A hash of descriptions for each field.  The key is the
#						field name and the value is the description
# Returns:
#	NONE
sub API_CheckFormFields {
   my($fields) = shift || API_ExitError("API_CheckFormFields: A hash with form data was not given");
   my($r) = ${$fields}{'required'};
   my($o) = ${$fields}{'optional'};
   my($n) = ${$fields}{'names'};
   my($k, $v);
   while(($k,$v) = each %{$r}) {
      if(API_CheckField(\$cgi->param($k), $v) < 1) {
         API_ExitError("Field ${$n}{$k} was not filled in correctly (" . 
            API_GetError() . ")");
      }
   }

   while(($k,$v) = each %{$o}) {
      if($cgi->param($k)) {
         if(API_CheckField(\$cgi->param($k), $v) < 1) {
            API_ExitError("Field ${$n}{$k} was not filled in correctly");
         }
      }
   }
}

# API_CheckField: Check a supplied value against a regex
# Input:
#	$field		Reference to the field to check
#	$type 		The regex to use
#	$regex		An optional regex to use instead a pre-defined one
# Returns:
#	-1 	Error
#	0		Required parameter not given
#	1		Successful
sub API_CheckField {
   my($field) = shift || (API_SetError('CheckFieldNoField') && return(0));
   my($type) = shift || (API_SetError('CheckFieldNoType') && return(0));
   my($regex) = shift || undef;
   Debug("field=${$field} type=$type regex=$regex", 5);
   my(%types) = (
         'email'		=>	'([\w\-\.\+]+)\@([\w\-\.]+)\.(\w{3}|\w{2})',
         'phone'		=>	'^[\d\-\(\)\ ]+$',
         'state'		=>	'([.\w{2}])',
         'zip'		=>	'([.\d{5}])',
         'ziplong'	=>	'([.\d{5}]\-[.\d{4}])',
         'zipboth'	=>	'^(\d{5}\-\d{4}|\d{5})$',
         'alpha'		=>	'^[\d\w\ \.\#\&\,\'\-\n\@\,\W]+$',
         'num'		=>	'^\d+$',
         'money'		=>	'^([\d\,\$]+)$',
         'date'		=>	'^(\d{1,2}\/\d{1,2}\/\d{2})$',
         'ccnum'		=>	'^[0-9]+$',
         'ipv4addr'	=> '^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$',
         'cidraddr'  => '^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$\/\d{1,2}$',
         'macaddr'	=>	'^[0-9a-f]{2}\:[0-9a-f]{2}\:[0-9a-f]{2}\:[0-9a-f]{2}\:[0-9a-f]{2}\:[0-9a-f]{2}$',
         'anything'	=>	'.*',
      );

   if($types{$type}) {
      if(${$field} !~ /$types{$type}/) {
         API_SetGenError("Defined regex match failed");
         return(-1);
      }
   }
   elsif($regex) {
      if(${$field} !~ /$regex/) {
         API_SetGenError("Custom regex match failed");
         return(-1);
      }
   }
   else {
      API_SetGenError("Regex not found");
      return(-1);
   }

   return(1);
}


# API_OutputPage: Output a HTML template page.  This function automatically
#					   calls API_ExitError on any errors
# Input:
#	$page 	The filename of the page relative to the 'html' directory
#	\%pitems A hash of tags to be replaced on the page.  The key should be
#				the tag name and the value should be what the tag needs to be
#				replaced with.
# Returns:
#	NONE
sub API_OutputPage {
   my($page) = shift || (API_ExitError("API_OutputPage: No page specified"));
   my($pitems) = shift;
   my($k, $v);
   my($template_dir);

   $template_dir = ConfigGetValue('Global','HTMLDir');

   API_OutputHeader();
   $page =~ s/(\.\.+)|(\/)//g;

   if(-f "$template_dir/$page") {
      open(P, "$template_dir/$page") || API_ExitError("API_OutputPage: $!");

      ${$pitems}{'script'} = ConfigGetValue('Global','ScriptURL');
      ${$pitems}{'tmpldir'} = ConfigGetValue('Global','TemplateURL');
      ${$pitems}{'servername'} = ConfigGetValue('Global','ServerName') || $ENV{'SERVER_NAME'};
      ${$pitems}{'nsadmin'} = ConfigGetValue('Global','AdminEmail');
      ${$pitems}{'nsversion'} = $NorthStar::VERSION;
      ${$pitems}{'nsusername'} = $NorthStar::username;
      ${$pitems}{'nsrealname'} = $NorthStar::user_realname;
      ${$pitems}{'sid'} = $NorthStar::sid;
      ${$pitems}{'depth'} = $cgi->param('depth');
      ${$pitems}{'path'} = $cgi->param('path');

      while(<P>) {
         while(($k, $v) = each %{$pitems}) {
            $_ =~ s/<!--$k-->/$v/g;
         }
         print $_;
      }

      close(P);
   }
   else {
      print "<H1>$page</H1>";
      while(($k, $v) = each %{$pitems}) {
         print "$k = $v<BR>";
      }
   }
}

# API_OutputHeader: Output the html header if it hasn't been outputted already
# Input:
#	Anything here will be passed to $cgi->header
# Returns:
#	NONE
sub API_OutputHeader {
   if($NorthStar::API::_headeroutputed != 1) {
      print $cgi->header(@_);
      $NorthStar::API::_headeroutputed = 1;
   }
}


# API_ExitError: Exit the program returning an error page to the user.
#                If a error string is not specified API_GetError() is 
#                automatically called and if an error message exists it
#                is displayed.
# Input:
#	$str		The error message to display
# Returns:
#	NONE
sub API_ExitError {
   my($str) = shift || (API_GetError() || "No Error Message Specified");

   my($d) = ConfigGetValue('Global','Debug');

   API_OutputHeader();
   print "<HTML>\n<HEAD><TITLE>Error Encountered</TITLE></HEAD>\n";
   print "<BODY><H1>The following <FONT COLOR=\"red\">error</FONT> has occured during your request.</H1>\n";
   print "<UL><LI>$str</LI></UL>\n";
   print "<H3>Please contact the site administrator for more details.</H3>\n";

   if($d > 0) {
      print "Debug Mode Enabled.  A stack trace has been dumped to STDERR ".
         "(check your server error log)</BODY></HTML>\n";
      confess();
   } else {
      print "</BODY></HTML>";
      exit(1);
   }
}

# API_DBConnect: Connect to the database
# Input:
#	\%dbinfo 	A reference to a hash containing the following info:
#		fh 			The db filehandle to use
#		driver		The DBI::DBD driver to use
#		host			The hostname to connect to
#		port			The port to connect to
#		dbname		The database to use
#		dbuser		The username to use
#		dbpass 		The password to use
# Returns:
#	-1 	Database Connect Failed
#	0		Required parameter not found
#	1		Connection succeded
sub API_DBConnect {
   my($d) = shift || (API_SetError('DBConnectNoHash') && return(0));

   my($fh) = ${$d}{fh} || (API_SetError('DBConnectNoFh') && return(0));
   my($driver) = ${$d}{driver} || (API_SetError('DBConnectNoDriver') && return(0));
   my($host) = ${$d}{host} || (API_SetError('DBConnectNoHost') && return(0));
   my($port) = ${$d}{port} || (API_SetError('DBConnectNoPort') && return(0));
   my($dbname) = ${$d}{dbname} || (API_SetError('DBConnectNoDB') && return(0));
   my($user) = ${$d}{dbuser} || (API_SetError('DBConnectNoUser') && return(0));
   my($password) = ${$d}{dbpass} || (API_SetError('DBConnectNoPass') && return(0));

   ${$fh} = DBI->connect("dbi:$driver:dbname=$dbname;host=$host;port=$port", "$user", "$password");

   if(${$fh} == undef) {
      API_SetGenError('API_DBConnect: ' . $DBI::errstr);		
      return(-1);
   }
   else {
      return(1);
   }
}

# API_DBResCheck: Check the result of a query
# Input:
#	\$sth 	A reference to a DBI sth object
#	$str		A freeform string that is prepended to the error message
#	$mode 	The mode to run in
#		0 or undef	Do not output an error message and exit the program
#		1				Exit the program displaying an error message		
# Returns:
#	-1 		An error occurred in the query
#	0			A required parameter was missing
#	1			The query was successful
sub API_DBResCheck {
   my($sth) = shift || (API_SetError('DBResCheckNoRef') && return(0));
   my($str) = shift || undef;
   my($mode) = shift || 0;

   if(!${$sth}) {
      API_ExitError("This should not happen.. please enable debug mode and send a stack trace");
   }

   if(${$sth}->err) {
      if($mode == 1) {
         API_ExitError(("$str: " || "") . ${$sth}->errstr);
      }
      else {
         API_SetGenError(("$str: " || "") . ${$sth}->errstr);
         return(-1);
      }
   }
   return(1);
}

# API_FormHash:  Get a hash with all currect CGI parameters in it
# Input:
#  NONE
# Returns:
#  %form    A hash with form key/value pairs
sub API_FormHash {
   my(%form);
   my(@names) = $cgi->param;
   foreach(@names) {
      $form{$_} = $cgi->param($_);
      Debug("key=$_ value=" . $cgi->param($_), 5);
   }
   return(%form);
}

# API_MenuAddSection:  Add a section to the user menu
# Input:
#  $name    The name of the section
#  $sort	An integer used for sorting.  1=top 1000=bottom
# Returns:
#  0        Required parameter not given
#  1        Successful
sub API_MenuAddSection {
   my($name) = shift || (API_SetError('MenuSectionNoName') && return(0));
   my($sort) = shift || 1001;
   $NorthStar::_usermenusections{$name} = $sort;
   return(1);
}

# API_MenuAddItem:  Add a item under a menu section
# Input:
#  $section    The section name to add this item to
#  $name       The name to be displayed for this item
#  $link       The url to link to
#  $append     Anything else to append at the end of the link
#  $sort	   An integer used for sorting 1=top 1000=bottom
# Returns:
#  0     Required parameter not given
#  1     Successful
sub API_MenuAddItem {
   my($section) = shift || (API_SetError('MenuItemNoSection') && return(0));
   my($name) = shift || (API_SetError('MenuItemNoName') && return(0));
   my($link) = shift || (API_SetError('MenuItemNoLink') && return(0));
   my($append) = shift || undef;
   my($sort) = shift || 1001;
   $NorthStar::_usermenuitems{$section}{$name}{'link'} = $link;
   $NorthStar::_usermenuitems{$section}{$name}{'append'} = $append;
   $NorthStar::_usermenuitems{$section}{$name}{'sort'} = $sort;
   return(1);
}


# API_AddAudit: Add an audit log entry
# Input: 
#   $section   Network,Owner,etc.    required
#   $action    add,del,upd,etc.      required             
#   $msg       message string        required
# Return Codes:
#	-1 General Error
#  0	Required Parameter not given
#  1	Success 
sub API_AddAudit {
   my($section) = shift || (API_SetError('AuditNoSection') && return(0));
   my($action) = shift || (API_SetError('AuditNoAction') && return(0));
   my($msg) = shift || (API_SetError('AuditNoMsg') && return(0));
   my($sth) = 
      $db->prepare("INSERT INTO auditlog (date,username,section,action,msg) ".
         "VALUES (NOW(),'$NorthStar::username','$section',".
         "'$action','$msg')");
   $sth->execute();
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError('API_AddAudit: ' . $sth->errstr);
      return(-1);
   }
   return(1);
}


# API_AddDevice: Add a device to the database
# Input:
#	%devinfo 	A hash with the following fields:
#		parent		The parent id of this device
#		templateid	The template id of this device
#		name			The name of this device
#		descr 		The description of the device
#		owner 		The owner id of this device
#		location 	The location id of this device
# Return Codes:
#	-1 General Error
#	0	Required Parameter not given
#	>0 The id of the new device
sub API_AddDevice {
   my($d) = shift || (API_SetError('DeviceNoHash') && return(0));

   my($sql) = "INSERT INTO devices (parent,templateid,name,descr,".
      "owner,location,assignuser,updateuser) VALUES (".
      "'${$d}{'parent'}','${$d}{'templateid'}','${$d}{'name'}',".
      "'${$d}{'descr'}','${$d}{'owner'}','${$d}{'location'}',".
      "'$NorthStar::user_id','$NorthStar::user_id');";
   my($sth) = $db->prepare($sql);
   $sth->execute();
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_AddDevice: Add: $sql:" . $sth->errstr);
      return(-1);
   }

   $sth = $db->prepare("SELECT id FROM devices WHERE name = '${$d}{'name'}' ".
         "AND descr = '${$d}{'descr'}' AND parent = '${$d}{'parent'}';");
   $sth->execute;
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_AddDevice: GetID: " . $sth->errstr);
      return(-1);
   }
   my($id) = ($sth->fetchrow_array)[0];

   if(API_AddAudit('device','add',"id=$id,name=${$d}{'name'}") < 1) {
      return(-1);
   }   
   return($id);
}

# API_UpdDevice: Update a device in the database
# Input:
#	%devinfo 	A hash with the following fields:
#		id 			The device id to update
#		name			The name of this device
#		descr 		The description of the device
#		owner 		The owner id of this device
#		location 	The location id of this device
# Return Codes:
#	-1 General Error
#	0	Required Parameter not given
#	1	Success
sub API_UpdDevice {
   my($d) = shift || (API_SetError('DeviceUpdNoHash') && return(0));

   if(!${$d}{'id'}) {
      API_SetGenError("A device id to update was not specified");
      return(-1);
   }

   my($sql) = "UPDATE devices SET name='${$d}{'name'}', descr='${$d}{'descr'}', ".
      "owner='${$d}{'owner'}',location='${$d}{'location'}', ".
      "updateuser='$NorthStar::user_id', upddate='now' ".
      "WHERE id = '${$d}{'id'}';";

   my($sth) = $db->prepare($sql);
   $sth->execute();
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_UpdDevice: Upd: $sql:" . $sth->errstr);
      return(-1);
   }

   if(API_AddAudit('device','update',"id=$id") < 1) {
      return(-1);
   }    

   return(1);
}

# API_DelDevice: Delete a device in the database
# Input:
#	$id			The id of the device to delete
#  $aryref		Reference to array to add deleted id's to
# Returns:
#  -1 General Error
#	0	Required parameter not given
#	1	Successful
sub API_DelDevice {
   my($id) = shift || (API_SetError('DeviceDelNoID') && return(0));
   my($aryref) = shift || (API_SetError('DeviceDelNoRef') && return(0));
   if(API_GenericDelete($id, 'devices', $aryref) < 1) {
      return(-1);
   } 

   if(API_AddAudit('device','delete',"id=$id") < 1) {
      return(-1);
   } 
   return(1)
}

# API_DeviceDropdown: Create a <SELECT> of devices
# Input:
#	$str			Reference to the string to put the dropdown in
#	$default 	The default item to be selected
#	$owner		The owner id to filter by
#	$location	The location id to filter by
#	$append		The string to append to the <SELECT> tag
# Returns:
#	-1 	General Error
#	0		Required parameter not given
#	1		Successful
sub API_DeviceDropdown {
   my($str) = shift || (API_SetError('DeviceDropdownNoRef') && return(0));
   my($default) = shift || undef;
   my($owner) = shift || undef;
   my($location) = shift || undef;
   my($append) = shift || undef;
   Debug("str=$str default=$default owner=$owner loc=$location append=$append", 5);
   my($sql, $i, @sort, %labels, %parents);
   if($owner && $location) {
      $sql = "WHERE (owner = '$owner' OR owner = '0') ".
         "AND (location = '$location' OR location = '0') AND id != '0'";
   } elsif($owner) { 
      $sql = "WHERE (owner = '$owner' OR owner = '0') AND id != '0'";
   } elsif ($location) {
      $sql = "WHERE (location = '$location' OR location = '0') AND id != '0'";
   } else { 
      $sql = "WHERE id != '0'";
   }
   my($sth) = $db->prepare("SELECT id,parent,name FROM devices ".
         "$sql ORDER BY parent,name");
   $sth->execute();
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_DeviceDropdown: " . $sth->errstr);
      return(-1);
   }

   for($i = 0; $i < $sth->rows; $i++) {
      my(@row) = $sth->fetchrow_array;
      push(@sort, $row[0]);
      $parents{$row[0]} = $row[1];
      $labels{$row[0]} = $row[2];
   }

   ${$str} = API_TreeCreateDropdown('device', $default, $append, 
         'Select a Device Object', \@sort, \%parents,
         \%labels);
   return(1);											
}


# API_GetDevice: Get device information
# Input:
#	$id			The id of the device to get
#	$mode 		The type of info to return (extended fields denoted * below)
#		0 or undef	Return raw info (db fields only)
#		1  			Return extended info (parent names resolved etc..)
# Returns:
#	%devinfo 	A hash with the following fields:
#	  Regular Mode:
#		id 			The id of this device
#		parent		The parent id of this device
#     children  	An array of the id's of children this device has
#		templateid	The template id of this device
#		name			The name of this device
#		descr 		The description of the device
#		owner 		The owner id of this device
#		location 	The location id of this device
#		assignuser	The user that added this device
#		assigndate	The date that this device was added
#		updateuser	The user that last updated this device
#		upddate		The date that this device was last updated
#    Extended Mode
#		id 			The id of this device
#		parentid		The parent id of this device
#		parent		The name of the parent device
#     children  	An array of the id's of children this device has
#		templateid	The template id of this device
#		name			The name of this device
#		descr 		The description of the device
#		ownerid 		The owner id of this device
#		owner 		The name of the owner this device is assigned to
#		locationid 	The location id of this device
#		location 	The name of the location this device is in
#		assignuser	The user that added this device
#		assigndate	The date that this device was added
#		updateuser	The user that last updated this device
#		upddate		The date that this device was last updated
#	null			An error occurred
sub API_GetDevice {
   my($id) = shift || (API_SetError('DeviceNoID') && return());
   my($mode) = shift || 0;

   my($sql);
   if($mode == 1) {	
      $sql = "SELECT d.id,d.parent,parent.name,d.templateid,d.name,d.descr,".
         "d.location,location.name,d.owner,owner.name,user1.username,".
         "d.assigndate,user2.username,d.upddate FROM devices d, ".
         "devices parent,locations location,owners owner, users user1, ".
         "users user2 WHERE d.id = '$id' AND d.parent = parent.id ".
         "AND d.location = location.id AND d.owner = owner.id ".
         "AND d.assignuser = user1.id AND d.updateuser = user2.id;";		
   } else {
      $sql = "SELECT * FROM devices WHERE id = '$id';";
   }
   my($sth) = $db->prepare($sql);
   $sth->execute();
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_GetDevice: $sql:" . $sth->errstr);
      return();
   }

   if($sth->rows != 1) {
      API_SetGenError("The requested device was not found");
      return();
   }
   my(@row) = $sth->fetchrow_array;

   my(%devinfo);
   if($mode == 1) {
      %devinfo = (
            'id'        => $row[0],
            'parentid'  => $row[1],
            'parent'    => $row[2],
            'children'	=> [],
            'templateid'=> $row[3],
            'name'      => $row[4],
            'descr'     => $row[5],
            'locationid'=> $row[6],
            'location'  => $row[7],
            'ownerid'   => $row[8],
            'owner'     => $row[9],
            'assignby'  => $row[10],
            'assigndate'=> $row[11],
            'updateby'  => $row[12],
            'updatedate'=> $row[13]
         );
   } else {
      %devinfo = (
            'id'			=> $row[0],
            'parent' 	=> $row[1],
            'children'	=> [],
            'templateid'=> $row[2],
            'name'		=> $row[3],
            'descr'		=> $row[4],
            'owner'		=> $row[5],
            'location'	=> $row[6],
            'assignuser'=> $row[7],
            'assigndate'=> $row[8],
            'updateuser'=> $row[9],
            'upddate'	=> $row[10]
         );
   }
   $sql = "SELECT id FROM devices WHERE parent = '$id';";
   $sth = $db->prepare($sql);
   $sth->execute();
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_GetDevice: GetChildren: $sql:" . $sth->errstr);
      return();
   }
   my($i, @children);
   my($aryref) = $devinfo{'children'};
   for($i = 0; $i < $sth->rows; $i++) {
      my(@row) = $sth->fetchrow_array;
      push(@$aryref, $row[0]);
   }

   if($mode == 1) {
      if($devinfo{'parentid'} == 0) {
         $devinfo{'parent'} = "Top-Level Device";
      }
   }

   return(%devinfo);
}

# API_GetDeviceList: Get a list of device objects and fields
# Input:
#  @fields        The fields to get.  This can be 1 or more of:
#		id 			The id of this device
#		parent		The parent id of this device
#		templateid	The template id of this device
#		name			The name of this device
#		descr 		The description of the device
#		owner 		The owner id of this device
#		location 	The location id of this device
#		assignuser	The user that added this device
#		assigndate	The date that this device was added
#		updateuser	The user that last updated this device
#		upddate		The date that this device was last updated
#  \@ary          An array of hashes containing the requested
#                 information for each location.  The key in 
#                 the hash is the field name and the value is
#                 the data
#  @order         The field ordering (goes after ORDER BY in the SQL)
#  @criteria         A array of hashes of extra criteria to match by.  The 
#                    hash should look like this:
#                       field => <fieldname>
#                       type  => (=,<,>,=<,>=,!,....)
#                       data  => the data to compare
# Returns:
#  -1    General Error
#  0     Required Parameter not given
#  1     Successful
sub API_GetDeviceList {
   my($fields) = shift || (API_SetError('GetDeviceListNoField') && return(0));
   my($ary) = shift || (API_SetError('GetDeviceListNoAry') && return(0));
   my($order) = shift || undef;
   my($criteria) = shift || undef;

   if(API_GenericGetInfo($fields, 'devices', $ary, $order, $criteria) < 1) {
      return(-1);
   }
   return(1);
}



# API_AddLocation: Add a location to the database
# Input:
#	%locinfo 	A hash with the following fields:
#		parent		The parent id of this device
#		templateid	The template id of this device
#		name			The name of this device
#		descr 		The description of the device
# Return Codes:
#	-1 General Error
#	0	Required Parameter not given
#	>0 The id of the new device

sub API_AddLocation {
   my($d) = shift || (API_SetError('LocationNoHash') && return(0));

   my($sql) = "INSERT INTO locations (parent,templateid,name,descr,".
      "assignuser,updateuser) VALUES (".
      "'${$d}{'parent'}','${$d}{'templateid'}','${$d}{'name'}',".
      "'${$d}{'descr'}','$NorthStar::user_id','$NorthStar::user_id');";

   my($sth) = $db->prepare($sql);
   $sth->execute();
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_AddLocation: Add: $sql:" . $sth->errstr);
      return(-1);
   }

   $sth = $db->prepare("SELECT id FROM locations WHERE name = '${$d}{'name'}' ".
         "AND descr = '${$d}{'descr'}' AND parent = '${$d}{'parent'}';");
   $sth->execute;
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_AddLocation: GetID: " . $sth->errstr);
      return(-1);
   }
   my($id) = ($sth->fetchrow_array)[0];
   if(API_AddAudit('location','add',"id=$id,name=${$d}{'name'}") < 1) {
      return(-1);
   } 
   return($id);
}

# API_UpdLocation: Update a location in the database
# Input:
#	%devinfo 	A hash with the following fields:
#		id 			The device id to update
#		name			The name of this device
#		descr 		The description of the device
# Return Codes:
#	-1 General Error
#	0	Required Parameter not given
#	1	Success

sub API_UpdLocation {
   my($d) = shift || (API_SetError('LocationUpdNoHash') && return(0));

   if(!${$d}{'id'}) {
      API_SetGenError("A location id to update was not specified");
      return(-1);
   }

   my($sql) = "UPDATE locations SET name='${$d}{'name'}', descr='${$d}{'descr'}', ".
      "updateuser='$NorthStar::user_id', upddate='now' ".
      "WHERE id = '${$d}{'id'}';";

   my($sth) = $db->prepare($sql);
   $sth->execute();
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_UpdLocation: Upd: $sql:" . $sth->errstr);
      return(-1);
   }

   if(API_AddAudit('location','update',"id=$id") < 1) {
      return(-1);
   } 

   return(1);
}

# API_DelLocation: Delete a location in the database
# Input:
#	$id			The id of the device to delete
#  $aryref		Reference to array to add deleted id's to
# Returns:
#  -1 General Error
#	0	Required parameter not given
#	1	Successful

sub API_DelLocation {
   my($id) = shift || (API_SetError('LocationDelNoID') && return(0));
   my($aryref) = shift || (API_SetError('LocationDelNoRef') && return(0));
   if(API_GenericDelete($id, 'locations', $aryref) < 1) {
      return(-1);
   } 

   if(API_AddAudit('location','delete',"id=$id") < 1) {
      return(-1);
   } 

   return(1)
}

# API_LocationDropdown: Create a <SELECT> of locations
# Input:
#	$str			Reference to the string to put the dropdown in
#	$default 	The default item to be selected
#	$append		The string to append to the <SELECT> tag
#	$global		Show the global location (boolean)
# Returns:
#	-1 	General Error
#	0		Required parameter not given
#	1		Successful
sub API_LocationDropdown {
   my($str) = shift || (API_SetError('DeviceDropdownNoRef') && return(0));
   my($default) = shift || undef;
   my($append) = shift || undef;
   my($global) = shift || 0;

   my($i, @sort, %labels, %parents, @data);
   API_GetLocationList([qw(id parent name)], \@data, [qw(parent name)]);

   foreach(@data) {
      push(@sort, ${$_}{'id'});
      $parents{${$_}{'id'}} = ${$_}{'parent'};
      $labels{${$_}{'id'}} = ${$_}{'name'};
   }

   if($global) {
      ${$str} = API_TreeCreateDropdown('location', $default, $append, 
            'Select a Location Object', \@sort, \%parents, 
            \%labels, 'Global Location');
   } else {
      ${$str} = API_TreeCreateDropdown('location', $default, $append, 
            'Select a Location Object', \@sort, \%parents, 
            \%labels);
   }
   return(1);											
}

# API_GetLocation: Get location information
# Input:
#	$id			The id of the device to get
#	$mode 		The type of info to return (extended fields denoted * below)
#		0 or undef	Return raw info (db fields only)
#		1  			Return extended info (parent names resolved etc..)
# Returns:
#	%devinfo 	A hash with the following fields:
#	  Regular Mode:
#		id 			The id of this device
#		parent		The parent id of this device
#     children  	An array of the id's of children this device has
#		templateid	The template id of this device
#		name			The name of this device
#		descr 		The description of the device
#		assignuser	The user that added this device
#		assigndate	The date that this device was added
#		updateuser	The user that last updated this device
#		upddate		The date that this device was last updated
#    Extended Mode
#		id 			The id of this device
#		parentid		The parent id of this device
#		parent		The name of the parent device
#     children  	An array of the id's of children this device has
#		templateid	The template id of this device
#		name			The name of this device
#		descr 		The description of the device
#		assignuser	The user that added this device
#		assigndate	The date that this device was added
#		updateuser	The user that last updated this device
#		upddate		The date that this device was last updated
#	null			An error occurred
sub API_GetLocation {
   my($id) = shift || (API_SetError('LocationNoID') && return());
   my($mode) = shift || 0;

   my($sql);
   if($mode == 1) {	
      $sql = "SELECT loc.id, loc.parent, parent.name, loc.templateid,".
         "loc.name, loc.descr, user1.username, loc.assigndate, ".
         "user2.username, loc.upddate FROM locations loc, ".
         "locations parent, users user1, users user2 WHERE ".
         "loc.id = '$id' AND loc.parent = parent.id ".
         "AND user1.id = loc.assignuser AND user2.id = loc.updateuser;";
   } else {
      $sql = "SELECT * FROM locations WHERE id = '$id';";
   }
   my($sth) = $db->prepare($sql);
   $sth->execute();
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_GetLocation: $sql:" . $sth->errstr);
      return();
   }

   if($sth->rows != 1) {
      API_SetGenError("The requested location was not found");
      return();
   }
   my(@row) = $sth->fetchrow_array;

   my(%locinfo);
   if($mode == 1) {
      %locinfo = (
            'id'        => $row[0],
            'parentid'  => $row[1],
            'parent'    => $row[2],
            'children'	=> [],
            'templateid'=> $row[3],
            'name'      => $row[4],
            'descr'     => $row[5],
            'assignby'  => $row[6],
            'assigndate'=> $row[7],
            'updateby'  => $row[8],
            'updatedate'=> $row[9]
         );
   } else {
      %locinfo = (
            'id'			=> $row[0],
            'parent' 	=> $row[1],
            'children'	=> [],
            'templateid'=> $row[2],
            'name'		=> $row[3],
            'descr'		=> $row[4],
            'assignuser'=> $row[5],
            'assigndate'=> $row[6],
            'updateuser'=> $row[7],
            'upddate'	=> $row[8]
         );
   }
   $sql = "SELECT id FROM locations WHERE parent = '$id';";
   $sth = $db->prepare($sql);
   $sth->execute();
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_GetLocation: GetChildren: $sql:" . $sth->errstr);
      return();
   }
   my($i, @children);
   my($aryref) = $locinfo{'children'};
   for($i = 0; $i < $sth->rows; $i++) {
      my(@row) = $sth->fetchrow_array;
      push(@$aryref, $row[0]);
   }

   if($mode == 1) {
      if($locinfo{'parentid'} == 0) {
         $locinfo{'parent'} = "Top-Level Location";
      }
   }

   return(%locinfo);
}

# API_GetLocationList: Get a list of location object and fields
# Input:
#  @fields        The fields to get.  This can be 1 or more of:
#     id             The location ID
#     parent         The location's parent
#     templateid     The location's template ID
#     name           The location name
#     descr          The location description
#     assignuser     The user ID that added the object
#     assigndate     The date the object was added
#     updateuser     The user ID that last updated the object
#     upddate        The date the object was last updated
#  \@ary          An array of hashes containing the requested
#                 information for each location.  The key in 
#                 the hash is the field name and the value is
#                 the data
#  @order         The field ordering (goes after ORDER BY in the SQL)
#  @criteria         A array of hashes of extra criteria to match by.  The 
#                    hash should look like this:
#                       field => <fieldname>
#                       type  => (=,<,>,=<,>=,!,....)
#                       data  => the data to compare
# Returns:
#  -1    General Error
#  0     Required Parameter not given
#  1     Successful
sub API_GetLocationList {
   my($fields) = shift || (API_SetError('GetLocationListNoField') && return(0));
   my($ary) = shift || (API_SetError('GetLocationListNoAry') && return(0));
   my($order) = shift || undef;
   my($criteria) = shift || undef;

   if(API_GenericGetInfo($fields, 'locations', $ary, $order, $criteria) < 1) {
      return(-1);
   }
   return(1);
}

# API_AddOwner: Add an owner to the database
# Input:
#	%ownerinfo 	A hash with the following fields:
#		templateid	The template id of this device
#		name			The name of this device
#		descr 		The description of the device
# Return Codes:
#	-1 General Error
#	0	Required Parameter not given
#	>0 The id of the new owner

sub API_AddOwner {
   my($d) = shift || (API_SetError('OwnerNoHash') && return(0));

   my($sql) = "INSERT INTO owners (templateid,name,descr) VALUES ".
      "('${$d}{'templateid'}','${$d}{'name'}','${$d}{'descr'}');";

   my($sth) = $db->prepare($sql);
   $sth->execute();
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_AddOwner: Add: $sql:" . $sth->errstr);
      return(-1);
   }

   $sth = $db->prepare("SELECT id FROM owners WHERE name = '${$d}{'name'}' ".
         "AND descr = '${$d}{'descr'}';");
   $sth->execute;
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_AddOwner: GetID: " . $sth->errstr);
      return(-1);
   }
   my($id) = ($sth->fetchrow_array)[0];
   if(API_AddAudit('owner','add',"id=$id,name=${$d}{'name'}") < 1) {
      return(-1);
   } 
   return($id);
}

# API_UpdOwner: Update a owner in the database
# Input:
#	%devinfo 	A hash with the following fields:
#		id 			The device id to update
#		name			The name of this device
#		descr 		The description of the device
# Return Codes:
#	-1 General Error
#	0	Required Parameter not given
#	1	Success

sub API_UpdOwner {
   my($d) = shift || (API_SetError('OwnerUpdNoHash') && return(0));

   if(!${$d}{'id'}) {
      API_SetGenError("A owner id to update was not specified");
      return(-1);
   }

   my($sql) = "UPDATE owners SET name='${$d}{'name'}', descr='${$d}{'descr'}' ".
      "WHERE id = '${$d}{'id'}';";

   my($sth) = $db->prepare($sql);
   $sth->execute();
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_UpdOwner: Upd: $sql:" . $sth->errstr);
      return(-1);
   }

   if(API_AddAudit('owner','update',"id=$id") < 1) {
      return(-1);
   } 

   return(1);
}

# API_DelOwner: Delete a owner in the database
# Input:
#	$id			The id of the device to delete
#  $aryref		Reference to array to add deleted id's to
# Returns:
#  -1 General Error
#	0	Required parameter not given
#	1	Successful

sub API_DelOwner {
   my($id) = shift || (API_SetError('OwnerDelNoID') && return(0));
   my($aryref) = shift || (API_SetError('OwnerDelNoRef') && return(0));
   if(API_GenericDelete($id, 'owners', $aryref, 1) < 1) {
      return(-1);
   } 

   if(API_AddAudit('owner','delete',"id=$id") < 1) {
      return(-1);
   } 

   return(1)
}

# API_OwnerDropdown: Create a <SELECT> of owners
# Input:
#	$str			Reference to the string to put the dropdown in
#	$default 	The default item to be selected
#	$append		The string to append to the <SELECT> tag
#	$global		Show the global location (boolean)
# Returns:
#	-1 	General Error
#	0		Required parameter not given
#	1		Successful
sub API_OwnerDropdown {
   my($str) = shift || (API_SetError('OwnerDropdownNoRef') && return(0));
   my($default) = shift || undef;
   my($append) = shift || undef;
   my($global) = shift || 0;
   Debug("str=$str default=$default append=$append global=$global", 5);
   my($i, @sort, %labels, %parents, @data);
   API_GetOwnerList([qw(id name)], \@data, [qw(name)]);

   foreach(@data) {
      push(@sort, ${$_}{'id'});
      $parents{${$_}{'id'}} = 0;
      $labels{${$_}{'id'}} = ${$_}{'name'};
   }

   if($global) {
      ${$str} = API_TreeCreateDropdown('owner', $default, $append, 
            'Select an Owner Object', \@sort, \%parents, 
            \%labels, 'Global Owner');
   } else {
      ${$str} = API_TreeCreateDropdown('owner', $default, $append, 
            'Select an Owner Object', \@sort, \%parents, 
            \%labels);
   }
   return(1);											
}

# API_GetOwner: Get owner information
# Input:
#	$id			The id of the owner to get
# Returns:
#	%ownerinfo 	A hash with the following fields:
#		id 			The id of this owner
#     templateid  The template id of this owner
#		name			The name of this owner
#		descr 		The description of the owner
#	null			An error occurred
sub API_GetOwner {
   my($id) = shift || (API_SetError('OwnerNoID') && return());

   my($sql) = "SELECT * FROM owners WHERE id = '$id';";
   my($sth) = $db->prepare($sql);
   $sth->execute();
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_GetOwner: $sql:" . $sth->errstr);
      return();
   }

   if($sth->rows != 1) {
      API_SetGenError("The requested owner was not found");
      return();
   }
   my(@row) = $sth->fetchrow_array;

   my(%ownerinfo) = (
         'id'			=> $row[0],
         'templateid'=> $row[1],
         'name'		=> $row[2],
         'descr'		=> $row[3],
      );
   return(%ownerinfo);
}

# API_GetOwnerList: Get a list of location object and fields
# Input:
#  @fields        The fields to get.  This can be 1 or more of:
#     id             The location ID
#     name           The location name
#     descr          The location description
#  \@ary          An array of hashes containing the requested
#                 information for each location.  The key in 
#                 the hash is the field name and the value is
#                 the data
#  @order         The field ordering (goes after ORDER BY in the SQL)
#  @criteria         A array of hashes of extra criteria to match by.  The 
#                    hash should look like this:
#                       field => <fieldname>
#                       type  => (=,<,>,=<,>=,!,....)
#                       data  => the data to compare
# Returns:
#  -1    General Error
#  0     Required Parameter not given
#  1     Successful
sub API_GetOwnerList {
   my($fields) = shift || (API_SetError('GetOwnerListNoField') && return(0));
   my($ary) = shift || (API_SetError('GeOwnerListNoAry') && return(0));
   my($order) = shift || undef;
   my($criteria) = shift || undef;

   if(API_GenericGetInfo($fields, 'owners', $ary, $order, $criteria) < 1) {
      return(-1);
   }
   return(1);
}

# API_AddNetwork: Add a network to the database and place it in the tree
# Input:
#	%netinfo 	A hash with the following fields:
#     name           The network's name
#     templateid     The network's template ID
#     viewid         The network's view ID
#     status         The block status
#     descr          The network's description
#     block          The block in CIDR format (x.x.x.x/x)
#     owner          The network's owner ID
#     device         The network's device ID
#     location       The network's location ID
#     priority       The block priority
# Return Codes:
#	-1 General Error
#	0	Required Parameter not given
#	>0 The id of the new network
sub API_AddNetwork {
   my($d) = shift || (API_SetError('NetNoHash') && return(0));

   my($net,$mask) = split(/\//, ${$d}{'block'});
   my($int) = IP_DQToInt($net);

   my($sql) = "INSERT INTO networks (name,templateid,viewid,status,".
      "descr,label,network,masklen,owner,device,location,priority,".
      "assignuser,updateuser) VALUES".
      "('${$d}{'name'}','${$d}{'templateid'}','${$d}{'viewid'}',".
      "'${$d}{'status'}','${$d}{'descr'}','${$d}{'block'}',".
      "'$int','$mask','${$d}{'owner'}','${$d}{'device'}',".
      "'${$d}{'location'}','${$d}{'priority'}',".
      "'$NorthStar::user_id','$NorthStar::user_id');";	

   my($sth) = $db->prepare($sql);
   $sth->execute();
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_AddNet: Add: $sql:" . $sth->errstr);
      return(-1);
   }

   $sth = $db->prepare("SELECT id FROM networks WHERE label = '${$d}{'block'}'".
         " AND viewid = '${$d}{'viewid'}'");
   $sth->execute;
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_AddNet: GetID: " . $sth->errstr);
      return(-1);
   }

   if($sth->rows != 1) {
      API_ExitError("Your database is in an inconsistent state.  This is not good.");
   }

   my($id) = ($sth->fetchrow_array)[0];
   if(API_NetworkTreeUpdate($id) < 1) {
      return(-1);
   }
   if(API_AddAudit('network','add',"id=$id,view=${$d}{'viewid'}") < 1) {
      return(-1);
   } 
   return($id);
}

# API_UpdNetwork: Update a network in the database
# Input:
#	%netinfo 	A hash with the following fields:
#     id             The network object to update
#     name           The network's name
#     status         The block status
#     descr          The network's description
#     owner          The network's owner ID
#     device         The network's device ID
#     location       The network's location ID
#     priority       The block priority
# Return Codes:
#	-1 General Error
#	0	Required Parameter not given
#	1	Success
sub API_UpdNetwork {
   my($d) = shift || (API_SetError('NetUpdNoHash') && return(0));

   if(!${$d}{'id'}) {
      API_SetGenError("A network id to update was not specified");
      return(-1);
   }

   my($sql) = "UPDATE networks SET name='${$d}{'name'}', ".
      "descr='${$d}{'descr'}', owner='${$d}{'owner'}', ".
      "device='${$d}{'device'}', updateuser='$NorthStar::user_id', ".
      "upddate='now', location='${$d}{'location'}', ".
      "status='${$d}{'status'}',priority='${$d}{'priority'}' ".
      "WHERE id = '${$d}{'id'}';";

   my($sth) = $db->prepare($sql);
   $sth->execute();
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_UpdNetwork: Upd: $sql:" . $sth->errstr);
      return(-1);
   }
   if(API_AddAudit('network','update',"id=$id,name=${$d}{'name'}") < 1) {
      return(-1);
   } 
   return(1);
}

# API_DelNetwork: Delete a device in the database
# Input:
#	$id			The id of the device to delete
#  $aryref		Reference to array to add deleted id's to
# Returns:
#  -1 General Error
#	0	Required parameter not given
#	1	Successful
sub API_DelNetwork {
   my($id) = shift || (API_SetError('NetworkDelNoID') && return(0));
   my($aryref) = shift || (API_SetError('NetworkDelNoRef') && return(0));
   if(API_GenericDelete($id, 'networks', $aryref) < 1) {
      return(-1);
   } 

   if(API_AddAudit('network','delete',"id=$id") < 1) {
      return(-1);
   } 

   return(1)
}

# API_GetNetwork: Get network information
# Input:
#	$id			The id of the network to get
#	$mode 		The type of info to return
#		0 or undef	Return raw info (db fields only)
#		1  			Return extended info (parent names resolved etc..)
# Returns:
#	%devinfo 	A hash with the following fields:
#	  Regular Mode:
#		id 			The id of this network
#		parent		The parent id of this network
#     children  	An array of the id's of children this network has
#		templateid	The template id of this network
#     viewid      The view this network belongs to
#     status      The status bit of this network
#		name			The name of this network
#		descr 		The description of the network
#     label       The network address in CIDR format
#     network     The network address in integer format
#     masklen     The network mask length
#		owner 		The owner id of this network
#     device      The device id of this network
#		location 	The location id of this network
#     priority    The block priority of this network
#		assignuser	The user that added this network
#		assigndate	The date that this network was added
#		updateuser	The user that last updated this network
#		upddate		The date that this network was last updated
#    Extended Mode
#		id 			The id of this network
#		parentid		The parent id of this network
#		parent		The name of the parent network
#     children  	An array of the id's of children this network has
#		templateid	The template id of this network
#     viewid      The view this network belongs to
#     view        The name of the view
#     statusid    The status bit of this network
#     status      The status name of this network
#		name			The name of this network
#		descr 		The description of the network
#     label       The network address in CIDR format
#     network     The network address in integer format
#     masklen     The network mask length
#		ownerid 		The owner id of this network
#		owner 		The name of the owner this network is assigned to
#		deviceid 	The device id of this network
#		device 		The name of the device this network is assigned to
#		locationid 	The location id of this network
#		location 	The name of the location this network is in
#     priority    The block priority of this network
#		assignuser	The user that added this network
#		assigndate	The date that this network was added
#		updateuser	The user that last updated this network
#		upddate		The date that this network was last updated
#	null			An error occurred
sub API_GetNetwork {
   my($id) = shift || (API_SetError('DeviceNoID') && return());
   my($mode) = shift || 0;

   my($sql);
   if($mode == 1) {	
      $sql = "SELECT n.id,n.parent,parent.name,n.templateid,n.viewid,view.name,".
         "n.status,n.name,n.descr,n.label,n.network,n.masklen,n.owner,".
         "owner.name,n.device,device.name,n.location,location.name,n.priority,".
         "user1.username,n.assigndate,user2.username,n.upddate ".
         "FROM networks n, networks parent, owners owner, ".
         "locations location,devices device,users user1, users user2, ".
         "views view WHERE n.id = '$id' ".
         "AND n.parent = parent.id AND n.owner = owner.id ".
         "AND n.device = device.id AND n.assignuser = user1.id ".
         "AND n.updateuser = user2.id AND n.viewid = view.id ".
         "AND n.location = location.id;";   
   } else {
      $sql = "SELECT * FROM networks WHERE id = '$id';";
   }
   my($sth) = $db->prepare($sql);
   $sth->execute();
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_GetNetwork: $sql:" . $sth->errstr);
      return();
   }

   if($sth->rows != 1) {
      API_SetGenError("The requested network was not found");
      return();
   }
   my(@row) = $sth->fetchrow_array;

   my(%netinfo);
   if($mode == 1) {
      %netinfo = (
            'id'			=> $row[0],
            'parentid' 	=> $row[1],
            'parent' 	=> $row[2],
            'children'	=> [],
            'templateid'=> $row[3],
            'viewid'    => $row[4],
            'view'      => $row[5],
            'statusid'  => $row[6],
            'status'    => API_GetNetStatusName($row[6]),
            'name'		=> $row[7],
            'descr'		=> $row[8],
            'label'     => $row[9],
            'network'   => $row[10],
            'masklen'   => $row[11],
            'ownerid'	=> $row[12],
            'owner'		=> $row[13],
            'deviceid'  => $row[14],
            'device'    => $row[15],
            'locationid'=> $row[16],
            'location'	=> $row[17],
            'priority'  => $row[18],
            'assignuser'=> $row[19],
            'assigndate'=> $row[20],
            'updateuser'=> $row[21],
            'upddate'	=> $row[22]
         );
   } else {
      %netinfo = (
            'id'			=> $row[0],
            'parent' 	=> $row[1],
            'children'	=> [],
            'templateid'=> $row[2],
            'viewid'    => $row[3],
            'status'    => $row[4],
            'name'		=> $row[5],
            'descr'		=> $row[6],
            'label'     => $row[7],
            'network'   => $row[8],
            'masklen'   => $row[9],
            'owner'		=> $row[10],
            'device'    => $row[11],
            'location'	=> $row[12],
            'priority'  => $row[13],
            'assignuser'=> $row[14],
            'assigndate'=> $row[15],
            'updateuser'=> $row[16],
            'upddate'	=> $row[17]
         );
   }
   $sql = "SELECT id FROM networks WHERE parent = '$id';";
   $sth = $db->prepare($sql);
   $sth->execute();
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_GetNetwork: GetChildren: $sql:" . $sth->errstr);
      return();
   }
   my($i, @children);
   my($aryref) = $netinfo{'children'};
   for($i = 0; $i < $sth->rows; $i++) {
      my(@row) = $sth->fetchrow_array;
      push(@$aryref, $row[0]);
   }

   if($mode == 1) {
      if($netinfo{'parentid'} == 0) {
         $netinfo{'parent'} = "Top-Level Network";
      }
   }

   while(($k,$v) = each %netinfo) { Debug("net: $k=$v");}
   return(%netinfo);
}

# API_CheckNetworkStatus: Check the given block to make sure it's 
#                         supernet's are not in RECLAIM or BLOCKED status
# Input:
#  $net     The network address
#  $mask    The network mask length
#  $view    The view ID
# Returns:
#  -1    General Error
#  0     Required parameter not given
#  1     Success (ok to add)
sub API_CheckNetworkStatus {
   my($net) = shift || (API_SetError('CheckNetworkStatusNoNet') && return(0));
   my($mask) = shift || (API_SetError('CheckNetworkStatusNoMask') && return(0));
   my($view) = shift || (API_SetError('CheckNetworkStatusNoView') && return(0));

   my($newfirst) = IP_DQToInt($net);
   my($newlast) = IP_DQToInt(IP_LastIP($net, $mask));
   my($pnet, $len) = IP_GetSupernet($net, $mask, $view);
   my($first) = IP_DQToInt($pnet);
   my($last) = IP_DQToInt(IP_LastIP($pnet, $len));

   my(@data);
   API_GetNetworkList([qw(id label network masklen)], \@data, [qw(network masklen)],
      [{ 'field' => 'network',
            'type'  => '>=',
            'data'  => $first
            },
         { 'field' => 'network',
            'type'  => '<=',
            'data'  => $last
            },
         { 'field' => 'masklen',
            'type'  => '<',
            'data'  => $mask
            },
         { 'field' => 'viewid',
            'type'  => '=',
            'data'  => $view
            }]);

   foreach(@data) {
      my($tfirst) = ${$_}{'network'};
      my($tlast) = IP_DQToInt(IP_LastIP(IP_IntToDQ(${$_}{'network'}), ${$_}{'masklen'}));
      if($newfirst >= $tfirst && $newlast <= $tlast) {      
         my($s) = API_GetNetStatus(${$_}{'id'});
         if($s == 3 || $s == 4) {
            API_SetGenError("The netblock ${$_}{'label'} has a RECLAIM or BLOCKED status.  Cannot add network.");
            return(-1);            
         }
      }
   }
   return(1);
}

# API_GetNetworkList: Get a list of network objects and fields
# Input:
#  @fields        The fields to get.  This can be 1 or more of:
#     id             The network ID
#     parent         The network's parent ID
#     templateid     The network's template ID
#     viewid         The network's view ID
#     status         The block status
#     name           The network's name
#     descr          The network's description
#     label          The block in CIDR format (x.x.x.x/x)
#     network        The block in integer format
#     masklen        The block mask length
#     owner          The network's owner ID
#     device         The network's device ID
#     location       The network's location ID
#     priority       The block priority
#		assignuser	The user that added this device
#		assigndate	The date that this device was added
#		updateuser	The user that last updated this device
#		upddate		The date that this device was last updated
#  \@ary          An array of hashes containing the requested
#                 information for each location.  The key in 
#                 the hash is the field name and the value is
#                 the data
#  @order         The field ordering (goes after ORDER BY in the SQL)
#  @criteria         A array of hashes of extra criteria to match by.  The 
#                    hash should look like this:
#                       field => <fieldname>
#                       type  => (=,<,>,=<,>=,!,....)
#                       data  => the data to compare
# Returns:
#  -1    General Error
#  0     Required Parameter not given
#  1     Successful
sub API_GetNetworkList {
   my($fields) = shift || (API_SetError('GetNetworkListNoField') && return(0));
   my($ary) = shift || (API_SetError('GetNetworkListNoAry') && return(0));
   my($order) = shift || undef;
   my($criteria) = shift || undef;

   if(API_GenericGetInfo($fields, 'networks', $ary, $order, $criteria) < 1) {
      return(-1);
   }
   return(1);
}



# API_GetNetStatus: Get the block status bit
# Input:
#  $id      The id of the object to get
# Returns:
#  -1    General Error
#  0     Required parameter no given
#  >0    The status bit
sub API_GetNetStatus {
   my($id) = shift || (API_SetError('GetNetStatusNoID') && return(0));
   my($sth) = $db->prepare("SELECT status FROM networks WHERE id = '$id'");
   $sth->execute();
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_GetNetStatus: " . $sth->errstr);
      return(-1);
   }    
   if($sth->rows == 0) {
      API_SetGenError("API_GetNetStatus: The object does not exist");
      return(-1);
   } 

   my($r) = ($sth->fetchrow_array)[0];
   return($r);
}


# API_NetPriorityDropdown: Create a dropdown of block priorities 
#                          named 'priority'
# Input:
#  \$str    A reference to a string to put the dropdown in
#  $default The default value to be selected
# Returns:
#  null     Required parameter missing
#  1        Successful
sub API_NetPriorityDropdown {
   my($str) = shift || return();
   my($default) = shift || undef;

   my(%labels) = (
         '0' => '0  - Lowest',
         '5' => '5  - Normal',
         '10'=> '10 - Highest'
      );
   my(@sort) = qw(10 9 8 7 6 5 4 3 2 1 0);

   if(!$default) {
      ${$str} = $cgi->popup_menu(
            -name	=> 'priority',
            -values => \@sort,
            -labels => \%labels
         );
   } else {
      ${$str} = $cgi->popup_menu(
            -name	=>	'priority',
            -values =>	\@sort,
            -labels => \%labels,
            -default=>	$default
         );
   }
   return(1);
}

# API_GetNetStatusName: Get the name associated with a particular status ID
# Input:
#  $id   The status id
# Returns:
#  null  Required parameter not given
#  $str  The name of the status id
sub API_GetNetStatusName {
   my($id) = shift || return();
   my(%statustypes) = (
         '1'	=> 'Allocated',
         '2'	=> 'Assigned',
         '3'	=> 'Reclaim',
         '4'	=> 'Blocked'
      );
   return($statustypes{$id});
}

# API_NetStatusDropdown: Create a dropdown of block status's named 'status'
# Input:
#  \$str    A reference to a string to put the dropdown in
#  $default The default value to be selected
# Returns:
#  null     Required parameter missing
#  1        Successful
sub API_NetStatusDropdown {
   my($str) = shift || return();
   my($default) = shift || undef;

   my(%statustypes) = (
         '1'	=> 'Allocated',
         '2'	=> 'Assigned',
         '3'	=> 'Reclaim',
         '4'	=> 'Blocked'
      );
   my(@sort) = qw(1 2 3 4);

   if(!$default) {
      ${$str} = $cgi->popup_menu(
            -name	=>	'status',
            -values =>	\@sort,
            -labels	=>	\%statustypes
         );
   } else {
      ${$str} = $cgi->popup_menu(
            -name	=>	'status',
            -values =>	\@sort,
            -labels	=>	\%statustypes,
            -default=>	$default
         );
   }
   return(1);
}



# API_NetStatusUpdate: Recursively updates the block status for a net 
#                      and ALL of it's children
# Input:
#  $id      The id of the block to update
#  $status  The status to set (1-4)
# Returns:
#  -1    General Error
#  0     Required Parameter not given
#  1     Successful
sub API_NetStatusUpdate {
   my($id) = shift || (API_SetError('NetStatusUpdateNoID') && return(0));
   my($status) = shift || (API_SetError('NetStatusUpdateNoPri') && return(0));
   my($mode) = shift || 0;
   Debug("starting");
   my(@updids);

   if(API_GenericRecursiveUpdate($id,'networks',
            { 'status' => $status },
            [
               { 'field' => 'masklen',
                  'type'  => '!=',
                  'data'  => '32'
                  }
            ],
            \@updids) < 1) {
      return(-1);
   }
   return(1);
}

# API_NetPriorityUpdate: Recursively updates the block priority for a net
#                        and ALL of it's children (anything under it)
# Input:
#  $id      The id of the block to update
#  $pri     The priority to set (1-10)
# Returns:
#  -1    General Error
#  0     Required Parameter not given
#  1     Successful
sub API_NetPriorityUpdate {
   my($id) = shift || (API_SetError('NetPriorityUpdateNoID') && return(0));
   my($pri) = shift || (API_SetError('NetPriorityUpdateNoPri') && return(0));
   Debug("starting");
   my(@updids);
   if(API_GenericRecursiveUpdate($id,'networks',
            { 'priority' => $pri }, 
            undef, \@updids) < 1) {
      return(-1);
   }
   return(1);
}

# API_NetReassignParent: Reassign the parent id of networks
# Input:
#  $old     The old parent ID
#  $new     The new parent ID (default is 0)
# Returns:
#  -1    General Error
#  0     Required parameter not given
#  1     Successful
sub API_NetReassignParent {
   my($old) = shift || (API_SetError('NetReassignParentNoOld') && return(0));
   my($new) = shift || 0;

   my($sql) = "UPDATE networks SET parent = '$new' WHERE parent = '$old'";
   my($sth) = $db->prepare($sql);
   $sth->execute();
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_NetReassignParent: " . $sth->errstr);
      return(-1);
   }
   return(1);
}

# API_NetCheckSmallerExist:  Check to see if there are any blocks
#                            smaller then the given one
# Input:
#  $net     The block network address
#  $mask    The block mask length
#  $view    The view ID
#  $mode    The mode to run in
#     0 or undef    Only return smaller nets (>)
#     1             Return smaller nets and the given net (>=)
# Returns:
#  -1    General Error
#  0     Required parameter not given
#  >0    The number of smaller nets
sub API_NetCheckSmallerExist {
   my($net) = shift || (API_SetError('NetCheckSmallerExistNoNet') && return(0));
   my($mask) = shift || (API_SetError('NetCheckSmallerExistNoMask') && return(0));
   my($view) = shift || (API_SetError('NetCheckSmallerExistNoView') && return(0));
   my($mode) = shift || 0;

   my($int) = IP_DQToInt($net);
   my($lastint) = IP_DQToInt(IP_LastIP($net, $mask));

   my($t);

   if($mode == 1) {
      $t = ">=";
   } else {
      $t = ">";
   }
   my($sth) = $db->prepare("SELECT id FROM networks WHERE network >= '$int' ".
         "AND network <= '$lastint' AND masklen $t '$mask' ".
         "AND viewid = '$view'");

   $sth->execute();
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError('API_NetCheckSmallerExist: ' . $sth->errstr);
      return(-1);
   }
   return($sth->rows);
}

# API_NetCheckExist: Check to see if a block exists already
# Input:
#  $net     The block network address
#  $mask    The block mask length
#  $view    The view ID to check in
# Returns:
#  -1    General Error or required parameter not given
#  0     No network found
#  >0    The id of network (if it exists)
sub API_NetCheckExist {
   my($net) = shift || (API_SetError('NetCheckExistNoNet') && return(-1));
   my($mask) = shift || (API_SetError('NetCheckExistNoMask') && return(-1));
   my($view) = shift || (API_SetError('NetCheckExistNoView') && return(-1));

   my($int) = IP_DQToInt($net);
   my($sth) = $db->prepare("SELECT id FROM networks WHERE network = '$int' ".
         "AND masklen = '$mask' AND viewid = '$view'");
   $sth->execute();
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError('API_NetCheckExist: ' . $sth->errstr);
      return(-1);
   }

   if($sth->rows != 1) {
      return(0);
   }

   return(($sth->fetchrow_array)[0]);
}

# API_NetworkAddrDropdown: Create a dropdown of blocks
# Input:
#  \$str       Reference to a string to put the dropdown in
#  $net        The block network address
#  $mask       The parent block mask length
#  $mask2      The mask length of blocks to list
#  $view       The view id
#  $append     The information to append to the <SELECT> tag
# Returns:
#  -1    General Error
#  0     Required parameter not given
#  1     Successful
sub API_NetworkAddrDropdown {
   my($str) = shift || (API_SetError('NetAddrDropdownNoRef') && return(0));
   my($net) = shift || (API_SetError('NetAddrDropdownNoNet') && return(0));
   my($mask) = shift || (API_SetError('NetAddrDropdownNoMask') && return(0));
   my($mask2) = shift || (API_SetError('NetAddrDropdownNoMask2') && return(0));
   my($view) = shift || (API_SetError('NetAddrDropdownNoView') && return(0));
   my($append) = shift || undef;
   Debug("starting");
   my(@sort) = API_NetworkGetSubnets($net,$mask,$mask2,$view);
   if(!@sort) {
      Debug("no nets, return -1");
      return(-1);
   }
   my(%labels);
   for (@sort) {
      $labels{$_} = $_;
   }

   ${$str} = $cgi->popup_menu(
         -name	=>	'netaddr',
         -values	=>	\@sort,
         -labels =>	\%labels,
         -onChange => $append || '',
      );
   return(1);
}

# API_NetworkGetSubnets: Get available blocks given an aggregate
# Input:
#  $net     The aggregate's network address
#  $mask    The aggregate's mask length
#  $mask2   The mask length of blocks to generate
#  $view    The view id
# Returns:
#  null     General Error
#  @ret     a list of available blocks
sub API_NetworkGetSubnets {
   my($net) = shift || (API_SetError('NetGetSubnetsNoNet') && return());
   my($mask) = shift || (API_SetError('NetGetSubnetsNoMask') && return());
   my($mask2) = shift || (API_SetError('NetGetSubnetsNoMask2') && return());
   my($view) = shift || (API_SetError('NetGetSubnetsNoView') && return());
   my(@ret);
   Debug("starting");
   my(@candidates) = IP_CalculateNets($net,$mask,$mask2);
   my($last) = IP_LastIP($net, $mask);
   Debug("last=$last");

   foreach(@candidates) {
      Debug("candidate=$_");
      if(API_NetCheckSmallerExist($_,$mask2,$view,1) == 0) {
         Debug("no smaller nets");
         my($supernet,$supermask) = IP_GetSupernet($_, $mask2,$view);
         my($supernetid) = API_NetCheckExist($supernet,$supermask,$view);
         my($int) = IP_DQToInt($_);
         Debug("supernet=$supernet supermask=$supermask supernetid=$supernetid");
         if( (($net == $supernet) && ($mask == $supermask)) || 
               ($supernetid != 0 && API_GetNetStatus($supernetid) == 1) || 
               $supernetid == 0)  
         {
            Debug("adding block $_");
            push(@ret, $_);
         }
      }
   }
   return(@ret);
}

# API_NetworkGetBlock:  Get a block to allocate
# Input:
#  %nets       A hash of aggregates with the key being the network id and the
#              value being the block in CIDR format
#  %priority   A hash of the priorities of each of the above nets.  The key 
#              is the network id
#  $netlen     The mask length of the block we want
#  $view       The view id to get the block in
# Returns:
#  null        General Error
#  $block      The block in CIDR format
sub API_NetworkGetBlock {
   my $nets = shift || (API_SetError('NetGetBlockNoNets') && return());
   my $priority = shift || (API_SetError('NetGetBlockNoPri') && return());
   my $netlen = shift || (API_SetError('NetGetBlockNoMask') && return());
   my $view = shift || (API_SetError('NetGetBlockNoView') && return());

   my(%netranks, %netpri);

   my($k,$v);
        #loop for each supernet we should have in %nets
   while(($k,$v) = each %{$nets}) {
      my($parentpri) = ${$priority}{$k}; 
      my($intblock, $len) = split(/\//, $v);
      my($block) = IP_IntToDQ($intblock);
                # get our nets that could work
      my(@candidates) = IP_CalculateNets($block, $len, $netlen);
      foreach(@candidates) {
                        # first check if the candidate already exists, then
                        # check if the candidate has any nets that are 
                        # smaller then it already assigned, then finally
                        # check to make sure no addresses are assigned under
                        # a supernet of the candidate
         if(API_NetCheckSmallerExist($_, $netlen, $view, 1) == 0) {
            my($supernet,$superlen) = IP_GetSupernet($_, $netlen,$view);
            my($supernetid) = API_NetCheckExist($supernet,$superlen,$view);
            my($int) = IP_DQToInt($_);
            if(($supernetid != 0 && API_GetNetStatus($supernetid) == 1) || $supernetid == 0) {
               $netranks{"$int/$netlen"} = API_NetRank($block, $len, $netlen, $view);
               $netpri{"$int/$netlen"} = $parentpri;
            } 
         }
      }
   }

        # sort by rank first then by key (key is the integer and gives
        # us the lowest integer network
   my(@s) = sort { 
      $netpri{$a} <=> $netpri{$b} || 
      $netranks{$a} <=> $netranks{$b} || 
      $b <=> $a 
   } keys(%netranks);
        # pop it off and return that biznitch
   my($tb, $tl) = split(/\//, pop(@s));
   my($block) = IP_IntToDQ($tb);
        # relax focker... the needles are jumping
   return("$block/$tl");
}

# API_NetRank: Rank a given block by seeing how many blocks of the 
#              same length exist under it's parent
# Input:
#  $net        The network address
#  $parentlen  The mask length of the parent
#  $childlen   The mask length of the child
#  $view       The view id
# Returns:
#  null     General Error
#  $rank    An integer of the number of blocks of childlen under the block
#           "net/parentlen"
sub API_NetRank {
   my($net) = shift || (API_SetError('NetRankNoNet') && return());
   my($parentlen) = shift || (API_SetError('NetRankNoParentLen') && return());
   my($childlen) = shift || (API_SetError('NetRankNoChildLen') && return());
   my($view) = shift || (API_SetError('NetRankNoView') && return());

   my($id) = API_NetCheckExist($net, $parentlen, $view);
   my($sth) = $db->prepare("SELECT id FROM networks WHERE parent = '$id' ".
         "AND masklen = '$childlen';");
   $sth->execute();
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_NetRank: $net($parentlen/$childlen): " . $sth->errstr);
      return();
   }

   return($sth->rows);
}


# API_NetworkTreeUpdate: handles placing new nets in the tree
# Input:
#  $id      The id of the new network
# Returns:
#  -1    General Error
#  0     Required parameter not given
#  1     Successful
sub API_NetworkTreeUpdate {
   my($id) = shift || (API_SetError('NetworkTreeUpdateNoID') && return(0));

   my($sth, @row, $first, $last, %parents, %labels, $i, @sort, @array, %nodes);
   my($len, $ip, $l, $pnet);
   my(%netinfo) = API_GetNetwork($id);
   if(!%netinfo) {
      API_SetGenError("API_NetworkTreeUpdate: The network id was not found");
   }
   my($view) = $netinfo{'viewid'};
   ($ip, $len) = split(/\//, $netinfo{'label'});


      # Ok, now were going to see if this network has smaller
      # nets that need to have there parents reassigned to 
      # this one.
      #
      # Here is out general strategy to do this:
      # 1. First the first and last integer of the new net
      # 2. Find the nets currently in the db in the range found in step 1
      #    AND with a masklen greater then the new net.
      # 3. Throw the results of step 3 into a super-duper pooper-scooper tree
      # 4. Get an array of nets at a depth of 2 (1=root, 2=first level) and
      #    set those nets parent to the id of the new net

   $first = IP_DQToInt($ip);
   $last = IP_DQToInt(IP_LastIP($ip, $len));
   $sth = $db->prepare("SELECT id,parent,label,network,masklen ".
         "FROM networks WHERE id != 0 AND ".
         "network >= '$first' AND network <= '$last' AND ".
         "id != '$netinfo{'id'}' AND ".
         "masklen > '$netinfo{'masklen'}' AND ".
         "viewid = '$view' ORDER BY network,masklen");
   $sth->execute();
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_NetworkTreeUpdate: $id: GetSmaller: " . $sth->errstr);
      return(-1);
   }
   for($i = 0; $i < $sth->rows; $i++) {
      my(@row) = $sth->fetchrow_array;
      push(@sort, $row[0]);
      $parents{$row[0]} = $row[1];
      $labels{$row[0]}{'label'} = $row[2];
      $labels{$row[0]}{'network'} = $row[3];
      $labels{$row[0]}{'masklen'} = $row[4];
   }

   my($n0) = {
      'name'          => 'IPv4',
      'network'       => '0',
      'masklen'       => '0',
      'id'            => '0',
      'parent'        => '0',
      'children'      => [],
   };
   %nodes = {};
   foreach(@sort) {
      $id = $_;
      $p = $parents{$_};
      $a = "n$id";
      $b = "n$p";
      $nodes{$a} = {                
         'label'         => $labels{$_}{'label'},
         'network'       => $labels{$_}{'network'},
         'masklen'       => $labels{$_}{'masklen'},
         'id'            => $_,
         'parent'        => $p,
         'children'      => [],
      };

      if($p == 0) {
         API_TreeGrow($n0, $nodes{$a});
      } else {
         if(!$nodes{$b}) {
            API_TreeGrow($n0, $nodes{$a});
         } else {
            API_TreeGrow($nodes{$b}, $nodes{$a});
         }
      }
   }

#    API_TreeDump($n0);
   API_TreeArray($n0, \@array, 2);

   foreach(@array){
      my($sth2) = $db->prepare("UPDATE networks SET parent = '$netinfo{'id'}' WHERE id = '$_';");
      $sth2->execute;
      if(API_DBResCheck(\$sth2) < 1) {
         API_SetGenError("API_NetworkTreeUpdate: $id: ReassignParentSmaller: " . $sth->errstr);
         return(-1);
      }
   }
      # Ok, now were going to see if this network needs to
      # be put under an existing network or should have a 
      # parent = 0

      # Here is out general strategy to do this:
      # 1. First the first and last integer of the largest CIDR net
      # 2. Get the nets currently in the db in the range found in step 1
      #    AND with a masklen that is less then the masklen of the new net
      # 3. Out of the nets above grab the networks with the lowest masklen
      # 4. For each remaining net do this:
      #    a. Get the first and last integer of this net
      #    b. Get the first and last integer of the new net
      #    c. if the range of the new net falls within the range of
      #       the net were looking at we've found a match.  Update the 
      #       new net's parent to the id of the net were looking at

   ($ip, $l) = split(/\//, $netinfo{'label'});
   ($pnet,$len) = IP_GetSupernet($ip, $l, $view);
   $first = IP_DQToInt($pnet);
   $last = IP_DQToInt(IP_LastIP($pnet, $len));

   $sth = $db->prepare("SELECT id,parent,label,network,masklen FROM networks WHERE id != 0 AND network >= '$first' AND network <= '$last' AND id != '$netinfo{'id'}' AND masklen < '$netinfo{'masklen'}' AND viewid = '$view' ORDER BY network,masklen");
   $sth->execute();
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_NetworkTreeUpdate: $id: GetLarger: " . $sth->errstr);
      return(-1);
   } 

   for($i = 0; $i < $sth->rows; $i++) {
      my(@row) = $sth->fetchrow_array;
      push(@sort, $row[0]);
      $parents{$row[0]} = $row[1];
      $labels{$row[0]}{'label'} = $row[2];
      $labels{$row[0]}{'network'} = $row[3];
      $labels{$row[0]}{'masklen'} = $row[4];
   }

   $n0 = {
      'name'          => 'IPv4',
      'network'       => '0',
      'masklen'       => '0',
      'id'            => '0',
      'parent'        => '0',
      'children'      => [],
   };
   %nodes = {};
   foreach(@sort) {
      $id = $_;
      $p = $parents{$_};
      $a = "n$id";
      $b = "n$p";
      $nodes{$a} = {

         'label'         => $labels{$_}{'label'},
         'network'       => $labels{$_}{'network'},
         'masklen'       => $labels{$_}{'masklen'},
         'id'            => $_,
         'parent'        => $p,
         'children'      => [],
      };

      API_TreeGrow($n0, $nodes{$a});
   }

#    API_TreeDump($n0);
   API_TreeArray($n0, \@array);

   ($ip, $len) = split(/\//, $netinfo{'label'});

   $first = IP_DQToInt($ip);
   $last = IP_DQToInt(IP_LastIP($ip, $len));

   my($currmask, $currnet, $currlabel, $currfirst, $currlast, @nets, $largestnet);
   foreach(@array){
      $currmask = $labels{$_}{'masklen'};
      $currnet = $labels{$_}{'network'};
      $currlabel = $labels{$_}{'label'};

      my($cip, $clen) = split(/\//, $currlabel);

      $currfirst = IP_DQToInt($cip);
      $currlast = IP_DQToInt(IP_LastIP($cip, $clen));

      if($first >= $currfirst && $last <= $currlast) {
         push(@nets, $_);
      }
   }

   my($largestmask) = 0;
   foreach(@nets) {
      $currmask = $labels{$_}{'masklen'};
      $currnet = $labels{$_}{'network'};
      $currlabel = $labels{$_}{'label'};
      if($currmask > $largestmask) {
         $largestmask = $currmask;
         $largestnet = $_;
      }            
   }    

   if($largestnet) {
      my($sth) = $db->prepare("UPDATE networks SET parent = '$largestnet' WHERE id = '$netinfo{'id'}';");
      $sth->execute;
      if(API_DBResCheck(\$sth) < 1) {
         API_SetGenError("API_NetworkTreeUpdate: $id: ReassignParentLarger: " . $sth->errstr);
         return(-1);
      }
   }

      # BAM!
   return(1);
}

# API_AddView: Add a view to the database
# Input:
#	%viewinfo 	A hash with the following fields:
#		name			The name of this view
#		descr 		The description of the view
# Return Codes:
#	-1 General Error
#	0	Required Parameter not given
#	>0 The id of the new device
sub API_AddView {
   my($d) = shift || (API_SetError('ViewNoHash') && return(0));

   my($sql) = "INSERT INTO views (name,descr)".
      "VALUES ('${$d}{'name'}','${$d}{'descr'}');";

   my($sth) = $db->prepare($sql);
   $sth->execute();
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_AddView: Add: $sql:" . $sth->errstr);
      return(-1);
   }

   if(API_AddAudit('view','add',"name=${$d}{'name'}") < 1) {
      return(-1);
   }

   $sth = $db->prepare("SELECT id FROM views WHERE name = '${$d}{'name'}' ".
         "AND descr = '${$d}{'descr'}';");
   $sth->execute;
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_AddView: GetID: " . $sth->errstr);
      return(-1);
   }
   my($id) = ($sth->fetchrow_array)[0];
   return($id);
}

# API_UpdView: Update a location in the database
# Input:
#	%viewinfo 	A hash with the following fields:
#		id 			The device id to update
#		name			The name of this device
#		descr 		The description of the device
# Return Codes:
#	-1 General Error
#	0	Required Parameter not given
#	1	Success

sub API_UpdView {
   my($d) = shift || (API_SetError('ViewUpdNoHash') && return(0));

   if(!${$d}{'id'}) {
      API_SetGenError("A view id to update was not specified");
      return(-1);
   }

   my($sql) = "UPDATE views SET name='${$d}{'name'}', descr='${$d}{'descr'}' ".
      "WHERE id = '${$d}{'id'}';";

   my($sth) = $db->prepare($sql);
   $sth->execute();
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_UpdView: Upd: $sql:" . $sth->errstr);
      return(-1);
   }

   if(API_AddAudit('view','update',"id=${$d}{'id'},name=${$d}{'name'}") < 1) {
      return(-1);
   }

   return(1);
}

# API_DelView: Delete a location in the database
# Input:
#	$id			The id of the device to delete
# Returns:
#  -1 General Error
#	0	Required parameter not given
#	1	Successful

sub API_DelView {
   my($id) = shift || (API_SetError('ViewDelNoID') && return(0));
   my(@junk);

   my(@data);
   API_GetNetworkList([qw(id)], \@data, [qw(id)],
      [{ 'field' => 'viewid',
            'type'  => '=',
            'data'  => $id
            }]);
   if(@data) {
      API_SetGenError("The view you are trying to delete is in use by ".
         "at least one network");
      return(-1);
   }

   if(API_GenericDelete($id, 'views', \@junk, 1) < 1) {
      return(-1);
   }

   if(API_AddAudit('view','delete',"id=$id") < 1) {
      return(-1);
   }

   return(1)
}

# API_ViewDropdown: Create a <SELECT> of views
# Input:
#	$str			Reference to the string to put the dropdown in
#	$default 	The default item to be selected
#	$append		The string to append to the <SELECT> tag
# Returns:
#	-1 	General Error
#	0		Required parameter not given
#	1		Successful
sub API_ViewDropdown {
   my($str) = shift || (API_SetError('ViewDropdownNoRef') && return(0));
   my($default) = shift || undef;
   my($append) = shift || undef;

   my($i, @sort, %labels, %parents, @data);
   API_GetViewList([qw(id name)], \@data, [qw(id)]);

   foreach(@data) {
      push(@sort, ${$_}{'id'});
      $parents{${$_}{'id'}} = 0;
      $labels{${$_}{'id'}} = ${$_}{'name'};
   }

   ${$str} = API_TreeCreateDropdown('view', $default, $append, 
         undef, \@sort, \%parents, 
            \%labels);

   return(1);											
}

# API_GetView: Get view information
# Input:
#	$id			The id of the device to get
# Returns:
#	%viewinfo 	A hash with the following fields:
#		id 			The id of this view
#		name			The name of this view
#		descr 		The description of the view
#	null			An error occurred
sub API_GetView {
   my($id) = shift || (API_SetError('ViewNoID') && return());

   my($sql) = "SELECT * FROM views WHERE id = '$id';";
   my($sth) = $db->prepare($sql);
   $sth->execute();
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_GetView: $sql:" . $sth->errstr);
      return();
   }

   if($sth->rows != 1) {
      API_SetGenError("The requested view was not found");
      return();
   }
   my(@row) = $sth->fetchrow_array;

   my(%viewinfo) = (
         'id'        => $row[0],
         'name'      => $row[1],
         'descr'     => $row[2],
      );

   return(%viewinfo);
}

# API_GetViewList: Get a list of view objects and fields
# Input:
#  @fields        The fields to get.  This can be 1 or more of:
#     id             The location ID
#     name           The location name
#     descr          The location description
#  \@ary          An array of hashes containing the requested
#                 information for each location.  The key in 
#                 the hash is the field name and the value is
#                 the data
#  @order         The field ordering (goes after ORDER BY in the SQL)
#  @criteria         A array of hashes of extra criteria to match by.  The 
#                    hash should look like this:
#                       field => <fieldname>
#                       type  => (=,<,>,=<,>=,!,....)
#                       data  => the data to compare
# Returns:
#  -1    General Error
#  0     Required Parameter not given
#  1     Successful
sub API_GetViewList {
   my($fields) = shift || (API_SetError('GetViewListNoField') && return(0));
   my($ary) = shift || (API_SetError('GetViewListNoAry') && return(0));
   my($order) = shift || undef;
   my($criteria) = shift || undef;

   if(API_GenericGetInfo($fields, 'views', $ary, $order, $criteria) < 1) {
      return(-1);
   }
   return(1);
}


# API_GetViewSingle: check to see if only a single view is defined. 
# Input:
#  NONE
# Returns:
#  $id      The id of the view if only one view exists
sub API_GetViewSingle {
   my($sth) = $db->prepare("SELECT id FROM views");
   $sth->execute();
   API_DBResCheck(\$sth, 'viewchecksingle: getcount', 1);

   if($sth->rows == 1) {
      my($id) = ($sth->fetchrow_array)[0];
      return($id);
   }
   return(0);
}

# API_AddUser: Add a user to the database
# Input:
#	%userinfo 	A hash with the following fields:
#		$username		 The username of this user
#     $realname       The user's real name
#     $passwd         The user's uncrypted password
#     \@perms         The permissions that the user CAN execute
# Return Codes:
#	-1 General Error
#	0	Required Parameter not given
#	>0 The id of the new user
sub API_AddUser {
   my($d) = shift || (API_SetError('UserNoHash') && return(0));

   my($username) = lc(${$d}{'username'});
   my($sql) = "SELECT id FROM users WHERE username = '$username';";
   my($sth) = $db->prepare($sql);
   $sth->execute();
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_AddUser: Check: $sql:" . $sth->errstr);
      return(-1);
   }

   if($sth->rows != 0) {
      API_SetGenError("The username is already used");
      return(-1);
   }

   my($ref) = ${$d}{'perms'};
   my($permmask) = API_PermissionsMaskCreate($ref);

   $sql = "INSERT INTO users (username,realname,passwd,permmask,enabled) " .
      "VALUES ('${$d}{'username'}','${$d}{'realname'}','*','$permmask',".
      "'1');";      

   $sth = $db->prepare($sql);
   $sth->execute();
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_AddUser: Add: $sql:" . $sth->errstr);
      return(-1);
   }

   if(API_AddAudit('user','add',"username=${$d}{'username'}") < 1) {
      return(-1);
   }

   $sth = $db->prepare("SELECT id FROM users WHERE username = ".
         "'${$d}{'username'}';");
   $sth->execute;
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_AddUser: GetID: " . $sth->errstr);
      return(-1);
   }
   my($id) = ($sth->fetchrow_array)[0];

   if(API_UserChangePass($id, ${$d}{'passwd'}) < 1) {
      return(-1);
   }

   return($id);
}

# API_UserChangePass: Change the password for a user
# Input: 
#  $id      The id of the user to change
#  $passwd  The plaintext password
# Return Codes:
#	-1 General Error
#	0	Required Parameter not given
#	1  Successful
sub API_UserChangePass {
   my($id) = shift;
   my($passwd) = shift || (API_SetError('UserChangePassNoPass') && return(0));  


   if(!$id && $id != 0) {
      API_SetError('UserChangePassNoID');
      return(0);
   }

   my($salt) = join('', ('.', '/', 0..9, 'A'..'Z', 'a'..'z')[rand 64, rand 64]);
   my($cp) = crypt($passwd, $salt);
   my($sql) = "UPDATE users SET passwd = '$cp' WHERE id = '$id';";
   Debug($sql,5);
   my($sth) = $db->prepare($sql);
   $sth->execute();
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_UserChangePass: $sql:" . $sth->errstr);
      return(-1);
   }
   return(1);
} 

# API_UpdUser: Update a user in the database
# Input:
#	%userinfo 	A hash with the following fields:
#     $id            The user id to update
#     $realname      The user's real name
#     $enabled       Boolean value 0=disabled 1=enabled
#     \@perms        The permissions that the user CAN execute
# Return Codes:
#	-1 General Error
#	0	Required Parameter not given
#	1  Success
sub API_UpdUser {
   my($d) = shift || (API_SetError('UserUpdNoHash') && return(0));

   if(!${$d}{'id'}) {
      API_SetGenError("A user id to update was not specified");
      return(-1);
   }

   my($ref) = ${$d}{'perms'};
   my($permmask) = API_PermissionsMaskCreate($ref);

   my($sql) = "UPDATE users SET realname='${$d}{'realname'}',".
      "enabled='${$d}{'enabled'}',permmask='$permmask' ".
      "WHERE id = '${$d}{'id'}';";

   my($sth) = $db->prepare($sql);
   $sth->execute();
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_UpdUser: Upd: $sql:" . $sth->errstr);
      return(-1);
   }

   if(API_AddAudit('user','update',"id=${$d}{'id'}") < 1) {
      return(-1);
   }

   return(1);
}

# API_DelUser: Delete a user in the database
# Input:
#	$id			The id of the user to delete
# Returns:
#  -1 General Error
#	0	Required parameter not given
#	1	Successful

sub API_DelUser {
   my($id) = shift || (API_SetError('UserDelNoID') && return(0));
   my(@junk);

   my(@tables) = qw(networks locations devices);
   my($sth, $sql);
   foreach(@tables) {
      $sql = "UPDATE $_ SET assignuser='0' WHERE assignuser = '$id'";
      $sth = $db->prepare($sql);
      $sth->execute;
      if(API_DBResCheck(\$sth) < 1) {
         API_SetGenError("API_DelUser: ChangeAssignID: $sql:" . $sth->errstr);
         return(-1);
      }

      $sql = "UPDATE $_ SET updateuser='0' WHERE updateuser = '$id'";
      $db->prepare($sql);
      $sth->execute;
      if(API_DBResCheck(\$sth) < 1) {
         API_SetGenError("API_DelUser: ChangeUpdateID: $sql:" . $sth->errstr);
         return(-1);
      }
   }

   if(API_GenericDelete($id, 'users', \@junk, 1) < 1) {
      return(-1);
   }

   if(API_AddAudit('user','delete',"id=$id") < 1) {
      return(-1);
   }

   return(1)
}

# API_UserDropdown: Create a <SELECT> of users
# Input:
#	$str			Reference to the string to put the dropdown in
#	$default 	The default item to be selected
#  $top        The item to put at the top of the list
#	$append		The string to append to the <SELECT> tag
# Returns:
#	-1 	General Error
#	0		Required parameter not given
#	1		Successful
sub API_UserDropdown {
   my($str) = shift || (API_SetError('UserDropdownNoRef') && return(0));
   my($default) = shift || undef;
   my($top) = shift || undef;
   my($append) = shift || undef;

   my($i, @sort, %labels, %parents, @data);
   API_GetUserList([qw(username)], \@data, [qw(username)]);

   if($top) {
      @sort = ($top);
   }

   foreach(@data) {
      push(@sort, ${$_}{'username'});
   }

   ${$str} = $cgi->popup_menu(
         -name		=>	'user',
         -values		=>	\@sort,
         -onChange 	=> $append,
         -default 	=> $default
      );                                 

   return(1);											
}

# API_GetUser: Get user information
# Input:
#	$id			The id of the user to get
# Returns:
#	%userinfo 	A hash with the following fields:
#	  id 			 The id of this user
#     username       The username of this user
#     realname       The user's real name
#     enabled        The user's status
#     permmask       The users permissions mask
#	null			An error occurred
sub API_GetUser {
   my($id) = shift || (API_SetError('UserNoID') && return());

   my($sql) = "SELECT id,username,realname,enabled,permmask ".
      "FROM users WHERE id = '$id';";
   my($sth) = $db->prepare($sql);
   $sth->execute();
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_GetUser: $sql:" . $sth->errstr);
      return();
   }

   if($sth->rows != 1) {
      API_SetGenError("The requested user was not found");
      return();
   }
   my(@row) = $sth->fetchrow_array;

   my(%userinfo) = (
         'id'          => $row[0],
         'username'    => $row[1],
         'realname'    => $row[2],
         'enabled'     => $row[3],
         'permmask'    => $row[4],
      );

   return(%userinfo);
}

# API_GetUserList: Get a list of user objects and fields
# Input:
#  @fields        The fields to get.  This can be 1 or more of:
#		id 			   The id of this user
#     username       The username of this user
#     realname       The user's real name
#     enabled        The user's status
#     permmask       The users permissions mask
#  \@ary          An array of hashes containing the requested
#                 information for each location.  The key in 
#                 the hash is the field name and the value is
#                 the data
#  @order         The field ordering (goes after ORDER BY in the SQL)
#  @criteria         A array of hashes of extra criteria to match by.  The 
#                    hash should look like this:
#                       field => <fieldname>
#                       type  => (=,<,>,=<,>=,!,....)
#                       data  => the data to compare
# Returns:
#  -1    General Error
#  0     Required Parameter not given
#  1     Successful
sub API_GetUserList {
   my($fields) = shift || (API_SetError('GetUserListNoField') && return(0));
   my($ary) = shift || (API_SetError('GetUserListNoAry') && return(0));
   my($order) = shift || undef;
   my($criteria) = shift || undef;

   if(API_GenericGetInfo($fields, 'users', $ary, $order, $criteria) < 1) {
      return(-1);
   }
   return(1);
}

# API_PermissionsRegisterPerm: This function is called to register a permissions 
#                              bit with it's associated user call. 
# Input:
#  $bit     The bit were trying to set (0-127)
#  $def     Whether this bit defaults to being on
#  $call    The call from the user
#  $descr   A short description of the permissions function
#  $module  The name of the module using this permissions bit
# Returns:
#  -1    General Error
#  0     Required parameter not given
#  1     Successful
sub API_PermissionsRegisterPerm {
   my($bit) = shift;
   my($def) = shift || 0;
   my($call) = shift || (API_SetError('PermRegNoCall') && return(0));
   my($descr) = shift || (API_SetError('PermRegNoDescr') && return(0));
   my($module) = shift || (API_SetError('PermRegNoModule') && return(0));

   Debug("starting (bit=$bit,def=$def,call=$call,descr=$descr,module=$module)", 10);

   if(!$bit && $bit != 0) {
      API_SetError('PermRegNoBit');
      return(0);
   }

   if($bit < 0 || $bit > 127) {
      Debug("bit value is bad, exiting", 10);
      API_SetGenError("The bit value specified is bad");
      return(-1);
   }

   my(%t) = API_PermissionsGetInfo($bit);
   if(%t) {
      Debug("permission bit already registered, exiting", 10);
      API_SetGenError("The permission is already registered");
      return(-1);
   }

   Debug("setting permissions info", 10);
   if(API_PermissionsSetInfo($bit, $def, $call, $descr, $module) < 1) {
      return(-1);
   }
}


# API_PermissionsGetInfo: Get the info registered for a particular perm bit
# Input:
#  $bit     The bit number you want to get info for (0-127)
# Returns:
#  %info 
#     bit      The bit number
#     def      Whether this bit defaults to being on
#     call     The user call for this bit
#     descr    The description field for this bit
#     module   The module that registered this bit
#  null        Error
sub API_PermissionsGetInfo {
   my($bit) = shift;

   if(!$bit && $bit != 0) {
      API_SetError('PermGetNoBit');
      return();
   }

   Debug("starting bit=$bit", 10);

   if($bit < 0 || $bit > 127) {
      Debug("bit value is bad, exiting", 10);
      API_SetGenError("The bit value specified is bad");
      return();
   }

   if(!$NorthStar::_permissionshash{$bit}) {
      Debug("bit not registered, returning", 10);
      API_SetGenError("The permissions bit is not registered");
      return();
   } else {
      Debug("bit registered, returning hash", 10);
      my(%r) = (
            'bit'   => $bit,
            'def'   => $NorthStar::_permissionshash{$bit}{'def'},
            'call'  => $NorthStar::_permissionshash{$bit}{'call'},
            'descr' => $NorthStar::_permissionshash{$bit}{'descr'},
            'module'=> $NorthStar::_permissionshash{$bit}{'module'}
         );
      return(%r);
   }
}


# API_PermissionsSetInfo: Set the information for a particular perm bit
# Input:
#  $bit      The bit number you want to set (0-127)
#  $def      Whether this bit defaults to being on
#  $call     The user call for this bit
#  $descr    The description field for this bit
#  $module   The module that registered this bit
# Returns:
#  0     Required parameter not given
#  1     Successful
sub API_PermissionsSetInfo {
   my($bit) = shift;
   my($def) = shift || 0;
   my($call) = shift || (API_SetError('PermSetNoCall') && return(0));
   my($descr) = shift || (API_SetError('PermSetNoDescr') && return(0));
   my($module) = shift || (API_SetError('PermSetNoModule') && return(0));
   Debug("starting (bit=$bit,def=$def,call=$call,descr=$descr,module=$module", 10);

   if(!$bit && $bit != 0) {
      API_SetError('PermSetNoBit');
      return(0);
   }

   $NorthStar::_permissionshash{$bit}{'bit'} = $bit;
   $NorthStar::_permissionshash{$bit}{'def'} = $def;
   $NorthStar::_permissionshash{$bit}{'call'} = $call;
   $NorthStar::_permissionshash{$bit}{'descr'} = $descr;
   $NorthStar::_permissionshash{$bit}{'module'} = $module;

   $NorthStar::_permissionscallmap{$call} = $bit;
   return(1);

}


# API_PermissionsSetPerms:  Set the current permissions for a user given
#                           their permissions mask
# Input:
#  $mask    A permissions mask string (128 chars, 1=true, 0=false, X=unused)
# Returns:
#  0     Required parameter not given
#  1     Successful
sub API_PermissionsSetPerms {
   my($mask) = shift || (API_SetError('PermUserSetNoMask') && return(0));

   Debug("starting mask=$mask", 10);

   my(@b) = split(//, $mask);
   my($i) = 0;
   for(@b) {
      if($b[$i] == '1') {
         $NorthStar::_permissionsuser{$i} = 1;
      } else {
         $NorthStar::_permissionsuser{$i} = 0;
      }
      $i++;
   }
   return(1);
}

# API_PermissionsCheckPerm:  Return whether the user can execute the given 
#                            function
# Input:
#  $call    The call name
# Returns:
#  0        The user CANNOT execute this function
#  1        The user CAN execute this function
sub API_PermissionsCheckPerm {
   my($call) = shift;

   Debug("starting call=$call", 7);

   my($bit) = $NorthStar::_permissionscallmap{$call};

   Debug("bit=$bit");

   my(@defaultcalls) = qw(shmenu shtmpl shmain);
   foreach(@defaultcalls) {
      if($call eq $_) {
         return(1);
      }
   }

   if($bit >= 0) {
      Debug("userpermbit=$NorthStar::_permissionsuser{$bit}", 10);
      if($NorthStar::_permissionsuser{$bit} == 1) {
         return(1);
      } else { 
         return(0);
      }
   }
}

# API_PermissionsCreateCheckboxes: This function creates HTML checkboxes for 
#                                  each permission in the system.  Permissions 
#                                  with def=1 are automatically checked.
# Input:
#  \$str    A reference to the string to put the checkboxes in
#  \@ary    An array of all the current permissions (checked by default)
# Returns:
#  -1       General Error
#  0        Required parameter not given
#  1        Successful
sub API_PermissionsCreateCheckboxes {
   my($str) = shift || (API_SetError('PermCreateChecksNoStr') && return(0));
   my($ary) = shift || undef;
   my(%g, @h, @d);
   for(0..127) {
      if($NorthStar::_permissionshash{$_}) {
         $g{$_} = $NorthStar::_permissionshash{$_}{'descr'};
         push(@h, $_);
         if($NorthStar::_permissionshash{$_}{'def'} == 1) {
            push(@d, $_);
         }
      }
   }

   if(!$ary) {
      ${$str} = $cgi->checkbox_group(
            -name	=> 'permissions',
            -values	=> \@h,
            -columns=> 5,
            -default=> \@d,
            -labels	=> \%g
         );
   } else {
      ${$str} = $cgi->checkbox_group(
            -name	=> 'permissions',
            -values	=> \@h,
            -columns=> 5,
            -default=> \@$ary,
            -labels	=> \%g
         );
   }
   return(1);
}

# API_PermissionsCreateTable: This function creates a HTML table for 
#                             each permission in the system.  Permissions 
#                             that are enabled have a green background.
# Input:
#  $mask    The user permissions mask
#  \$str    A reference to the string to put the table in
# Returns:
#  -1       General Error
#  0        Required parameter not given
#  1        Successful
sub API_PermissionsCreateTable {
   my($mask) = shift || (API_SetError('PermCreateTableNoMask') && return(0));
   my($str) = shift || (API_SetError('PermCreateTableNoStr') && return(0));
   my(@permorder);
   my(@pm) = split(//, $mask);
   my($i, $j) = 0;

   my(%permtable, %permtablex);
   foreach(@pm) {
      my($d) = $NorthStar::_permissionshash{$i}{'descr'};
      if($NorthStar::_permissionshash{$i}{'descr'}) {
         $permtable{$i}{'value'} = $pm[$i];
         $permtable{$i}{'description'} = $NorthStar::_permissionshash{$i}{'descr'};
         push(@permorder, $i);
      }
      $i++;
   }

   my($c) = @permorder;
   my($cols, $ccol, $crow);

   $cols = 5;
   $ccol = 1;
   $crow = 1;
   foreach(@permorder) {
      if($ccol > $cols) {
         $ccol = 1;
         $crow++;
      }
      if($permtable{$_}{value} == 1) {
         $permtablex{$crow}{$ccol} =  "<TD BGCOLOR=GREEN>$permtable{$_}{description}</TD>";
      } else {
         $permtablex{$crow}{$ccol} =  "<TD BGCOLOR=RED>$permtable{$_}{description}</TD>";
      }
      $ccol++;
   }

   ${$str} = "<TABLE BORDER=1 CELLPADDING=3 CELLSPACING=0>";
   for($i = 1 ; $i <= keys(%permtablex); $i++) {
      my($h) = $permtablex{$i};
      ${$str} .= "<tr>\n";
      for($j = 1; $j <= keys(%$h); $j++) {
         ${$str} .= "$permtablex{$i}{$j}\n";
      }
      ${$str} .= "</tr>\n";
   }
   ${$str} .= "</TABLE>";
}

# API_PermissionsMaskCreate:  Create a permissions mask
# Input:
#  \@ary    A reference to an array that contains only permissions that CAN
#           be executed by the current user
# Returns:
#  $mask    The permissions mask
#  null     Error
sub API_PermissionsMaskCreate {
   my($ary) = shift || (API_SetError('PermMaskCreate') && return());
   my($mask, %perms);

   foreach(@{$ary}) {
      $perms{$_} = 1;
   }

   for(0..127) {
      if($perms{$_}) {
         $mask .= "1";
      } elsif (!$NorthStar::_permissionshash{$_}) {
         $mask .= "X";
      } else {
         $mask .= "0";
      }
   }
   return($mask);
}

# API_PermissionsCreateBitArray:  Create an array of bits that are set to 1 given
#                                a mask
# Input:
#  $mask    The permissions mask
# Returns:
#  @ary     An array that contains all the bits that are set to 1
#  null     Error
sub API_PermissionsCreateBitArray {
   my($mask) = shift || (API_SetError('PermCreateBitAry') && return());
   my(@ret);
   my(@m) = split(//, $mask);
   for(0..127) {
      if($m[$_] == 1) {
         push(@ret, $_);
      }
   }
   return(@ret);
}


# API_TreeGrow: Add the given node to the given parent
# Input:
#  $parent     A reference to the parent node
#  $node       A reference to the node to add
# Returns:
#  0     A required parameter was not given
#  1     Successful
sub API_TreeGrow {
   my($parent) = shift || (API_SetError('TreeGrowNoParent') && return(0));
   my($node) = shift || (API_SetError('TreeGrowNoNode') && return(0));
   push(@{$parent->{'children'}}, $node);
   return(1);
}


# API_TreeDump: Dump the given tree and to the given depth to STDERR
# Input:
#  $node       The root node of the tree
#  $depth      The depth to dump to
# Returns:
#  0     Required parameter not given
#  1     Successful
sub API_TreeDump {
   my($node) = shift || (API_SetError('TreeDumpNoRoot') && return(0));
   my($depth) = shift || 0;

   print STDERR "  " x $depth;
   foreach(keys(%{$node})) {
      print STDERR $_, "=", $node->{$_}, " ";
   }
   print STDERR ")\n";
   foreach(@{$node->{'children'}}) {
      API_TreeDump($_, $depth + 1);
   }
}

# API_TreeDumpTable:  Dump an html table of a tree
# Input:
#  $html       The html template to use
#  $node       The root node of the tree
#  $path       The path to the node to be automatically opened
#  $maxdepth   The maximum depth the tree should be opened to
#  $depth      The current depth (used internally)
#  $cmdone     The string to append to the r= portion of the view link
#  $cmdtwo     The string to append to the r= portion of the folder icons
#  $background The background color id (used internally)
#  $lastbit    Used internally
#  $pitems     The hash of tags to be passed to API_OutputPage
# Returns:
#  NONE
sub API_TreeDumpTable {
   my($html) = shift;
   my($n) = shift;
   my($id) = shift;
   my($maxdepth) = shift || 0;
   my($depth) = shift || 0;
   my($cmdone) = shift || "";
   my($cmdtwo) = shift || "";
   $bg = shift || 1;
   my($lastbit) = shift;
   my($pageitems) = shift;

   my($pd, $x, %idhash, $t);

   if(!$bg || $bg == 2) {
      $bgcolor = "yellow";
      $bg = 1;
   } else { 
      $bgcolor = "white";
      $bg = 2;
   }

   foreach(split(/\:/, $id)) {
      $idhash{$_} = 1;
   }

   if($n->{'id'} != 0) {
      push(@idpath, $n->{'id'});
      $path = join(':', @idpath);
      my($action);
      my($url) = ConfigGetValue('Global','ScriptURL');
      my($turl) = ConfigGetValue('Global','TemplateURL');
      my($imgend) = 'width="18" align="left" border="0" vspace="0" hspace="0"';

      $pd = @idpath;

      if(@{$n->{'children'}} == 0) {

         if($lastbit) { 
            $x = "pixel";
         } else {
            $x = "bar";
         }

         $action .= "<A HREF=\"$url?r=$cmdone&id=" . $n->{'id'} . "\" target=\"bottom\">";

         if($n->{'last'} == 1 && !$lastbit) {
#print "in 1a; last=", $n->{'last'}, " last2=", $lastbit2, " id=", $n->{'id'}, "<br>";
            $action .= "<IMG SRC=\"$turl/images/menu_bar.gif\" $imgend>" x ($depth - 1);
            $action .= "<IMG SRC=\"$turl/images/menu_corner.gif\" $imgend>";
         } elsif($n->{'last'} == 2 && !$lastbit) {
#print "in 1b; last=", $n->{'last'}, " last2=", $lastbit2, " id=", $n->{'id'}, "<br>";
            $action .= "<IMG SRC=\"$turl/images/menu_pixel.gif\" $imgend>" x ($depth - 1);
            $action .= "<IMG SRC=\"$turl/images/menu_corner.gif\" $imgend>";
         } elsif($n->{'last'} == 0 && $lastbit == 1) {
#print "in 1c; last=", $n->{'last'}, " last2=", $lastbit2, " id=", $n->{'id'}, "<br>";
            $action .= "<IMG SRC=\"$turl/images/menu_bar.gif\" $imgend>";
            $action .= "<IMG SRC=\"$turl/images/menu_pixel.gif\" $imgend>" x ($depth - 2);
            $action .= "<IMG SRC=\"$turl/images/menu_tee.gif\" $imgend>";
         } elsif($n->{'last'} == 0 && $lastbit == 2) {
#print "in 1d; last=", $n->{'last'}, " last2=", $lastbit2, " id=", $n->{'id'}, "<br>";
            $action .= "<IMG SRC=\"$turl/images/menu_pixel.gif\" $imgend>";
            $action .= "<IMG SRC=\"$turl/images/menu_bar.gif\" $imgend>" x ($depth - 2);
            $action .= "<IMG SRC=\"$turl/images/menu_tee.gif\" $imgend>";
         } elsif($n->{'last'} == 1 && $lastbit == 1) {
#print "in 1e; last=", $n->{'last'}, " last2=", $lastbit2, " id=", $n->{'id'}, "<br>";
            $action .= "<IMG SRC=\"$turl/images/menu_bar.gif\" $imgend>" x ($depth - 2);
            $action .= "<IMG SRC=\"$turl/images/menu_pixel.gif\" $imgend>";
            $action .= "<IMG SRC=\"$turl/images/menu_corner.gif\" $imgend>";
         } elsif($n->{'last'} == 1 && $lastbit == 2) {
#print "in 1f; last=", $n->{'last'}, " last2=", $lastbit2, " id=", $n->{'id'}, "<br>";
            $action .= "<IMG SRC=\"$turl/images/menu_pixel.gif\" $imgend>";
            $action .= "<IMG SRC=\"$turl/images/menu_bar.gif\" $imgend>" x ($depth - 2);
            $action .= "<IMG SRC=\"$turl/images/menu_corner.gif\" $imgend>";
         } elsif($n->{'last'} == 0 && !$lastbit && $n->{'parent'} == 0) {
#print "in 1g; last=", $n->{'last'}, " last2=", $lastbit2, " id=", $n->{'id'}, "<br>";
            $action .= "<IMG SRC=\"$turl/images/menu_pixel.gif\" $imgend>" x ($depth - 1);
            $action .= "<IMG SRC=\"$turl/images/menu_tee.gif\" $imgend>";
         } else {
#print "in 1h; last=", $n->{'last'}, " last2=", $lastbit2, " id=", $n->{'id'}, "<br>";
            $action .= "<IMG SRC=\"$turl/images/menu_bar.gif\" $imgend>" x ($depth - 1);
            $action .= "<IMG SRC=\"$turl/images/menu_tee.gif\" $imgend>";
         }                

         $action .= "<IMG SRC=\"$turl/images/menu_link_local.gif\" $imgend></A>";

      } elsif($idhash{$n->{'id'}}) {
         if($lastbit) { 
            $x = "pixel";
         } else {
            $x = "bar";
         }

         my(@temp) = @idpath;
         pop(@temp);
         $path = join(':', @temp);

         $action .= "<A HREF=\"$url?r=$cmdtwo&depth=$depth&path=$path\">";

         if($n->{'last'} == 1 && !$lastbit) {
#print "in 2a; last=", $n->{'last'}, " last2=", $lastbit2, " id=", $n->{'id'}, "<br>";
            $action .= "<IMG SRC=\"$turl/images/menu_bar.gif\" $imgend>" x ($depth - 1);
            $action .= "<IMG SRC=\"$turl/images/menu_corner_minus.gif\" $imgend>";
         } elsif($n->{'last'} == 2 && !$lastbit) {
#print "in 2b; last=", $n->{'last'}, " last2=", $lastbit2, " id=", $n->{'id'}, "<br>";
            $action .= "<IMG SRC=\"$turl/images/menu_pixel.gif\" $imgend>" x ($depth - 1);
            $action .= "<IMG SRC=\"$turl/images/menu_corner_minus.gif\" $imgend>";
         } elsif($n->{'last'} == 0 && $lastbit == 1) {
#print "in 2c; last=", $n->{'last'}, " last2=", $lastbit2, " id=", $n->{'id'}, "<br>";
            $action .= "<IMG SRC=\"$turl/images/menu_bar.gif\" $imgend>";
            $action .= "<IMG SRC=\"$turl/images/menu_pixel.gif\" $imgend>" x ($depth - 2);
            $action .= "<IMG SRC=\"$turl/images/menu_tee_minus.gif\" $imgend>";
         } elsif($n->{'last'} == 0 && $lastbit == 2) {
#print "in 2d; last=", $n->{'last'}, " last2=", $lastbit2, " id=", $n->{'id'}, "<br>";
            $action .= "<IMG SRC=\"$turl/images/menu_pixel.gif\" $imgend>";
            $action .= "<IMG SRC=\"$turl/images/menu_bar.gif\" $imgend>" x ($depth - 2);
            $action .= "<IMG SRC=\"$turl/images/menu_tee_minus.gif\" $imgend>";
         } elsif($n->{'last'} == 1 && $lastbit == 1) {
#print "in 2e; last=", $n->{'last'}, " last2=", $lastbit2, " id=", $n->{'id'}, "<br>";
            $action .= "<IMG SRC=\"$turl/images/menu_bar.gif\" $imgend>";
            $action .= "<IMG SRC=\"$turl/images/menu_pixel.gif\" $imgend>" x ($depth - 2);
            $action .= "<IMG SRC=\"$turl/images/menu_corner_minus.gif\" $imgend>";
         } elsif($n->{'last'} == 1 && $lastbit == 2) {
#print "in 2f; last=", $n->{'last'}, " last2=", $lastbit2, " id=", $n->{'id'}, "<br>";
            $action .= "<IMG SRC=\"$turl/images/menu_pixel.gif\" $imgend>";
            $action .= "<IMG SRC=\"$turl/images/menu_bar.gif\" $imgend>" x ($depth - 2);
            $action .= "<IMG SRC=\"$turl/images/menu_corner_minus.gif\" $imgend>";
         } elsif($n->{'last'} == 0 && !$lastbit && $n->{'parent'} == 0) {
#print "in 2g; last=", $n->{'last'}, " last2=", $lastbit2, " id=", $n->{'id'}, "<br>";
            $action .= "<IMG SRC=\"$turl/images/menu_pixel.gif\" $imgend>" x ($depth - 1);
            $action .= "<IMG SRC=\"$turl/images/menu_tee_minus.gif\" $imgend>";
         } else {
#print "in 2h; last=", $n->{'last'}, " last2=", $lastbit2, " id=", $n->{'id'}, "<br>";
            $action .= "<IMG SRC=\"$turl/images/menu_bar.gif\" $imgend>" x ($depth - 1);
            $action .= "<IMG SRC=\"$turl/images/menu_tee_minus.gif\" $imgend>";
         }                

         $action .= "<IMG SRC=\"$turl/images/menu_folder_open.gif\" $imgend></A>";

      } else {
         if($lastbit) { 
            $x = "pixel";
         } else {
            $x = "bar";
         }

         $action .= "<A HREF=\"$url?r=$cmdtwo&depth=$depth&path=$path\">";

         if($n->{'last'} == 1 && !$lastbit) {
#print "in 3a; last=", $n->{'last'}, " last2=", $lastbit2, " id=", $n->{'id'}, "<br>";
            $action .= "<IMG SRC=\"$turl/images/menu_bar.gif\" $imgend>" x ($depth - 1);
            $action .= "<IMG SRC=\"$turl/images/menu_corner_plus.gif\" $imgend>";
         } elsif($n->{'last'} == 2 && !$lastbit) {
#print "in 3b; last=", $n->{'last'}, " last2=", $lastbit2, " id=", $n->{'id'}, "<br>";
            $action .= "<IMG SRC=\"$turl/images/menu_pixel.gif\" $imgend>" x ($depth - 1);
            $action .= "<IMG SRC=\"$turl/images/menu_corner_plus.gif\" $imgend>";
         } elsif($n->{'last'} == 0 && $lastbit == 1) {
#print "in 3c; last=", $n->{'last'}, " last2=", $lastbit2, " id=", $n->{'id'}, "<br>";
            $action .= "<IMG SRC=\"$turl/images/menu_bar.gif\" $imgend>";
            $action .= "<IMG SRC=\"$turl/images/menu_pixel.gif\" $imgend>" x ($depth - 2);
            $action .= "<IMG SRC=\"$turl/images/menu_tee_plus.gif\" $imgend>";
         } elsif($n->{'last'} == 0 && $lastbit == 2) {
#print "in 3d; last=", $n->{'last'}, " last2=", $lastbit2, " id=", $n->{'id'}, "<br>";
            $action .= "<IMG SRC=\"$turl/images/menu_pixel.gif\" $imgend>";
            $action .= "<IMG SRC=\"$turl/images/menu_bar.gif\" $imgend>" x ($depth - 2);
            $action .= "<IMG SRC=\"$turl/images/menu_tee_plus.gif\" $imgend>";
         } elsif($n->{'last'} == 1 && $lastbit == 2) {
#print "in 3e; last=", $n->{'last'}, " last2=", $lastbit2, " id=", $n->{'id'}, "<br>";
            $action .= "<IMG SRC=\"$turl/images/menu_pixel.gif\" $imgend>" x ($depth - 1);
            $action .= "<IMG SRC=\"$turl/images/menu_bar.gif\" $imgend>";
            $action .= "<IMG SRC=\"$turl/images/menu_corner_plus.gif\" $imgend>";
         } elsif($n->{'last'} == 1 && $lastbit == 1) {
#print "in 3f; last=", $n->{'last'}, " last2=", $lastbit2, " id=", $n->{'id'}, "<br>";
            $action .= "<IMG SRC=\"$turl/images/menu_bar.gif\" $imgend>";
            $action .= "<IMG SRC=\"$turl/images/menu_pixel.gif\" $imgend>" x ($depth - 2);
            $action .= "<IMG SRC=\"$turl/images/menu_corner_plus.gif\" $imgend>";
#                } elsif($n->{'last'} == 1 && $lastbit == 2) {
#print "in 3g; last=", $n->{'last'}, " last2=", $lastbit2, " id=", $n->{'id'}, "<br>";
#                    $action .= "<IMG SRC=\"$turl/images/menu_pixel.gif\" $imgend>" x ($depth - 1);
#                    $action .= "<IMG SRC=\"$turl/images/menu_corner_plus.gif\" $imgend>";
         } elsif($n->{'last'} == 0 && !$lastbit && $n->{'parent'} == 0) {
#print "in 3h; last=", $n->{'last'}, " last2=", $lastbit2, " id=", $n->{'id'}, "<br>";
            $action .= "<IMG SRC=\"$turl/images/menu_pixel.gif\" $imgend>" x ($depth - 1);
            $action .= "<IMG SRC=\"$turl/images/menu_tee_plus.gif\" $imgend>";
         } else {
#print "in 3i; last=", $n->{'last'}, " last2=", $lastbit2, " id=", $n->{'id'}, "<br>";
            $action .= "<IMG SRC=\"$turl/images/menu_bar.gif\" $imgend>" x ($depth - 1);
            $action .= "<IMG SRC=\"$turl/images/menu_tee_plus.gif\" $imgend>";
         }                

         $action .= "<IMG SRC=\"$turl/images/menu_folder_closed.gif\" $imgend></A>";

      }

      while(my($k,$v) = each %{$n}) {
         $pitems{$k} = $v;
      }

      while(my($k,$v) = each %{$pageitems}) {
         if(!$pitems{$k}) {
            $pitems{$k} = $v;
         }
      }

      $pitems{'tableback'} = $bgcolor;
      $pitems{'indent'} = $action;
      if($n->{'html'}) {
         API_OutputPage($n->{'html'}, \%pitems);
      } else {
         API_OutputPage($html, \%pitems);
      }
      pop(@idpath);
   }
   push(@idpath, $n->{'id'});

   my($nl);
   if( ( $depth <= $maxdepth && $idhash{$n->{'id'}} ) || $n->{'id'} == 0) {
      if($lastbit == 2) {
         $nl = 2;
      } else {
         $nl = $n->{'last'};
      }

      for(@{$n->{'children'}}) {
         API_TreeDumpTable($html, $_, $id, $maxdepth, $depth + 1, $cmdone, $cmdtwo, $bg, $nl, $pageitems);
      }
   }
   pop(@idpath);
}

# API_TreeArray: Return an array of the 'id' fields of every node in a tree
# Input:
#  $node       The root node of the tree
#  $ref        The reference to the array to add the id's to
#  $maxdepth   The maximum depth to go to
#  $depth      Used Internally
# Returns:
#  0     A required parameter was not given
#  1     Successful
sub API_TreeArray {
   my($n) = shift || (API_SetError('TreeArrayNoRoot') && return(0));
   my($ref) = shift || (API_SetError('TreeArrayNoRef') && return(0));
   my($maxdepth) = shift || -1;
   my($depth) = shift || 0;

   if($depth != $maxdepth) {
      if($n->{'id'} != 0) {
         push(@{$ref}, $n->{'id'});
      }
      foreach(@{$n->{'children'}}) {
         API_TreeArray($_, $ref, $maxdepth, $depth + 1);
      }
   }
}

# API_TreeHash: Return a hash of the id,name pair for each node in the tree
# Input:
#  $node       The root node of the tree
#  $ref        The reference to the hash to add items to
#  $indent     The string to use to indent items
#  $pre        The string to put just before the name field
#  $post       The string to put just after the name field
#  $depth      Used internally
# Returns:
#  0     A required parameter was not given
#  1     Successful
sub API_TreeHash {
   my($n) = shift || (API_SetError('TreeHashNoRoot') && return(0));
   my($ref) = shift || (API_SetError('TreeHashNoRef') && return(0));
   my($indent) = shift || '';
   my($pre) = shift || ''; 
   my($post) = shift || '';
   my($depth) = shift || 0;
   if($n->{'id'} != 0) {
      ${$ref}{$n->{'id'}} = ($indent x ($depth - 1)). $pre . 
         $n->{'name'} . $post;
   }
   foreach(@{$n->{'children'}}) {
      API_TreeHash($_, $ref, $indent, $pre, $post, $depth + 1);
   }
}

# API_TreeCreateDropdown:  Create a <SELECT> based on a tree
# Input:
#  $name       The name of the <SELECT> (<SELECT NAME="xxx">)
#  $default    The default item id to be selected
#  $onchange   The string to add to the onChange tag
#  $topitem    The item to put at the very top of the list
#  $idarray    A reference to the array that holds the id's of each item
#  $parents    A reference to the hash with the key being the id and the value
#              being the id of the parent item
#  $labels     A refence to a hash containing the labels for each id
#  $defaultitem  The string to show with an id of 0
# Returns:
#  null    Required parameter not given
#  $str    Successful 
sub API_TreeCreateDropdown {
   my($name) = shift || (API_SetError('TreeDropdownNoName') && return());
   my($default) = shift || undef;
   my($onchange) = shift || undef;
   my($topitem) = shift || undef;
   my($sort) = shift || (API_SetError('TreeDropdownNoIDs') && return());
   my($parents) = shift || (API_SetError('TreeDropdownNoParents') && return());
   my($labels) = shift || (API_SetError('TreeDropdownNoLabels') && return());
   my($defaultitem) = shift || undef;
   my(@dropsort, %droplabels, $str);

   my($n0) = {
      'name'          => 'Root',
      'id'            => '0',
      'children'      => [],
   };

   foreach(@{$sort}) {
      $id = $_;
      $p = ${$parents}{$_};
      $a = "n$id";
      $b = "n$p";
      ${$a} = {
         'name'          => ${$labels}{$_},
         'id'            => $_,
         'children'      => [],
      };
      if($p == 0) {
         API_TreeGrow($n0, ${$a});
      } else {
         if(${$b}) {
            API_TreeGrow(${$b}, ${$a});
         } else {
            API_TreeGrow($n0, ${$a});
         }
      }
   }

   if($topitem) {
      @dropsort = ('');
      $droplabels{''} = $topitem;
   }

   if($defaultitem) {
      push(@dropsort,'0');
      $droplabels{'0'} = $defaultitem;
   }
   API_TreeArray($n0, \@dropsort);
   API_TreeHash($n0, \%droplabels, '&nbsp;&nbsp;&nbsp;', '', '');
   foreach(@dropsort) {
      Debug("dropsort=$_");
   }
   while(($k,$v) = each %droplabels) {
      Debug("label k=$k v=$v");
   }

   $cgi->autoEscape(0);
   Debug("default=$default");
   $str = $cgi->popup_menu(
         -name       => $name,
         -values     => \@dropsort,
         -default    => $default,
         -labels     => \%droplabels,
         -onChange   => $onchange || '',
         -override,
      );
   return($str);
}

# API_AddRegex: Add a regex to the database
# Input:
#	%regexinfo 	A hash with the following fields:
#		name			The name of this regex
#     example     The example string
#     regex       The regex itself
# Return Codes:
#	-1 General Error
#	0	Required Parameter not given
#	>0 The id of the new device
sub API_AddRegex {
   my($d) = shift || (API_SetError('RegexNoHash') && return(0));

   ${$d}{'regex'} =~ s/\\/\\\\/g;
   ${$d}{'regex'} =~ s/\'/\\'/g;		
   my($sql) = "INSERT INTO regex (name,example,regex)".
      "VALUES ('${$d}{'name'}','${$d}{'example'}','${$d}{'regex'}');";

   my($sth) = $db->prepare($sql);
   $sth->execute();
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_AddRegex: Add: $sql:" . $sth->errstr);
      return(-1);
   }

   if(API_AddAudit('regex','add',"name=${$d}{'name'}") < 1) {
      return(-1);
   }

   $sth = $db->prepare("SELECT id FROM regex WHERE name = '${$d}{'name'}' ".
         "AND example = '${$d}{'example'}';");
   $sth->execute;
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_AddRegex: GetID: " . $sth->errstr);
      return(-1);
   }
   my($id) = ($sth->fetchrow_array)[0];
   return($id);
}

# API_UpdRegex: Update a regex in the database
# Input:
#	%regexinfo 	A hash with the following fields:
#     id          The regex id to update
#		name			The name of this regex
#     example     The example string
#     regex       The regex itself
# Return Codes:
#	-1 General Error
#	0	Required Parameter not given
#	1	Success
sub API_UpdRegex {
   my($d) = shift || (API_SetError('RegexUpdNoHash') && return(0));

   if(!${$d}{'id'}) {
      API_SetGenError("A regex id to update was not specified");
      return(-1);
   }

   ${$d}{'regex'} =~ s/\\/\\\\/g;
   ${$d}{'regex'} =~ s/\'/\\'/g;		
   my($sql) = "UPDATE regex SET name='${$d}{'name'}', regex='${$d}{'regex'}', ".
      "example='${$d}{'example'}' WHERE id = '${$d}{'id'}';";

   my($sth) = $db->prepare($sql);
   $sth->execute();
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_UpdRegex: Upd: $sql:" . $sth->errstr);
      return(-1);
   }

   if(API_AddAudit('regex','update',"id=${$d}{'id'},name=${$d}{'name'}") < 1) {
      return(-1);
   }

   return(1);
}

# API_DelRegex: Delete a regex in the database
# Input:
#	$id			The id of the regex to delete
# Returns:
#  -1 General Error
#	0	Required parameter not given
#	1	Successful
sub API_DelRegex {
   my($id) = shift || (API_SetError('RegexDelNoID') && return(0));
   my(@junk);

   my(@data);
   if(API_GetTemplateFieldList([qw(id name)], \@data, [qw(id)],
            [{ 'field' => 'regex',
                  'type'  => '=',
                  'data'  => $id
                  }]) < 1) {
      API_ExitError();
   }

   if(@data) {
      API_SetGenError("The Regex you are trying to delete is still used ".
         "by at least one field.");
      return(-1);
   }

   if(API_GenericDelete($id, 'regex', \@junk, 1) < 1) {
      return(-1);
   }

   if(API_AddAudit('regex','delete',"id=$id") < 1) {
      return(-1);
   }

   return(1)
}

# API_RegexDropdown: Create a <SELECT> of views
# Input:
#	$str			Reference to the string to put the dropdown in
#	$default 	The default regex to be selected
#	$append		The string to append to the <SELECT> tag
# Returns:
#	-1 	General Error
#	0		Required parameter not given
#	1		Successful
sub API_RegexDropdown {
   my($str) = shift || (API_SetError('RegexDropdownNoRef') && return(0));
   my($default) = shift || undef;
   my($append) = shift || undef;

   my($i, @sort, %labels, %parents, @data);
   API_GetRegexList([qw(id name)], \@data, [qw(id)]);

   foreach(@data) {
      push(@sort, ${$_}{'id'});
      $parents{${$_}{'id'}} = 0;
      $labels{${$_}{'id'}} = ${$_}{'name'};
   }

   ${$str} = API_TreeCreateDropdown('regex_select', $default, $append, 
         undef, \@sort, \%parents, 
            \%labels);

   return(1);											
}

# API_RegexCheck:  Check a string against a regex in the database
# Input:
#  $id      The regex id to use
#  $str     A reference to the string to check
# Returns:
#  -1       General Error
#  0        Did NOT match
#  1        Did match
sub API_RegexCheck {
   my($id) = shift || (API_SetError('RegexCheckNoID') && return(-1));
   my($str) = shift || (API_SetError('RegexCheckNoStr') && return(-1));

   my(%regexinfo) = API_GetRegex($id);
   if(!%regexinfo) {
      Debug("ret -1");
      return(-1);
   }
   my($r) = $regexinfo{'regex'};
   Debug("id=$id str=${$str} regex=$regexinfo{'regex'} r=$r");    
   if(${$str} !~ /$r/) {
      Debug("ret 0");    
      return(0);
   }
   Debug("ret 1");    
   return(1);
}

# API_GetRegex: Get regex information
# Input:
#	$id			The id of the regex to get
# Returns:
#	%regexinfo 	A hash with the following fields:
#     id          The regex id to update
#	  name	      The name of this regex
#     example     The example string
#     regex       The regex itself
#	null	    An error occurred
sub API_GetRegex {
   my($id) = shift || (API_SetError('RegexNoID') && return());

   my($sql) = "SELECT * FROM regex WHERE id = '$id';";
   my($sth) = $db->prepare($sql);
   $sth->execute();
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_GetRegex: $sql:" . $sth->errstr);
      return();
   }

   if($sth->rows != 1) {
      API_SetGenError("The requested regex was not found");
      return();
   }
   my(@row) = $sth->fetchrow_array;

   my(%regexinfo) = (
         'id'        => $row[0],
         'name'      => $row[1],
         'example'   => $row[2],
         'regex'     => $row[3],
      );

   return(%regexinfo);
}

# API_GetRegexList: Get a list of regex's
# Input:
#  @fields        The fields to get.  This can be 1 or more of:
#     id          The regex id to update
#		name			The name of this regex
#     example     The example string
#     regex       The regex itself
#  \@ary          An array of hashes containing the requested
#                 information for each location.  The key in 
#                 the hash is the field name and the value is
#                 the data
#  @order         The field ordering (goes after ORDER BY in the SQL)
#  @criteria         A array of hashes of extra criteria to match by.  The 
#                    hash should look like this:
#                       field => <fieldname>
#                       type  => (=,<,>,=<,>=,!,....)
#                       data  => the data to compare
# Returns:
#  -1    General Error
#  0     Required Parameter not given
#  1     Successful
sub API_GetRegexList {
   my($fields) = shift || (API_SetError('GetRegexListNoField') && return(0));
   my($ary) = shift || (API_SetError('GetRegexListNoAry') && return(0));
   my($order) = shift || undef;
   my($criteria) = shift || undef;

   if(API_GenericGetInfo($fields, 'regex', $ary, $order, $criteria) < 1) {
      return(-1);
   }
   return(1);
}

# API_AddTemplate: Add a template to the database
# Input:
#	%tmplinfo 	A hash with the following fields:
#		name			The name of this view
#		descr 		The description of the view
#     objtype     The object type this template is used for
# Return Codes:
#	-1 General Error
#	0	Required Parameter not given
#	>0 The id of the new template
sub API_AddTemplate {
   my($d) = shift || (API_SetError('TemplateNoHash') && return(0));

   my($sql) = "INSERT INTO templates (name,descr,objtype)".
      "VALUES ('${$d}{'name'}','${$d}{'descr'}','${$d}{'objtype'}');";

   my($sth) = $db->prepare($sql);
   $sth->execute();
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_AddTemplate: Add: $sql:" . $sth->errstr);
      return(-1);
   }

   if(API_AddAudit('template','add',"name=${$d}{'name'}") < 1) {
      return(-1);
   }

   $sth = $db->prepare("SELECT id FROM templates WHERE name = '${$d}{'name'}' ".
         "AND descr = '${$d}{'descr'}' ".
         "AND objtype = '${$d}{'objtype'}';");
   $sth->execute;
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_AddTemplate: GetID: " . $sth->errstr);
      return(-1);
   }
   my($id) = ($sth->fetchrow_array)[0];
   return($id);
}

# API_UpdTemplate: Update a location in the database
# Input:
#	%tmplinfo 	A hash with the following fields:
#		id 			 The device id to update
#		name			 The name of this device
#		descr 		 The description of the device
#     fieldorder   A reference to an array containing the order of the fields
#                  that are under this template.  The elements in the array
#                  should be the field id number
# Return Codes:
#	-1 General Error
#	0	Required Parameter not given
#	1	Success

sub API_UpdTemplate {
   my($d) = shift || (API_SetError('TemplateUpdNoHash') && return(0));

   if(!${$d}{'id'}) {
      API_SetGenError("A template id to update was not specified");
      return(-1);
   }

   if(API_TemplateFieldSetSortKeys(${$d}{'fieldorder'}) < 1) {
      return(-1);
   }
   my($sql)="UPDATE templates SET name='${$d}{'name'}',descr='${$d}{'descr'}' ".
      "WHERE id = '${$d}{'id'}';";

   my($sth) = $db->prepare($sql);
   $sth->execute();
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_UpdTemplate: Upd: $sql:" . $sth->errstr);
      return(-1);
   }

   if(API_AddAudit('template','update',"id=${$d}{'id'},name=${$d}{'name'}") <1){
      return(-1);
   }

   return(1);
}

# API_DelTemplate: Delete a template in the database
# Input:
#	$id			The id of the template to delete
# Returns:
#  -1 General Error
#	0	Required parameter not given
#	1	Successful

sub API_DelTemplate {
   my($id) = shift || (API_SetError('TmplDelNoID') && return(0));
   my(@junk);

   my(@data);

   my(%tmplinfo) = API_GetTemplate($id);
   if(!%tmplinfo) {
      return(-1);
   }

   if($tmplinfo{'id'} == $tmplinfo{'objtype'}) {
      API_SetGenError("You cannot delete the default template for an object");
      return(-1);
   }

      # Check to see that this template has no fields assigned
   API_GetTemplateFieldList([qw(id)], \@data, [qw(id)],
      [{ 'field' => 'templateid',
            'type'  => '=',
            'data'  => $id
            }]);
   if(@data) {
      API_SetGenError("The template you are trying to delete has fields ".
         "assigned.  Please delete them first.");
      return(-1);
   }


      # Check to see if any of the objects are still using this template
   if($tmplinfo{'objtype'} == 1) {
      API_GetOwnerList([qw(id)], \@data, [qw(id)],
         [{ 'field'=>'templateid','type'=>'=','data'=>$id}]);
   } elsif ($tmplinfo{'objtype'} == 2) {
      API_GetLocationList([qw(id)], \@data, [qw(id)],
         [{ 'field'=>'templateid','type'=>'=','data'=>$id}]);
   } elsif ($tmplinfo{'objtype'} == 3) {
      API_GetDeviceList([qw(id)], \@data, [qw(id)],
         [{ 'field'=>'templateid','type'=>'=','data'=>$id}]);
   } elsif ($tmplinfo{'objtype'} == 4 || $tmplinfo{'objtype'} == 5) {
      API_GetNetworkList([qw(id)], \@data, [qw(id)],
         [{ 'field'=>'templateid','type'=>'=','data'=>$id}]);
   } else {
      API_SetGenError("The object type does not have a proper delete handler");
      return(-1);
   }

   if(@data) {
      API_SetGenError("The template you are trying to delete is in use.  ".
         "Please delete objects using this template first");
      return(-1);
   }

   if(API_GenericDelete($id, 'templates', \@junk, 1) < 1) {
      return(-1);
   }

   if(API_AddAudit('template','delete',"id=$id") < 1) {
      return(-1);
   }

   return(1)
}


# API_GetTemplate: Get template information
# Input:
#	$id			The id of the template to get
# Returns:
#	%tmplinfo 	A hash with the following fields:
#		id 			The id of this template
#		name			The name of this template
#		descr 		The description of the template
#     objtype     The object type of this template
#	null			An error occurred
sub API_GetTemplate {
   my($id) = shift || (API_SetError('ViewNoID') && return());

   my($sql) = "SELECT id,name,descr,objtype FROM templates WHERE id = '$id';";
   my($sth) = $db->prepare($sql);
   $sth->execute();
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_GetTemplate: $sql:" . $sth->errstr);
      return();
   }

   if($sth->rows != 1) {
      API_SetGenError("The requested template was not found");
      return();
   }
   my(@row) = $sth->fetchrow_array;

   my(%tmplinfo) = (
         'id'        => $row[0],
         'name'      => $row[1],
         'descr'     => $row[2],
         'objtype'   => $row[3],      
      );

   return(%tmplinfo);
}

# API_GetTemplateList: Get a list of view objects and fields
# Input:
#  @fields        The fields to get.  This can be 1 or more of:
#     id             The template ID
#     name           The template name
#     descr          The template description
#     objtype        The template object type
#  \@ary          An array of hashes containing the requested
#                 information for each location.  The key in 
#                 the hash is the field name and the value is
#                 the data
#  @order         The field ordering (goes after ORDER BY in the SQL)
#  @criteria         A array of hashes of extra criteria to match by.  The 
#                    hash should look like this:
#                       field => <fieldname>
#                       type  => (=,<,>,=<,>=,!,....)
#                       data  => the data to compare
# Returns:
#  -1    General Error
#  0     Required Parameter not given
#  1     Successful
sub API_GetTemplateList {
   my($fields) = shift || (API_SetError('GetTmplListNoField') && return(0));
   my($ary) = shift || (API_SetError('GetTmplListNoAry') && return(0));
   my($order) = shift || undef;
   my($criteria) = shift || undef;

   if(API_GenericGetInfo($fields, 'templates', $ary, $order, $criteria) < 1) {
      return(-1);
   }
   return(1);
}


# API_AddTemplateField: Add a field to the database
# Input:
#	%fieldinfo 	A hash with the following fields:
#     templateid     The template this field is assigned to
#     name           The field name
#     descr          The field description
#     required       If the field is required or not (boolean value)
#     regex          The regex id that this field is using
#     minlength      The minimum length of this field
#     maxlength      The maximum length of this field
#     fieldtype      The field type (TEXT, PASSWORD, TEXTAREA, etc..)
#     updatemode     The way this field is updated (APPEND,OVERWRITE)
# Return Codes:
#	-1 General Error
#	0	Required Parameter not given
#	>0 The id of the new field
sub API_AddTemplateField {
   my($d) = shift || (API_SetError('FieldNoHash') && return(0));


   my($sql) = "INSERT INTO fielddefs (templateid,name,descr,required,regex,".
      "minlength,maxlength,sortkey,fieldtype,updatemode) VALUES ".
      "('${$d}{'templateid'}','${$d}{'name'}','${$d}{'descr'}',".
      "'${$d}{'required'}','${$d}{'regex'}','${$d}{'minlength'}',".
      "'${$d}{'maxlength'}','-1','${$d}{'fieldtype'}',".
      "'${$d}{'updatemode'}');";

   my($sth) = $db->prepare($sql);
   $sth->execute();
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_AddTemplateField: Add: $sql:" . $sth->errstr);
      return(-1);
   }

   if(API_AddAudit('templatefield','add',"name=${$d}{'name'}") < 1) {
      return(-1);
   }

   $sth = $db->prepare("SELECT id FROM fielddefs WHERE name = '${$d}{'name'}' ".
         "AND descr = '${$d}{'descr'}' ".
         "AND templateid = '${$d}{'templateid'}';");
   $sth->execute;
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_AddTemplateField: GetID: " . $sth->errstr);
      return(-1);
   }
   my($id) = ($sth->fetchrow_array)[0];
   return($id);
}

# API_UpdTemplateField: Update a field in the database
# Input:
#	%fieldinfo 	A hash with the following fields:
#		id 			   The field id to update
#     name           The field name
#     descr          The field description
#     required       If the field is required or not (boolean value)
#     regex          The regex id that this field is using
#     minlength      The minimum length of this field
#     maxlength      The maximum length of this field
#     fieldtype      The field type (TEXT, PASSWORD, TEXTAREA, etc..)
#     updatemode     The way this field is updated (APPEND,OVERWRITE)
# Return Codes:
#	-1 General Error
#	0	Required Parameter not given
#	1	Success

sub API_UpdTemplateField {
   my($d) = shift || (API_SetError('FieldUpdNoHash') && return(0));

   if(!${$d}{'id'}) {
      API_SetGenError("A field id to update was not specified");
      return(-1);
   }

   my($sql) = "UPDATE fielddefs SET name='${$d}{'name'}',".
      "descr='${$d}{'descr'}', required='${$d}{'required'}', ".
      "regex='${$d}{'regex'}', minlength='${$d}{'minlength'}', ".
      "maxlength='${$d}{'maxlength'}',fieldtype='${$d}{'fieldtype'}',".
      "updatemode='${$d}{'updatemode'}' WHERE id = '${$d}{'id'}';";

   my($sth) = $db->prepare($sql);
   $sth->execute();
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_UpdTemplateField: Upd: $sql:" . $sth->errstr);
      return(-1);
   }

   if(API_AddAudit('templatefield','update',
            "id=${$d}{'id'},name=${$d}{'name'}") < 1) {
      return(-1);
   }

   return(1);
}

# API_DelTemplateField: Delete a field in the database
# Input:
#	$id			The id of the field to delete
# Returns:
#  -1 General Error
#	0	Required parameter not given
#	1	Successful
sub API_DelTemplateField {
   my($id) = shift || (API_SetError('FieldDelNoID') && return(0));
   my(@junk);

   my(@data);
   if(API_GenericDelete($id, 'fielddefs', \@junk, 1) < 1) {
      return(-1);
   }

   if(API_AddAudit('templatefield','delete',"id=$id") < 1) {
      return(-1);
   }

   return(1)
}


# API_GetTemplateField: Get field information
# Input:
#	$id			The id of the template to get
# Returns:
#	%fieldinfo 	A hash with the following fields:
#     id             The field ID
#     templateid     The template this field is assigned to
#     name           The field name
#     descr          The field description
#     required       If the field is required or not (boolean value)
#     regex          The regex id that this field is using
#     minlength      The minimum length of this field
#     maxlength      The maximum length of this field
#     sortkey        The integer showing the sort index of this field
#     fieldtype      The field type (TEXT, PASSWORD, TEXTAREA, etc..)
#     fieldopt       The field type options
#     updatemode     The way this field is updated (APPEND,OVERWRITE)
#	null			An error occurred
sub API_GetTemplateField {
   my($id) = shift || (API_SetError('ViewNoID') && return());

   my($sql) = "SELECT * FROM fielddefs WHERE id = '$id';";
   my($sth) = $db->prepare($sql);
   $sth->execute();
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_GetTemplateField: $sql:" . $sth->errstr);
      return();
   }

   if($sth->rows != 1) {
      API_SetGenError("The requested field was not found");
      return();
   }
   my(@row) = $sth->fetchrow_array;

   my(%fieldinfo) = (
         'id'         => $row[0],
         'templateid' => $row[1],
         'name'       => $row[2],
         'descr'      => $row[3],
         'required'   => $row[4],
         'regex'      => $row[5],
         'minlength'  => $row[6],
         'maxlength'  => $row[7],
         'sortkey'    => $row[8],
         'fieldtype'  => $row[9],
         'fieldopt'   => $row[10],
         'updatemode' => $row[11]
      );

   return(%fieldinfo);
}

# API_GetTemplateFieldList: Get a list of template fields
# Input:
#  @fields        The fields to get.  This can be 1 or more of:
#     id             The field ID
#     templateid     The template this field is assigned to
#     name           The field name
#     descr          The field description
#     required       If the field is required or not (boolean value)
#     regex          The regex id that this field is using
#     minlength      The minimum length of this field
#     maxlength      The maximum length of this field
#     sortkey        The integer showing the sort index of this field
#     fieldtype      The field type (TEXT, PASSWORD, TEXTAREA, etc..)
#     fieldopt       The field type options
#     updatemode     The way this field is updated (APPEND,OVERWRITE)
#  \@ary          An array of hashes containing the requested
#                 information for each location.  The key in 
#                 the hash is the field name and the value is
#                 the data
#  @order         The field ordering (goes after ORDER BY in the SQL)
#  @criteria         A array of hashes of extra criteria to match by.  The 
#                    hash should look like this:
#                       field => <fieldname>
#                       type  => (=,<,>,=<,>=,!,....)
#                       data  => the data to compare
# Returns:
#  -1    General Error
#  0     Required Parameter not given
#  1     Successful
sub API_GetTemplateFieldList {
   my($fields) = shift ||(API_SetError('GetTmplFieldListNoField') && return(0));
   my($ary) = shift || (API_SetError('GetTmplFieldListNoAry') && return(0));
   my($order) = shift || undef;
   my($criteria) = shift || undef;

   if(API_GenericGetInfo($fields, 'fielddefs', $ary, $order, $criteria) < 1) {
      return(-1);
   }
   return(1);
}

# API_TemplateSelectPage:  Output the page to select the template given the
#                          object type
# Input:
#  $objtype    The object type (see API_TemplateGetObjectType)
#  \%pitems    A reference to a hash containing tags to be replaced on the page
# Returns:
#  -1    General Error
#  0     Required parameter not given
#  1     Successful
sub API_TemplateSelectPage {
   my($objtype) = shift || (API_SetError('TmplSelectPageNoObj') && return(0));
   my($p) = shift || undef;

   my(%pitems) = %{$p};

   my(@fields) = qw(r parent netaddr netmask owner location view path status 
                                        netblock mode);
   foreach(@fields) {
      if(!$pitems{$_}) {
         $pitems{$_} = '';
      }
   }

   if(API_TemplateDropdown(\$pitems{'templateselect'},$objtype, undef) < 1) {
      return(-1);
   }
   API_OutputPage('template_select.html',\%pitems);
   return(1);
}

# API_TemplateDropdown:    Create a dropdown of templates given and object type
# Input:
#  \$str       Reference to a string to put the dropdown in
#  $objtype    The object type to create a dropdown of
#  $default    The id of the template to be selected by default
# Returns:
#  -1    General Error
#  0     Required parameter not given
#  1     Successful
sub API_TemplateDropdown {
   my($str) = shift || (API_SetError('TmplDropdownNoStr') && return(0));
   my($objtype) = shift|| (API_SetError('TmplDropdownNoObj') && return(0));
   my($default) = shift || undef;
   my(@data, @sort);

   if(API_GetTemplateList([qw(id name)], \@data, [qw(id)],
            [{ 'field' => 'objtype',
                  'type'  => '=',
                  'data'  => $objtype
                  }]) < 1) {
      return(-1);
   }

   my(%labels);
   foreach(@data) {
      $labels{${$_}{'id'}} = ${$_}{'name'};
      push(@sort, ${$_}{'id'});
   }

   if(!$default) {
      ${$str} = $cgi->popup_menu(
            -name		=>	'template_select',
            -values		=>	\@sort,
            -labels		=>	\%labels
         );
   } else {
      ${$str} = $cgi->popup_menu(
            -name		=>	'template_select',
            -values		=>	\@sort,
            -labels		=>	\%labels,
            -default	=>	$default
         );
   }
   return(1);
}

# API_TemplateGetObjectType:  Returns the string associated with an object type
# Input:
#  $obj     The object type
# Returns:
#  $str     The object type name
#  null     Error occured
sub API_TemplateGetObjectType {
   my($o) = shift || return();
   my(%objtypes) = (
         '1' => 'Owner',
         '2' => 'Location',
         '3' => 'Device',
         '4' => 'Network',
         '5' => 'Address'
      );
   return($objtypes{$o});
}

# API_TemplateFieldSetValue:  Set the value of a specific field
# Input:
#  $id      The id of the object (owner,device,network, etc..)
#  $objtype The id of the object type (1-5)
#  $fieldid The id of the field to set
#  $data    The data to put in the field
# Returns:
#  -1    General Error
#  0     Required parameter not given
#  1     Successful
sub API_TemplateFieldSetValue {
   my($id) = shift || (API_SetError('FieldSetNoId') && return(0));
   my($objtype) = shift || (API_SetError('FieldSetNoObj') && return(0));
   my($fieldid) = shift || (API_SetError('FieldSetNoField') && return(0));
   my($data) = shift || "";
   my($sth, $sql, $noupdate);
   my($d);
   $d = localtime;

   my(%fieldinfo) = API_GetTemplateField($fieldid);
   if(!%fieldinfo) {
      return(-1);
   }

   my($currdata) = API_TemplateFieldGetValue($id, $objtype, $fieldid, 1);
   if($currdata) {
      if($fieldinfo{'updatemode'} == 1) {
         if($fieldinfo{'fieldtype'} ne 'TEXTAREA') {
            if(length($data) > 0) {
               $sql = "UPDATE fielddata SET data='$currdata $data' ".
                  "WHERE id = '$id' AND fieldid = '$fieldid' AND ".
                  "objtype = '$objtype';";
            } else {
               $noupdate = 1;
            }
         } else {
            if(length($data) > 0) {
               $sql = "UPDATE fielddata SET ".
                  "data='$currdata\n-------------------------\n".
                  "Added by $NorthStar::username on $d\n\n$data\n' ".
                  "WHERE id = '$id' AND fieldid = '$fieldid' AND ".
                  "objtype = '$objtype';";
            } else {
               $noupdate = 1;
            }
         }
      } elsif($fieldinfo{'updatemode'} == 2) {
         $sql = "UPDATE fielddata SET data='$data' WHERE id = '$id' AND ".
            "fieldid = '$fieldid' AND objtype = '$objtype';";                
      } else {
         return(-1);
      }

      if(!$noupdate) {
         $sth = $db->prepare($sql);      
         $sth->execute();
         if(API_DBResCheck(\$sth) < 1) {
            API_SetGenError("API_TemplateFieldSetValue: Upd: $sql: " . 
               $sth->errstr);
            return(-1);
         }
      }
   } else {
      if($fieldinfo{'fieldtype'} ne 'TEXTAREA') {
         $sql = "INSERT INTO fielddata VALUES('$id','$objtype','$fieldid',".
            "'$data');";
      } else {
         $sql = "INSERT INTO fielddata VALUES('$id','$objtype','$fieldid',".
            "'Added by $NorthStar::username on $d\n\n$data\n');";
      }
      $sth = $db->prepare($sql);
      $sth->execute;
      if(API_DBResCheck(\$sth) < 1) {
         API_SetGenError("API_TemplateFieldSetValue: Add: $sql: ".$sth->errstr);
         return(-1);
      }
   }
}


# API_TemplateFieldGetValue:  Get the value of a specific field
# Input:
#  $id      The id number of the object
#  $objtype The id number of the object type
#  $fieldid The id number of the field
#  $mode    The mode to return in
#     0 or undef  <PRE> tags are added
#     1           <PRE> tags are not added    
# Returns:
#     $str  The data
#     null  General Error
sub API_TemplateFieldGetValue {
   my($id) = shift || (API_SetError('FieldGetValNoID') && return());
   my($objtype) = shift || (API_SetError('FieldGetValNoObj') && return());
   my($fieldid) = shift || (API_SetError('FieldGetValNoField') && return());
   my($mode) = shift || undef;

   my(%fieldinfo) = API_GetTemplateField($fieldid);
   if(!%fieldinfo) {
      return();
   }

   my($sql) = "SELECT data FROM fielddata WHERE id = '$id' ".
      "AND fieldid = '$fieldid' AND objtype='$objtype';";
   my($sth) = $db->prepare($sql);
   $sth->execute;
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_TemplateFieldGetValue: $sql: " . $sth->errstr);
   }

   if($sth->rows != 1) {
      return("");
   }

   my(@row) = $sth->fetchrow_array;	 
   if(!$mode) {
      if($fieldinfo{'fieldtype'} eq 'TEXTAREA') {
         return("<PRE>" . $row[0] . "</PRE>");
      } else {
         return($row[0]);
      }
   } else {
      return($row[0]);
   }

}

# API_TemplateFieldDeleteAllValues:  Delete all the fields for a given 
#                                    object id and object type
# Input:
#  $id      The object id
#  $objtype The object type
# Returns:
#  -1    General Error
#  0     Required parameter not given
#  1     Successful
sub API_TemplateFieldDeleteAllValues {
   my($id) = shift || (API_SetError('FieldDelAllNoID') && return(0));
   my($objtype) = shift || (API_SetError('FieldDelAllNoObj') && return(0));
   my($sth) = $db->prepare("DELETE FROM fielddata WHERE id = '$id' ".
         "AND objtype = '$objtype';");
   $sth->execute;
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_TemplateFieldDeleteAllValues: " . $sth->errstr);
      return(-1);
   }
   return(1);
}

# API_TemplateFieldIsRequired:   Return whether a field is required or not
# Input:
#  $id      The field id
# Returns:
#  -1    General Error
#  0     Not Required
#  1     Required
sub API_TemplateFieldIsRequired {
   my($id) = shift || (API_SetError('FieldRequiredNoID') && return(-1));

   my($sth) = $db->prepare("SELECT required FROM fielddefs WHERE id = '$id';");
   $sth->execute;
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_TemplateFieldIsRequired: " . $sth->errstr);
      return(-1);
   }
   my(@row) = $sth->fetchrow_array;	 

   if($row[0] == 1) {
      return(1);
   } else {
      return(0);
   }
}

# API_TemplateFieldGetExample:   Return the regex example for a field
# Input:
#  $id      The field id
# Returns:
#  $str     The example
#  null     Error
sub API_TemplateFieldGetExample {
   my($id) = shift || (API_SetError('FieldGetExampleNoID') && return());

   my($sth) = $db->prepare("SELECT regex FROM fielddefs WHERE id = '$id';");
   $sth->execute;
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_TemplateFieldGetExample: " . $sth->errstr);
      return();
   }
   my(@row) = $sth->fetchrow_array;	
   my(%regexinfo) = API_GetRegex($row[0]);
   if(!%regexinfo) { 
      return(-1);
   } 
   return($regexinfo{'example'});
}

# API_TemplateFieldCheckInput:  Check data against the regex defined for a field
# Input:
#  $id      The field id
#  \$str    Reference to the string to check
# Returns:
#  -1       General Error
#  0        Required parameter not given
#  1        Success
#  2        The field was required but had no data
#  3        The data was too short
#  4        The data was too long
#  5        The data was not correctly formatted
sub API_TemplateFieldCheckInput {
   my($id) = shift || (API_SetError('FieldCheckNoID') && return(0));
   my($str) = shift || (API_SetError('FieldCheckNoStr') && return(0));

   my($sth) = $db->prepare("SELECT required,minlength,maxlength,regex ".
         "FROM fielddefs WHERE id = '$id';");
   $sth->execute;
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_TemplateFieldCheckInput: " . $sth->errstr);
      return(-1);
   }
   my(@row) = $sth->fetchrow_array;	 

   if($row[0] == 1 && !${$str}) {
      return(2);
   }

   if(${$str}) {		
      if(length(${$str}) < $row[1]) {
         return(3);
      }

      if(length(${$str}) > $row[2]) {
         return(4);
      }

      if(API_RegexCheck($row[3], $str) < 1) {
         return(5);
      }
   }

   return(1);
}

# API_TemplateFieldDropdown:  Create a <SELECT> of fields
# Input:
#  \$str    The string to put the dropdown in
#  $default The field id to be selected by default
#  $id      The template id to get fields from
#  $size    The size of the SELECT
# Returns:
#  -1    General Error
#  0     Required parameter not given
#  1     Successful
sub API_TemplateFieldDropdown {
   my($str) = shift || (API_SetError('FieldDropdownNoStr') && return(0));
   my($default) = shift || undef;
   my($id) = shift || undef;
   my($size) = shift || 1;
   my(@data);
   if(!$id) {
      if(API_GetTemplateFieldList([qw(id name)], \@data, [qw(id)]) < 1) {
         return(-1);
      }
   } else {
      if(API_GetTemplateFieldList([qw(id name)], \@data, [qw(sortkey)],
               [{ 'field' => 'templateid',
                     'type'  => '=',
                     'data'  => $id
                     }]) < 1) {
         return(-1);
      }
   }

   my(%g);
   my(@order);
   foreach(@data) {
      push(@order, ${$_}{'id'});
      $g{${$_}{'id'}} = ${$_}{'name'};
   }

   if(!$default) {
      ${$str} = $cgi->popup_menu(
            -name	=>	'field_select',
            -values =>	\@order,
            -labels	=>	\%g,
            -size	=>	$size
         );
   } else {
      ${$str} = $cgi->popup_menu(
            -name	=>	'field_select',
            -values =>	\@order,
            -labels	=>	\%g,
            -size	=>	$size
            -default=>	$default
         );
   }
   return(1);
}

# API_TemplateFieldSetSortKeys:  Set the field ordering of the specified fields
# Input:
#  \@ary       An array of field id's in the order that they should be set
# Returns:
#  -1    General Error
#  0     Required parameter not given
#  1     Successful
sub API_TemplateFieldSetSortKeys {
   my($ary) = shift || (API_SetError('TmplFieldSetSort') && return(0));
   my($i) = 0;

   foreach(@{$ary}) {      
      my($sql) = "UPDATE fielddefs SET sortkey = '$i' WHERE id = '$_';";
      my($sth) = $db->prepare($sql);
      $sth->execute;
      if(API_DBResCheck(\$sth) < 1) {
         API_SetGenError("API_TemplateFieldSetSortKeys: $sql: " . $sth->errstr);
         return(-1);
      }
      $i++;
   }
   return(1);
}

# API_TemplateFieldGetAssignedFields:  Get the fields assigned to a template
# Input:
#  $id      The template id
# Returns:
#  %fields  A hash where the key is field id and the value is the field name
#  null     Error
sub API_TemplateFieldGetAssignedFields {
   my($id) = shift || (API_SetError('FieldGetAssigned') && return());
   my(%r, @data);

   if(API_GetTemplateFieldList([qw(id name)], \@data, [qw(id)],
            [{ 'field' => 'templateid',
                  'type'  => '=',
                  'data'  => $id
                  }]) < 1) {
      return();
   }

   foreach(@data) {
      $r{${$_}{'id'}} = ${$_}{'name'};
   }

   return(%r);
}

# API_TemplateFieldGetAssignedFieldsOrder:  Get the order of fields assigned 
#                                           to a template
# Input:
#  $id      The template id
# Returns:
#  @order  An array with the id's of the fields in order
#  null     Error
sub API_TemplateFieldGetAssignedFieldsOrder {
   my($id) = shift || (API_SetError('FieldGetAssignedOrder') && return());
   my(@r, @data);

   if(API_GetTemplateFieldList([qw(id)], \@data, [qw(sortkey)],
            [{ 'field' => 'templateid',
                  'type'  => '=',
                  'data'  => $id
                  }]) < 1) {
      return();
   }

   foreach(@data) {
      push(@r, ${$_}{'id'});
   }

   return(@r);
}

# API_TemplateFieldTypeDropdown:  Create of dropdown of field types
# Input:
#  \$str    A reference to a string to put the dropdown in
#  $default The default value to select
# Returns:
#  0     Required parameter not given
#  1     Successful
sub API_TemplateFieldTypeDropdown {
   my($str) = shift || (API_SetError('FieldTypeDropdown') && return(0));
   my($default) = shift || undef;
   my(@g) = qw(TEXT PASSWORD TEXTAREA);

   if(!$default) {
      ${$str} = $cgi->popup_menu(
            -name		=>	'fieldtype_select',
            -values		=>	\@g
         );
   } else {
      ${$str} = $cgi->popup_menu(
            -name		=>	'fieldtype_select',
            -values		=>	\@g,
            -default	=>	$default
         );
   }
   return(1);
}

# API_TemplateFieldUpdateDropdown:  Create of dropdown of update types
# Input:
#  \$str    A reference to a string to put the dropdown in
#  $default The default value to select
# Returns:
#  0     Required parameter not given
#  1     Successful
sub API_TemplateFieldUpdateDropdown {
   my($str) = shift || (API_SetError('FieldUpdateDropdown') && return(0));
   my($default) = shift || undef;;

   my(%u) = (
         '1'	=>	'APPEND',
         '2'	=>	'OVERWRITE'
      );
   my(@o) = qw(2 1);
   if(!$default) {
      ${$str} = $cgi->popup_menu(
            -name		=>	'updatemode_select',
            -values		=>	\@o,
            -labels     =>  \%u
         );
   } else {
      ${$str} = $cgi->popup_menu(
            -name		=>	'updatemode_select',
            -values		=>	\@o,
            -labels     =>  \%u,
            -default	=>	$default
         );
   }
   return(1);
}

# API_TemplateFieldUpdateName: The string associated with an update mode
# Input:
#  $id      The update mode id
# Returns:
#  $str     The name of the update mode
sub API_TemplateFieldUpdateName {
   my($id) = shift;
   my(%u) = (
         '1'	=>	'APPEND',
         '2'	=>	'OVERWRITE'
      );

   return($u{$id});
}

# API_TemplateFieldCreateHTML:   Create the html for a field input
# Input:
#  $id      The field id
#  $val     The current value of the field
# Returns:
#  $str     The html
#  null     Error 
sub API_TemplateFieldCreateHTML {
   my($id) = shift;
   my($val) = shift;
   my($r, $x);

   my(%fieldinfo) = API_GetTemplateField($id);
   if(!%fieldinfo) {
      return();
   }

   my($ft) = $fieldinfo{'fieldtype'};
   my($um) = $fieldinfo{'updatemode'};
   if($um == 1) {
      $r = "<PRE>$val</PRE><BR><b>Enter text to add:</b><br>";
      $val = "";
   } else {
      $r = "";
   }

   if($ft eq 'TEXT') {
      $x = "<INPUT TYPE=\"TEXT\" NAME=\"$id\" VALUE=\"$val\">";
   } elsif($ft eq 'PASSWORD') {
      $x = "<INPUT TYPE=\"PASSWORD\" NAME=\"$id\" VALUE=\"$val\">";
   } elsif($ft eq 'TEXTAREA') {
      $x = "<TEXTAREA COLS=60 ROWS=5 NAME=\"$id\">$val</TEXTAREA>";
   } else {
      $x = "<INPUT TYPE=\"TEXT\" NAME=\"$id\" VALUE=\"$val\">";
   }

   $r .= $x;
   return($r);
}

# API_TemplateCheckSingle: check to see if only a single template
#                  is defined in the system.  If only
#                  one template exists return it's ID else
#                  return 0
sub API_TemplateCheckSingle {
   my($objtype) = shift || (API_SetError('TmplCheckSingle') && return(-1));
   my($sth) = $db->prepare("SELECT id FROM templates WHERE objtype = '$objtype'");;
   $sth->execute();
   API_DBResCheck(\$sth, 'API_TemplateCheckSingle: getcount', 1);

   if($sth->rows == 1) {
      my($id) = ($sth->fetchrow_array)[0];
      Debug("1 template found, returning $id", 5);        
      return($id);
   }
   Debug(">1 templates not found, returning 0", 5);        
   return(0);
}

sub ConfigGetSection {
   my $sec = shift;
   my(%t);

   my($h) = $NorthStar::CONFIG{$sec};
   while(my($k,$v) = each %$h) {
      $t{$k} = $v;
   }
   return(%t);
}

sub ConfigGetValue {
   if(scalar(@_) == 3) {
      return($NorthStar::CONFIG{$_[0]}{$_[1]}{$_[2]});
   } elsif(scalar(@_) == 2) {
      return($NorthStar::CONFIG{$_[0]}{$_[1]});
   } else {
      return(undef);
   }
}

sub ConfigInit {
   my $cf = shift;
   my($cs, $csv, $cn, $cv, @j, $x);
   open(_CONFIG, "<$cf") || die("config file open failed: $cf: $!\n");
   while(<_CONFIG>) {
      chomp;
      s/^[\s\t]+//g; #strip leading whitespace
      next if /^#/;
      next if /^$/;
      s/\s+$//g; #strip trailing whitespace

      if(/\#/) {
         /(.*)\#.*/;
         $x = $1;
      } else {
         $x = $_;
      }

                                # given:
                                # <Section SectionVariable>
                                #                Name = Value
                                # </Section>
                                # $cs  = Section                Config Section        
                                # $csv = SectionVariable        Config Section Variable 
                                # $cn  = Name                   Config Name
                                # $cv  = Value                  Config Variable
                                # 
                                # $csv may not be defined if a SectionVariable was not present
      if($x =~ /^<(.*)>$/ && $x !~ /^<\/.*>/) {           # handler for section starts <Section>
         if(!defined($cs)) {              # check to see if the previous section was closed with </Section> 
            $cs = $1;
         } else {
            die("syntax error in configuration file \"$cf\" at line $.: " .
                  "previous configuration section not closed (maybe you forgot </$cs>)\n");
         }

                                                # sub-handler for sections with a variable <Section SectionVariable>
         if($x =~ /^<(.*) (.*)>$/ && (@j = split(/\s+/, $x)) == 2) {
            $cs = $1;
            $csv = $2;
         } elsif($x =~ /^<(.*)>$/) { # dont clobber sections with no SectionVariable (<Section> ..)
            $cs = $1;
         } else {
            die("syntax error in configuration file \"$cf\" at line $.: " .
                  "configuration sections may only have one parameter (<Section Parameter>)\n");
         }
         next;
      } elsif($x =~ /^<\/(.*)>$/) {            #handler for section closes </Section>
         if(defined($cs)) {         #check to see if this section was ever opened
            undef($cs);
            undef($csv);
         } else {
            $cs = $1;          #set section name for error message
            $cs =~ s/\///g;    #strip leading '/'
            die("syntax error in configuration file \"$cf\" at line $.: " .
                  "configuration section not opened (maybe you forgot <$cs>)\n");
         }
         next;
      } else {
         if($x =~ /(.*) \"(.*)\"/) {               #process line "Name = "Value In Here"
            $cn = $1;                                
            $cv = $2;
            $cv =~ s/\\(.)/$1/g;              #normalize escaped charecters like: \"
         } elsif((@j = split(/\s+/, $x)) > 2) {    #process line "Name = Value1 Value2..."
            $cn = shift(@j);
            $cv = join(',', @j);              #make a comma delimited list
         } elsif((@j = split(/\s+/, $x)) == 2) {   #process line "Name = Value"
            $cn = $j[0];
            $cv = $j[1];
         } else {
            die("syntax error in configuration file \"$cf\" at line $.\n");
         }

         if($csv) { #if SectionVariable was set this as CONFIG->Section->SectionVariable->Name=Value
                #$self->Debug("add value: CONFIG->$cs->$csv->$cn=$cv");
            $NorthStar::CONFIG{$cs}{$csv}{$cn} = $cv;
         } else {   #else set this value under CONFIG->Section->Name=Value
                #$self->Debug("add value: CONFIG->$cs->$cn=$cv");
            $NorthStar::CONFIG{$cs}{$cn} = $cv;
         }

         next;
      }


   }
                #$self->Debug("closing config file");
   close(_CONFIG) || die("close failed: $cf: $!\n");

   return 1;
}

sub ConfigPrint {
   if(!defined(%NorthStar::CONFIG)) {
      return;
   }

   while(my($k,$v) = each %NorthStar::CONFIG) {
      print STDERR "$k\n";
      while(my($y,$z) = each %$v) {
         if(ref($z) eq "HASH") {
            print STDERR " $y\n";
            while(my($a,$b) = each %$z) {
               print STDERR "  $a=$b\n";
            }
         } else {
            print STDERR " $y=$z\n";
         }
      }
      print STDERR "\n";
   }
}

sub Debug {
   my($d) = ConfigGetValue('Global','Debug');
   if($d > 0) {
      my($m) = shift;
      my($l) = shift || 0;
      my(@x) = caller(1);
      if($l <= $d) {
         print STDERR "[$x[3]] $m\n";
      }
   }
   return();
}

sub Die {
   die(@_);
}

sub ModulesInit {
   Debug("initializing modules");
   my($i, %mi, $mn);
   my(@coremodules) = qw(CORE Network Owner Device Location Search User
                          Session Template Regex View);


   foreach(@coremodules) {
      my($i);
      if($_ eq "CORE") {
         $i = \&NorthStar::init;
      } else {
         $i = "NorthStar::" . $_ . "::init";
      }
      %mi = &$i();
      $mn = $_;
      $i = $mi{'export_hooks'};
      foreach(@$i) {
         ModuleAddHook($mn, $_);
      }

      $i = $mi{'user_hooks'};
      while(my($k,$v) = each %$i) {
         ModuleAddUserHook($mn, $k, $v);
      }

      $i = $mi{'import_hooks'};
      while(my($k,$v) = each %$i) {
         while(my($a,$b) = each %$v) {
            ModuleRegisterHook($mn, $k, $a, $b);
         }
      }
   }

   my(@modules) = split(/\,/, ConfigGetValue('Global','UseModules'));
   foreach(@modules) {
      require "NorthStar/Modules/$_.pm";

      my($i) = "NorthStar::Modules::" . $_ . "::init";
      %mi = &$i();
      $mn = $mi{'module_info'}{'modulename'};
      $i = $mi{'export_hooks'};
      foreach(@$i) {
         ModuleAddHook($mn, $_);
      }

      $i = $mi{'user_hooks'};
      while(my($k,$v) = each %$i) {
         ModuleAddUserHook($mn, $k, $v);
      }

      $i = $mi{'import_hooks'};
      while(my($k,$v) = each %$i) {
         while(my($a,$b) = each %$v) {
            ModuleRegisterHook($mn, $k, $a, $b);
         }
      }
   }
}

sub ModuleDispatch {
   my($mod) = shift;
   my($hook) = shift;
   my($ref) = shift;
   my(%mh, $k, $v);
   Debug("attempting to dispatch calls for $mod->$hook", 7);
   if(!defined($NorthStar::_HOOKDB{$mod}{$hook})) {
      Die("The module $mod tryed to dispatch calls for $hook which is not registered");
   }
   %mh = ModuleGetRegisteredHooks($mod, $hook);
   while(my($k,$v) = each %mh) {
      Debug("calling $v which registered hook '$hook'", 7);
      &{$v}($ref);
   }	
}

sub ModuleAddHook {	
   my $mn = shift;
   my $hn = shift;

   if(!defined($NorthStar::_HOOKDB{$mn}{$hn})) {
      $NorthStar::_HOOKDB{$mn}{$hn} = 1;
      Debug("registered hook for module \"$mn\": $hn", 7);
   } else {
      if(!$ENV{'MOD_PERL'}) {
         Die("the module $mn tryed to add hook $hn which already exists"); 
      }
   }
}

sub ModuleAddUserHook {
   my $mn = shift;
   my $hn = shift;
   my $cn = shift;

   if(!defined($NorthStar::_USERHOOKDB{$hn})) {
      $NorthStar::_USERHOOKDB{$hn} = $cn;
      Debug("registered USER hook for module \"$mn\": $hn->$cn", 7);
   } else {
      if(!$ENV{'MOD_PERL'}) {
         Die("the module $mn tryed to add USER hook $hn->$cn which already exists");
      }
   }
}

sub ModuleRegisterHook {
   my $mn = shift;
   my $hm = shift;
   my $hn = shift;
   my $cn = shift;

   if(!defined($NorthStar::_REGHOOKS{$hm}{$hn}{$mn}) && $NorthStar::_HOOKDB{$hm}{$hn}) {
      $NorthStar::_REGHOOKS{$hm}{$hn}{$mn} = $cn;
      Debug("registered hook for module $mn for internal hook $hm->$hn to ".
         "call $cn", 7);
   } else {
      if(!$ENV{'MOD_PERL'}) {
         Die("module $mn tryed to register a hook $hm->$hn to call $cn. This ".
            "hook may already be registered or the module $hm has not ".
            "exported hook $hn");
      }
   }
}

sub ModuleGetRegisteredHooks {
   my $mn = shift;
   my $hn = shift;
   my($i, %mh);

   $i = $NorthStar::_REGHOOKS{$mn}{$hn};
   while(my($k,$v) = each %$i) {
      $mh{$k} = $v;
      Debug("adding hook for module $mn->$hn to $k->$v", 7);
   }

   return(%mh);
}

# API_GenericRecursiveUpdate:  recursively update the given field(s) in the 
#                              given table
# Input:
#	$id			The id number to update
#	$table		The table to update
#  %fields     Hash of the fields to update.  The key is the field name
#              and the value is the new value
#  @criteria   A array of hashes of extra criteria to match by.  The 
#              hash should look like this:
#                 field => <fieldname>
#                 type  => (=,<,>,=<,>=,!=,....)
#                 data  => the data to compare
#  $aryref		Reference to array to add updated id's to
# Returns:
#  -1 General Error
#	0	Required parameter not given
#	1	Successful
sub API_GenericRecursiveUpdate {
   my($id) = shift || return(0);
   my($table) = shift || return(0);
   my($fields) = shift || return(0);
   my($criteria) = shift || undef;
   my($aryref) = shift || return(0);
   my($origid) = shift || $id;
   Debug("starting id=$id table=$table fields=$fields criteria=$criteria");
   my($i);
   my($sth) = $db->prepare("SELECT id FROM $table WHERE parent = '$id';");
   $sth->execute();
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_GenericRecursiveUpdate: $table($id): " . $sth->errstr);
      return(-1);
   }

   my($updatestr) = undef;
   while(my($k,$v) = each %{$fields}) {
      if($updatestr) {
         $updatestr .= ", $k='$v'";
      } else {
         $updatestr = "$k='$v'";
      }
   }
   Debug("updatestr=$updatestr");

   my($criteriastr) = undef;
   if($criteria) {
      foreach(@{$criteria}) {
         $criteriastr .= "AND ${$_}{'field'} ${$_}{'type'} '${$_}{'data'}' ";
      }
   }
   Debug("criteriastr=$criteriastr");

   for($i = 0; $i < $sth->rows; $i++) {
      my($xid) = ($sth->fetchrow_array)[0];
      my($sth2) = $db->prepare("UPDATE $table SET $updatestr ".
            "WHERE id = '$xid' $criteriastr");
      $sth2->execute();
      if(API_DBResCheck(\$sth2) < 1) {
         API_SetGenError("API_GenericRecursiveUpdate: UpdChild: $table($id): " . $sth->errstr);
         return(-1);
      }

      push(@{$aryref}, $xid);
      if(API_GenericRecursiveUpdate($xid,$table,$fields,
               $criteria,$aryref,$id) < 1) {
         return(-1);
      }		
   }

   if($origid == $id) {
      my($sth2) = $db->prepare("UPDATE $table SET $updatestr ".
            "WHERE id = '$id' $criteriastr");
      $sth2->execute();
      if(API_DBResCheck(\$sth2) < 1) {
         API_SetGenError("API_GenericRecursiveUpdate: UpdParent: $table($id): " . $sth->errstr);
         return(-1);
      }
      push(@{$aryref}, $id);
   }
   return(1);
}



# API_GenericGetInfo: Get the specified fields from the specified table
#                     and returns an array of hashes of each object
# Input:
#  @fields           The fields to get
#  $table            The table to query
#  \@ary             The array to put the data in
#  @order            The field ordering
#  @criteria         A array of hashes of extra criteria to match by.  The 
#                    hash should look like this:
#                       field => <fieldname>
#                       type  => (=,<,>,=<,>=,!,....)
#                       data  => the data to compare
# Returns:
#  -1    General Error
#  0     Required parameter not given
#  1     Successful
sub API_GenericGetInfo {
   my($fields) = shift || (API_SetError('GenGetInfoNoFields') && return(0));
   my($table) = shift || (API_SetError('GenGetInfoNoTable') && return(0));
   my($ary) = shift || (API_SetError('GenGetInfoNoRef') && return(0));
   my($order) = shift || undef;
   my($criteria) = shift || undef;

   my($fieldstr) = join(',', @{$fields});
   Debug("get fields: $fieldstr", 8);

   my($orderstr) = "";
   if($order) {
      $orderstr = "ORDER BY " . join(',', @{$order});
   }

   my($criteriastr) = undef;
   if($criteria) {
      foreach(@{$criteria}) {
         $criteriastr .= "AND ${$_}{'field'} ${$_}{'type'} '${$_}{'data'}' ";
      }
   }

   my($sql) = "SELECT $fieldstr FROM $table WHERE id != 0 $criteriastr $orderstr";
   Debug("sql=$sql");
   my($sth) = $db->prepare($sql);
   $sth->execute();
   if(API_DBResCheck(\$sth)  < 1) {
      API_SetGenError("API_GenericGetInfo: $table($fieldstr): " . $sth->errstr);
      return(-1);
   }

   my($i);
   for($i = 0; $i < $sth->rows; $i++) {
      my(@row) = $sth->fetchrow_array();
      Debug("got row: " . join(',', @row), 8);
      push(@{$ary}, API_BuildHash($fields, \@row));
   }
   return(1);
}

# API_BuildHash: Build a hash given an array of keys and values
# Input:
#  \$keys      An array of key names
#  \$vals      An array of values (in the same order as the keys)
# Returns:
#  \%hash      The reference to an anonymous hash
#  null        Required parameter not given
sub API_BuildHash {
   my($keys) = shift || return();
   my($vals) = shift || return();
   my(@tempval) = reverse(@{$vals});

   Debug("keys=" . join(',',@{$keys}) . " vals=" . join(',', @tempval), 8);
   my($i);
   my($temp) = {};
   foreach(@{$keys}) {
      my($val) = pop(@tempval);
      Debug("adding: $_=>$val");
      ${$temp}{$_} = $val;
   }
   return($temp);
}

# API_GenericDelete: Dispatches a delete request to the right routines
#							based on the database in use (Postgres,MySQL,...)
# Input:
#	$id		The id to delete
#	$table	The table to delete from
#	$aryref	An array reference to stick deleted id's in
#  $mode    The mode to run in
#     0 or undef     The object type being deleted is nestable (has children)
#     1              The object type being deleted isn't nestable (owners,..)
sub API_GenericDelete {
   my($id) = shift || return(0);
   my($table) = shift || return(0);
   my($aryref) = shift || return(0);
   my($mode) = shift || 0;

   if($mode == 1) {
      my($sth) = $db->prepare("DELETE FROM $table WHERE id = '$id'");
      $sth->execute;
      if(API_DBResCheck(\$sth) < 1) {
         API_SetGenError("API_GenericDelete: NoChildren: $table($id): " . $sth->errstr);
         return(-1);
      }
      push(@{$aryref}, $id);
      return(1);
   }

   if(ConfigGetValue('Database','DatabaseDriver') =~ /mysql/i) {
      if(API_GenericRecursiveDelete($id, $table, $aryref) < 1) {
         return(-1);
      }
   } else {
      if(API_RecursiveGetChildren($id, $table, $aryref) < 1) {
         return(-1);
      } 

      my($sth) = $db->prepare("DELETE FROM $table WHERE id = '$id'");
      $sth->execute;
      if(API_DBResCheck(\$sth) < 1) {
         API_SetGenError("API_GenericDelete: $table($id): " . $sth->errstr);
         return(-1);
      }
   }	
   return(1);
}


# API_GenericRecursiveDelete:  recursively deletes the given id and all of its
#                              children from a table (use with DB's that don't
#							   support CASCADE ON DELETE....)
# Input:
#	$id			The id number to delete
#	$table		The table to delete from
#  $aryref		Reference to array to add deleted id's to
# Returns:
#  -1 General Error
#	0	Required parameter not given
#	1	Successful
sub API_GenericRecursiveDelete {
   my($id) = shift || return(0);
   my($table) = shift || return(0);
   my($retary) = shift || return(0);
   my($origid) = shift || $id;

   my($i);
   my($sth) = $db->prepare("SELECT id FROM $table WHERE parent = '$id';");
   $sth->execute();
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_GenericRecursiveDelete: $table($id): " . $sth->errstr);
      return(-1);
   }

   for($i = 0; $i < $sth->rows; $i++) {
      my($xid) = ($sth->fetchrow_array)[0];
      my($sth2) = $db->prepare("DELETE FROM $table WHERE id = '$xid'");
      $sth2->execute();
      if(API_DBResCheck(\$sth2) < 1) {
         API_SetGenError("API_GenericRecursiveDelete: DelChild: $table($id): " . $sth->errstr);
         return(-1);
      }

      push(@{$retary}, $xid);
      if(API_GenericRecursiveDelete($xid,$table,$retary, $id) < 1) {
         return(-1);
      }
   }

   if($origid == $id) {
      my($sth2) = $db->prepare("DELETE FROM $table WHERE id = '$id'");
      $sth2->execute();
      if(API_DBResCheck(\$sth2) < 1) {
         API_SetGenError("API_GenericRecursiveDelete: DelParent: $table($id): " . $sth->errstr);
         return(-1);
      }
      push(@{$retary}, $id);
   }

   return(1);
}

# API_RecursiveGetChildren:  recursively get all children for an id from a table
# Input:
#	$id			The id number to delete
#	$table		The table to delete from
#  $aryref		Reference to array to add deleted id's to
# Returns:
#  -1 General Error
#	0	Required parameter not given
#	1	Successful
sub API_RecursiveGetChildren {
   my($id) = shift || return(0);
   my($table) = shift || return(0);
   my($aryref) = shift || return(0);
   my($origid) = shift || $id;

   my($i);
   my($sth) = $db->prepare("SELECT id FROM $table WHERE parent = '$id';");
   $sth->execute();
   if(API_DBResCheck(\$sth) < 1) {
      API_SetGenError("API_RecursiveGetChildren: $table($id): " . $sth->errstr);
      return(-1);
   }

   for($i = 0; $i < $sth->rows; $i++) {
      my($xid) = ($sth->fetchrow_array)[0];
      push(@{$aryref}, $xid);
      if(API_RecursiveGetChildren($xid, $table, $aryref, $id) < 1) {
         return(-1);
      }
   }

   if($origid == $id) {
      push(@{$aryref}, $id);
   }

   return(1);
}


# API_SetError: Set the API Error Message with an error id
# Input:
#	$errorid		The id of the error message in %errormsg in API_GetErrorStr	
# Return Codes:
#  1	Success
sub API_SetError {
   my($errorid) = shift || return();
   Debug("****** setting error $errorid", 7);
   $NorthStar::API::ErrorMsg = API_GetErrorStr($errorid);
   return(1);
}

# API_SetGenError: Set the API Error Message with a general string
# Input:
#	$errormsg 	The error message to set
# Return Codes:
#	1	Success
sub API_SetGenError {
   my($errormsg) = shift || return();
   Debug("****** setting gen error $errormsg", 7);
   $NorthStar::API::ErrorMsg = $errormsg;
   return(1);
}


# API_GetError: Get the last error message
# Input:
#	NONE
# Return Codes:
#	string	The error message
sub API_GetError {
   return($NorthStar::API::ErrorMsg);
}


# API_GetErrorStr: Get the error message associated with a specified id
# Input:
#	$errorid	The id of the error message
# Return Codes:
#	string	The error message or "ERROR ID NOT FOUND" if id is not present
sub API_GetErrorStr {   
   my($errorid) = shift || return();

   my(%errormsg) = (
         'AuditNoSection'	=> 'API_AddAudit: Audit Section not provided',
         'AuditNoAction'	=> 'API_AddAudit: Audit Action not provided',
         'AuditNoMsg'		=> 'API_AddAudit: Audit Message not provided',
         'DeviceNoHash' 	=> 'API_AddDevice: Device info hash was not provided',
         'DeviceUpdNoHash'	=> 'API_UpdDevice: Device info hash was not provided',
         'DeviceNoID' 		=> 'API_GetDevice: Device id was not provided',
         'DeviceDelNoID' 	=> 'API_DelDevice: Device id was not provided',
         'DeviceDelNoRef' 	=> 'API_DelDevice: Array reference was not provided',
         'DeviceDropdownNoRef'=>'API_DeviceDropdown: String reference was not provided',
         'DBConnectNoHash' => 'API_DBConnect: DB info hash was not provided',
         'DBConnectNoFh'	=> 'API_DBConnect: The database filehandle was not provided',
         'DBConnectNoDriver'=> 'API_DBConnect: The database driver was not provided',
         'DBConnectNoHost'	=> 'API_DBConnect: The database hostname was not provided',
         'DBConnectNoPort'	=> 'API_DBConnect: The database port number was not provided',
         'DBConnectNoDB'	=> 'API_DBConnect: The database name was not provided',
         'DBConnectNoUser'	=> 'API_DBConnect: The database username was not provided',
         'DBConnectNoPass'	=> 'API_DBConnect: The database password was not provided',
         'DBResCheckNoRef' => 'API_DBResCheck: A statment handle reference was not provided',
         'CheckFieldNoField'=>'API_CheckField: A field was not given',
         'CheckFieldNoType'=> 'API_CheckField: A regex type was not given',

      );

   if($errormsg{$errorid}) {
      return($errormsg{$errorid});
   } else {
      return("ERROR ID NOT FOUND ($errorid)");
   }
}
return 1;   
